C:::::      ,,,,,REDUCE.....
C
C
C
C**********************************************************************
C        1         2         3         4         5         6         7*
C**********************************************************************
C**                                                                 ***
C**                                                                 ***
      subroutine REDUCE(ROUTIN,swork,string,lstr,myproc,
     &                  nproc,tids,nmsg)
C**                                                                 ***
C**                                                                 ***
C**********************************************************************
C**                                                                 ***
C**      R E D U C E    performs the reduce operation on the        ***
C**                     routine <ROUTIN> on all processors.         ***
C**                                                                 ***
C**********************************************************************
C**                                                                 ***
!**    Copyright (C) 1998 University Karlsruhe                      ***
!**    PROGRAMMER:    H. Haefner                                    ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
C**                                                                 ***
C**********************************************************************
C**                                                                 ***
      USE constants

      implicit none
C**                                                                 ***
C**-----------------------------------------------------------------***
C**                                                                 ***
C**      FORMAL PARAMETERS :                                        ***
C**      -----------------                                          ***
C**                                                                 ***
      integer            lstr,myproc,nproc,nmsg 
      integer            tids(nproc)
C**                                                                 ***
      character*1        swork(lstr),string(lstr)                
      external           ROUTIN
C**                                                                 ***
C**-----------------------------------------------------------------***
C**                                                                 ***
C**   LIST OF FORMAL PARAMETERS :                                   ***
C**   -------------------------                                     ***
C**                                                                 ***
C--------I------I-----I------------------------------------------------
C NAME   I TYPE I I/O I     MEANING
C--------I------I-----I------------------------------------------------
C--------I------I-----I------------------------------------------------
C ROUTIN I  C   I in  I  name of the routine to be called            
C--------I------I-----I------------------------------------------------
C swork  I  C   I in  I  string vector for routine <ROUTIN>          
C--------I------I-----I------------------------------------------------
C string I  C   I i/o I  string vector for routine <ROUTIN>          
C--------I------I-----I------------------------------------------------
C lstr   I  I   I in  I  length of the above mentioned vectors in Bytes 
C--------I------I-----I------------------------------------------------
C myproc I  I   I in  I  process(or) id
C--------I------I-----I------------------------------------------------
C nproc  I  I   I in  I  number of processes (processors)
C--------I------I-----I------------------------------------------------
C tids   I  I   I in  I  set of tids                array : tids(nproc)
C--------I------I-----I------------------------------------------------
C nmsg   I  I   I i/o I  number of calls of communication routines
C--------I------I-----I------------------------------------------------
C**                                                                 ***
C**                                                                 ***
C**        LOCAL PARAMETERS :                                       ***
C**        ----------------                                         ***
C**                                                                 ***
      integer           i,j,jend,to,totid,ierr,msgtp,frtid,tofrom,
     &                  mids,midr,nlist,nredstp,nxtpow,pdist
      integer           plist(maxnpe)
C**                                                                 ***
C**-----------------------------------------------------------------***
C**                                                                 ***
C
C
C**** START OF CALCULATION :
C     ---------------------
C
      msgtp=0
 
      nlist = 0
      nredstp = int(log(real(nproc))/log(2.))
      if (nproc .gt. 2**nredstp) nredstp = nredstp+1
      nxtpow  = nproc

C
C*** Number of REDUCE steps is ceiling[log2(nproc)]
C
      do 10 i=1,nredstp
	msgtp = msgtp+1
	pdist = 2**(i-1)
	if (mod(nxtpow,2**i) .ne. 0) nxtpow = nxtpow+pdist
	if (mod(myproc-1,2**i) .lt. pdist) then
C
C*** SEND to the left and RECEIVE from the right
C
	  tofrom = myproc + pdist
	  if (nlist .gt. 0) then
	    jend = nlist
	    do 20 j=1,jend
	      if (mod(plist(j)-1,2**i) .lt. pdist) then
	        to    = plist(j) + pdist
	        nlist = nlist+1
	        plist(nlist) = to
	      endif
   20       continue
	  endif
	  if (tofrom .le. nproc .or. nproc+pdist .gt. nxtpow) then
	    if (tofrom .le. nproc) then
	      frtid = tids(tofrom)
            else
	      frtid = -1
	    endif
	    call MPRCVA(frtid,nmsg+msgtp,lstr,swork,midr,ierr)
	  endif
	  if (tofrom .le. nproc) then
	    totid=tids(tofrom)
	    call MPSNDA(totid,nmsg+msgtp,lstr,string,mids,ierr)
	    call MPSNDW(totid,nmsg+msgtp,lstr,string,mids,ierr)
	  endif
C
C*** Behave as if you are processor with logical processor-id <tofrom>
C***           (only if there is not yet another processor)
C
	  if (tofrom .gt. nproc .and. nproc+pdist .le. nxtpow) then
	    nlist = nlist+1
	    plist(nlist) = tofrom
	  endif
	  if (tofrom .le. nproc .or. nproc+pdist .gt. nxtpow) then
	    if (tofrom .le. nproc) then
	      frtid = tids(tofrom)
            else
	      frtid = -1
	    endif
	    call MPRCVW(frtid,nmsg+msgtp,lstr,swork,midr,ierr)
	    call ROUTIN(swork,string,lstr)
	  endif
	else
C
C*** SEND to the right and RECEIVE from the left
C
	  tofrom = myproc - pdist
	  frtid = tids(tofrom)
	  call MPRCVA(frtid,nmsg+msgtp,lstr,swork,midr,ierr)
	  totid=tids(tofrom)
	  call MPSNDA(totid,nmsg+msgtp,lstr,string,mids,ierr)
	  call MPSNDW(totid,nmsg+msgtp,lstr,string,mids,ierr)
C
C*** SEND to the right and RECEIVE from the left
C
	  do 30 j=1,nlist
	    to = plist(j) - pdist
	    totid = tids(to)
	    call MPSNDA(totid,nmsg+msgtp,lstr,string,mids,ierr)
	    call MPSNDW(totid,nmsg+msgtp,lstr,string,mids,ierr)
   30     continue
	  frtid = tids(tofrom)
	  call MPRCVW(frtid,nmsg+msgtp,lstr,swork,midr,ierr)
	  call ROUTIN(swork,string,lstr)
	endif
   10 continue
C
      nmsg = MOD(nmsg+msgtp+1,nmsgmax)
C
C**** END OF CALCULATION
C     ------------------
C
      r e t u r n
C-----END OF REDUCE--------------------------------------------------
      e    n    d
