!:::::      ,,,,,MPSCAT.....
!
!
!
!**********************************************************************
!        1         2         3         4         5         6         7*
!**********************************************************************
!**                                                                 ***
!**                                                                 ***
      subroutine MPSCAT(svect,lstr,root,myproc,nproc,tids,nmsg,ierr)
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**      M P S C A T   broadcasts the computed values stored in     ***
!**                    <svect> to all processes (processors).       ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**    Copyright (C) 1999 University Karlsruhe                      ***
!**    PROGRAMMER:    H. Haefner                                    ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
      USE constants

      implicit none
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**      FORMAL PARAMETERS :                                        ***
!**      -----------------                                          ***
!**                                                                 ***
      integer            lstr,myproc,nproc,nmsg,ierr
      integer            tids(nproc)
      logical            root
!**                                                                 ***
      integer            svect(lstr)                
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**   LIST OF FORMAL PARAMETERS :                                   ***
!**   -------------------------                                     ***
!**                                                                 ***
!--------I------I-----I------------------------------------------------
! NAME   I TYPE I I/O I     MEANING
!--------I------I-----I------------------------------------------------
!--------I------I-----I------------------------------------------------
! svect  I  C   I i/o I  string vector for routine <ROUTIN>          
!--------I------I-----I------------------------------------------------
! lstr   I  I   I in  I  length of the above mentioned vectors in Bytes 
!--------I------I-----I------------------------------------------------
! root   I  L   I in  I  is true, if the vector to be scattered is on
!        I      I     I  my processor;
!        I      I     I  is false, if the vector to be scattered is not
!        I      I     I  on my processor.
!--------I------I-----I------------------------------------------------
! myproc I  I   I in  I  process(or) id
!--------I------I-----I------------------------------------------------
! nproc  I  I   I in  I  number of processes (processors)
!--------I------I-----I------------------------------------------------
! tids   I  I   I in  I  set of tids                array : tids(nproc)
!--------I------I-----I------------------------------------------------
! nmsg   I  I   I i/o I  global message counter, must have the same
!        I      I     I  value on all processes (processors) on entry
!--------I------I-----I------------------------------------------------
! ierr   I  I   I out I  error number
!--------I------I-----I------------------------------------------------
!**                                                                 ***
!**                                                                 ***
!**        LOCAL PARAMETERS :                                       ***
!**        ----------------                                         ***
!**                                                                 ***
      integer           from,frtid,i,j,msgtp,mids,midr,nsteps,to
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!
!
!**** START OF CALCULATION :
!     ---------------------
!
      nsteps = CEILING(LOG(REAL(nproc))/LOG(2.))

      if (root .and. myproc /= 1) then
	call MPSNDA(tids(1),nmsg,lstr,svect,mids,ierr)
	call MPSNDW(tids(1),nmsg,lstr,svect,mids,ierr)
      endif
      frtid  = -1
      if (.not. root .and. myproc == 1) then
	call MPRCVA(frtid,nmsg,lstr,svect,midr,ierr)
	call MPRCVW(frtid,nmsg,lstr,svect,midr,ierr)
      endif

      msgtp  = 1
 
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**   THE COMPUTED VALUES ARE BROADCASTED WITH CASCADE              ***
!**   -----------------------------------------------               ***
!**                                                                 ***
      do i=nsteps,1,-1
	do j=1,nproc,2**i
	  to   = j + 2**(i-1)
	  from = j
	  if (to <= nproc) then
	    if (myproc == from) then
	      call MPSNDA(tids(to),nmsg+msgtp,lstr,svect,mids,ierr)
	      call MPSNDW(tids(to),nmsg+msgtp,lstr,svect,mids,ierr)
            endif
	    if (myproc == to) then
	      call MPRCVA(tids(from),nmsg+msgtp,lstr,svect,midr,ierr)
	      call MPRCVW(tids(from),nmsg+msgtp,lstr,svect,midr,ierr)
	    endif
	  endif
	  msgtp = msgtp + 1
	enddo
      enddo
!
      nmsg = MOD(nmsg+msgtp,nmsgmax)
!
!**** END OF CALCULATION
!     ------------------
!
      r e t u r n
!-----END OF MPSCAT---------------------------------------------------
      e    n    d
