C**********************************************************************
C       1         2         3         4         5         6         7 *
C**********************************************************************
C**                                                                 ***
C**                                                                 ***
      subroutine distribute_linsol_matrix(matrix,mat,lmat,index,lindex,
     #             info,ia1,rhs,l,lsym,nvtloc,ierr,lmatbk,ptrmbk,ilin,
     #             infmt,nrhs,myproc,nproc,tids,nmsg)
C**                                                                 ***
C**                                                                 ***
C**********************************************************************
C**                                                                 ***

C**                                                                 ***
C**********************************************************************
C**                                                                 ***
C**      Copyright (C) 1997 University Karlsruhe                    ***
C**      PROGRAMMER:   Hartmut Haefner                              ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
C**                                                                 ***
C**                                                                 ***
C**********************************************************************
C**                                                                 ***
      USE constants

      implicit none
C**                                                                 ***
C**-----------------------------------------------------------------***
C**                                                                 ***
C**      FORMAL PARAMETERS :                                        ***
C**      -----------------                                          ***
C**                                                                 ***
      integer myproc,nproc,nmsg,lmat,lindex,ia1,nvtloc,ierr,
     #        tids(nproc),index(lindex),info(ia1,ia2),infmt,
     #        lmatbk(nproc),ptrmbk(nproc+1),ilin(nilin),l,nrhs
      double precision  mat(lmat),rhs(l)
      character         matrix*72
      logical           lsym
      
C**                                                                 ***
C**      LOCAL VARIABLES :                                          ***
C**      -----------------                                          ***
C**                                                                 ***
      integer     iounit,mids,midr,to,from,ityp,iar,iac,lvt,nvec
      integer     i,j,ibuf(ia2),indbuf(l),irbuf(l),lout,idoku
      integer     endmat,endind,ivt,iertag
      integer     lmatd,lindd,ld,nvtd
      real*4      rbuf(l)
      double precision  buf(l),x(l)
      logical           lsymd

c**********************************************************************
c**    Start of calculation                                         ***
c**********************************************************************

      iounit = 11
      ivt    = 1
      endmat = 0
      endind = 0
      iertag = 0
      lout   = ilin(12)
      idoku  = ilin(13)

      if (infmt == 0) then
        if (idoku /= 0) write(lout,1100) matrix,iounit
        open(UNIT=iounit,FILE=matrix,FORM='UNFORMATTED')
        read (iounit) lmatd,lindd,ld,lsymd,nvtd
      else
        if (idoku /= 0) write(lout,1150) matrix,iounit
        open(UNIT=iounit,FILE=matrix,FORM='FORMATTED')
        read (iounit,*) lmatd,lindd,ld,lsymd,nvtd
      endif

!*********************************************************
!   Calculation of the right hand side rhs so that
!   x=(1,2**(1/3),3**(1/3),...,l**(1/3))
!   is the solution of the linear system.
!*********************************************************
      if (nrhs < 0) then
	do i=1,l
          x(i) = dble(i)**(1./3.)
        enddo
      endif

!*********************************************************
!   Starting loop over all storage patterns
!*********************************************************

 100  if (infmt == 0) then
        read(iounit,END=999) ityp,nvec
      else
        read(iounit,*,END=999) ityp,nvec
      endif
      ibuf(1) = ityp
      ibuf(2) = nvec

c   tells every processor the number of LTSEP.. - Calls
   
      do i=2,nproc
        to = tids(i)
        call MPSNDA(to,nmsg+i,2*iint,ibuf(1),mids,ierr)
        call MPSNDW(to,nmsg+i,2*iint,ibuf(1),mids,ierr)
      enddo
      nmsg = nmsg+nproc-1

c   read nvec-times one of the different storage types,
c   fill buffer with information and call separation-routine

      ibuf(1) = ityp
      ibuf(2) = 0
      ibuf(6) = 0
      ibuf(7) = 0
      do j=1,nvec
        if (ityp == 1) then
          if (infmt == 0) then
            read(iounit) (rbuf(i),i=1,l)
	    buf = dble(rbuf)
	  else
            read(iounit,*) (buf(i),i=1,l)
	  endif
          if (nrhs < 0) then
            do i=1,l
              rhs(i) = rhs(i) + buf(i)*x(i)
	    enddo
	  endif
              
          ibuf(4) = 0
          ibuf(5) = 0
          call LINSP1(mat,lmat,info,ivt,ia1,tids,
     #                lmatbk,ptrmbk,nproc,myproc,nmsg,
     #                ierr,iertag,buf,l,ibuf,endmat)

        elseif (ityp.eq.2) then
          if (infmt == 0) then
            read(iounit) lvt,iac,iar
            read(iounit) (rbuf(i),i=1,lvt)
	    do i=1,lvt
	      buf(i) = dble(rbuf(i))
	    enddo
	  else
            read(iounit,*) lvt,iac,iar
            read(iounit,*) (buf(i),i=1,lvt)
	  endif
          if (nrhs < 0) then
            do i=1,lvt
              rhs(i+iar) = rhs(i+iar) + buf(i)*x(i+iac)
	    enddo
            if (lsym) then
              do i=1,lvt
	        rhs(i+iac) = rhs(i+iac) + buf(i)*x(i+iar)
	      enddo
	    endif
	  endif

          ibuf(3) = lvt
          ibuf(4) = iac
          ibuf(5) = iar
          call LINSP2(mat,lmat,info,ivt,ia1,tids,
     #                lmatbk,ptrmbk, nproc,myproc,nmsg,
     #                ierr,iertag,buf,l,ibuf,endmat)

        elseif (ityp.eq.3) then
          if (infmt == 0) then
            read(iounit) lvt,iac,iar
            read(iounit) (indbuf(i),i=1,lvt)
            read(iounit) (rbuf(i),i=1,lvt)
	    do i=1,lvt
	      buf(i) = dble(rbuf(i))
	    enddo
	  else
            read(iounit,*) lvt,iac,iar
            read(iounit,*) (indbuf(i),i=1,lvt)
            read(iounit,*) (buf(i),i=1,lvt)
	  endif
          if (nrhs < 0) then
            include 'norec.h'
            do i=1,lvt
              rhs(indbuf(i)+iar) = rhs(indbuf(i)+iar) + buf(i)*
     #                             x(indbuf(i)+iac)
	    enddo
            if (lsym) then
              do i=1,lvt
	        rhs(indbuf(i)+iac) = rhs(indbuf(i)+iac) + buf(i)*
     #                               x(indbuf(i)+iar)
	      enddo
	    endif
	  endif

          ibuf(3) = lvt
          ibuf(4) = iac
          ibuf(5) = iar
          call LINSP3(mat,lmat,info,ivt,ia1,tids,
     #                lmatbk,ptrmbk,nproc,myproc,nmsg,
     #                ierr,iertag,buf,l,ibuf,endmat,
     #                index,lindex,endind,indbuf)

        elseif (ityp.eq.4) then
          if (infmt == 0) then
            read(iounit) lvt,iac,iar
            read(iounit) (indbuf(i),i=1,lvt)
            read(iounit) (rbuf(i),i=1,lvt)
	    do i=1,lvt
	      buf(i) = dble(rbuf(i))
	    enddo
	  else
            read(iounit,*) lvt,iac,iar
            read(iounit,*) (indbuf(i),i=1,lvt)
            read(iounit,*) (buf(i),i=1,lvt)
	  endif
          if (nrhs < 0) then
            include 'norec.h'
            do i=1,lvt
              rhs(i+iar) = rhs(i+iar) + buf(i)*x(indbuf(i))
	    enddo
            if (lsym) then
              do i=1,lvt
	        rhs(indbuf(i)) = rhs(indbuf(i)) + buf(i)*x(i+iar)
              enddo
	    endif
	  endif

          ibuf(3) = lvt
          ibuf(4) = iac
          ibuf(5) = iar
          call LINSP4(mat,lmat,info,ivt,ia1,tids,
     #                lmatbk,ptrmbk,nproc,myproc,nmsg,
     #                ierr,iertag,buf,l,ibuf,endmat,
     #                index,lindex,endind,indbuf)

        elseif (ityp.eq.5) then
          if (infmt == 0) then
            read(iounit) lvt,iac,iar
            read(iounit) (indbuf(i),i=1,lvt)
            read(iounit) (rbuf(i),i=1,lvt)
	    do i=1,lvt
	      buf(i) = dble(rbuf(i))
	    enddo
	  else
            read(iounit,*) lvt,iac,iar
            read(iounit,*) (indbuf(i),i=1,lvt)
            read(iounit,*) (buf(i),i=1,lvt)
	  endif
          if (nrhs < 0) then
            do i=1,lvt
              rhs(indbuf(i)) = rhs(indbuf(i)) + buf(i)*x(i+iac)
	    enddo
            if (lsym) then
              do i=1,lvt
	        rhs(i+iac) = rhs(i+iac) + buf(i)*x(indbuf(i))
              enddo
	    endif
	  endif

          ibuf(3) = lvt
          ibuf(4) = iac
          ibuf(5) = iar
          call LINSP5(mat,lmat,info,ivt,ia1,tids,
     #                lmatbk,ptrmbk,nproc,myproc,nmsg,
     #                ierr,iertag,buf,l,ibuf,endmat,index,
     #                lindex,endind,indbuf)

        elseif (ityp.eq.6) then
          if (infmt == 0) then
            read(iounit) lvt
            read(iounit) (indbuf(i),i=1,lvt)
            read(iounit) (irbuf(i),i=1,lvt)
            read(iounit) (rbuf(i),i=1,lvt)
	    do i=1,lvt
	      buf(i) = dble(rbuf(i))
	    enddo
	  else
            read(iounit,*) lvt
            read(iounit,*) (indbuf(i),i=1,lvt)
            read(iounit,*) (irbuf(i),i=1,lvt)
            read(iounit,*) (buf(i),i=1,lvt)
	  endif
          if (nrhs < 0) then
            do i=1,lvt
              rhs(irbuf(i)) = rhs(irbuf(i)) + buf(i)*x(indbuf(i))
	    enddo
            if (lsym) then
              do i=1,lvt
	        rhs(indbuf(i)) = rhs(indbuf(i)) + buf(i)*x(irbuf(i))
              enddo
	    endif
	  endif

          ibuf(3) = lvt
          ibuf(4) = 0
          ibuf(5) = 0
          call LINSP6(mat,lmat,info,ivt,ia1,tids,
     #                lmatbk,ptrmbk,nproc,myproc,nmsg,
     #                ierr,iertag,buf,l,ibuf,endmat,index,
     #                lindex,endind,indbuf,irbuf)

        elseif (ityp.eq.7) then
          if (infmt == 0) then
            read(iounit) lvt,iac,iar
            read(iounit) (rbuf(i),i=1,lvt)
	    do i=1,lvt
	      buf(i) = dble(rbuf(i))
	    enddo
	  else
            read(iounit,*) lvt,iac,iar
            read(iounit,*) (buf(i),i=1,lvt)
	  endif
          if (nrhs < 0) then
            do i=1,lvt
              rhs(1+iar) = rhs(1+iar) + buf(i)*x(i+iac)
	    enddo
            if (lsym) then
              do i=1,lvt
	        rhs(iac+i) = rhs(iac+i) + buf(i)*x(iar+1)
              enddo
	    endif
	  endif

          ibuf(3) = lvt
          ibuf(4) = iac
          ibuf(5) = iar
          call LINSP7(mat,lmat,info,ivt,ia1,tids,
     #                lmatbk,ptrmbk, nproc,myproc,nmsg,
     #                ierr,iertag,buf,l,ibuf,endmat)

        elseif (ityp.eq.8) then
          if (infmt == 0) then
            read(iounit) lvt,iac,iar
            read(iounit) (rbuf(i),i=1,lvt)
	    do i=1,lvt
	      buf(i) = dble(rbuf(i))
	    enddo
	  else
            read(iounit,*) lvt,iac,iar
            read(iounit,*) (buf(i),i=1,lvt)
	  endif
          if (nrhs < 0) then
            do i=1,lvt
              rhs(i+iar) = rhs(i+iar) + buf(i)*x(1+iac)
	    enddo
            if (lsym) then
              do i=1,lvt
	        rhs(iac+1) = rhs(iac+1) + buf(i)*x(iar+i)
              enddo
	    endif
	  endif

          ibuf(3) = lvt
          ibuf(4) = iac
          ibuf(5) = iar
          call LINSP8(mat,lmat,info,ivt,ia1,tids,
     #                lmatbk,ptrmbk, nproc,myproc,nmsg,
     #                ierr,iertag,buf,l,ibuf,endmat)

        elseif (ityp.eq.9) then
          if (infmt == 0) then
            read(iounit) lvt,iac,iar
            read(iounit) (indbuf(i),i=1,lvt)
            read(iounit) (rbuf(i),i=1,lvt)
	    do i=1,lvt
	      buf(i) = dble(rbuf(i))
	    enddo
	  else
            read(iounit,*) lvt,iac,iar
            read(iounit,*) (indbuf(i),i=1,lvt)
            read(iounit,*) (buf(i),i=1,lvt)
	  endif
          if (nrhs < 0) then
            do i=1,lvt
              rhs(1+iar) = rhs(1+iar) + buf(i)*x(indbuf(i))
	    enddo
            if (lsym) then
              do i=1,lvt
	        rhs(indbuf(i)) = rhs(indbuf(i)) + buf(i)*x(iar+1)
              enddo
	    endif
	  endif

          ibuf(3) = lvt
          ibuf(4) = iac
          ibuf(5) = iar
          call LINSP9(mat,lmat,info,ivt,ia1,tids,
     #                lmatbk,ptrmbk,nproc,myproc,nmsg,
     #                ierr,iertag,buf,l,ibuf,endmat,
     #                index,lindex,endind,indbuf)

        elseif (ityp.eq.11) then
          if (infmt == 0) then
            read(iounit) lvt,iac,iar
            read(iounit) (indbuf(i),i=1,lvt)
            read(iounit) (rbuf(i),i=1,lvt)
	    do i=1,lvt
	      buf(i) = dble(rbuf(i))
	    enddo
	  else
            read(iounit,*) lvt,iac,iar
            read(iounit,*) (indbuf(i),i=1,lvt)
            read(iounit,*) (buf(i),i=1,lvt)
	  endif
          if (nrhs < 0) then
            include 'norec.h'
            do i=1,lvt
              rhs(indbuf(i)) = rhs(indbuf(i)) + buf(i)*x(1+iac)
	    enddo
            if (lsym) then
              do i=1,lvt
	        rhs(iac+1) = rhs(iac+1) + buf(i)*x(indbuf(i))
              enddo
	    endif
	  endif

          ibuf(3) = lvt
          ibuf(4) = iac
          ibuf(5) = iar
          call LINSP11(mat,lmat,info,ivt,ia1,tids,
     #                 lmatbk,ptrmbk,nproc,myproc,nmsg,
     #                 ierr,iertag,buf,l,ibuf,endmat,
     #                 index,lindex,endind,indbuf)

        endif
      enddo

      goto 100
    
c   tell the other processors that matrix-reading complete
        
 999  ibuf(1) = 0
      ibuf(2) = 0
      close(iounit)
      do i=2,nproc
        to = tids(i)
        call MPSNDA(to,nmsg+i,2*iint,ibuf(1),mids,ierr)
        call MPSNDW(to,nmsg+i,2*iint,ibuf(1),mids,ierr)
      enddo
      nmsg = nmsg+nproc-1

      nvtloc = ivt-1      
      ilin(39) = iertag
      ilin(33) = endmat
      ilin(34) = endind

 1100 format(/'  Reading binary file ',/
     &       '  ',a72,/
     &       '  on unit ',i3,' ...'/)
 1150 format(/'  Reading file ',/
     &       '  ',a72,/
     &       '  on unit ',i3,' ...'/)

      r e t u r n
c---- END OF DISTRIBUTE_LINSOL_MATRIX ---------------------------------
      e n d
