!***********************************************************************
!        1         2         3         4         5         6         7 *
!***********************************************************************
!**                                                                  ***
!**                                                                  ***
      subroutine LININ(MAT,lmat,index,lindex,info,ia1,nvt,lloc,l,lsym,
     &                 lin,lout,ierr)
!**                                                                  ***
!**                                                                  ***
!***********************************************************************
!**                                                                  ***
!**    L I N I N   loads a matrix in LINSOL-format                   ***
!**                                                                  ***
!***********************************************************************
!**                                                                  ***
!**    Copyright (C) 1999 University Karlsruhe                       ***
!**    Programmer : H. Haefner                                       ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                  ***
!***********************************************************************
!**                                                                  ***
      USE constants

      implicit none
!**                                                                  *** 
!***********************************************************************
!**                                                                  ***
!**    formal parameters :                                           ***
!**                                                                  ***
!***********************************************************************
      integer,intent(in)  ::  ia1,lin,l,lout
      integer,intent(inout) ::  lmat,lindex
      integer,intent(out) ::  nvt,ierr,lloc
      integer,intent(out) ::  info(ia1,ia2),index(lindex)
      double precision, intent(out) ::  MAT(lmat)
      logical, intent(out) ::  lsym
!***********************************************************************
!**                                                                  ***
!**    list of formal parameters :                                   ***
!**                                                                  ***
!**                                                                  ***
!---------I------I-----I------------------------------------------------
! name    I type I i/o I meaning
!---------I------I-----I------------------------------------------------
! l       I int  I out I dimension of the matrix
!---------I------I-----I------------------------------------------------
! lmat    I int  I i/o I number of nonzero elements
!---------I------I-----I------------------------------------------------
! lindex  I int  I i/o I number of indices
!---------I------I-----I------------------------------------------------
! nvt     I int  I out I total number of vector-terms
!---------I------I-----I------------------------------------------------
! ia1     I int  I in  I 1.dimension of info-array
!---------I------I-----I------------------------------------------------
! ia2     I int  I in  I 2.dimension of info-array
!---------I------I-----I------------------------------------------------
! info    I int  I out I array with matrix-structure-information
!---------I------I-----I------------------------------------------------
! index   I int  I out I array with indices
!---------I------I-----I------------------------------------------------
! MAT     I real I out I array with nonzero matrix-elements
!---------I------I-----I------------------------------------------------
! lsym    I l    I out I indicates if matrix is symmetrical
!---------I------I-----I------------------------------------------------
! lin     I int  I in  I i/o-unit to read from
!---------I------I-----I------------------------------------------------
! ierr    I int  I out I error-indication
!---------I------I-----I------------------------------------------------
!**                                                                  ***
!**                                                                  ***
!**    local variables :                                             ***
!**                                                                  ***
      integer lvt,nvec,infpos,iac,iar,ptrind,ptrmat,buf2,i,j
      integer iostat,ityp
      double precision buf
!**                                                                  ***
!**                                                                  ***
!***********************************************************************
!**    start of calculation                                          ***
!***********************************************************************
!**                                                                  
!**    init local variables
!**
      ptrmat=0
      ptrind=0
      infpos = 1
      ierr=0
       
!**
!**    load some important parameters
!**
      read(lin,*,IOSTAT=iostat) lmat,lindex,lloc,lsym,nvt
      if (iostat > 0) then
	write(lout,1000)
	write(lout,1001)
	write(lout,1100)
	ierr = iostat
	goto 999
      endif
      if (iostat < 0) then
	write(lout,1000)
	write(lout,1002)
	write(lout,1100)
	ierr = iostat
	goto 999
      endif
      if ((nvt > ia1) .or. (lloc > lmat) .or. (lmat <= 0) .or.
     &    (lindex < 0) .or. (lloc <= 0) .or. (nvt <= 0))  then
	write(lout,1000)
	write(lout,1003)
	write(lout,1100)
        ierr=1
	goto 999
      endif
!**
!**    begin of loop to read the different storage-types
!**

  100 read(lin,*,IOSTAT=iostat,END=999) ityp,nvec
      if (iostat > 0) then
	write(lout,1000)
	write(lout,1004)
	write(lout,1100)
	ierr = iostat
	goto 999
      endif
      if (iostat < 0) then
	write(lout,1000)
	write(lout,1005) ityp,nvec
	write(lout,1100)
	ierr = iostat
	goto 999
      endif
      if ((ityp < 1) .or. (ityp > 11) .or. (nvec < 0)) then
	write(lout,1000)
	write(lout,1006) ityp,nvec
	write(lout,1100)
        ierr=1
	goto 999
      endif

!**
!**    now try to read nvec-times storage-typ ityp
!**
      if (ityp == 1) then
        if (ierr == 0) then
          read(lin,*,IOSTAT=iostat) (MAT(ptrmat+i),i=1,lloc)
          if (iostat > 0) then
            write(lout,1000)
	    write(lout,1007)
	    write(lout,1100)
	    ierr = iostat
	    goto 999
          endif
          info(infpos,1)=1
          info(infpos,2)=ptrmat
          info(infpos,3)=lloc
          info(infpos,4)=0
          info(infpos,5)=0
          info(infpos,6)=0
          info(infpos,7)=0
        else
          read(lin,*,IOSTAT=iostat) (buf,i=1,lloc)
          if (iostat > 0) then
            write(lout,1000)
	    write(lout,1007)
	    write(lout,1100)
	    ierr = iostat
	    goto 999
          endif
        endif
        ptrmat=ptrmat+lloc
        infpos=infpos+1

      elseif ((ityp == 2) .or. (ityp == 7) .or. (ityp == 8)) then
!**   
!**      read diagonal, full row or full column 
!**
        do i=1,nvec
          read(lin,*,IOSTAT=iostat) lvt,iac,iar
          if (iostat > 0) then
            write(lout,1000)
	    write(lout,1008)
	    write(lout,1100)
	    ierr = iostat
	    goto 999
          endif
          if (iostat < 0) then
	    write(lout,1000)
	    write(lout,1009) i,nvec
	    write(lout,1100)
	    ierr = iostat
	    goto 999
          endif
          if ((lvt <= 0) .or. (lvt > l) .or. (iac < 0) .or.
     &        (iac >= l) .or. (iar < 0) .or. (iar >= l))  then
	    write(lout,1000)
	    write(lout,1011)
	    write(lout,1100)
            ierr=1
	    goto 999
          endif
          if (ptrmat+lvt > lmat) then
            ierr=1
          endif
          if (ierr == 0) then
            read(lin,*,IOSTAT=iostat) (MAT(ptrmat+j),j=1,lvt)
            if (iostat > 0) then
              write(lout,1000)
	      write(lout,1008)
	      write(lout,1100)
	      ierr = iostat
	      goto 999
            endif
            if (iostat < 0 .and. i < nvec) then
	      write(lout,1000)
	      write(lout,1010) i,nvec
	      write(lout,1100)
	      ierr = iostat
	      goto 999
            endif
            info(infpos,1)=ityp
            info(infpos,2)=ptrmat
            info(infpos,3)=lvt
            info(infpos,4)=iac
            info(infpos,5)=iar
            info(infpos,6)=0
            info(infpos,7)=0
          else
            read(lin,*,IOSTAT=iostat) (buf,j=1,lvt)
            if (iostat > 0) then
              write(lout,1000)
	      write(lout,1008)
	      write(lout,1100)
	      ierr = iostat
	      goto 999
            endif
            if (iostat < 0 .and. i < nvec) then
	      write(lout,1000)
	      write(lout,1009) i,nvec
	      write(lout,1100)
	      ierr = iostat
	      goto 999
            endif
          endif
          ptrmat=ptrmat+lvt
          infpos=infpos+1
        enddo

      elseif ((ityp == 3) .or. (ityp == 4) .or. (ityp == 5) .or.
     &        (ityp == 9) .or. (ityp == 11)) then
!**
!**      read packed diagonal,row,column,packed row or packed column
!**
        do i=1,nvec
          read(lin,*,IOSTAT=iostat) lvt,iac,iar
          if (iostat > 0) then
            write(lout,1000)
	    write(lout,1012)
	    write(lout,1100)
	    ierr = iostat
	    goto 999
          endif
          if (iostat < 0) then
	    write(lout,1000)
	    write(lout,1009) i,nvec
	    write(lout,1100)
	    ierr = iostat
	    goto 999
          endif
          if ((lvt <= 0) .or. (lvt > l) .or. (iac < 0) .or.
     &        (iac >= l) .or. (iar < 0) .or. (iar >= l))  then
	    write(lout,1000)
	    write(lout,1013)
	    write(lout,1100)
            ierr=1
	    goto 999
          endif
          if ((ptrmat+lvt > lmat) .or. (ptrind+lvt > lindex)) then
            ierr=1
          endif
          if (ierr == 0) then
            read(lin,*,IOSTAT=iostat) (index(ptrind+j),j=1,lvt)
            if (iostat > 0) then
              write(lout,1000)
	      write(lout,1012)
	      write(lout,1100)
	      ierr = iostat
	      goto 999
            endif
            if (iostat < 0) then
	      write(lout,1000)
	      write(lout,1014) i,nvec
	      write(lout,1100)
	      ierr = iostat
	      goto 999
            endif
            read(lin,*,IOSTAT=iostat) (MAT(ptrmat+j),j=1,lvt)
            if (iostat > 0) then
              write(lout,1000)
	      write(lout,1012)
	      write(lout,1100)
	      ierr = iostat
	      goto 999
            endif
            if (iostat < 0 .and. i < nvec) then
	      write(lout,1000)
	      write(lout,1010) i,nvec
	      write(lout,1100)
	      ierr = iostat
	      goto 999
            endif
            info(infpos,1)=ityp
            info(infpos,2)=ptrmat
            info(infpos,3)=lvt
            info(infpos,4)=iac
            info(infpos,5)=iar
            if ((ityp == 3) .or. (ityp == 4) .or. (ityp == 9)) then
              info(infpos,6)=ptrind
              info(infpos,7)=0
            elseif ((ityp == 5) .or. (ityp == 11)) then
              info(infpos,6)=0
              info(infpos,7)=ptrind
            endif
          else
            read(lin,*,IOSTAT=iostat) (buf2,j=1,lvt)
            if (iostat > 0) then
              write(lout,1000)
	      write(lout,1012)
	      write(lout,1100)
	      ierr = iostat
	      goto 999
            endif
            if (iostat < 0) then
	      write(lout,1000)
	      write(lout,1014) i,nvec
	      write(lout,1100)
	      ierr = iostat
	      goto 999
            endif
            read(lin,*,IOSTAT=iostat) (buf,j=1,lvt)
            if (iostat > 0) then
              write(lout,1000)
	      write(lout,1012)
	      write(lout,1100)
	      ierr = iostat
	      goto 999
            endif
            if (iostat < 0 .and. i < nvec) then
	      write(lout,1000)
	      write(lout,1010) i,nvec
	      write(lout,1100)
	      ierr = iostat
	      goto 999
            endif
          endif
          ptrmat=ptrmat+lvt
          ptrind=ptrind+lvt
          infpos=infpos+1
        enddo

      elseif (ityp == 6) then
!**
!**      read info for sky-format
!**
        do i=1,nvec
          read(lin,*,IOSTAT=iostat) lvt
          if (iostat > 0) then
            write(lout,1000)
	    write(lout,1015)
	    write(lout,1100)
	    ierr = iostat
	    goto 999
          endif
          if (iostat < 0) then
	    write(lout,1000)
	    write(lout,1009) i,nvec
	    write(lout,1100)
	    ierr = iostat
	    goto 999
          endif
          if ((lvt <= 0) .or. (lvt > l)) then
	    write(lout,1000)
	    write(lout,1016)
	    write(lout,1100)
            ierr=1
	    goto 999
          endif
          if ((ptrmat+lvt > lmat) .or. (ptrind+2*lvt > lindex)) then
            ierr=1
          endif
          if (ierr == 0) then
            read(lin,*,IOSTAT=iostat) (index(ptrind+j),j=1,lvt)
            if (iostat > 0) then
              write(lout,1000)
	      write(lout,1015)
	      write(lout,1100)
	      ierr = iostat
	      goto 999
            endif
            if (iostat < 0) then
	      write(lout,1000)
	      write(lout,1014) i,nvec
	      write(lout,1100)
	      ierr = iostat
	      goto 999
            endif
            read(lin,*,IOSTAT=iostat) (index(ptrind+lvt+j),j=1,lvt)
            if (iostat > 0) then
              write(lout,1000)
	      write(lout,1015)
	      write(lout,1100)
	      ierr = iostat
	      goto 999
            endif
            if (iostat < 0) then
	      write(lout,1000)
	      write(lout,1014) i,nvec
	      write(lout,1100)
	      ierr = iostat
	      goto 999
            endif
            read(lin,*,IOSTAT=iostat) (MAT(ptrmat+j),j=1,lvt)
            if (iostat > 0) then
              write(lout,1000)
	      write(lout,1015)
	      write(lout,1100)
	      ierr = iostat
	      goto 999
            endif
            if (iostat < 0 .and. i < nvec) then
	      write(lout,1000)
	      write(lout,1010) i,nvec
	      write(lout,1100)
	      ierr = iostat
	      goto 999
            endif
            info(infpos,1)=ityp
            info(infpos,2)=ptrmat
            info(infpos,3)=lvt
            info(infpos,4)=0
            info(infpos,5)=0
            info(infpos,6)=ptrind
            info(infpos,7)=ptrind+lvt
          else
            read(lin,*,IOSTAT=iostat) (buf2, j=1,lvt)
            if (iostat > 0) then
              write(lout,1000)
	      write(lout,1015)
	      write(lout,1100)
	      ierr = iostat
	      goto 999
            endif
            if (iostat < 0) then
	      write(lout,1000)
	      write(lout,1014) i,nvec
	      write(lout,1100)
	      ierr = iostat
	      goto 999
            endif
            read(lin,*,IOSTAT=iostat) (buf2, j=1,lvt)
            if (iostat > 0) then
              write(lout,1000)
	      write(lout,1015)
	      write(lout,1100)
	      ierr = iostat
	      goto 999
            endif
            if (iostat < 0) then
	      write(lout,1000)
	      write(lout,1014) i,nvec
	      write(lout,1100)
	      ierr = iostat
	      goto 999
            endif
            read(lin,*,IOSTAT=iostat) (buf, j=1,lvt)
            if (iostat > 0) then
              write(lout,1000)
	      write(lout,1015)
	      write(lout,1100)
	      ierr = iostat
	      goto 999
            endif
            if (iostat < 0 .and. i < nvec) then
	      write(lout,1000)
	      write(lout,1010) i,nvec
	      write(lout,1100)
	      ierr = iostat
	      goto 999
            endif
          endif
          ptrmat=ptrmat+lvt    
          ptrind=ptrind+2*lvt 
          infpos=infpos+1
        enddo
      endif 
      goto 100
!**
!**    error-handling and feedback-information
!**         

      infpos = infpos - 1
      if (nvt /= infpos) ierr = 1

  888 if (ierr > 0) then
        write(lout,1000) 
        if (nvt > ia1) then
          write(lout,1017) nvt, infpos
	endif
        if (ptrmat > lmat) then
          write(lout,1018) lmat, ptrmat
        endif
        if (ptrind > lindex) then
           write(lout,1019) lindex, ptrind
        endif
        write(lout,1100) 
      endif

  999 continue

 1000 format(' *****************************************************'/
     &       ' ***  ERROR  occured in LININ (reading matrix):    ***')
 1001 format(' ***         reading parameters of the matrix      ***'/
     &       ' ***         fails!                                ***')
 1002 format(' ***         after reading first row of the file   ***'/
     &       ' ***         the end of file (EOF) is reached!     ***')
 1003 format(' ***         parameters of the matrix are not      ***'/
     &       ' ***         correct!                              ***')
 1004 format(' ***         reading number of vector terms for    ***'/
     &       ' ***         a stored storage pattern fails!       ***')
 1005 format(' ***         after reading type of storage pattern ***'/
     &       ' ***        ',i6,' and the number of vector terms  ***'/
     &       ' ***        ',i6,' end of file (EOF) is reached!   ***')
 1006 format(' ***         type of storage pattern ',i6,' or no. ***'/
     &       ' ***         of vector terms ',i6,' not correct!   ***')
 1007 format(' ***         reading storage pattern 1 fails!      ***')
 1008 format(' ***         reading storage pattern 2 or 7 or 8   ***'/
     &       ' ***         fails!                                ***')
 1009 format(' ***         while reading ',i6,' vector terms of  ***'/
     &       ' ***         expected ',i6,' vector terms end of   ***'/
     &       ' ***         file (EOF) is reached!                ***')
 1010 format(' ***         after reading ',i6,' vector terms of  ***'/
     &       ' ***         expected ',i6,' vector terms end of   ***'/
     &       ' ***         file (EOF) is reached!                ***')
 1011 format(' ***         parameters of storage pattern 2 or 7  ***'/
     &       ' ***         or 8 are not correct!                 ***')
 1012 format(' ***         reading storage pattern 3 or 4 or 5   ***'/
     &       ' ***         or 9 or 11 fails!                     ***')
 1013 format(' ***         parameters of storage pattern 3 or 4  ***'/
     &       ' ***         or 5 or 9 or 11 are not correct!      ***')
 1014 format(' ***         after reading ',i6,' vector term      ***'/
     &       ' ***         index vectors of expected ',i6,'      ***'/
     &       ' ***         vector term index vectors end of file ***'/
     &       ' ***         (EOF) is reached!                     ***')
 1015 format(' ***         reading storage pattern 6 fails!      ***')
 1016 format(' ***         parameter of storage pattern 6 is     ***'/
     &       ' ***         not correct!                          ***')
 1017 format(' ***         nvt =',i9,' is wrong! Set it to    ***'/
     &       ' ***                               ',i9,'       ***')
 1018 format(' ***         lmat =',i9,' is too small! Set it  ***'/
     &       ' ***                                  to ',i9,' ***')
 1019 format(' ***         lindex =',i9,' is too small! Set   ***'/
     &       ' ***                               it to ',i9,' ***')
 1100 format(' *****************************************************')

       r e t u r n
!-----END OF LININ----------------------------------------------------
       e    n    d
