!***********************************************************************
!        1         2         3         4         5         6         7 *
!***********************************************************************
!**                                                                  ***
!**    L I N S O L   is designed to run on a message passing system! ***
!**    L I N S O L   loads a matrix in LINSOL- or Harwell/Boeing-    ***
!**                  format on processor 1. Then processor 1         ***
!**                  distributes the matrix on <nproc> processors.   ***
!**                                                                  ***
!**    Copyright (C) 1996 University Karlsruhe                       ***
!**    Programmer : H. Haefner                                       ***
!**                                                                  ***
!**    This file is part of LINSOL.                                  ***
!**    LINSOL is free software; you can redistribute it and/or       ***
!**    modify it under the terms of the GNU General Public License   ***
!**    as published by the Free Software Foundation; either          ***
!**    version 2 of the License, or (at your option) any later       ***
!**    version.                                                      ***
!**                                                                  ***
!**    LINSOL is distributed in the hope that it will be useful,     ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty   ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.       ***
!**    See the GNU General Public License for more details.          ***
!**                                                                  ***
!**    You should have received a copy of the GNU General Public     ***
!**    License along with LINSOL; if not, write to the Free Software ***
!**    Foundation, Inc., 59 Temple Place - Suite 330, Boston,        ***
!**    MA  02111-1307, USA.                                          *** 
!**                                                                  ***
!***********************************************************************
!**                                                                  ***
!**    Parameters :                                                  ***
!**                                                                  ***
!***********************************************************************
      USE constants

      USE solex

      implicit none
!***********************************************************************
!**                                                                  ***
!**    Important input-parameters for LINSOL:                        ***
!**                                                                  ***
!***********************************************************************

      character        exname*8,matrix*72,outnam*72,rhsin*72,solout*72,
     &                 solin*72,tmpdir*72,inrannam*72
      integer          nproc,infmt,outfmt,outdev,split,startsol,method,
     &                 itmax,noprec,idoku,msprec,smooth,nrhs,n_rhs,
     &                 isprec,msnorm,misc,optim,bufsize,bwoalg,runacc
      double precision rlin(nrlin),zero

!***********************************************************************
!**                                                                  ***
!**    Static variables :                                            ***
!**                                                                  ***
!***********************************************************************

      integer           lmat,lindex,ia1,nvt,nvtloc,ierr,iconv,ntids,
     &                  myproc,nmsg,i,j,k,l,lout,irhs,ihelp(2),
     &                  nr,lprec,ilin(nilin),iounit,llocal
      integer           lmatd,lixd
      logical           lsym
      character         c,name*72,s1*5,nostr*5,yesstr*5,date*8,tims*10
      character*34      methstr,normstr,precstr,nsysstr,osysstr
      character*34      nsmostr,smostr
      character*29      str291,str293
      character*25      str251,str253
      character*27      str231,str233

!***********************************************************************
!**                                                                  ***
!**    Dynamic arrays :                                              ***
!**                                                                  ***
!***********************************************************************

      integer, pointer :: index(:),
     &                    lmatbk(:), ptrmbk(:), info(:,:), tids(:)
      double precision, pointer :: mat(:),rhs(:),x(:)

!***********************************************************************
!**    Interface of external subroutines                             ***
!***********************************************************************
      interface
        subroutine READ_MATRIX_PARAM(l,llocal,MAT,lmat,index,lindex,
     &                    lsym,info,ia1,nvt,n_rhs,idoku,lout,infmt,
     &                    split,optim,isprec,msprec,bwoalg,
     &                    lmatbk,ptrmbk,matrix,tids,nproc,
     &                    myproc,nmsg,ierr)
         USE           constants
         implicit      none
         integer       l,llocal,lmat,lindex,nvt,n_rhs,ia1,ierr,myproc,
     &                 idoku,lout,infmt,optim,split,nproc,nmsg,isprec,
     &                 msprec,bwoalg
         integer       lmatbk(nproc),ptrmbk(nproc+1),tids(nproc)
         character*72  matrix
         logical       lsym
         integer, pointer :: index(:),info(:,:)
         double precision, pointer :: MAT(:)
        end subroutine 

        subroutine READ_MATRIX_PART(myproc,mat,lmat,index,lindex,info,
     &                        ia1,nvt,lsym,tids,nproc,nmsg,ierr,matrix,
     &                        l,llocal,lmatbk,ptrmbk,idoku,infmt,lout,
     &                        isprec,msprec,bwoalg,optim)
         USE           constants
         implicit      none

         integer myproc,nproc,nmsg,lmat,lindex,ia1,nvt,ierr,
     &           lmatbk(nproc),ptrmbk(nproc+1),tids(nproc),l,llocal,
     &           idoku,infmt,lout,isprec,msprec,bwoalg,optim
         integer, pointer :: index(:),info(:,:)
         double precision, pointer :: mat(:)
         character*72 matrix
         logical lsym
        end subroutine 
        
        subroutine random(lmat,mat,lindex,index,ia1,nvt,
     & 			  lout,info,l,lsym,ilin,inrannam,ierr)
         USE           constants
         implicit      none
         integer  ierr,lmat,lindex,ia1,nvt,
     &            l,lout,ilin(nilin)
         integer, pointer :: index(:),info(:,:)
         double precision, pointer :: mat(:)
         character*72 inrannam
         logical lsym
       end subroutine 
       
      end interface
!**********************************************************************     
!**    End Interface block                                          ***
!**********************************************************************


      parameter (zero = 0.0)

!***********************************************************************
!**    Start of calculation                                          ***
!***********************************************************************

!   Init parameters for parallel processing and initialize communication

      exname = 'plinsol'
      ntids = maxnpe
      allocate (tids(ntids),stat=ierr)
      call COMBGN(nproc,myproc,ntids,tids,exname,ierr)

      ilin = 0

      nmsg = 0
      lout = 6

!   read parameters from parameter-file on processor 1
      if (myproc == 1) then
        call READ_PARAM_FILE(infmt,split,matrix,outfmt,   
     &                       outnam,inrannam,nrhs,rhsin,solout,startsol,
     &                       solin,tmpdir,method,itmax,outdev,isprec,
     &                       msprec,noprec,idoku,msnorm,smooth,misc,
     &                       optim,bufsize,bwoalg,runacc,
     &                       rlin,nproc,tids,nmsg,ierr)
      else

!   receive parameters from processor 1

        call RCV_INPUT_PARAM(infmt,split,outfmt,method,itmax,outdev,
     &                       isprec,bufsize,msprec,noprec,
     &                       idoku,msnorm,smooth,misc,optim,nrhs,
     &                       startsol,bwoalg,runacc,
     &                       rlin,matrix,rhsin,outnam,
     &                       nproc,myproc,tids,nmsg)
      endif

      call LL4INM(1,1,1,ierr,ihelp(1),ihelp(2),myproc,nproc,tids,nmsg)
      if (ihelp(1) > 0) then
	ierr = ihelp(1)
	iconv = 99
	if (myproc == 1) then
          write(lout,1200)
          write(lout,1201)
          write(lout,1300)
	endif
	goto 9999
      endif

      if (outdev /= 6) then
        if ((idoku > 0) .and. (myproc == 1)) then
          write(lout,1003)
        endif
        if (idoku /= 0) then
          write(lout,1004) myproc
        endif
        write(name,2000) myproc
        open(lout,file=name)
 2000   format ('stdout_',i4.4)
      endif

      if (method == 1)   methstr='........................... PRES20'
      if (method == 2)   methstr='............................. BICO'
      if (method == 3)   methstr='........................ Bi-CGSTAB'
      if (method == 4)   methstr='.......................... AT-PRES'
      if (method == 5)   methstr='.............................. CGS'
      if (method == 6)   methstr='.................... QMS-Simulator'
      if (method == 7)   methstr='....................... Bi-CGSTAB2'
      if (method == 9)   methstr='............................ CG-AT'
      if (method == 15)  methstr='........................... GMERR5'
      if (method == 16)  methstr='.......................... GMERR20'
      if (method == 100) methstr='..................... Classical CG'
      if (method == 101) methstr='........................... GMERRS'
      if ((method >= 200) .and. (method <= 212))
     &                   methstr='.................... Polyalgorithm'
      if ((method >= 220) .and. (method <= 222))
     &                   methstr='. Polyalgorithm for a symm. matrix'
      if (msprec == 0)   precstr='............... NO preconditioning'
      if (msprec == 63)  precstr='... Gauss preconditioning (middle)'
      if (msprec == 66)  precstr='.... Gauss preconditioning (right)'
      if (msprec == 69)  precstr='..... Gauss preconditioning (left)'
      if (msprec == 72)  precstr='.. Indexed Gauss precond. (middle)'
      if (msprec == 75)  precstr='... Indexed Gauss precond. (right)'
      if (msprec == 78)  precstr='.... Indexed Gauss precond. (left)'
      if (msnorm == 0)   normstr=' .  .  .  .  .  . NO normalization'
      if (msnorm == 1)   normstr=' .  .  .  .  Row sum normalization'
      if (msnorm == 2)   normstr=' .  .  Main diagonal normalization'
      if (msnorm == 3)   normstr=' .  .  .  Square sum normalization'
      if (msnorm == 4)   normstr=' .  .  .   Frobenius normalization'
      if (msnorm == 11)  normstr=' .  .  Symm. row sum normalization'
      if (msnorm == 12)  normstr=' Symm. main diagonal normalization'
      if (msnorm == 13)  normstr=' .  Symm. square sum normalization'
      if (msnorm == 14)  normstr=' .   Symm. Frobenius normalization'
                          yesstr='  YES'
                           nostr='   NO'
                         nsysstr=' .  .  .  .  .   Normalized system'
                         osysstr=' .  .  .  .  .  .  Original system'
                         nsmostr='..................... NON-smoothed'
                          smostr='......................... Smoothed'
                          str291='.............................'
                          str293=' .  .  .  .  .  .  .  .  .  .'
                          str251='.........................'
                          str253=' .  .  .  .  .  .  .  .  '
                          str231='.......................'
                          str233=' .  .  .  .  .  .  .  .'


      if ((idoku > 0) .and. (myproc == 1)) then
         write(lout,1000)
         write(lout,1001)
         write(lout,1002)
         write(lout,1010)
         write(lout,1011) methstr
         write(lout,1012) str293,nproc
         write(lout,1013) str291,nrhs
         if (startsol .eq. 0) write(lout,1014) str293,nostr
         if (startsol .eq. 1) write(lout,1015) str293,yesstr
	 write(lout,1016) str251,itmax
         if (isprec .eq. 0) write(lout,1017) str293,nostr
         if (isprec .eq. 1) write(lout,1018) str293,yesstr
	 write(lout,1019) precstr
         if (noprec .eq. 0) write(lout,1020) nsysstr
         if (noprec .eq. 1) write(lout,1021) osysstr
	 write(lout,1022) str291,idoku
	 write(lout,1023) normstr
         if (smooth == 0) write(lout,1024) nsmostr
         if (smooth /= 0) write(lout,1025) smostr
	 write(lout,1026) str293,misc
	 write(lout,1027) str251,optim
	 write(lout,1091) str293,bufsize
	 write(lout,1096) str291,runacc
	 write(lout,1097) str293,bwoalg
	 write(lout,1028) str251,rlin(1)
	 write(lout,1092) str253,rlin(4)
	 write(lout,1093) str251,rlin(2)
	 write(lout,1094) str253,rlin(3)
	 write(lout,1095) str251,rlin(5)
         if (infmt == 0) write(lout,1030)
         if (infmt == 1) write(lout,1031)
         if (infmt == 2) write(lout,1032)
         if (infmt == 3) write(lout,1033)
	 if (infmt /= 3) write(lout,1041) matrix
         if (outfmt > 0) write(lout,1042) outnam
         if (nrhs > 0)   write(lout,1043) rhsin
         if (startsol == 1) write(lout,1044) solin
         if (nrhs >= 0)  write(lout,1045) solout
      endif
      if (idoku < 0) then
         write(lout,1000)
         write(lout,1001)
         write(lout,1002)
         write(lout,1010)
         write(lout,1011) methstr
         write(lout,1012) str293,nproc
         write(lout,1013) str291,nrhs
         if (startsol .eq. 0) write(lout,1014) str293,nostr
         if (startsol .eq. 1) write(lout,1015) str293,yesstr
	 write(lout,1016) str251,itmax
         if (isprec .eq. 0) write(lout,1017) str293,nostr
         if (isprec .eq. 1) write(lout,1018) str293,yesstr
	 write(lout,1019) precstr
         if (noprec .eq. 0) write(lout,1020) nsysstr
         if (noprec .eq. 1) write(lout,1021) osysstr
	 write(lout,1022) str291,idoku
	 write(lout,1023) normstr
         if (smooth == 0) write(lout,1024) nsmostr
         if (smooth /= 0) write(lout,1025) smostr
	 write(lout,1026) str293,misc
	 write(lout,1027) str251,optim
	 write(lout,1091) str293,bufsize
	 write(lout,1096) str291,runacc
	 write(lout,1097) str293,bwoalg
	 write(lout,1028) str251,rlin(1)
	 write(lout,1092) str253,rlin(4)
	 write(lout,1093) str251,rlin(2)
	 write(lout,1094) str253,rlin(3)
	 write(lout,1095) str251,rlin(5)
         if (infmt == 0) write(lout,1030)
         if (infmt == 1) write(lout,1031)
         if (infmt == 2) write(lout,1032)
         if (infmt == 3) write(lout,1033)
	 if (infmt /= 3) write(lout,1041) matrix
         if (outfmt > 0) write(lout,1042) outnam
         if (nrhs > 0)   write(lout,1043) rhsin
         if (startsol == 1) write(lout,1044) solin
         if (nrhs >= 0)  write(lout,1045) solout
      endif


!*****************************************************************
!   Matrix not distributed, read matrix-parameters on processor 1
!*****************************************************************

!     if (idoku > 0) then
!       if (myproc == 1) write(lout,1060) nproc*iint
!     endif
!     if (idoku < 0) write(lout,1060) nproc*iint
      allocate (lmatbk(nproc),stat=ierr)
!     if (idoku > 0) then
!       if (myproc == 1) write(lout,1061) (nproc+1)*iint
!     endif
!     if (idoku < 0) write(lout,1061) (nproc+1)*iint
      allocate (ptrmbk(nproc+1),stat=ierr)

      call LL4INM(1,1,1,ierr,ihelp(1),ihelp(2),myproc,nproc,tids,nmsg)
      if (ihelp(1) > 0) then
	ierr = ihelp(1)
	iconv = 99
	if (myproc == 1) then
          write(lout,1200)
          write(lout,1202)
          write(lout,1300)
	endif
	goto 9999
      endif
         
      if ((split == 0) .and. (infmt /= 3)) then
        call READ_MATRIX_PARAM(l,llocal,MAT,lmat,index,lindex,lsym,
     &                         info,ia1,nvt,n_rhs,idoku,lout,infmt,
     &                         split,optim,isprec,msprec,bwoalg,
     &                         lmatbk,ptrmbk,matrix,
     &                         tids,nproc,myproc,nmsg,ierr)
	if (ierr /= 0) goto 9999

	if ((idoku /= 0) .and. (nproc == 1)) then
	  if (lsym) then
            write(lout,1070)
            write(lout,1072)
            write(lout,1075) l
            write(lout,1080)
	  else
            write(lout,1070)
            write(lout,1071)
            write(lout,1075) l
            write(lout,1080)
	  endif
        endif
	if (nproc > 1) then
	  if ((idoku /= 0) .and. (myproc == 1)) then
	    if (lsym) then
              write(lout,1070)
              write(lout,1072)
              write(lout,1075) l
              write(lout,1074)
              write(lout,1075) llocal
              write(lout,1080)
	    else
              write(lout,1070)
              write(lout,1071)
              write(lout,1075) l
              write(lout,1073)
              write(lout,1075) llocal
              write(lout,1080)
	    endif
          endif
	  if ((idoku < 0) .and. (myproc > 1)) then
	    if (lsym) then
              write(lout,1070)
              write(lout,1074)
              write(lout,1075) l
              write(lout,1080)
	    else
              write(lout,1070)
              write(lout,1073)
              write(lout,1075) l
              write(lout,1080)
	    endif
          endif
        endif
        if (myproc > 1) then
!   IMPORTANT : worst case dimensions for arrays
!               lindex not correctly dimensioned (if typ 4/5 used) 
!         if (idoku .gt. 0) then
!           if (myproc .eq. 1) write(lout,1062) lmat*ireal
!         endif
!         if (idoku < 0) write(lout,1062) lmat*ireal
          allocate (mat(lmat),stat=ierr)
!         if (idoku < 0) write(lout,1063) lindex*iint
          allocate (index(lindex),stat=ierr)
!         if (idoku < 0) write(lout,1064) ia1*ia2*iint
          allocate (info(ia1,ia2),stat=ierr)
        endif

        call LL4INM(1,1,1,ierr,ihelp(1),ihelp(2),myproc,nproc,tids,nmsg)
        if (ihelp(1) > 0) then
	  ierr = ihelp(1)
	  iconv = 99
	  if (myproc == 1) then
            write(lout,1200)
            write(lout,1203)
            write(lout,1300)
	  endif
	  goto 9999
        endif
      endif

!*********************************************************
!   Matrix (in LINSOL-format) is already distributed
!   Each processor reads his matrix-part
!*********************************************************

      if ((split == 1) .and. (infmt <= 1)) then
	if ((idoku > 0) .and. (myproc == 1)) then
          write(lout,1051) matrix,myproc
        endif
        if (idoku < 0) then
          write(lout,1051) matrix,myproc
        endif

        call READ_MATRIX_PART(myproc,mat,lmat,index,lindex,info,ia1,
     &        nvtloc,lsym,tids,nproc,nmsg,ierr,matrix,l,llocal,lmatbk,
     &        ptrmbk,idoku,infmt,lout,isprec,msprec,bwoalg,optim)
      endif


      if ((infmt == 3) .and. (myproc == 1)) then
        call RANDOM(lmat,mat,lindex,index,ia1,nvt,
     &	            lout,info,l,lsym,ilin,inrannam,ierr)
        call INIT_LINSOL_PARAM(lmatbk,ptrmbk,llocal,l,nproc)
      endif

      call LL4INM(1,1,1,ierr,ihelp(1),ihelp(2),myproc,nproc,tids,nmsg)
      if (ihelp(1) > 0) then
        ierr = ihelp(1)
        iconv = 99
	if (myproc == 1) then
          write(lout,1200)
          write(lout,1206)
          write(lout,1300)
	endif
	goto 9999
      endif
      
      if (myproc > 1) then
!       if (idoku > 0) then
!         if (myproc == 1) write(lout,1065) llocal*ireal
!       endif
!       if (idoku < 0) write(lout,1065) llocal*ireal
        allocate (x(llocal),stat=ierr)
!       if (idoku > 0) then
!         if (myproc == 1) write(lout,1066) llocal*ireal
!       endif
!       if (idoku < 0) write(lout,1066) llocal*ireal
        allocate (rhs(llocal),stat=ierr)
      else
!       if (idoku > 0) then
!         if (myproc == 1) write(lout,1065) l*ireal
!       endif
!       if (idoku < 0) write(lout,1065) l*ireal
        allocate (x(l),stat=ierr)
!       if (idoku > 0) then
!         if (myproc == 1) write(lout,1066) l*ireal
!       endif
!       if (idoku < 0) write(lout,1066) l*ireal
        allocate (rhs(l),stat=ierr)
        do i=1,l
          x(i)   = zero
          rhs(i) = zero
        enddo
      endif

      call LL4INM(1,1,1,ierr,ihelp(1),ihelp(2),myproc,nproc,tids,nmsg)
      if (ihelp(1) > 0) then
        ierr = ihelp(1)
        iconv = 99
	if (myproc == 1) then
          write(lout,1200)
          write(lout,1204)
          write(lout,1300)
	endif
	goto 9999
      endif

!*********************************************************
!   Now start reading matrix and distributing if necessary
!*********************************************************
      if (nproc == 1) then

!   no distributing if only one processor !!!

         if (infmt == 0) then
           iounit = 11
           open(UNIT=iounit,FILE=matrix,FORM='UNFORMATTED')
           if (idoku /= 0) then
             write(lout,1110) matrix,iounit
           endif
           call LINBIN(mat,lmatd,index,lixd,info,ia1,nvt,l,l,lsym,
     &                 iounit,lout,ierr)
           close(iounit)
         endif
         if (infmt == 1) then
           iounit = 11
           open(UNIT=iounit,FILE=matrix,FORM='FORMATTED')
           if (idoku /= 0) then
             write(lout,1100) matrix,iounit
           endif
           call LININ(mat,lmatd,index,lixd,info,ia1,nvt,l,l,lsym,
     &                iounit,lout,ierr)
           close(iounit)
         endif
      endif

!*************************************************************
!   LINSOL-statistics not available for (nproc>1 and infmt<=1)
!*************************************************************
      if (nproc == 1 .or. infmt > 1) then
        if (mod(optim,100) >= 10 .and. myproc == 1) then
	  call LINSTATISTICS(l,mat,lmat,lindex,info,ia1,nvt,ilin,
     &                       idoku,lout)
	endif
      endif

      ilin(12) = lout
      ilin(13) = idoku

      if ((split == 0) .and. (nproc > 1)) then
!*********************************************************
!   distribute matrix in LINSOL-Format physically 
!*********************************************************
        if (infmt <= 1) then
          if (myproc == 1) then
            call DISTRIBUTE_LINSOL_MATRIX(matrix,mat,lmat,index,
     &      lindex,info,ia1,rhs,l,lsym,nvtloc,ierr,lmatbk,ptrmbk,
     &      ilin,infmt,nrhs,myproc,nproc,tids,nmsg)
          else
            call RCV_LINSOL_MATRIX(mat,lmat,index,lindex,ilin,
     &                  info,ia1,llocal,nvtloc,ierr,lmatbk,ptrmbk,
     &                  myproc,nproc,tids,nmsg)
          endif
        endif
 
!*********************************************************
!   Separate matrix in compressed-sparse-row-format 
!*********************************************************
        if (infmt == 2) then
          call DISTRIBUTE_HB_MATRIX(myproc,nproc,tids,nmsg,
     &        mat,lmat,index,lindex,info,ia1,ilin,l,nvtloc,ierr,
     &        lmatbk,ptrmbk)
        endif
	 
!*********************************************************
!   Seperate random-matrix
!*********************************************************
        if (infmt == 3) then
	  call DISTRIBUTE_RANDOM_MATRIX(myproc,nproc,tids,nmsg,
     &			mat,lmat,index,lindex,info,ia1,ilin,l,
     &			nvt,nvtloc,ierr,lmatbk,ptrmbk)  
	endif
      endif

!*********************************************************
!   Save matrix in distributed LINSOL-format, if specified
!*********************************************************

      if (outfmt == 1) then
         iounit = 23
	 if (nproc > 1) then
           write (s1,'(A1,i4.4)') '_',myproc
           outnam = trim(outnam) // s1
	 endif
         if ((idoku > 0) .and. (myproc == 1)) then
            write(lout,1052) outnam,myproc
         endif
         if (idoku < 0) then
            write(lout,1052) outnam,myproc
         endif
         open(UNIT=iounit,FILE=outnam,FORM='FORMATTED')
	 if (mod(misc,1000) < 100) misc = misc + 100
      endif

      if (outfmt == 2) then
         iounit = 23
	 if (nproc > 1) then
           write (s1,'(A1,i4.4)') '_',myproc
           outnam = trim(outnam) // s1
	 endif
         if ((idoku > 0) .and. (myproc == 1)) then
            write(lout,1053) outnam,myproc
         endif
         if (idoku < 0) then
            write(lout,1053) outnam,myproc
         endif
         open(UNIT=iounit,FILE=outnam,FORM='UNFORMATTED')
	 if (mod(misc,1000) < 200) misc = misc + 200
      endif

!   init ilin array 

      ilin(1) = method
      ilin(2) = itmax
      ilin(3) = nmsg
      ilin(4) = nrhs
      if (nproc > 1) then
        ilin(5) = nvtloc
      else
        ilin(5) = nvt
      endif
      ilin(6) = isprec
      ilin(7) = bufsize
      ilin(8) = msprec
      ilin(9) = noprec
      if ((msprec >= 63 .or. MOD(optim,1000000) >= 100000) .and.
     &    (nrhs > 1)) then
        ilin(14) = 1
      else
        ilin(14) = 0
      endif
      ilin(15) = msnorm
      ilin(17) = myproc
      ilin(18) = smooth
      ilin(19) = misc
      ilin(20) = optim
      ilin(21) = runacc
      ilin(22) = bwoalg
      ilin(41) = 21
      ilin(42) = 22
      ilin(43) = iounit
      ilin(44) = 24
      ilin(45) = 25
      ilin(49) = split
       
      if (MOD(misc,10) > 0 .and. MOD(misc,10) < 4) then
         write(name,2001) myproc
         open(ilin(41),file=name,FORM='FORMATTED')
 2001    format ('linout_',i4.4)
      endif

      if ((nrhs < 0) .and. (nproc == 1) .and.
     &    (mod(misc,10000) >= 1000)) then
        allocate(xexact(l),stat=ierr)
        if (ierr > 0) then
	  iconv = 99
          write(lout,1200)
          write(lout,1205)
          write(lout,1300)
	  goto 9999
        endif
      endif
      if (mod(misc,100000) >= 10000) then
         call DATE_AND_TIME(DATE = date, TIME = tims)
         write(name,2002) date(3:8),tims(1:6)
         open(ilin(44),file=name,FORM='FORMATTED')
 2002    format ('plot_',a6,'_',a6)
      endif

!*********************************************************
!   now read right-hand-sides and start-solutions and
!   calculate solution-vector x
!*********************************************************

      irhs = 1
      if (nrhs > 0) nr = nrhs
      if (nrhs < 0) nr = 1
      if (nrhs == 0) nr = n_rhs

      m1: do while (irhs <= nr)
        if (myproc == 1) then
          call READ_RHS_IGUESS(MAT,lmat,index,lindex,lsym,info,ia1,
     &                         x,rhs,l,llocal,nvt,lmatbk,ptrmbk,
     &                         infmt,solin,rhsin,misc,startsol,split,
     &                         nrhs,irhs,idoku,lout,ierr,
     &                         tids,nproc,myproc,nmsg)
	else
	  call RCV_RHS_IGUESS(x,rhs,llocal,lmatbk,rhsin,startsol,split,
     &                        infmt,ierr,nrhs,irhs,idoku,lout,
     &                        tids,nproc,myproc,nmsg)
	endif

        if (startsol == 1) then
          ilin(16) = 4711
        else
          ilin(16) = 0
        endif

!*********************************************************
!   Subroutine LSOLP now can be called on all processors
!**********************************************************

	call LSOLP(lmat,lindex,llocal,nproc,lsym,ia1,info,mat,
     &             x,rhs,index,rlin,ilin,lmatbk,tids,iconv,ierr)

        if (MOD(misc,10) > 0 .and. MOD(misc,10) < 4) then
	  ilin(19) = ilin(19) - misc
	  misc     = ilin(19)
	  close(ilin(41))
	endif

        if (ierr /= 0) goto 9999

	if (myproc == 1) then
          call WRITE_SOLUTION(x,l,lmatbk,ptrmbk,ilin,tmpdir,
     &                        solout,infmt,irhs,nrhs,ierr,
     &                        tids,nproc,myproc,ilin(3))
	else
	  call SEND_SOLUTION(x,llocal,lmatbk,ierr,
     &                       tids,nproc,myproc,ilin(3))
	endif
        call LL4INM(1,1,1,ierr,ihelp(1),ihelp(2),
     &              myproc,nproc,tids,nmsg)
        if (ihelp(1) /=  0) then
          ierr = ihelp(1)
          iconv = 99
	  if (myproc == 1) then
            write(lout,1200)
            write(lout,1207)
            write(lout,1300)
	  endif
	  goto 9999
        endif
	irhs = irhs+1
      enddo m1

 9999 call COMEND(nproc,ierr)

      if ((nrhs < 0) .and. (nproc == 1) .and.
     &    (MOD(misc,10000) >= 1000)) then
        deallocate(xexact)
      endif

 1000 format('  ---------------------------------------------------- '/
     &       ' | -------------------------------------------------- |')
 1001 format(' ||      L I N S O L - Version 1.0  (January 2002)   ||')
 1002 format(' | -------------------------------------------------- |'/
     &       '  ---------------------------------------------------- '/)
 1003 format('  LINSOL stores the standard output in files:')
 1004 format('        <stdout_',i4,'>'/)
 1010 format('  INPUT PARAMETERS of LINSOL: ') 
 1011 format('   Used iterative method .............',a34)
 1012 format('   Number of processors .  .  .  .  . ',a29,i5)
 1013 format('   Number of right-hand-sides (nrhs)..',a29,i5) 
 1014 format('   Use of initial guess .  .  .  .  . ',a29,a5) 
 1015 format('   Use of initial guess(es)   .  .  . ',a29,a5) 
 1016 format('   Maximal number of iteration steps .',a25,i9)
 1017 format('   Is the matrix already normalized?  ',a29,a5)
 1018 format('   Is the matrix already normalized?  ',a29,a5)
 1019 format('   Used preconditioning method .......',a34)
 1020 format('   Checking of stopping criterion by  ',a34)
 1021 format('   Checking of stopping criterion by  ',a34)
 1022 format('   Amount of output (idoku)...........',a29,i5)
 1023 format('   Used normalization method  .  .  . ',a34)
 1024 format('   Kind of solution ..................',a34)
 1025 format('   Kind of solution ..................',a34)
 1026 format('   Miscellaneous (misc) .  .  .  .  . ',a29,i5)
 1027 format('   Optimization strategy (optim)......',a25,i9)
 1091 format('   Temporary buffer size   .  .  .  . ',a29,i5)
 1028 format('   Relative stopping factor (rlin(1)) ',a25,e9.2)
 1092 format('   Fill in factor {Gauss} (rlin(4)) . ',a25,f9.3)
 1093 format('   1st threshold value {Gauss}........',a25,f9.6)
 1094 format('   2nd threshold value {Gauss}   .  . ',a25,f9.6)
 1095 format('   memory utilization factor (rlin(5))',a25,f9.6,//)
 1096 format('   runtime acceleration value ........',a29,i5)
 1097 format('   bandwidth optimization algorithm . ',a29,i5)
 1030 format('  INPUT-Matrix is given in binary LINSOL-Format '/)
 1031 format('  INPUT-Matrix is given in LINSOL-Format '/)
 1032 format('  INPUT-Matrix is given in H/B-Format '/)
 1033 format('  INPUT-Matrix is given as RANDOM-Matrix '/)
 1041 format(/'  DIRECTORY and FILENAME of INPUT-Matrix: ',/
     &       '   ',a72) 
 1042 format('  DIRECTORY and FILENAME of OUTPUT-Matrix: ',/
     &       '   ',a72) 
 1043 format('  DIRECTORY and FILENAME of Right-Hand-Side(s): ',/
     &       '   ',a72) 
 1044 format('  DIRECTORY and FILENAME of initial guess(es): ',/
     &       '   ',a72) 
 1045 format('  DIRECTORY and FILENAME of solution(s): ',/
     &       '   ',a72/) 
 1051 format(/'  DIRECTORY and FILENAME of INPUT-Matrix: ',/
     &       '   ',a72,/
     &       '   with processor-id as suffix (_',i4,')'/)
 1052 format('  DIRECTORY and FILENAME of OUTPUT-Matrix: ',/
     &       '   ',a72,/
     &       '   with processor-id as suffix (_',i4,')'/)
 1053 format('  DIRECTORY and FILENAME of binary OUTPUT-Matrix: ',/
     &       '   ',a72,/
     &       '   with processor-id as suffix (_',i4,')'/)
!1060 format(//'  Allocating storage of size (in Bytes) ',i11,/
!    &       '  for integer array <lmatbk> ......................'/)
!1061 format('  Allocating storage of size (in Bytes) ',i11,/
!    &       '  for integer array <ptrmbk> ......................'/)
!1062 format(/'  Allocating storage of size (in Bytes) ',i11,/
!    &       '  for matrix array <MAT> ..........................'/)
!1063 format('  Allocating storage of size (in Bytes) ',i11,/
!    &       '  for index array <index> .........................'/)
!1064 format('  Allocating storage of size (in Bytes) ',i11,/
!    &       '  for matrix information array <info> .............'/)
!1065 format(//'  Allocating storage of size (in Bytes) ',i11,/
!    &       '  for solution vector <x> .........................'/)
!1066 format('  Allocating storage of size (in Bytes) ',i11,/
!    &       '  for right hand side <rhs> .......................'/)
 1070 format(/'   ------------------------------------------------ ')
 1071 format('  || The dimension of the (global) unsymmetric    ||')
 1072 format('  || The dimension of the (global) symmetric      ||')
 1073 format('  || The dimension of the (local) unsymmetric     ||')
 1074 format('  || The dimension of the (local) symmetric       ||')
 1075 format('  || matrix is ',i11,'!                       ||')
 1080 format('   ------------------------------------------------ '/)
 1100 format(/'  Reading file ',/
     &        '  ',a72,/
     &        '  on unit ',i3,' ...'/)
 1110 format(/'  Reading binary file ',/
     &        '  ',a72,/
     &        '  on unit ',i3,' ...'/)
 1200 format(' **************************************************'/
     &       ' ***  ERROR occured in LINSOL:                  ***')
 1201 format(' ***   some important environment variables     ***'/
     &       ' ***   are not defined!                         ***')
 1202 format(' ***   allocation of array <lmatbk> or <ptrmbk> ***'/
     &       ' ***   failed!                                  ***')
 1203 format(' ***   allocation of array <mat> or <index> or  ***'/
     &       ' ***   <info> on some processors (from 2 up to  ***'/
     &       ' ***   nproc) failed!                           ***')
 1204 format(' ***   allocation of array <x> or <rhs> failed! ***')
 1205 format(' ***   allocation of array <xexact> failed!     ***')
 1206 format(' ***   errorstatus in routine RANDOM <> 0!      ***')
 1207 format(' ***   file handling failed!                    ***')
 1300 format(' **************************************************')
!*********************************************************
!     end of main program
!*********************************************************
      end program
