!***********************************************************************
!        1         2         3         4         5         6         7 *
!***********************************************************************
!**                                                                  ***
!**                                                                  ***
      subroutine LINSTATISTICS(l,mat,lmat,lindex,info,ia1,nvt,ilin,
     &                         idoku,lout)
!**                                                                  ***
!**                                                                  ***
!***********************************************************************
!**                                                                  ***
!**    LINSTATISTICS   enters statistical parameters into array      ***
!**                    <ilin> and prints statistical information     ***
!**                                                                  ***
!***********************************************************************
!**                                                                  ***
!**    Copyright (C) 1997 University Karlsruhe                       ***
!**    PROGRAMMER : H. Haefner                                       ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                  ***
!***********************************************************************
!**                                                                  ***
      USE constants

      implicit none
!**                                                                  *** 
!***********************************************************************
!**                                                                  ***
!**    Formal Parameters :                                           ***
!**                                                                  ***
!***********************************************************************
      integer            l,lmat,lindex,lout,ia1,idoku,nvt
      integer            info(ia1,ia2),ilin(nilin)
      double precision   mat(lmat)
!***********************************************************************
!**                                                                  ***
!**    List of Formal Parameters :                                   ***
!**                                                                  ***
!**                                                                  ***
!---------I------I-----I------------------------------------------------
! NAME    I TYPE I I/O I MEANING
!---------I------I-----I------------------------------------------------
! l       I int  I in  I dimension of the matrix
!---------I------I-----I------------------------------------------------
! lmat    I int  I in  I number of nonzero elements
!---------I------I-----I------------------------------------------------
! lindex  I int  I in  I number of indices
!---------I------I-----I------------------------------------------------
! nvt     I int  I in  I total number of vector-terms
!---------I------I-----I------------------------------------------------
! ia1     I int  I in  I 1.dimension of info-array
!---------I------I-----I------------------------------------------------
! info    I int  I in  I array with matrix-structure-information
!---------I------I-----I------------------------------------------------
! ilin    I int  I out I array with LINSOL-parameters
!---------I------I-----I------------------------------------------------
!**                                                                  ***
!**                                                                  ***
!**    Local Variables :                                             ***
!**                                                                  ***
      integer        i,j,maxi,maxind,maxadd
      integer        imin(ntyp),imax(ntyp),icnt(ntyp)
      real           ravg(ntyp)

!***********************************************************************
!**                                                                  ***
!**    List of Local Variables :                                     ***
!**                                                                  ***
!**                                                                  ***
!---------I------I-----I------------------------------------------------
! NAME    I TYPE I I/O I MEANING
!---------I------I-----I------------------------------------------------
!  imin   I int  I  -  I Minimal v-length of vector terms of type 2-ntyp
!---------I------I-----I------------------------------------------------
!  ravg   I real I  -  I Average v-length of vector terms of type 2-ntyp
!---------I------I-----I------------------------------------------------
!  imax   I int  I  -  I Maximal v-length of vector terms of type 2-ntyp
!---------I------I-----I------------------------------------------------
!***********************************************************************


!***********************************************************************
!**    Start of Calculation                                          ***
!***********************************************************************

!** Size of matrix on processor 1
      maxadd = 0
      do i = 1,nvt
        if (info(i,2) > maxadd) then
	  maxadd = info(i,2)
          maxi   = i
        endif
      enddo
      ilin(30) = info(maxi,2) + info(maxi,3)
      ilin(33) = ilin(30)
!** Size of index array  on processor 1
      maxind = -1
      do i = 1,nvt
	if (info(i,1) == 3 .or. info(i,1) == 4 .or.
     &      info(i,1) == 9) then
	  if (info(i,6) > maxind) then
	    maxind = info(i,6)
	    maxi   = i
          endif
        endif
	if (info(i,1) == 5 .or. info(i,1) == 11) then
	  if (info(i,7) > maxind) then
	    maxind = info(i,7)
	    maxi   = i
          endif
        endif
	if (info(i,1) == 6) then
	  if (info(i,6) > maxind) then
	    maxind = info(i,6)
	    maxi   = i
          endif
	  if (info(i,7) > maxind) then
	    maxind = info(i,7)
	    maxi   = i
          endif
        endif
      enddo
      if (maxind > -1) then
        ilin(31) = maxind + info(maxi,3) 
      else
        ilin(31) = 0
      endif
      ilin(34) = ilin(31)

!** Size of info array on processor 1
      ilin(32) = nvt
      ilin(35) = ilin(32)

      imin = l+1
      ravg = 0.0
      imax = 0
      icnt = 0

!** Vector length statistics
      do i=1,nvt
        if (info(i,1) == 1) then
          do j=1,info(i,3)
            if (mat(info(i,2)+j) /= 0.d0) imax(1) = imax(1) + 1
          enddo
          ravg(1) = ravg(1)+info(i,3)
          icnt(1) = icnt(1)+1
        endif
	if (info(i,1) == 2) then
	  if (info(i,3) < imin(2)) imin(2) = info(i,3)
	  if (info(i,3) > imax(2)) imax(2) = info(i,3)
	  ravg(2) = ravg(2)+info(i,3)
	  icnt(2) = icnt(2)+1
        endif
	if (info(i,1) == 3) then
	  if (info(i,3) < imin(3)) imin(3) = info(i,3)
	  if (info(i,3) > imax(3)) imax(3) = info(i,3)
	  ravg(3) = ravg(3)+info(i,3)
	  icnt(3) = icnt(3)+1
        endif
	if (info(i,1) == 4) then
	  if (info(i,3) < imin(4)) imin(4) = info(i,3)
	  if (info(i,3) > imax(4)) imax(4) = info(i,3)
	  ravg(4) = ravg(4)+info(i,3)
	  icnt(4) = icnt(4)+1
        endif
	if (info(i,1) == 5) then
	  if (info(i,3) < imin(5)) imin(5) = info(i,3)
	  if (info(i,3) > imax(5)) imax(5) = info(i,3)
	  ravg(5) = ravg(5)+info(i,3)
	  icnt(5) = icnt(5)+1
        endif
	if (info(i,1) == 6) then
	  if (info(i,3) < imin(6)) imin(6) = info(i,3)
	  if (info(i,3) > imax(6)) imax(6) = info(i,3)
	  ravg(6) = ravg(6)+info(i,3)
	  icnt(6) = icnt(6)+1
        endif
	if (info(i,1) == 7) then
	  if (info(i,3) < imin(7)) imin(7) = info(i,3)
	  if (info(i,3) > imax(7)) imax(7) = info(i,3)
	  ravg(7) = ravg(7)+info(i,3)
	  icnt(7) = icnt(7)+1
        endif
	if (info(i,1) == 8) then
	  if (info(i,3) < imin(8)) imin(8) = info(i,3)
	  if (info(i,3) > imax(8)) imax(8) = info(i,3)
	  ravg(8) = ravg(8)+info(i,3)
	  icnt(8) = icnt(8)+1
        endif
	if (info(i,1) == 9) then
	  if (info(i,3) < imin(9)) imin(9) = info(i,3)
	  if (info(i,3) > imax(9)) imax(9) = info(i,3)
	  ravg(9) = ravg(9)+info(i,3)
	  icnt(9) = icnt(9)+1
        endif
	if (info(i,1) == 11) then
	  if (info(i,3) < imin(11)) imin(11) = info(i,3)
	  if (info(i,3) > imax(11)) imax(11) = info(i,3)
	  ravg(11) = ravg(11)+info(i,3)
	  icnt(11) = icnt(11)+1
        endif
	if (info(i,3) < imin(10)) imin(10) = info(i,3)
	if (info(i,3) > imax(10)) imax(10) = info(i,3)
	ravg(10) = ravg(10)+info(i,3)
      enddo

      icnt(10) = nvt
      do i=1,ntyp
	if (icnt(i) == 0) imin(i) = 0
      enddo
      do i=1,ntyp
	if (icnt(i) /= 0) then
	  ravg(i) = ravg(i)/icnt(i)
	else
	  ravg(i) = -1.0
	endif
      enddo
      ilin(50) = int(ravg(1))

      if (idoku > 0) then
	write(lout,1000)
	write(lout,1001) lmat
	write(lout,1002) ilin(30)
	write(lout,1003) lindex
	write(lout,1004) ilin(31)
	write(lout,1005) ia1
	write(lout,1006) ia2
	write(lout,1008) nvt
	write(lout,1009) imin(10)
	write(lout,1010) ravg(10)
	write(lout,1011) imax(10)
	write(lout,1007)
	write(lout,1112) icnt(1)
	if (icnt(1) /= 0) then
	  write(lout,1010) ravg(1)
	  write(lout,1111) imax(1)
	endif
	write(lout,1007)
	write(lout,1012) icnt(2)
	if (icnt(2) /= 0) then
	  write(lout,1009) imin(2)
	  write(lout,1010) ravg(2)
	  write(lout,1011) imax(2)
	endif
	write(lout,1007)
	write(lout,1013) icnt(3)
	if (icnt(3) /= 0) then
	  write(lout,1009) imin(3)
	  write(lout,1010) ravg(3)
	  write(lout,1011) imax(3)
	endif
	write(lout,1007)
	write(lout,1014) icnt(4)
	if (icnt(4) /= 0) then
	  write(lout,1009) imin(4)
	  write(lout,1010) ravg(4)
	  write(lout,1011) imax(4)
	endif
	write(lout,1007)
	write(lout,1015) icnt(5)
	if (icnt(5) /= 0) then
	  write(lout,1009) imin(5)
	  write(lout,1010) ravg(5)
	  write(lout,1011) imax(5)
	endif
	write(lout,1007)
	write(lout,1016) icnt(6)
	if (icnt(6) /= 0) then
	  write(lout,1009) imin(6)
	  write(lout,1010) ravg(6)
	  write(lout,1011) imax(6)
	endif
	write(lout,1007)
	write(lout,1017) icnt(7)
	if (icnt(7) /= 0) then
	  write(lout,1009) imin(7)
	  write(lout,1010) ravg(7)
	  write(lout,1011) imax(7)
	endif
	write(lout,1007)
	write(lout,1018) icnt(8)
	if (icnt(8) /= 0) then
	  write(lout,1009) imin(8)
	  write(lout,1010) ravg(8)
	  write(lout,1011) imax(8)
	endif
	write(lout,1007)
	write(lout,1019) icnt(9)
	if (icnt(9) /= 0) then
	  write(lout,1009) imin(9)
	  write(lout,1010) ravg(9)
	  write(lout,1011) imax(9)
	endif
	write(lout,1007)
	write(lout,1020) icnt(11)
	if (icnt(11) /= 0) then
	  write(lout,1009) imin(11)
	  write(lout,1010) ravg(11)
	  write(lout,1011) imax(11)
	endif
	write(lout,1021)
      endif



!***********************************************************************
!**    End of Calculation                                            ***
!***********************************************************************

 1000 format(/'  LINSOL STATISTICS: ')
 1001 format('   Dimension of (global) matrix array (lmat) ......:',i10)
 1002 format('   Size of (global) matrix . . . . . . . . . . . . :',i10)
 1003 format('   Dimension of (global) index array (lindex) .....:',i10)
 1004 format('   Size of (global) index array . . . . . . . . . .:',i10)
 1005 format('   First Dimension of (global) info array (ia1) ...:',i10)
 1006 format('   Second Dimension of (global) info array (ia2)   :',i10)
 1007 format('   -------------------------------------------------')
 1008 format('   Number of (global) vector terms ................:',i10)
 1009 format('   Minimal vector length . . . . . . . . . . . . . :',i10)
 1010 format('   Average vector length ..........................:',
     &       f10.2)
 1011 format('   Maximal vector length . . . . . . . . . . . . . :',i10)
 1111 format('   Number of non-zero elements . . . . . . . . . . :',i10)
 1112 format('   Number of (global) vector terms of type 1 ......:',i10)
 1012 format('   Number of (global) vector terms of type 2 ......:',i10)
 1013 format('   Number of (global) vector terms of type 3 ......:',i10)
 1014 format('   Number of (global) vector terms of type 4 ......:',i10)
 1015 format('   Number of (global) vector terms of type 5 ......:',i10)
 1016 format('   Number of (global) vector terms of type 6 ......:',i10)
 1017 format('   Number of (global) vector terms of type 7 ......:',i10)
 1018 format('   Number of (global) vector terms of type 8 ......:',i10)
 1019 format('   Number of (global) vector terms of type 9 ......:',i10)
 1020 format('   Number of (global) vector terms of type 11 .....:',i10)
 1021 format('   -------------------------------------------------'//)
       r e t u r n
!---- END OF LINSTATISTICS -----------------------------------------
       e    n    d
