!***********************************************************************
!        1         2         3         4         5         6         7 *
!***********************************************************************
!**                                                                  ***
!**                                                                  ***
      subroutine RCV_RHS_IGUESS(x,rhs,llocal,lmatbk,rhsin,startsol,
     &                          split,infmt,ierr,nrhs,irhs,idoku,lout,
     &                          tids,nproc,myproc,nmsg)
!**                                                                  ***
!**                                                                  ***
!***********************************************************************
!**                                                                  ***
!**    RCV_RHS_IGUESS   receives parts of right hand sides and of    ***
!**                     one initial guess                            ***
!**                                                                  ***
!***********************************************************************
!**                                                                  ***
!**    Copyright (C) 1997 University Karlsruhe                       ***
!**    PROGRAMMER : H. Haefner                                       ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                  ***
!***********************************************************************
!**                                                                  ***
      USE constants

      implicit none
!**                                                                  *** 
!***********************************************************************
!**                                                                  ***
!**    Formal Parameters :                                           ***
!**                                                                  ***
!***********************************************************************
      integer          llocal,ierr,myproc,startsol,split,nproc,nmsg
      integer          idoku,infmt,lout,nrhs,irhs
      integer          lmatbk(nproc),tids(nproc)
      double precision x(llocal),rhs(llocal)
      character*72     rhsin
!***********************************************************************
!**                                                                  ***
!**    List of Formal Parameters :                                   ***
!**                                                                  ***
!**                                                                  ***
!---------I------I-----I------------------------------------------------
! NAME    I TYPE I I/O I MEANING
!---------I------I-----I------------------------------------------------
! llocal  I int  I in  I dimension of the local matrices
!---------I------I-----I------------------------------------------------
! startsolI int  I in  I indicates, if initial guess is used
!---------I------I-----I------------------------------------------------
! ierr    I int  I out I error-indication
!---------I------I-----I------------------------------------------------
!**                                                                  ***
!**                                                                  ***
!**    Local Variables :                                             ***
!**                                                                  ***
      integer          i,j,iermax,ihelp,iounit,from,midr,lfile,nfile
      integer          lmyblk
      double precision dw(abs(nrhs))
      real*4           realbuf(abs(nrhs))
      character*5      rhssufix


!***********************************************************************
!**    Start of Calculation                                          ***
!***********************************************************************

      ierr = 0000

      lmyblk = lmatbk(myproc)

      if (split == 1 .and. nrhs > 0) then
        write (rhssufix,'(A1,i4.4)') '_',myproc
	rhsin = trim(rhsin) // rhssufix
        iounit = 12
        if (infmt == 0) then
          open(UNIT=iounit,FILE=rhsin,FORM='UNFORMATTED')
        else
          open(UNIT=iounit,FILE=rhsin,FORM='FORMATTED')
        endif
	if (idoku < 0) write(lout,1090) rhsin,iounit
        if (infmt == 0) read (iounit) lfile,nfile
        if (infmt > 0)  read (iounit,*) lfile,nfile
        if (lfile > llocal .or. lfile < lmyblk) then
          ierr = 0001
          write(lout,1010)
          write(lout,1011) lfile,llocal
          write(lout,1020)
          goto 999
        endif
        if (nfile .ne. nrhs) then
          write(lout,1030)
          write(lout,1012) nfile,nrhs
          write(lout,1020)
	  nrhs = nfile
	endif
        if (infmt == 0) then
          do i=1,lmyblk
            read (iounit) (realbuf(j),j=1,nrhs)
            rhs(i) = DBLE(realbuf(irhs))
          enddo
        endif
        if (infmt > 0) then
          do i=1,lmyblk
            read (iounit,*) (dw(j),j=1,nrhs)
            rhs(i) = dw(irhs)
          enddo
        endif
        close(iounit)
      endif

  999 call LL4INM(1,1,1,ierr,iermax,ihelp,myproc,nproc,
     &            tids,nmsg)
      if (iermax .ne. 0) then
        ierr = iermax
        stop
      endif
!**********************************************************
!   receiving parts from right-hand-side and initial guess 
!   on processors <> 1
!**********************************************************
      from = tids(1)
      if (split == 0) then
        call MPRCVA(from,nmsg+myproc-1,lmatbk(myproc)*ireal,
     &              rhs(1),midr,ierr)
        call MPRCVW(from,nmsg+myproc-1,lmatbk(myproc)*ireal,
     &              rhs(1),midr,ierr)
      endif
      if (startsol == 1) then
        call MPRCVA(from,nmsg+myproc,lmatbk(myproc)*ireal,
     &              x(1),midr,ierr)
        call MPRCVW(from,nmsg+myproc,lmatbk(myproc)*ireal,
     &              x(1),midr,ierr)
      endif
      nmsg = nmsg+nproc

!***********************************************************************
!**    End of Calculation                                            ***
!***********************************************************************

 1010 format('  ************************************************'/
     &       '  ***  LINSOL: F A T A L  E R R O R            ***')
 1030 format('  ************************************************'/
     &       '  ***  LINSOL: W A R N I N G                   ***')
 1011 format('  ***   The number of coefficients of the      ***'/
     &       '  ***   RHS ',i7,' does not fit to the        ***'/
     &       '  ***   dimension of the matrix ',i7,'        ***')
 1012 format('  ***   The number of Right Hand Sides         ***'/
     &       '  ***   ',i6,' read from file is not equal     ***'/
     &       '  ***   to the number of RHS chosen by the     ***'/
     &       '  ***   user ',i6,': the value read from       ***'/
     &       '  ***   file will be taken !!!                 ***')
 1020 format('  ************************************************'/)
 1090 format(/'  Reading file ',/
     &       '  ',a72,/
     &       '  on unit ',i3,' ...'/)

       r e t u r n
!-----END OF RCV_RHS_IGUESS -------------------------------------------
       e    n    d
