!***********************************************************************
!        1         2         3         4         5         6         7 *
!***********************************************************************
!**                                                                  ***
!**                                                                  ***
      subroutine READ_MATRIX_PARAM(l,llocal,MAT,lmat,index,lindex,lsym,
     &                info,ia1,nvt,n_rhs,idoku,lout,infmt,split,optim,
     &                isprec,msprec,bwoalg,
     &                lmatbk,ptrmbk,matrix,tids,nproc,myproc,nmsg,ierr)
!**                                                                  ***
!**                                                                  ***
!***********************************************************************
!**                                                                  ***
!**    READ_MATRIX_PARAM  reads important matrix-parameters from     ***
!**                       file                                       ***
!**                                                                  ***
!***********************************************************************
!**                                                                  ***
!**    Copyright (C) 1996 University Karlsruhe                       ***
!**    PROGRAMMER : H. Haefner                                       ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                  ***
!***********************************************************************
!**                                                                  ***
      USE scriptparms
      USE constants

      implicit none
!**                                                                  *** 
!***********************************************************************
!**                                                                  ***
!**    Formal Parameters :                                           ***
!**                                                                  ***
!***********************************************************************
      integer            l,llocal,lmat,lindex,nvt,n_rhs,ia1,ierr,myproc,
     &                   ier,idoku,lout,infmt,optim,split,nproc,nmsg,
     &                   isprec,msprec,bwoalg
      integer            lmatbk(nproc),ptrmbk(nproc+1),tids(nproc)
      integer,pointer :: info(:,:),index(:)
      double precision,pointer :: MAT(:)
      character*72     matrix
      logical          lsym
!***********************************************************************
!**                                                                  ***
!**    List of Formal Parameters :                                   ***
!**                                                                  ***
!**                                                                  ***
!---------I------I-----I------------------------------------------------
! NAME    I TYPE I I/O I MEANING
!---------I------I-----I------------------------------------------------
! l       I int  I in  I dimension of the matrix
!---------I------I-----I------------------------------------------------
! lmat    I int  I in  I number of nonzero elements
!---------I------I-----I------------------------------------------------
! lindex  I int  I in  I number of indices
!---------I------I-----I------------------------------------------------
! nvt     I int  I out I total number of vector-terms
!---------I------I-----I------------------------------------------------
! ia1     I int  I in  I 1.dimension of info-array
!---------I------I-----I------------------------------------------------
! info    I int  I out I array with matrix-structure-information
!---------I------I-----I------------------------------------------------
! index   I int  I out I array with indices
!---------I------I-----I------------------------------------------------
! MAT     I real I out I array with nonzero matrix-elements
!---------I------I-----I------------------------------------------------
! lsym    I l    I out I indicates if matrix is symmetric
!---------I------I-----I------------------------------------------------
! lin     I int  I in  I i/o-unit to read from
!---------I------I-----I------------------------------------------------
! ierr    I int  I out I error-indication
!---------I------I-----I------------------------------------------------
!**                                                                  ***
!**                                                                  ***
!**    Local Variables :                                             ***
!**                                                                  ***
      integer        i,iounit,ibuf(5),mids,midr,to,from,iermax,ihelp
!***********************************************************************
!**    Variables for matrices in Harwell/Boeing-format :             ***
!***********************************************************************
      character      title*72, key*8, type*3, ptrfmt*16, indfmt*16,
     &               valfmt*20, rhsfmt*20, rhstyp*3, guesol*2, c
      integer        totcrd, ptrcrd, indcrd, valcrd, rhscrd, nrow,
     &               ncol, nnz, neltvl, nrwindx
      integer        lmatbuf(nproc),lindbuf(nproc),ia1buf(nproc)
      integer,allocatable :: ia(:),iacol(:),indcol(:)
      real           matfac,indfac,infofac
      double precision,allocatable :: matcol(:)

!***********************************************************************
!**                                                                  ***
!**    List of Local Variables :                                     ***
!**                                                                  ***
!**                                                                  ***
!---------I------I-----I------------------------------------------------
! NAME    I TYPE I I/O I MEANING
!---------I------I-----I------------------------------------------------
!  nnz    I int  I  -  I Length of matrix in H/B-Format
!---------I------I-----I------------------------------------------------
!  nrow   I int  I  -  I Number of rows of matrix in H/B-Format
!---------I------I-----I------------------------------------------------
!  ia     I int  I  -  I contains the pointers the beginning of each row
!         I      I     I in the H/B-matrix and -indices, array:ia(nrow+1)
!---------I------I-----I------------------------------------------------
!***********************************************************************


!***********************************************************************
!**    Start of Calculation                                          ***
!***********************************************************************

      ierr = 0

      if (myproc == 1) then
        iounit = 11
        if (infmt == 0) then
          open(UNIT=iounit,FILE=matrix,FORM='UNFORMATTED')
        else
          open(UNIT=iounit,FILE=matrix,FORM='FORMATTED')
	endif
      endif
      
!*********************************************************
!   reading parameters for matrix in LINSOL-Format
!   IMPORTANT : parameters for the complete matrix
!*********************************************************
      if ((infmt == 0) .and. (myproc == 1)) then
        if (idoku /= 0) write(lout,1100) matrix,iounit
        read (iounit) lmat,lindex,l,lsym,nvt
!       if (lindex == 0) lindex=1
        close(iounit)
      endif
      if ((infmt == 1) .and. (myproc == 1)) then
        if (idoku /= 0) write(lout,1150) matrix,iounit
        read (iounit,*) lmat,lindex,l,lsym,nvt
!       if (lindex == 0) lindex=1
        close(iounit)
      endif

!*********************************************************
!   reading parameters for matrix in Harwell/Boeing-format
!*********************************************************
      if ((infmt == 2) .and. (myproc == 1)) then
        if (idoku /= 0) write(lout,1200) matrix,iounit
        read (iounit,1000) title, key, totcrd, ptrcrd, 
     &               indcrd,valcrd, rhscrd, type, nrow, ncol, 
     &               nnz, neltvl, ptrfmt,indfmt, valfmt, rhsfmt
 1000   format (a72,a8/5i14/a3,11x,4i14/2a16,2a20)
        if (rhscrd > 0) then
          read (iounit,1001) rhstyp, n_rhs, nrwindx
        endif
 1001   format (a3,11x,i14,i14)

!   check if symmetric matrix

        read (type, '(1X,A1)') c
        if (c == 'S') then
          lsym = .true.
        else
          lsym = .false.
        endif

!   convert harwell/boeing-parameters to linsol parameters

        lmat   = nnz
        l      = nrow
        lindex = nnz
        nvt    = nrow

!*********************************************************
!   Init parameters on all processors for matrix distributing 
!   IMPORTANT: ia1 is dimensioned very high
!*********************************************************
        ia1    = nvt + 1

!   allocate arrays for matrix in harwell/boeing format

!       if (idoku > 0) then
!         if (myproc == 1) write(lout,1058) (ncol+1)*iint
!       endif
!       if (idoku < 0) write(lout,1058) (ncol+1)*iint
        allocate(iacol(ncol+1),stat=ierr)
!       if (idoku > 0) then
!         if (myproc == 1) write(lout,1060) lmat*ireal
!       endif
!       if (idoku < 0) write(lout,1060) lmat*ireal
        allocate (matcol(lmat),stat=ierr)
!       if (idoku > 0) then
!         if (myproc == 1) write(lout,1061) lindex*iint
!       endif
!       if (idoku < 0) write(lout,1061) lindex*iint
        allocate (indcol(lindex),stat=ierr)

!   read indices and matrix-elements

        if (idoku /= 0) write(lout,1250) matrix,iounit
        read (iounit,ptrfmt) (iacol(i),i=1,ncol+1)
        read (iounit,indfmt) (indcol(i),i=1,lindex)
        read (iounit,valfmt) (matcol(i),i=1,lmat)

!       if (idoku > 0) then
!         if (myproc == 1) write(lout,1059) (nrow+1)*iint
!       endif
!       if (idoku < 0) write(lout,1059) (nrow+1)*iint
        allocate(ia(nrow+1),stat=ierr)

	call getKeyword('LINSOL_MATRIX_FACTOR',matfac,ier)
        if (ier /= 0) then
          matfac = 1.0
	  if (idoku /= 0) write(lout,1330)
        endif
        lmat = MAX(lmat,INT(lmat*matfac/nproc))
!       if (idoku > 0) then
!         if (myproc == 1) write(lout,1062) lmat*ireal
!       endif
!       if (idoku < 0) write(lout,1062) lmat*ireal
        allocate (mat(lmat),stat=ierr)

	call getKeyword('LINSOL_INDEX_FACTOR',indfac,ier)
        if (ier /= 0) then
          indfac = 1.0
	  if (idoku /= 0) write(lout,1340)
        endif
        lindex = INT(lindex*indfac)
!       if (idoku > 0) then
!         if (myproc == 1) write(lout,1063) lindex*iint
!       endif
!       if (idoku < 0) write(lout,1063) lindex*iint
        allocate (index(lindex),stat=ierr)

        call CSRCSC(l,1,1,matcol,indcol,iacol,mat,index,ia)
	deallocate(iacol)
!       if (idoku > 0) then
!         if (myproc == 1) write(lout,1065)
!       endif
!       if (idoku < 0) write(lout,1065)
	deallocate(matcol)
!       if (idoku > 0) then
!         if (myproc == 1) write(lout,1066)
!       endif
!       if (idoku < 0) write(lout,1066)
	deallocate(indcol)
!       if (idoku > 0) then
!         if (myproc == 1) write(lout,1067)
!       endif
!       if (idoku < 0) write(lout,1067)

	call getKeyword('LINSOL_INFO_FACTOR',infofac,ier)
        if (ier /= 0) then
          infofac = 1.0
	  if (idoku /= 0) write(lout,1350)
        endif
        ia1  = INT(ia1*infofac)
!       if (idoku > 0) then
!         if (myproc == 1) write(lout,1064) ia1*ia2*iint
!       endif
!       if (idoku < 0) write(lout,1064) ia1*ia2*iint
        allocate(info(ia1,ia2),stat=ierr)

        call CSRLIN(ia,l,info,ia1)

	deallocate(ia)
!       if (idoku > 0) then
!         if (myproc == 1) write(lout,1068)
!       endif
!       if (idoku < 0) write(lout,1068)

        call INIT_LINSOL_PARAM(lmatbk,ptrmbk,llocal,l,nproc)
        call SET_ARRAY_HB(lmatbk,ptrmbk,info,ia1,nproc,matfac,
     &                    indfac,infofac,lmatbuf,lindbuf,ia1buf)

        if (n_rhs == 0) close(iounit)
      endif

!*********************************************************
!   parameters for matrix in LINSOL-Format are initialized
!*********************************************************
      if ((infmt <= 1) .and. (myproc == 1)) then
        call INIT_LINSOL_PARAM(lmatbk,ptrmbk,llocal,l,nproc)
        call SET_ARRAY_LINSOL(lmatbk,ptrmbk,lmat,lindex,nvt,nproc,
     &                        lmatbuf,lindbuf,ia1buf,optim,isprec,
     &                        msprec,bwoalg,ier)
	if (ier/=0) then
	  if (idoku /= 0) write(lout,1300)
	endif
!***  Could be optimized in view of memory request !!!
        lmat   = lmatbuf(1)
        lindex = lindbuf(1)
        ia1    = ia1buf(1)
!       if (idoku > 0) then
!         if (myproc == 1) write(lout,1062) lmat*ireal
!       endif
!       if (idoku < 0) write(lout,1062) lmat*ireal
        allocate (mat(lmat),stat=ierr)
!       if (idoku > 0) then
!         if (myproc == 1) write(lout,1063) lindex*iint
!       endif
!       if (idoku < 0) write(lout,1063) lindex*iint
        allocate (index(lindex),stat=ierr)
!       if (idoku > 0) then
!         if (myproc == 1) write(lout,1064) ia1*ia2*iint
!       endif
!       if (idoku < 0) write(lout,1064) ia1*ia2*iint
        allocate (info(ia1,ia2),stat=ierr)
      endif

      call LL4INM(1,1,1,ierr,iermax,ihelp,myproc,nproc,tids,nmsg)
      if (iermax /= 0) then
        ierr  = iermax
        if ((infmt <= 1) .and. (myproc == 1)) then
          write(lout,1020)
          write(lout,1021)
          write(lout,1030)
        endif
        if ((infmt == 2) .and. (myproc == 1)) then
          write(lout,1020)
          write(lout,1022)
          write(lout,1030)
        endif
        goto 999
      endif


!   fill buffer with parameters and send buffer to all processors

      if (nproc > 1) then
        if (myproc == 1) then
          ibuf(1) = llocal
          if (lsym) then
            ibuf(5) = 1
          else
            ibuf(5) = 0
          endif
          do i=2,nproc
            ibuf(2) = lmatbuf(i)
            ibuf(3) = lindbuf(i)
            ibuf(4) = ia1buf(i)
            to = tids(i)
            call MPSNDA(to,nmsg+i,5*iint,ibuf(1),mids,ierr)
            call MPSNDW(to,nmsg+i,5*iint,ibuf(1),mids,ierr)
            call MPSNDA(to,nmsg+nproc+i,nproc*iint,lmatbk,mids,ierr)
            call MPSNDW(to,nmsg+nproc+i,nproc*iint,lmatbk,mids,ierr)
            call MPSNDA(to,nmsg+2*nproc+i,(nproc+1)*iint,ptrmbk,
     *                  mids,ierr)
            call MPSNDW(to,nmsg+2*nproc+i,(nproc+1)*iint,ptrmbk,
     *                  mids,ierr)
          enddo
          nmsg = nmsg+3*nproc
        else 

!   receive important dimension-parameters from processor 1 
!   if matrix is in LINSOL- or Harwell/Boeing-Format

          from = tids(1)
          call MPRCVA(from,nmsg+myproc,5*iint,ibuf(1),midr,ierr)
          call MPRCVW(from,nmsg+myproc,5*iint,ibuf(1),midr,ierr)
          call MPRCVA(from,nmsg+nproc+myproc,nproc*iint,lmatbk,
     *                midr,ierr)
          call MPRCVW(from,nmsg+nproc+myproc,nproc*iint,lmatbk,
     *                midr,ierr)
          call MPRCVA(from,nmsg+2*nproc+myproc,(nproc+1)*iint,
     *                ptrmbk,midr,ierr)
          call MPRCVW(from,nmsg+2*nproc+myproc,(nproc+1)*iint,
     *                ptrmbk,midr,ierr)
          nmsg = nmsg+3*nproc

          llocal = ibuf(1)
          lmat   = ibuf(2)
          lindex = ibuf(3)
          ia1    = ibuf(4)
          if (ibuf(5) == 1) then
            lsym = .true.
          else
            lsym = .false.
          endif
	endif
      endif

  999 continue

!***********************************************************************
!**    End of Calculation                                            ***
!***********************************************************************

 1020 format(' **************************************************'/
     &       ' ***  ERROR occured in LINSOL:                  ***')
 1021 format(' ***   allocation of arrays needed to read      ***'/
     &       ' ***   matrix in LINSOL-fileformat on           ***'/
     &       ' ***   processor 1 failed!                      ***')
 1022 format(' ***   allocation of arrays needed to read      ***'/
     &       ' ***   matrix in H/B-fileformat failed!         ***')
 1030 format(' **************************************************')
!1058 format(/'  Allocating storage of size (in Bytes) ',i11,/
!    &       '  for Harwell/Boeing pointer array <iacol> ........'/)
!1059 format('  Allocating storage of size (in Bytes) ',i11,/
!    &       '  for Harwell/Boeing pointer array <ia> ...........'/)
!1060 format('  Allocating storage of size (in Bytes) ',i11,/
!    &       '  for H/B-matrix array <MATCOL> ...................'/)
!1061 format('  Allocating storage of size (in Bytes) ',i11,/
!    &       '  for H/B-index array <indcol> ....................'/)
!1062 format(/'  Allocating storage of size (in Bytes) ',i11,/
!    &       '  for matrix array <MAT> ..........................'/)
!1063 format('  Allocating storage of size (in Bytes) ',i11,/
!    &       '  for index array <index> .........................'/)
!1064 format('  Allocating storage of size (in Bytes) ',i11,/
!    &       '  for matrix information array <info> .............'/)
!1065 format('  Storage for array  <iacol> deallocated ')
!1066 format('  Storage for array <MATCOL> deallocated ')
!1067 format('  Storage for array <indcol> deallocated '/)
!1068 format('  Storage for array     <ia> deallocated '/)
 1100 format(/'  Reading parameters of binary file ',/
     &       '  ',a72,/
     &       '  on unit ',i3,' ...'/)
 1150 format(/'  Reading parameters of file ',/
     &       '  ',a72,/
     &       '  on unit ',i3,' ...'/)
 1200 format(/'  Reading parameters of file in H/B-Format',/
     &       '  ',a72,/
     &       '  on unit ',i3,' ...'/)
 1250 format(/'  Reading file in H/B-Format',/
     &       '  ',a72,/
     &       '  on unit ',i3,' ...'/)
 1300 format(/'  Warning:',/
     &       '     At least one of the environment variables',/
     &       '     LINSOL_{MAT,INDEX,INFO}_FACTOR in parameter ',/
     &       '     file not correctly set --> the variable(s) ',/
     &       '     are set to 1!',/)
 1330 format(/'  Warning:',/
     &       '     The environment variable LINSOL_MATRIX_FACTOR in',/
     &       '     parameter file is not correctly set --> the',/
     &       '     environment variable is set to 1! '/)
 1340 format(/'  Warning:',/
     &       '     The environment variable LINSOL_INDEX_FACTOR in',/
     &       '     parameter file is not correctly set --> the',/
     &       '     environment variable is set to 1! '/)
 1350 format(/'  Warning:',/
     &       '     The environment variable LINSOL_INFO_FACTOR in',/
     &       '     parameter file is not correctly set --> the',/
     &       '     environment variable is set to 1! '/)
       r e t u r n
!---- END OF READ_MATRIX_PARAM -----------------------------------------
       e    n    d
