!**********************************************************************
!        1         2         3         4         5         6         7*
!**********************************************************************
!**                                                                 ***
!**                                                                 ***
      subroutine READ_MATRIX_PART(myproc,mat,lmat,index,lindex,info,
     &           ia1,nvt,lsym,tids,nproc,nmsg,ierr,matrix,l,llocal,
     &           lmatbk,ptrmbk,idoku,infmt,lout,isprec,msprec,bwoalg,
     &           optim)
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**      Copyright (C) 1996 University Karlsruhe                    ***
!**      PROGRAMMER:    M.Peter  M.Geisel                           ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
      USE scriptparms
      USE constants

      implicit none

!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**      FORMAL PARAMETERS :                                        ***
!**      -----------------                                          ***
!**                                                                 ***
      integer myproc,nproc,nmsg,l,llocal,lmat,lindex,ia1,nvt,ierr,
     &        lmatbk(nproc),ptrmbk(nproc+1),tids(nproc),idoku,
     &        infmt,lout,isprec,msprec,bwoalg,optim
      integer, pointer :: index(:),info(:,:)
      double precision, pointer :: mat(:)
      character*72 matrix
      logical      lsym
!**                                                                 ***
!**      LOCAL VARIABLES :                                          ***
!**      -----------------                                          ***
!**                                                                 ***
      integer     iounit,mids,midr,to,from,llocmax,lvt,i,ihelp,ptr
      integer     lmatd,lixd
      real        pmatfac,pindfac,pinfofac
      character*5 matsufix

!**********************************************************************
!**    Start of calculation                                         ***
!**********************************************************************

      iounit = 23
      write (matsufix,'(A1,i4.4)') '_',myproc
      matrix = trim(matrix) // matsufix
      if (infmt == 0) open(UNIT=iounit,FILE=matrix,FORM='UNFORMATTED')
      if (infmt == 1) open(UNIT=iounit,FILE=matrix,FORM='FORMATTED')

      if (idoku > 0) then
        if (myproc == 1) write(lout,1000) matrix,iounit
      endif
      if (idoku < 0) then
        write(lout,1000) matrix,iounit
      endif
!   read important dimension-parameters
!   IMPORTANT : these variables are local

      if (infmt == 0) read (iounit) lmat,lindex,llocal,lsym,nvt
      if (infmt == 1) read (iounit,*) lmat,lindex,llocal,lsym,nvt
      close (iounit)

      call getKeyword('LINSOL_MATRIX_FACTOR',pmatfac,ierr)
      if (ierr /= 0) then
!       iermax = ierr
        pmatfac = 1.0
      endif
      call getKeyword('LINSOL_INDEX_FACTOR',pindfac,ierr)
      if (ierr /= 0) then
!       if (ierr > iermax) iermax = ierr
        pindfac = 1.0
      endif
      call getKeyword('LINSOL_INFO_FACTOR',pinfofac,ierr)
      if (ierr /= 0) then
!       if (ierr > iermax) iermax = ierr
        pinfofac = 1.0
      endif
      ierr = 0

      lmat = int(lmat*pmatfac)
      lindex = int(lindex*pindfac)
      if (((msprec >= 63) .and. (isprec < 10)) .or. (bwoalg > 0) .or.
     &    (mod(optim,100000) >= 10000) .or.
     &    (mod(optim,200000) >= 100000)) lindex = lmat
      ia1  = int(nvt*pinfofac)

      allocate (mat(lmat), index(lindex), info(ia1,ia2))

      lmatd = lmat
      lixd  = lindex

!   send llocal to processor 1 to calculate global l

      if (myproc == 1) then
	 lmatbk(1) = llocal
         do i=2,nproc
            from = tids(i)   
            call MPRCVA(from,nmsg+i,1*iint,lmatbk(i),midr,ierr)
            call MPRCVW(from,nmsg+i,1*iint,lmatbk(i),midr,ierr)
         enddo
      else
         to = tids(1)
         call MPSNDA(to,nmsg+myproc,1*iint,llocal,mids,ierr)
         call MPSNDW(to,nmsg+myproc,1*iint,llocal,mids,ierr)
      endif
      nmsg = nmsg+nproc

!   send array lmatbk to all processors

      if (myproc == 1) then
         do i=2,nproc
            to = tids(i)  
            call MPSNDA(to,nmsg+i,nproc*iint,lmatbk,mids,ierr)
            call MPSNDW(to,nmsg+i,nproc*iint,lmatbk,mids,ierr)
         enddo
      else
         from = tids(1)
         call MPRCVA(from,nmsg+myproc,nproc*iint,lmatbk,mids,ierr)
         call MPRCVW(from,nmsg+myproc,nproc*iint,lmatbk,mids,ierr)
      endif
      nmsg = nmsg+nproc

      l = 0
      do i=1,nproc
	l = l+lmatbk(i)
      enddo

      if (infmt == 0) then
        iounit = 23
        open(UNIT=iounit,FILE=matrix,FORM='UNFORMATTED')
        if (idoku > 0) then
          if (myproc == 1) write(lout,1110) matrix,iounit
        endif
        if (idoku < 0) then
          write(lout,1110) matrix,iounit
        endif
        call LINBIN(mat,lmatd,index,lixd,info,ia1,nvt,llocal,l,
     &              lsym,iounit,lout,ierr)
        close(iounit)
      endif
      if (infmt == 1) then
        iounit = 23
        open(UNIT=iounit,FILE=matrix,FORM='FORMATTED')
        if (idoku > 0) then
          if (myproc == 1) write(lout,1100) matrix,iounit
        endif
        if (idoku < 0) then
          write(lout,1100) matrix,iounit
        endif
        call LININ(mat,lmatd,index,lixd,info,ia1,nvt,llocal,l,lsym,
     &             iounit,lout,ierr)
        close(iounit)
      endif

      call LL4INM(1,1,1,llocal,llocmax,ihelp,myproc,nproc,tids,nmsg)
      llocal = llocmax

      ptrmbk(1) = 0
      do i=2,nproc+1
	ptrmbk(i) = ptrmbk(i-1) + lmatbk(i-1)
      enddo
      
 1000 format(/'  Reading parameters of file ',/
     &       '  ',a72,/
     &       '  on unit ',i3,' ...'/)
 1100 format(/'  Reading file ',/
     &        '  ',a72,/
     &        '  on unit ',i3,' ...'/)
 1110 format(/'  Reading binary file ',/
     &        '  ',a72,/
     &        '  on unit ',i3,' ...'/)
     
      r e t u r n
C---- END OF READ_MATRIX_PART -----------------------------------------
      e    n    d
