!**********************************************************************
!        1         2         3         4         5         6         7*
!**********************************************************************
!**                                                                 ***
!**                                                                 ***
      subroutine READ_PARAM_FILE(infmt,split,matrix,outfmt,
     #                           outnam,inrannam,nrhs,rhsin,solout,
     #                           startsol,solin,tmpdir,method,itmax,
     #                           outdev,isprec,msprec,noprec,idoku,
     #                           msnorm,smooth,misc,optim,bufsize,
     #                           bwoalg,runacc,rlin,
     #                           nproc,tids,nmsg,ierr)
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**      Read input-parameters on processor 1 and send              ***
!**      them to all other processors                               ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**      Copyright (C) 1999 University Karlsruhe                    ***
!**      PROGRAMMER:    H. Haefner                                  ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
      USE scriptparms
      USE constants

      implicit none

!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**      FORMAL PARAMETERS :                                        ***
!**      -----------------                                          ***
!**                                                                 ***
      integer,intent(in) ::  nproc,tids(nproc)
      integer,intent(inout) ::  nmsg
      integer,intent(out) ::  infmt,split,outfmt,nrhs,startsol,method
      integer,intent(out) ::  outdev,noprec,idoku,msprec,smooth,misc
      integer,intent(out) ::  isprec,msnorm,optim,itmax,bufsize,ierr
      integer,intent(out) ::  bwoalg,runacc
      character*72,intent(out) ::  matrix,outnam,rhsin,solout,solin
      character*72,intent(out) ::  inrannam,tmpdir
      double precision,intent(out) ::  rlin(nrlin)
!**                                                                 ***
!**                                                                 ***
!**      LOCAL VARIABLES :                                          ***
!**      -----------------                                          ***
!**                                                                 ***
      integer,parameter ::  nparm=19, nrparm=6
      integer i,ier,mids,to,ibuf(nparm),lout
      real    epslin,gaussfac,thres1,thres2,ldrop,memfac

!**********************************************************************
!**    Start of calculation                                         ***
!**********************************************************************

      lout   =  6
      ierr   =  0

      bwoalg  = 0
      bufsize = 0
      runacc  = 0

      thres1 = 0.0
      thres2 = 0.0
      gaussfac = 0.0
      memfac = 0.0

      call getKeyword('LINSOL_INPUT_FORMAT',infmt,ier)
      if (ier /= 0) then
	ierr = 1
	write(lout,1000)
	write(lout,1001)
      endif
      if (infmt == 3) then
        call getKeyword('LINSOL_RANDOM_PARAMETER_FILENAME',inrannam,ier)
        if (ier /= 0) then
	  if (ierr == 0) write(lout,1000)
	  ierr = 1
	  write(lout,1002)
        endif
      endif
      call getKeyword('LINSOL_MATRIX_DISTRIBUTED',split,ier)
      if (ier /= 0) then
	if (ierr == 0) write(lout,1000)
	ierr = 1
	write(lout,1003)
      endif
      call getKeyword('LINSOL_MATRIX_INPUT_FILENAME',matrix,ier)
      if (ier /= 0) then
	if (ierr == 0) write(lout,1000)
	ierr = 1
	write(lout,1004)
      endif
      call getKeyword('LINSOL_MATRIX_OUTPUT_FORMAT',outfmt,ier)
      if (ier /= 0) then
	if (ierr == 0) write(lout,1000)
	ierr = 1
	write(lout,1005)
      endif
      if (outfmt > 0) then
        call getKeyword('LINSOL_MATRIX_OUTPUT_FILENAME',outnam,ier)
        if (ier /= 0) then
	  if (ierr == 0) write(lout,1000)
	  ierr = 1
	  write(lout,1006)
        endif
      endif
      call getKeyword('LINSOL_NUMBER_RHS',nrhs,ier)
      if (ier /= 0) then
	if (ierr == 0) write(lout,1000)
	ierr = 1
	write(lout,1007)
      endif
      if (nrhs > 0) then
        call getKeyword('LINSOL_RHS_INPUT_FILENAME',rhsin,ier)
        if (ier /= 0) then
	  if (ierr == 0) write(lout,1000)
	  ierr = 1
	  write(lout,1008)
        endif
      endif
      call getKeyword('LINSOL_SOLUTIONS_FILENAME',solout,ier)
      if (ier /= 0) then
	if (ierr == 0) write(lout,1000)
	ierr = 1
	write(lout,1009)
      endif
      call getKeyword('LINSOL_INITGUESS',startsol,ier)
      if (ier /= 0) then
	if (ierr == 0) write(lout,1000)
	ierr = 1
	write(lout,1010)
      endif
      if (startsol == 1) then
        call getKeyword('LINSOL_INITGUESS_INPUT_FILENAME',solin,ier)
        if (ier /= 0) then
	  if (ierr == 0) write(lout,1000)
	  ierr = 1
	  write(lout,1011)
        endif
      endif
      call getKeyword('LINSOL_METHOD',method,ier)
      if (ier /= 0) then
	if (ierr == 0) write(lout,1000)
	ierr = 1
	write(lout,1012)
      endif
      call getKeyword('LINSOL_ITERATION_MAXIMUM',itmax,ier)
      if (ier /= 0) then
	if (ierr == 0) write(lout,1000)
	ierr = 1
	write(lout,1013)
      endif
      call getKeyword('LINSOL_OUTPUT_DEVICE',outdev,ier)
      if (ier /= 0) then
	if (ierr == 0) write(lout,1000)
	ierr = 1
	write(lout,1014)
      endif
      call getKeyword('LINSOL_MATRIX_NORMALIZED',isprec,ier)
      if (ier /= 0) then
	if (ierr == 0) write(lout,1000)
	ierr = 1
	write(lout,1015)
      endif
      call getKeyword('LINSOL_PRECONDITIONING_METHOD',msprec,ier)
      if (ier /= 0) then
	if (ierr == 0) write(lout,1000)
	ierr = 1
	write(lout,1016)
      endif
      if (msprec >= 63) then
        call getKeyword('LINSOL_GAUSS_FACTOR',gaussfac,ier)
        if (ier /= 0) then
	  if (ierr == 0) write(lout,1000)
	  ierr = 1
	  write(lout,1024)
        endif
        call getKeyword('LINSOL_GAUSS_THRES1',thres1,ier)
        if (ier /= 0) then
	  if (ierr == 0) write(lout,1000)
	  ierr = 1
	  write(lout,1025)
        endif
        call getKeyword('LINSOL_GAUSS_THRES2',thres2,ier)
        if (ier /= 0) then
	  if (ierr == 0) write(lout,1000)
	  ierr = 1
	  write(lout,1026)
        endif
        call getKeyword('LINSOL_LDROP_FACTOR',ldrop,ier)
        if (ier /= 0) then
	  ldrop = 0.0
	  write(lout,1032)
        endif
      endif
      call getKeyword('LINSOL_CHECK_SYSTEM',noprec,ier)
      if (ier /= 0) then
	if (ierr == 0) write(lout,1000)
	ierr = 1
	write(lout,1017)
      endif
      call getKeyword('LINSOL_VERBOSE_LEVEL',idoku,ier)
      if (ier /= 0) then
	if (ierr == 0) write(lout,1000)
	ierr = 1
	write(lout,1018)
      endif
      call getKeyword('LINSOL_NORMALIZATION_METHOD',msnorm,ier)
      if (ier /= 0) then
	if (ierr == 0) write(lout,1000)
	ierr = 1
	write(lout,1019)
      endif
      call getKeyword('LINSOL_SMOOTHED',smooth,ier)
      if (ier /= 0) then
	if (ierr == 0) write(lout,1000)
	ierr = 1
	write(lout,1020)
      endif
      call getKeyword('LINSOL_MISC',misc,ier)
      if (ier /= 0) then
	if (ierr == 0) write(lout,1000)
	ierr = 1
	write(lout,1021)
      endif
      call getKeyword('LINSOL_OPTIMIZE',optim,ier)
      if (ier /= 0) then
	if (ierr == 0) write(lout,1000)
	ierr = 1
	write(lout,1022)
      endif
      call getKeyword('LINSOL_THRESHOLD',epslin,ier)
      if (ier /= 0) then
	if (ierr == 0) write(lout,1000)
	ierr = 1
	write(lout,1023)
      endif
      call getKeyword('LINSOL_TEMP_DIR',tmpdir,ier)
      if (ier /= 0) then
	if (ierr == 0) write(lout,1000)
	ierr = 1
	write(lout,1031)
      endif
      if (msprec >= 63 .or. optim >= 10000) then
        call getKeyword('LINSOL_BUFFER_SIZE',bufsize,ier)
        if (ier /= 0) then
	  bufsize = 5
	  write(lout,1027)
        endif
      endif
!     if (optim == 1000) then
!       call getKeyword('LINSOL_MEMORY_FACTOR',memfac,ier)
!       if (ier /= 0) then
!         if (ierr == 0) write(lout,1000)
!         ierr = 1
!         write(lout,1028)
!       endif
!     endif
      if (optim >= 200000) then
        call getKeyword('LINSOL_BANDW_ALG',bwoalg,ier)
        if (ier /= 0) then
	  bwoalg = 1
	  write(lout,1029)
        endif
      endif
      if (msprec >= 63 .or. optim >= 1000) then
        call getKeyword('LINSOL_RUNTIME_ACCELERATION',runacc,ier)
        if (ier /= 0) then
	  runacc = 0
	  write(lout,1030)
        endif
      endif

      rlin(1) = DBLE(epslin)
      rlin(2) = DBLE(thres1)
      rlin(3) = DBLE(thres2)
      rlin(4) = DBLE(gaussfac)
      rlin(5) = DBLE(memfac)
      rlin(6) = DBLE(ldrop)


!   send parameters to the other processors

      ibuf(1) = infmt
      ibuf(2) = split
      ibuf(3) = outfmt
      ibuf(4) = method
      ibuf(5) = itmax
      ibuf(6) = outdev
      ibuf(7) = isprec
      ibuf(8) = msprec
      ibuf(9) = noprec
      ibuf(10) = idoku
      ibuf(11) = msnorm
      ibuf(12) = smooth
      ibuf(13) = misc
      ibuf(14) = optim
      ibuf(15) = nrhs
      ibuf(16) = startsol
      ibuf(17) = bufsize
      ibuf(18) = bwoalg
      ibuf(19) = runacc
      do i=2,nproc
        to = tids(i)
        call MPSNDA(to,nmsg+i,nparm*iint,ibuf(1),mids,ierr)
        call MPSNDW(to,nmsg+i,nparm*iint,ibuf(1),mids,ierr)
        call MPSNDA(to,nmsg+i,nrparm*ireal,rlin,mids,ierr)
        call MPSNDW(to,nmsg+i,nrparm*ireal,rlin,mids,ierr)
        if (outfmt > 0) then
          call MPSNDA(to,nmsg+i,72,outnam,mids,ierr)
          call MPSNDW(to,nmsg+i,72,outnam,mids,ierr)
        endif
        if (split == 1) then
          call MPSNDA(to,nmsg+i,72,matrix,mids,ierr)
          call MPSNDW(to,nmsg+i,72,matrix,mids,ierr)
          call MPSNDA(to,nmsg+i,72,rhsin,mids,ierr)
          call MPSNDW(to,nmsg+i,72,rhsin,mids,ierr)
        endif
      enddo
      nmsg = nmsg + nproc

 1000 format(/'  ERROR occured in routine READ_PARAM_FILE:')
 1001 format('    Environment variable LINSOL_INPUT_FORMAT not defined')
 1002 format('    Environment variable LINSOL_RANDOM_PARAMETER_FILENAME'
     &      ,' not defined')
 1003 format('    Environment variable LINSOL_MATRIX_DISTRIBUTED not'
     &      ,' defined')
 1004 format('    Environment variable LINSOL_MATRIX_INPUT_FILENAME not'
     &      ,' defined')
 1005 format('    Environment variable LINSOL_MATRIX_OUTPUT_FORMAT not'
     &      ,' defined')
 1006 format('    Environment variable LINSOL_MATRIX_OUTPUT_FILENAME'
     &      ,' not defined')
 1007 format('    Environment variable LINSOL_NUMBER_RHS not defined')
 1008 format('    Environment variable LINSOL_RHS_FILENAME not'
     &      ,' defined')
 1009 format('    Environment variable LINSOL_SOLUTIONS_FILENAME not'
     &      ,' defined')
 1010 format('    Environment variable LINSOL_INITGUESS not defined')
 1011 format('    Environment variable LINSOL_INITGUESS_FILENAME' 
     &      ,' not defined')
 1012 format('    Environment variable LINSOL_METHOD not defined')
 1013 format('    Environment variable LINSOL_ITERATION_MAXIMUM not'
     &      ,' defined')
 1014 format('    Environment variable LINSOL_OUTPUT_DEVICE not'
     &      ,' defined')
 1015 format('    Environment variable LINSOL_MATRIX_NORMALIZED not'
     &      ,' defined')
 1016 format('    Environment variable LINSOL_PRECONDITIONING_METHOD'
     &      ,' not defined')
 1017 format('    Environment variable LINSOL_CHECK_SYSTEM not'
     &      ,' defined')
 1018 format('    Environment variable LINSOL_VERBOSE_LEVEL not'
     &      ,' defined')
 1019 format('    Environment variable LINSOL_NORMALIZATION_METHOD' 
     &      ,' not defined')
 1020 format('    Environment variable LINSOL_SMOOTHED not defined')
 1021 format('    Environment variable LINSOL_MISC not defined')
 1022 format('    Environment variable LINSOL_OPTIMIZE not defined')
 1023 format('    Environment variable LINSOL_THRESHOLD not defined')
 1024 format('    Environment variable LINSOL_GAUSS_FACTOR not defined')
 1025 format('    Environment variable LINSOL_GAUSS_THRES1 not defined')
 1026 format('    Environment variable LINSOL_GAUSS_THRES2 not defined')
 1027 format('    Environment variable LINSOL_BUFFER_SIZE not defined'/
     *      ,'    --> the buffer size is set to 5 MByte')
 1028 format('    Environment variable LINSOL_MEMORY_FACTOR not'
     &      ,' defined')
 1029 format('    Environment variable LINSOL_BANDW_ALG not defined'/
     *      ,'    --> the GPS algorithm is chosen (ilin(22) = 1)')
 1030 format('    Environment variable LINSOL_RUNTIME_ACCELERATION not'/
     *      ,'    defined --> version with minimum storage'/
     *      ,'    requirements is used (ilin(21) = 0)')
 1031 format('    Environment variable LINSOL_TEMP_DIR not defined')
 1032 format('    Environment variable LINSOL_LDROP_FACTOR not defined'/
     *      ,'    --> no dropping of elements is chosen (rlin(6) = 0.)')

      R E T U R N
!***  E N D  OF  READ_PARAM_FILE ***************************************
      E N D
