!***********************************************************************
!        1         2         3         4         5         6         7 *
!***********************************************************************
!**                                                                  ***
!**                                                                  ***
      subroutine READ_RHS_IGUESS(MAT,lmat,index,lindex,lsym,info,ia1,x,
     &                           rhs,l,llocal,nvt,lmatbk,ptrmbk,
     &                           infmt,solin,rhsin,misc,startsol,split,
     &                           nrhs,irhs,idoku,lout,ierr,
     &                           tids,nproc,myproc,nmsg)
!**                                                                  ***
!**                                                                  ***
!***********************************************************************
!**                                                                  ***
!**    READ_RHS_IGUESS  reads right hand sides and one initial       ***
!**                     guess from file                              ***
!**                                                                  ***
!***********************************************************************
!**                                                                  ***
!**    Copyright (C) 1996 University Karlsruhe                       ***
!**    PROGRAMMER : H. Haefner                                       ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                  ***
!***********************************************************************
!**                                                                  ***
      USE solex
      USE constants

      implicit none
!**                                                                  *** 
!***********************************************************************
!**                                                                  ***
!**    Formal Parameters :                                           ***
!**                                                                  ***
!***********************************************************************
      integer          l,llocal,lmat,lindex,nvt,ia1,ierr,myproc,idoku
      integer          infmt,lout,misc,startsol,nrhs,irhs,nproc,nmsg
      integer          split
      integer          lmatbk(nproc),ptrmbk(nproc+1)
      integer          info(ia1,ia2),index(lindex),tids(nproc)
      double precision MAT(lmat),x(l)
      double precision rhs(l)
      character*72     solin,rhsin
      logical          lsym
!***********************************************************************
!**                                                                  ***
!**    List of Formal Parameters :                                   ***
!**                                                                  ***
!**                                                                  ***
!---------I------I-----I------------------------------------------------
! NAME    I TYPE I I/O I MEANING
!---------I------I-----I------------------------------------------------
! l       I int  I in  I dimension of the global matrix
!---------I------I-----I------------------------------------------------
! lmat    I int  I in  I number of matrix elements
!---------I------I-----I------------------------------------------------
! lindex  I int  I in  I number of indices
!---------I------I-----I------------------------------------------------
! nvt     I int  I in  I total number of vector-terms
!---------I------I-----I------------------------------------------------
! startsolI int  I in  I indicates, if initial guess is used
!---------I------I-----I------------------------------------------------
! nrhs    I int  I in  I number of right hand sides
!---------I------I-----I------------------------------------------------
! irhs    I int  I in  I counter of right hand sides
!---------I------I-----I------------------------------------------------
! info    I int  I in  I array with matrix-structure-information
!---------I------I-----I------------------------------------------------
! index   I int  I in  I array with indices
!---------I------I-----I------------------------------------------------
! MAT     I real I in  I array with matrix-elements
!---------I------I-----I------------------------------------------------
! lsym    I l    I in  I indicates if matrix is symmetric
!---------I------I-----I------------------------------------------------
! solin   I c*72 I in  I name of file containing the solution
!---------I------I-----I------------------------------------------------
! rhsin   I c*72 I in  I name of file containing the right hand sides
!---------I------I-----I------------------------------------------------
! ierr    I int  I out I error-indication
!---------I------I-----I------------------------------------------------
!**                                                                  ***
!**                                                                  ***
!**    Local Variables :                                             ***
!**                                                                  ***
      integer          i,j,iounit,istride,to,mids,ll,lfile,nfile
      integer          iermax,ihelp,lmyblk
      double precision dw(abs(nrhs)),xsum,xmax
      real*4           realbuf(abs(nrhs))
      character*5      rhssufix
      double precision, parameter ::  zero = 0.0

!***********************************************************************
!**                                                                  ***
!**    List of Local Variables :                                     ***
!**                                                                  ***
!**                                                                  ***
!---------I------I-----I------------------------------------------------
! NAME    I TYPE I I/O I MEANING
!---------I------I-----I------------------------------------------------
!   dw    I  dp  I  -  I work array, array: dw(nrhs)
!---------I------I-----I------------------------------------------------
!***********************************************************************


!***********************************************************************
!**    Start of Calculation                                          ***
!***********************************************************************

      ierr = 0000

      lmyblk = lmatbk(myproc)

!*********************************************************
!   Calculation of the right hand side rhs so that
!   x=(1,2**(1/3),3**(1/3),...,l**(1/3))
!   is the solution of the linear system.
!*********************************************************
!     if ((nrhs < 0) .and. (split == 0)) then
!       do i=1,l
!         x(i) = dble(i)**(1./3.)
!       enddo
!       if ((nproc == 1) .and. (mod(misc,10000) >= 1000)) then
!         do i=1,l
!           xexact(i) = x(i)
!         enddo
!       endif
!       xsum = zero
!       do i=1,l
!         xsum = xsum + x(i)*x(i)
!       enddo
!       xsum = sqrt(xsum)
!       xmax = zero
!       do i=1,l
!         if (abs(x(i)) .gt. xmax)  xmax = abs(x(i))
!       enddo
!       write(lout,1015)
!       write(lout,1525)
!       write(lout,1021)
!       if (l .gt. 8) then
!         istride = (l+7)/8
!       else
!         istride = 1
!       endif
!       write(lout,1528) istride
!       write(lout,1530) (x(i),i=1,l,istride)
!       write(lout,1526) xsum
!       write(lout,1527) xmax

!       if (nproc == 1 .or. infmt == 2) then
!         if (lsym) then
!           call LINASX(mat,x,rhs,l,nvt,lmat,lindex,ia1,info,index)
!         else
!           call LINAX(mat,x,rhs,l,nvt,lmat,lindex,ia1,info,index)
!         endif
!       endif
!     endif

!***  if nrhs > 0 read rhs from file

      if (nrhs > 0) then
        iounit = 12
        if (split == 1) then
	  write (rhssufix,'(A1,i4.4)') '_',myproc
          rhsin = trim(rhsin) // rhssufix
        endif
        if (infmt == 0) then
          open(UNIT=iounit,FILE=rhsin,FORM='UNFORMATTED')
          write(lout,1531) rhsin,iounit 
        else
          open(UNIT=iounit,FILE=rhsin,FORM='FORMATTED')
          write(lout,1532) rhsin,iounit 
        endif
        if (infmt == 0) read (iounit) lfile,nfile
        if (infmt > 0)  read (iounit,*) lfile,nfile
        if (split == 0) ll = l
        if (split == 1) ll = llocal
        if (lfile > ll .or. lfile < lmyblk) then
          ierr = 0001
          write(lout,1010)
          write(lout,1011) lfile,ll
          write(lout,1020)
          goto 999
        endif
        if (nfile .ne. nrhs) then
          write(lout,1030)
          write(lout,1012) nfile,nrhs
          write(lout,1020)
          nrhs = nfile
        endif
        if (infmt == 0) then
          do i=1,lfile
            read (iounit) (realbuf(j),j=1,nrhs)
            rhs(i) = DBLE(realbuf(irhs))
          enddo
        endif
	if (infmt > 0) then
          do i=1,lfile
            read (iounit,*) (dw(j),j=1,nrhs)
            rhs(i) = dw(irhs)
          enddo
        endif
        close(iounit)
      endif

!*********************************************************
!   reading right-hand-side from H/B-matrix
!*********************************************************
!     if (nrhs == 0) then
!     endif


!*********************************************************
!   reading existing initial guess on processor 1
!*********************************************************

      if (startsol == 1) then
        iounit = 13
        if (infmt == 0) then
          open(UNIT=iounit,FILE=solin,FORM='UNFORMATTED')
          write(lout,1531) solin,iounit 
	else
          open(UNIT=iounit,FILE=solin,FORM='FORMATTED')
          write(lout,1532) solin,iounit 
	endif
	if (infmt == 0) then
	  read (iounit) lfile,nfile
	else
	  read (iounit,*) lfile,nfile
	endif
        if (lfile /= l) then
	  ierr = 0002
	  write(lout,1010)
	  write(lout,1013) lfile,l
	  write(lout,1020)
	  goto 999
	endif
        if (nfile .lt. abs(nrhs)) then
	  write(lout,1040)
	  write(lout,1014) nfile,abs(nrhs)
	  write(lout,1020)
          if (irhs .eq. nfile) then
	    startsol = 0
	  endif
	endif
        do i=1,lfile
	  if (infmt == 0) then
            read (iounit) (realbuf(j),j=1,nfile)
            x(i) = DBLE(realbuf(irhs))
	  else
            read (iounit,*) (dw(j),j=1,nfile)
            x(i) = dw(irhs)
	  endif
        enddo
        close(iounit)
      endif

!**
!***  if ierr <> 0 then send ierr to all processors
!**

  999 call LL4INM(1,1,1,ierr,iermax,ihelp,myproc,nproc,
     &            tids,nmsg)
      if (iermax .ne. 0) then
        ierr = iermax
        stop
      endif

!*********************************************************
!   now send parts from right-hand-side and initial guess
!   from processor 1 to all processors
!*********************************************************

      do i=2,nproc
        to = tids(i)
        if (split == 0) then
          call MPSNDA(to,nmsg+i-1,lmatbk(i)*ireal,
     &                rhs(ptrmbk(i)+1),mids,ierr)
          call MPSNDW(to,nmsg+i-1,lmatbk(i)*ireal,
     &                rhs(ptrmbk(i)+1),mids,ierr)
        endif
        if (startsol == 1) then
          call MPSNDA(to,nmsg+i,lmatbk(i)*ireal,
     &                x(ptrmbk(i)+1),mids,ierr)
          call MPSNDW(to,nmsg+i,lmatbk(i)*ireal,
     &                x(ptrmbk(i)+1),mids,ierr)
        endif
      enddo

      nmsg = nmsg+nproc

!***********************************************************************
!**    End of Calculation                                            ***
!***********************************************************************

 1010 format('  ************************************************'/
     &       '  ***  LINSOL: F A T A L  E R R O R            ***')
 1030 format('  ************************************************'/
     &       '  ***  LINSOL: W A R N I N G                   ***')
 1040 format('  ************************************************'/
     &       '  ***  LINSOL: N O T E                         ***')
 1011 format('  ***   The number of coefficients of the      ***'/
     &       '  ***   RHS ',i7,' does not fit to the        ***'/
     &       '  ***   dimension of the matrix ',i7,'        ***')
 1012 format('  ***   The number of Right Hand Sides         ***'/
     &       '  ***   ',i6,' read from file is not equal     ***'/
     &       '  ***   to the number of RHS chosen by the     ***'/
     &       '  ***   user ',i6,': the value read from       ***'/
     &       '  ***   file will be taken !!!                 ***')
 1013 format('  ***   The number of coefficients of the      ***'/
     &       '  ***   initial guess ',i7,' does not fit to  ***'/
     &       '  ***   the dimension of the matrix ',i7,'    ***')
 1014 format('  ***   The number of initial guesses ',i6,'   ***'/
     &       '  ***   read from file is less than the number ***'/
     &       '  ***   of RHS chosen by the user ',i6,': the  ***'/
     &       '  ***   latest linear systems are solved       ***'/
     &       '  ***   without initial guesses !!!            ***')
 1020 format('  ************************************************'/)
 1021 format('  ***************************************************'/)
 1015 format('  ***************************************************')
 1525 format('  ***   The linear system Ax=b will be solved     ***'/
     &       '  ***   with a RHS b so that holds:               ***'/
     &       '  ***   x = (1,2**(1/3),...,l**(1/3))             ***')
 1526 format(' ',/
     &       '  The Euclidian norm of the solution is:',/
     &       '   ||x||2   = ',d10.4)
 1527 format('  The Maximum norm of the solution is:',/
     &       '   ||x||max = ',d10.4//)
 1528 format('  Each',i5,'th element of the solution is: ')
 1530 format(1x,4(d17.10,2x),/,1x,4(d17.10,2x))
 1531 format(/'  Reading binary file ',/
     &       '  ',a72,/
     &       '  on unit ',i3,' ...'/) 
 1532 format(/'  Reading file ',/
     &       '  ',a72,/
     &       '  on unit ',i3,' ...'/) 
       r e t u r n
!-----END OF READ_RHS_IGUESS -------------------------------------------
       e    n    d
