!======================================================================+
!                                                                      |
! MODULE     :  scriptparms                                            |
! ------                                                               |
!                                                                      |
! FONCTION   :                                                         |
! --------                                                             |
!                                                                      |
! Low-level procedures .                                               |
!                                                                      |
!***********************************************************************
!**                                                                  ***
!**    Copyright (C) 1999 University Karlsruhe                       ***
!**    PROGRAMMER:    H. Haefner                                     ***
!**                                                                  ***
!**    This file is part of LINSOL.                                  ***
!**    LINSOL is free software; you can redistribute it and/or       ***
!**    modify it under the terms of the GNU General Public License   ***
!**    as published by the Free Software Foundation; either          ***
!**    version 2 of the License, or (at your option) any later       ***
!**    version.                                                      ***
!**                                                                  ***
!**    LINSOL is distributed in the hope that it will be useful,     ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty   ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.       ***
!**    See the GNU General Public License for more details.          ***
!**                                                                  ***
!**    You should have received a copy of the GNU General Public     ***
!**    License along with LINSOL; if not, write to the Free          ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,       ***
!**    Boston, MA  02111-1307, USA.                                  ***
!**                                                                  ***
!======================================================================+

MODULE scriptparms                      
!----------------------------------------------------------------------+
!    Public operators and procedures .                                 |
!----------------------------------------------------------------------+
      IMPLICIT NONE
!----------------------------------------------------------------------+
!     Global Variables .                                               |
!----------------------------------------------------------------------+
      PRIVATE
      PUBLIC :: GetKeyword                                                  
!----------------------------------------------------------------------+
!     Interface blocks .                                               |
!----------------------------------------------------------------------+
      INTERFACE GetKeyword                  
          MODULE PROCEDURE GetKeywordChar, GetKeywordR4, GetKeywordI4, &
                           GetKeywordR8
      END INTERFACE

CONTAINS   
                          

      SUBROUTINE GetKeywordChar (keyword,kvalue,ierr) 
!======================================================================+
!                                                                      |
! PROCEDURE  :  GetKeywordChar                                         |
! ---------                                                            |
!                                                                      |
! FONCTION   :                                                         |
! --------                                                             |
!                                                                      |
! Lecture d'un Mot Clef                                                |
!                                                                      |
! PARAMETRES :                                                         |
! ----------                                                           |
!                                                                      |
! keyword    :  Nom du mot clef                                        |
! kvalue     :  Valeur du mot clef                                     |
!                                                                      |
!======================================================================+
      IMPLICIT NONE 

      CHARACTER*(*),INTENT(IN) ::  keyword 
      CHARACTER*(*),INTENT(OUT)::  kvalue 
      INTEGER,INTENT(OUT)      ::  ierr

      INTEGER                 k , length0 , length
      CHARACTER*80            string  
      CHARACTER*80            buffer 
      CHARACTER*1             schar
      LOGICAL                 conv         
      length0 = LEN_TRIM(keyword) 
      string = keyword(1:length0) // CHAR(0)
      CALL get_env_keyword (string,buffer,length) 
      if (length > 0) then
        kvalue = buffer(1:length)
        ierr = 0
      else
        ierr = -1
      endif
      RETURN 
      END SUBROUTINE GetKeywordChar 

                                              
    
      SUBROUTINE GetKeywordR4 (keyword,rvalue,ierr) 
!======================================================================+
!                                                                      |
! PROCEDURE  :  GetKeywordR4 (WS)                                      |
! ---------                                                            |
!                                                                      |
! FONCTION   :                                                         |
! --------                                                             |
!                                                                      |
! Read keyword of real type .                                          |
!                                                                      |
! PARAMETRES :                                                         |
! ----------                                                           |
!                                                                      |
! keyword    :  Name of the keyword                                    |
! rvalue     :  Value of the keyword                                   |
!                                                                      |
!======================================================================+
      IMPLICIT NONE 

      CHARACTER*(*),INTENT(IN) ::  keyword 
      REAL,INTENT(OUT)         ::  rvalue 
      INTEGER,INTENT(OUT)      ::  ierr

      INTEGER                 k , length0 , length
      CHARACTER*80            buffer
      CHARACTER*80            string   
      CHARACTER*1             schar
      LOGICAL                 conv  
      INTEGER                 ios
!----------------------------------------------------------------------+
!     Interface blocks .                                               |
!----------------------------------------------------------------------+       
      length0 = LEN_TRIM(keyword) 
      string = keyword(1:length0) // CHAR(0)
      CALL get_env_keyword (string,buffer,length) 
      if (length > 0) then
      READ (UNIT=buffer(1:length) , FMT=* , IOSTAT=ios , ERR=100) rvalue  
        ierr = 0
      else
        ierr = -2
      endif
      RETURN          
  100 CONTINUE
      ierr = -3
      RETURN 
      END SUBROUTINE GetKeywordR4 


    
      SUBROUTINE GetKeywordR8 (keyword,rvalue,ierr) 
!======================================================================+
!                                                                      |
! PROCEDURE  :  GetKeywordR8 (WS)                                      |
! ---------                                                            |
!                                                                      |
! FONCTION   :                                                         |
! --------                                                             |
!                                                                      |
! Read keyword of real*8 type .                                        |
!                                                                      |
! PARAMETRES :                                                         |
! ----------                                                           |
!                                                                      |
! keyword    :  Name of the keyword                                    |
! rvalue     :  Value of the keyword                                   |
!                                                                      |
!======================================================================+
      IMPLICIT NONE 

      CHARACTER*(*),INTENT(IN)     ::  keyword 
      DOUBLE PRECISION,INTENT(OUT) ::  rvalue 
      INTEGER,INTENT(OUT)          ::  ierr

      INTEGER                 k , length0 , length
      CHARACTER*80            buffer
      CHARACTER*80            string   
      CHARACTER*1             schar
      LOGICAL                 conv  
      INTEGER                 ios
!----------------------------------------------------------------------+
!     Interface blocks .                                               |
!----------------------------------------------------------------------+       
      length0 = LEN_TRIM(keyword) 
      string = keyword(1:length0) // CHAR(0)
      CALL get_env_keyword (string,buffer,length) 
      if (length > 0) then
      READ (UNIT=buffer(1:length) , FMT=* , IOSTAT=ios , ERR=100) rvalue  
        ierr = 0
      else
        ierr = -2
      endif
      RETURN          
  100 CONTINUE
      ierr = -3
      RETURN 
      END SUBROUTINE GetKeywordR8 


    
      SUBROUTINE GetKeywordI4 (keyword,ivalue,ierr) 
!======================================================================+
!                                                                      |
! PROCEDURE  :  GetKeywordI4 (WS)                                      |
! ---------                                                            |
!                                                                      |
! FONCTION   :                                                         |
! --------                                                             |
!                                                                      |
! Read keyword of integer type .                                       |
!                                                                      |
! PARAMETRES :                                                         |
! ----------                                                           |
!                                                                      |
! keyword    :  Name of the keyword                                    |
! rvalue     :  Value of the keyword                                   |
!                                                                      |
!======================================================================+
      IMPLICIT NONE 

      CHARACTER*(*),INTENT(IN) ::  keyword 
      INTEGER,INTENT(OUT)      ::  ivalue, ierr 

      INTEGER                 k , length0 , length
      CHARACTER*80            buffer
      CHARACTER*80            string   
      CHARACTER*1             schar
      LOGICAL                 conv  
      INTEGER                 ios
!----------------------------------------------------------------------+
!     Interface blocks .                                               |
!----------------------------------------------------------------------+       
      length0 = LEN_TRIM(keyword) 
      string = keyword(1:length0) // CHAR(0)
      CALL get_env_keyword (string,buffer,length) 
      if (length > 0) then
      READ (UNIT=buffer(1:length) , FMT=* , IOSTAT=ios , ERR=100) ivalue  
        ierr = 0
      else
        ierr = -4
      endif
      RETURN          
  100 CONTINUE
      ierr = -5
      RETURN 
      END SUBROUTINE GetKeywordI4 
                                                                        

END MODULE scriptparms                                           
