!***********************************************************************
!        1         2         3         4         5         6         7 *
!***********************************************************************
!**                                                                  ***
!**                                                                  ***
      subroutine SET_ARRAY_LINSOL(lmatbk,ptrmbk,lmat,lindex,nvt,nproc,
     &                            lmatbuf,lindbuf,ia1buf,optim,isprec,
     &                            msprec,bwoalg,ierr)
!**                                                                  ***
!**                                                                  ***
!***********************************************************************
!**                                                                  ***
!**    SET_ARRAY_LINSOL  sets the dimensions of the arrays <MAT>,    ***
!**                      <index> and <info> for LINSOL-Format        ***
!**                                                                  ***
!***********************************************************************
!**                                                                  ***
!**    Copyright (C) 1996 University Karlsruhe                       ***
!**    PROGRAMMER : H. Haefner                                       ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                  ***
!***********************************************************************
!**                                                                  ***
      USE scriptparms

      implicit none
!**                                                                  *** 
!***********************************************************************
!**                                                                  ***
!**    Formal Parameters :                                           ***
!**                                                                  ***
!***********************************************************************
      integer          ierr,nproc,lmat,lindex,nvt,optim
      integer          isprec,msprec,bwoalg
      integer          lmatbk(nproc),ptrmbk(nproc+1)
      integer          lmatbuf(nproc),lindbuf(nproc)
      integer          ia1buf(nproc)

!***********************************************************************
!**                                                                  ***
!**    List of Formal Parameters :                                   ***
!**                                                                  ***
!**                                                                  ***
!---------I------I-----I------------------------------------------------
! NAME    I TYPE I I/O I MEANING
!---------I------I-----I------------------------------------------------
! lmatbuf I int  I out I dimensions of the distributed matrices
!         I      I     I                      array: lmatbuf(nproc)
!---------I------I-----I------------------------------------------------
! lindbuf I int  I out I dimensions of the distributed index-arrays
!         I      I     I (lindbuf = lmatbuf)  array: lindbuf(nproc)
!---------I------I-----I------------------------------------------------
! ia1buf  I int  I out I first dimensions of the distributed 
!         I      I     I information-arrays   array: ia1buf(nproc)
!---------I------I-----I------------------------------------------------
!**                                                                  ***
!**                                                                  ***
!**    Local Variables :                                             ***
!**                                                                  ***
       integer  i,iermax
       real     matfac,indfac,infofac


!***********************************************************************
!**    Start of Calculation                                          ***
!***********************************************************************

      iermax = 0

      call getKeyword('LINSOL_MATRIX_FACTOR',matfac,ierr)
      if (ierr /= 0) then
        iermax = ierr
        matfac = 1.0
      endif
      call getKeyword('LINSOL_INDEX_FACTOR',indfac,ierr)
      if (ierr /= 0) then
        if (ierr > iermax) iermax = ierr
        indfac = 1.0
      endif
      call getKeyword('LINSOL_INFO_FACTOR',infofac,ierr)
      if (ierr /= 0) then
        if (ierr > iermax) iermax = ierr
        infofac = 1.0
      endif

      if (nproc > 1) then
        do i=1,nproc
          lmatbuf(i) = int((lmat/nproc)*matfac)
          if (((msprec >= 63) .and. (isprec < 10)) .or. (bwoalg > 0)
     &        .or. (mod(optim,100000) >= 10000)
     &        .or. (mod(optim,200000) >= 100000)) then
            lindbuf(i) = lmatbuf(i)
            ia1buf(i)  = max(lmatbk(i)+1,(nvt + 1))*infofac
	  else
            lindbuf(i) = int((lindex/nproc)*indfac)
            ia1buf(i)  = (nvt + 1)*infofac
	  endif
        enddo
      else
        lmatbuf(1) = (lmat + lmatbk(1))*matfac
        if (((msprec >= 63) .and. (isprec < 10)) .or. (bwoalg > 0)
     &      .or. (mod(optim,100000) >= 10000)
     &      .or. (mod(optim,200000) >= 100000)) then
          lindbuf(1) = lmatbuf(1)
          ia1buf(1)  = max(lmatbk(1)+1,(nvt + 1))*infofac
	else
          lindbuf(1) = (lindex + lmatbk(1))*indfac
          ia1buf(1)  = (nvt + 1)*infofac
	endif
      endif

      ierr = iermax
      continue
!***********************************************************************
!**    End of Calculation                                            ***
!***********************************************************************

       r e t u r n
!---- END OF SET_ARRAY_LINSOL ------------------------------------------
       e    n    d
