!***********************************************************************
!        1         2         3         4         5         6         7 *
!***********************************************************************
!**                                                                  ***
!**                                                                  ***
      subroutine WRITE_SOLUTION(x,l,lmatbk,ptrmbk,ilin,tmpdir,
     &                          solout,infmt,irhs,nrhs,ierr,
     &                          tids,nproc,myproc,nmsg)
!**                                                                  ***
!**                                                                  ***
!***********************************************************************
!**                                                                  ***
!**    WRITE_SOLUTION   gathers the solution on processor 1          ***
!**                     and writes the solutions to file             ***
!**                                                                  ***
!***********************************************************************
!**                                                                  ***
!**    Copyright (C) 1996 University Karlsruhe                       ***
!**    PROGRAMMER : H. Haefner                                       ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                  ***
!***********************************************************************
!**                                                                  ***
      USE constants

      implicit none
!**                                                                  *** 
!***********************************************************************
!**                                                                  ***
!**    Formal Parameters :                                           ***
!**                                                                  ***
!***********************************************************************
      integer          l,ierr,myproc,infmt
      integer          ilin(nilin),irhs,nrhs,nproc,nmsg
      integer          lmatbk(nproc),ptrmbk(nproc+1),tids(nproc)
      double precision x(l)
      character        tmpdir*72,solout*72
!***********************************************************************
!**                                                                  ***
!**    List of Formal Parameters :                                   ***
!**                                                                  ***
!**                                                                  ***
!---------I------I-----I------------------------------------------------
! NAME    I TYPE I I/O I MEANING
!---------I------I-----I------------------------------------------------
! l       I int  I in  I dimension of the matrix
!---------I------I-----I------------------------------------------------
! irhs    I int  I in  I counter of right hand sides
!---------I------I-----I------------------------------------------------
! tmpdir  I c*80 I in  I name of temporary directory
!---------I------I-----I------------------------------------------------
! ierr    I int  I out I error-indication
!---------I------I-----I------------------------------------------------
!**                                                                  ***
!**                                                                  ***
!**    Local Variables :                                             ***
!**                                                                  ***
      integer          i,j,from,midr,lh,nrhsh
      integer          count,imin,imax,idoku,lout
      double precision dw(ABS(nrhs)),esum,emax,emin,xsum,xmax
      real*4           rbuf(ABS(nrhs))

      integer, parameter :: iconst = 4, ounit1 = 14, ounit2 = 15
      integer, parameter :: ounit3 = 16
      double precision, parameter ::  zero = 0.0

      integer          itop(iconst),ibutom(iconst)
      double precision top(iconst),buttom(iconst)
      character        s1*72,s2*72,chelp1*8,chelp2*8

!***********************************************************************
!**                                                                  ***
!**    List of Local Variables :                                     ***
!**                                                                  ***
!**                                                                  ***
!---------I------I-----I------------------------------------------------
! NAME    I TYPE I I/O I MEANING
!---------I------I-----I------------------------------------------------
!   dw    I  dp  I  -  I work array, array: dw(nrhs)
!---------I------I-----I------------------------------------------------
!***********************************************************************


!***********************************************************************
!**    Start of Calculation                                          ***
!***********************************************************************

      lout  = ilin(12)
      idoku = ilin(13)


!*********************************************************
!   Receive parts of solution-vector on processor 1
!   and write solution on file
!*********************************************************
      do i=2,nproc
        from = tids(i)
        call MPRCVA(from,nmsg+i-1,lmatbk(i)*ireal,
     &              x(ptrmbk(i)+1),midr,ierr)
        call MPRCVW(from,nmsg+i-1,lmatbk(i)*ireal,
     &              x(ptrmbk(i)+1),midr,ierr)
      enddo
      nmsg = nmsg+nproc-1

      chelp1 = '/lintmp1'
      chelp2 = '/lintmp2'
!   writing solution-vector x to file

      if (ABS(nrhs) == 1) then
        solout = trim(solout)
	if (infmt == 0) then
          open(UNIT=ounit3,FILE=solout,IOSTAT=ierr,FORM='UNFORMATTED')
	  if (ierr /= 0) then
            write(lout,1600) s1 
	    goto 999
	  endif
          write(lout,1544) solout, ounit3 
        else
          open(UNIT=ounit3,FILE=solout,IOSTAT=ierr,FORM='FORMATTED')
	  if (ierr /= 0) then
            write(lout,1601) s1 
	    goto 999
	  endif
          write(lout,1546) solout, ounit3 
        endif
	if (infmt == 0) then
          write(ounit3) l,ABS(nrhs)
          do i=1,l
            write(ounit3) REAL(x(i))
          enddo
	else
          write(ounit3,*) l,ABS(nrhs)
          do i=1,l
            write(ounit3,*) x(i)
          enddo
        endif
        close(ounit3)
      endif
      if (nrhs >= 2) then
        if (irhs == 1) then
          s1 = TRIM(tmpdir) // chelp1
	  if (infmt == 0) then
            open(UNIT=ounit1, FILE=s1, IOSTAT=ierr, FORM='UNFORMATTED')
	    if (ierr /= 0) then
              write(lout,1600) s1 
	      goto 999
	    endif
            write(lout,1544) s1, ounit1 
          else
            open(UNIT=ounit1, FILE=s1, IOSTAT=ierr, FORM='FORMATTED')
	    if (ierr /= 0) then
              write(lout,1601) s1 
	      goto 999
	    endif
            write(lout,1546) s1, ounit1 
          endif
	  if (infmt == 0) then
            write(ounit1) l,ABS(nrhs)
            do i=1,l
              write(ounit1) REAL(x(i))
            enddo
	  else
            write(ounit1,*) l,ABS(nrhs)
            do i=1,l
              write(ounit1,*) x(i)
            enddo
          endif
          close(ounit1)
        endif

        if ((irhs < nrhs) .and. (irhs /= 1)) then
          if (mod(irhs,2) == 0) then
            s1 = trim(tmpdir) // chelp1
            s2 = trim(tmpdir) // chelp2
          else
            s1 = trim(tmpdir) // chelp2
            s2 = trim(tmpdir) // chelp1
          endif
	  if (infmt == 0) then
            open(UNIT=ounit1, FILE=s1, IOSTAT=ierr, FORM='UNFORMATTED')
	    if (ierr /= 0) then
              write(lout,1600) s1 
	      goto 999
	    endif
            open(UNIT=ounit2, FILE=s2, IOSTAT=ierr, FORM='UNFORMATTED')
	    if (ierr /= 0) then
              write(lout,1600) s2 
	      goto 999
	    endif
            write(lout,1543) s1, ounit1 
            write(lout,1544) s2, ounit2  
          else
            open(UNIT=ounit1, FILE=s1, IOSTAT=ierr, FORM='FORMATTED')
	    if (ierr /= 0) then
              write(lout,1601) s1 
	      goto 999
	    endif
            open(UNIT=ounit2, FILE=s2, IOSTAT=ierr, FORM='FORMATTED')
	    if (ierr /= 0) then
              write(lout,1601) s2 
	      goto 999
	    endif
            write(lout,1545) s1, ounit1 
            write(lout,1546) s2, ounit2 
          endif
	  if (infmt == 0) then
            read(ounit1) lh,nrhsh
            write(ounit1) l,ABS(nrhs)
            do i=1,l
              read(ounit1) (rbuf(j),j=1,irhs-1)
	      rbuf(irhs) = REAL(x(i))
              write(ounit2) (rbuf(j),j=1,irhs)
            enddo
	  else
            read(ounit1,*) lh,nrhsh
            write(ounit1,*) l,ABS(nrhs)
            do i=1,l
              read(ounit1,*) (dw(j),j=1,irhs-1)
              dw(irhs) = x(i)
              write(ounit2,*) (dw(j),j=1,irhs)
            enddo
          endif
          close(ounit1)
          close(ounit2)
        endif

        if (irhs == nrhs) then
          if (mod(irhs,2) == 0) then
            s1 = trim(tmpdir) // chelp1
          else
            s1 = trim(tmpdir) // chelp2
          endif
	  if (infmt == 0) then
            open(UNIT=ounit1, FILE=s1, IOSTAT=ierr, FORM='UNFORMATTED')
	    if (ierr /= 0) then
              write(lout,1600) s1 
	      goto 999
	    endif
            open(UNIT=ounit3,FILE=solout,IOSTAT=ierr,FORM='UNFORMATTED')
	    if (ierr /= 0) then
              write(lout,1600) solout 
	      goto 999
	    endif
            write(lout,1543) s1, ounit1 
            write(lout,1544) solout, ounit3  
          else
            open(UNIT=ounit1, FILE=s1, IOSTAT=ierr, FORM='FORMATTED')
	    if (ierr /= 0) then
              write(lout,1601) s1 
	      goto 999
	    endif
            open(UNIT=ounit3,FILE=solout,IOSTAT=ierr,FORM='FORMATTED')
	    if (ierr /= 0) then
              write(lout,1601) solout
	      goto 999
	    endif
            write(lout,1545) s1, ounit1 
            write(lout,1546) solout, ounit3 
          endif
	  if (infmt == 0) then
            read(ounit1) lh,nrhsh
            write(ounit3) l,ABS(nrhs)
            do i=1,l
              read(ounit1) (rbuf(j),j=1,irhs-1)
	      rbuf(irhs) = REAL(x(i))
              write(ounit3) (rbuf(j),j=1,irhs)
            enddo
	  else
            read(ounit1,*) lh,nrhsh
            write(ounit3,*) l,ABS(nrhs)
            do i=1,l
              read(ounit1,*) (dw(j),j=1,irhs-1)
              dw(irhs) = x(i)
              write(ounit3,*) (dw(j),j=1,irhs)
            enddo
          endif
          close(ounit1)
          close(ounit3)
        endif
      endif


!     if (nrhs == -1) then
!       xsum = zero
!       do i=1,l
!         xsum = xsum + (dble(i)**(1./3.))**2
!       enddo
!       xsum = sqrt(xsum)
!       xmax = dble(l)**(1./3.)
!       do i=1,iconst
!         top(i) = zero
!         buttom(i) = zero
!         itop(i) = 0
!         ibutom(i) = 0
!       enddo
!       do i=1,l
!         x(i) = abs(x(i)-dble(i)**(1./3.))
!       enddo
!       esum = zero
!       do i=1,l
!         esum = esum + x(i)*x(i)
!       enddo

!       count = 1

!       m1: do while (count .le. iconst)
!          emax = 0
!          do i=1,l
!            if (top(1) .eq. zero) then
!              if (abs(x(i)) .gt. emax) then
!                emax = abs(x(i))
!                imax = i
!              endif
!            else
!              if ((abs(x(i)) .gt. emax) .and.
!    &              abs(x(i)) .lt. top(count-1)) then
!                emax = abs(x(i))
!                imax = i
!              endif
!            endif
!          enddo
!          top(count) = emax
!          itop(count) = imax
!          count = count + 1
!       enddo m1

!       count = 1

!       m2: do while (count .le. iconst)
!          emin  = top(1)
!          do i=1,l
!            if (buttom(1) .eq. zero) then
!              if (abs(x(i)) .lt. emin) then
!                emin = abs(x(i))
!                imin = i
!              endif
!            else
!              if ((abs(x(i)) .lt. emin) .and.
!    &              abs(x(i)) .gt. buttom(count-1)) then
!                emin = abs(x(i))
!                imin = i
!              endif
!            endif
!          enddo
!          buttom(count) = emin
!          ibutom(count) = imin
!          count = count + 1
!       enddo m2
!       esum = sqrt(esum)/xsum
!       emax = top(1)/xmax
!       write(lout,1541)
!       write(lout,1539) (itop(i),i=1,iconst)
!       write(lout,1540) (top(i),i=1,iconst)
!       write(lout,1542)
!       write(lout,1539) (ibutom(i),i=1,iconst)
!       write(lout,1540) (buttom(i),i=1,iconst)
!       write(lout,1527) esum
!       write(lout,1530) emax
!     endif

!***********************************************************************
!**    End of Calculation                                            ***
!***********************************************************************

 1527 format(' ',/
     &       '  The relative Euclidian norm of the error is:',/
     &       '   ||error||2   = ',d10.4)
 1530 format('  The relative Maximum norm of the error is:',/
     &       '   ||error||max = ',d10.4/)
 1539 format(1x,4(i17,2x))
 1540 format(1x,4(d17.10,2x))
 1541 format(' ',//
     &       '  The 4 largest errors are (indices --> errors):')
 1542 format(' ',/
     &       '  The 4 smallest errors are (indices --> errors):')
 1543 format(//'  Reading binary file ',/
     &       '  ',a72,/
     &       '  on unit ',i3,' ...')
 1544 format(// '  Writing binary file ',/
     &       '  ',a72,/
     &       '  on unit ',i3,' ...'//)
 1545 format(//'  Reading file ',/
     &       '  ',a72,/
     &       '  on unit ',i3,' ...')
 1546 format(// '  Writing file ',/
     &       '  ',a72,/
     &       '  on unit ',i3,' ...'//)
 1600 format(/'  ERROR in routine WRITE_SOLUTION:'/
     &       '     binary file'/
     &       '     ',a72,/
     &       '     can not be opened!')
 1601 format(/'  ERROR in routine WRITE_SOLUTION:'/
     &       '     ascii file'/
     &       '     ',a72,/
     &       '     can not be opened!')

  999  r e t u r n
!-----END OF WRITE_SOLUTION --------------------------------------------
       e    n    d
