!:::::      ,,,,,BMPROW.....
!
!
!
!**********************************************************************
!        1         2         3         4         5         6         7*
!**********************************************************************
!**                                                                 ***
!**                                                                 ***
      subroutine BMPROW(lvt,bw,ilin,mflops,ierr)
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**     B M P R O W :  measures a simulated matrix vector           ***
!**                    multiplication for the storage pattern       ***
!**                    "packed row" with vector length <lvt>.       ***
!**                    The maximum distance of the indices of       ***
!**                    vector elements is the bandwidth <bw>!       ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**    Copyright (C) 2001 University Karlsruhe                      ***
!**    PROGRAMMER: H. Haefner                                       ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**     THE IMPLICIT NONE-INSTRUCTION HAS THE AIM,                  ***
!**     THAT ALL VARIABLES MUST BE DECLARED EXPLICITLY.             ***
!**                                                                 ***
      USE constants

      implicit none
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**      FORMAL PARAMETERS :                                        ***
!**      -----------------                                          ***
!**                                                                 ***
      integer, intent(in) ::  lvt,bw
!     integer, intent(in) ::  ilin(nilin)
      integer, intent(in) ::  ilin(50)
      integer, intent(out) ::  ierr
      double precision, intent(out) ::  mflops
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**   LIST OF IMPORTANT PARAMETERS :                                ***
!**   ------------------------------                                ***
!**                                                                 ***
!--------I------I-----I------------------------------------------------
! NAME   I TYPE I I/O I     MEANING
!--------I------I-----I------------------------------------------------
!--------I------I-----I------------------------------------------------
! bw     I  I   I in  I  average bandwidth of all (l) packed rows
!--------I------I-----I------------------------------------------------
! time   I  DP  I --  I  measured time of the simulated mvm for the
!        I      I     I  storage pattern "packed row" in microsec.
!--------I------I-----I------------------------------------------------
! mflops I  DP  I out I  MFlop/s rate for above mentioned simulation
!--------I------I-----I------------------------------------------------
!**                                                                 ***
!**                                                                 ***
!**      LOCAL PARAMETERS :                                         ***
!**      -----------------                                          ***
!**                                                                 ***
      integer    i,j,k,idoku,ixdist,lout,repfactor
      integer    ix(lvt)
      double precision   a(lvt), b(bw), c(1)
      double precision   cok, tim1, tim2, tdum, time, eps, LL8EPS
      double precision, parameter ::  zero = 0.0, two = 2.0
      double precision, parameter ::  lowthres  = 100000.0
!     double precision, parameter ::  highthres = 1000.0
!
!
!**** START OF CALCULATION :
!     ---------------------
!
      lout   = ilin(12)
      idoku  = ilin(13)

      ierr   = 0
      eps    = LL8EPS(1.d-10)

      repfactor = 1000

      ixdist = INT(bw/lvt)
      ix(1) = 1
      do i=2,lvt-1
	ix(i) = ixdist*(i-1)
      enddo
      ix(lvt) = bw

      j = 1
      a = 1.d0
      do i=1,bw
	b(i) = DBLE(i)
      enddo

  111 call SECONDS(tim1,tdum)
      do k=1,repfactor
	call DUMMY(a,lvt,b,bw,c,1,ix,lvt)
	do i=1,lvt
	  c(j) = c(j) + a(i) * b(ix(i))
	enddo
      enddo
      call SECONDS(tim2,tdum)
      time = (tim2 - tim1) * 1.d6
      if (time == zero) then
	repfactor = repfactor*10
	goto 111
      endif
      if (time < lowthres) then
	repfactor = INT(DBLE(repfactor) * two*(lowthres/time))
	goto 111
      endif
!     if (time > highthres) then
!       repfactor = INT(DBLE(repfactor) * two*(highthres/time))
!       goto 111
!     endif

      cok = zero
      do i=1,lvt
	cok = cok + DBLE(ix(i))
      enddo
      if (ABS(cok-c(1)) > eps) then
	write(lout,1100)
	write(lout,1101)
	write(lout,1110)
	ierr = 10123
	goto 999
      endif

      mflops = DBLE(repfactor)*DBLE(2*lvt)/time

      if (idoku /= 0) then
        write(lout,1000)
        write(lout,1001) lvt
        write(lout,1002) bw
        write(lout,1003) repfactor
        write(lout,1004) time
        write(lout,1005) mflops
      endif

  999 continue

 1000 format('  Performance data for the packed rows:')
 1001 format('   average vector length is ...............',i10)
 1002 format('   average bandwidth is . . . . . . . . . .',i10)
 1003 format('   repeat count for the benchmark is ......',i10)
 1004 format('   the measured time is (in microsec.)  . .',f10.2)
 1005 format('   the measured MFlop/s rate is ...........',f10.2,/)
 1100 format(' **************************************************'/
     &       ' ***  ERROR occured in BMPROW:                  ***')
 1101 format(' ***    wrong results! The optimization level   ***'/
     &       ' ***    for this routine is probably too high!  ***')
 1110 format(' **************************************************'/)

      r e t u r n
!-----END OF BMPROW-----------------------------------------------------      
      e n d
