!:::::      ,,,,,BNDWTHLU.....
!
!
!
!**********************************************************************
!        1         2         3         4         5         6         7*
!**********************************************************************
!**                                                                 ***
!**                                                                 ***
      subroutine BNDWTHLU(index,lindex,infoLU,ia1LU,nvtLU,ilin,tids,
     *                    nproc,bwl,bwu,gbwl,gbwu,lbwopt)
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**     B N D W T H L U :   computes the local and the global       ***
!**                         bandwidth of the matrix <mat>.          ***
!**                         (works only on the preconditioning      ***
!**                         matrix!                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**    Copyright (C) 2000 University Karlsruhe                      ***
!**    PROGRAMMER: H. Haefner                                       ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**     THE IMPLICIT NONE-INSTRUCTION HAS THE AIM,                  ***
!**     THAT ALL VARIABLES MUST BE DECLARED EXPLICITLY.             ***
!**                                                                 ***
      USE constants

      implicit none
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**      FORMAL PARAMETERS :                                        ***
!**      -----------------                                          ***
!**                                                                 ***
      integer, intent(in) ::  ia1LU,lindex,nproc,nvtLU
      integer, intent(in) ::  infoLU(ia1LU,ia2LU),index(lindex)
      integer, intent(in) ::  tids(nproc)
      integer, intent(inout) ::  ilin(nilin)
      integer, intent(out) ::  bwl,bwu,gbwl,gbwu
      logical, intent(in) ::  lbwopt
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**   LIST OF FORMAL PARAMETERS :                                   ***
!**   -------------------------                                     ***
!**                                                                 ***
!--------I------I-----I------------------------------------------------
! NAME   I TYPE I I/O I     MEANING
!--------I------I-----I------------------------------------------------
!--------I------I-----I------------------------------------------------
! bwl    I  I   I out I  bandwidth below the main diagonal
!--------I------I-----I------------------------------------------------
! bwu    I  I   I out I  bandwidth above the main diagonal
!--------I------I-----I------------------------------------------------
! gbwl   I  I   I out I  global bandwidth below the main diagonal
!--------I------I-----I------------------------------------------------
! gbwu   I  I   I out I  global bandwidth above the main diagonal
!--------I------I-----I------------------------------------------------
!**                                                                 ***
!**   (see also Manual-Pages <lsolp>)                               ***
!**                                                                 ***
!**                                                                 ***
!**      LOCAL PARAMETERS :                                         ***
!**      -----------------                                          ***
!**                                                                 ***
      integer    i,iar1,idistu,idistl,myproc,nmsg,ihelp(9),idoku,lout
      integer    bw
      integer, parameter ::  izero = 0
!
!
!**** START OF CALCULATION :
!     ---------------------
!
      nmsg   = ilin(3)
      lout   = ilin(12)
      idoku  = ilin(13)
      myproc = ilin(17)

      bwl = 0
      bwu = 0
      do i=1,nvtLU
	iar1  = infoLU(i,3) + 1
	idistu = max(index(infoLU(i,1)+infoLU(i,2))-iar1,izero)
        if (idistu > bwu) bwu = idistu
	idistl = max(iar1-index(infoLU(i,1)+1),izero)
        if (idistl > bwl) bwl = idistl
      enddo
      bw = max(bwl,bwu)

      if (nproc > 1) then
        ihelp(1) = bwu
        ihelp(2) = bwl
        ihelp(3) = bw
        call LL4INM(1,3,1,ihelp(1),ihelp(4),ihelp(7),myproc,nproc,
     *              tids,nmsg)
        gbwu = ihelp(4)
        gbwl = ihelp(5)
	if (idoku > 0 .and. myproc == 1) then
	  if (lbwopt) then
	    write(lout,1010)
	  else
	    write(lout,1009)
	  endif
	  write(lout,1011) bwu
	  write(lout,1012) bwl
	  write(lout,1013) bw
	  write(lout,1014) ihelp(4)
	  write(lout,1015) ihelp(5)
	  write(lout,1016) ihelp(6)
	endif
	if (idoku < 0) then
	  if (lbwopt) then
	    write(lout,1010)
	  else
	    write(lout,1009)
	  endif
	  write(lout,1011) bwu
	  write(lout,1012) bwl
	  write(lout,1013) bw
	  write(lout,1014) ihelp(4)
	  write(lout,1015) ihelp(5)
	  write(lout,1016) ihelp(6)
	endif
      else
        gbwu = bwu
        gbwl = bwl
        if (idoku /= 0) then
	  if (lbwopt) then
	    write(lout,1010)
	  else
	    write(lout,1009)
	  endif
	  write(lout,1017) bwu
	  write(lout,1018) bwl
	  write(lout,1019) bw
        endif
      endif

      ilin(3) = nmsg

 1009 format('  Bandwidth (BW) parameters of the matrix:')
 1010 format('  Bandwidth (BW) parameters of the BW-optimized matrix:')
 1011 format('   The local upper bandwidth is ...........',i7)
 1012 format('   The local lower bandwidth is . . . . . .',i7)
 1013 format('   The local bandwidth is .................',i7)
 1014 format('   The global upper bandwidth is ..........',i7)
 1015 format('   The global lower bandwidth is  . . . . .',i7)
 1016 format('   The global bandwidth is ................',i7,/)
 1017 format('   The upper bandwidth is .................',i7)
 1018 format('   The lower bandwidth is . . . . . . . . .',i7)
 1019 format('   The bandwidth is .......................',i7,/)

      r e t u r n
!-----END OF BNDWTHLU--------------------------------------------------      
      e n d
