!:::::      ,,,,,CHOLSYM.....
!
!
!
!**********************************************************************
!        1         2         3         4         5         6         7*
!**********************************************************************
!**                                                                 ***
!**                                                                 ***
      subroutine CHOLSYM(mat,lmat,index,lindex,l,infoL,nvt,ia1,
     #                   bw,rlin,ilin,opcount,ierr)
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**      C H O L S Y M   eliminates a 'packed row' symmetric matrix ***
!**                      described by the triple (mat,index,info)   ***
!**                      within a buffer described by (gbuf,infbuf) ***
!**                      into the same matrix described by the      ***
!**                      triple (mat,index,infoL) for the matrix L. ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**    Copyright (C) 2001 University Karlsruhe                      ***
!**    PROGRAMMER:    H. Haefner                                    ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
      USE constants

      implicit none
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**      FORMAL PARAMETERS :                                        ***
!**      -----------------                                          ***
!**                                                                 ***
      integer,intent(in)    ::  lmat,lindex,l,ia1,nvt
      integer,intent(in)    ::  bw
      integer,intent(out)   ::  ierr
      integer,intent(inout) ::  ilin(nilin),index(lindex)
      integer,intent(inout) ::  infoL(ia1,ia2LU)
      integer*8,intent(out) ::  opcount
      double precision,intent(in) ::  rlin(nrlin)
      double precision,intent(inout) ::  mat(lmat)
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**   LIST OF FORMAL PARAMETERS :                                   ***
!**   -------------------------                                     ***
!**                                                                 ***
!**   see Manual-Pages <lsolp>                                      ***
!**                                                                 ***
!**                                                                 ***
!**                                                                 ***
!**                                                                 ***
!**        LOCAL PARAMETERS :                                       ***
!**        ----------------                                         ***
!**                                                                 ***
      integer ::  ia1buf  
      integer ::  lbuf
      integer    i,idoku,ii,il,irow,j,jj,k,lout
      integer    ihelp,indc,lvt,lvtbuf,infoH(2),no2pct,cntpct
      integer    norows,endgbuf,ptrmat,ptrlu
      integer    maxcol(l),minrow(l)
      integer, parameter ::  ifive = 5, ififty = 50
      integer,allocatable ::  infbuf(:,:)
      double precision  Ltemp,eps,LL8EPS,rdiag,rinv,rsub,thrshld2
      double precision,allocatable ::  gbuf(:)
      double precision, parameter ::  zero = 0.0, one=1.0
!
!**                                                                 ***
!**   LIST OF LOCAL PARAMETERS :                                    ***
!**   -------------------------                                     ***
!
!--------I------I-----I------------------------------------------------
! NAME   I TYPE I I/O I     MEANING
!--------I------I-----I------------------------------------------------
!--------I------I-----I------------------------------------------------
! gbuf   I  DP  I --- I  is a one-dimensional array with the actual
!        I      I     I  unpacked matrix rows.        array: gbuf(lbuf)
!--------I------I-----I------------------------------------------------
! infbuf I  I   I --- I  is a 2-dimensional information array for gbuf.
!        I      I     I  array: infbuf(ia1buf,ia2buf)
!        I      I     I  infbuf(...,1) - pointer to gbuf
!        I      I     I  infbuf(...,2) - global row number
!        I      I     I  infbuf(...,3) - length of vector in gbuf (lvt)
!        I      I     I  infbuf(1,...) is actual row number, further
!        I      I     I  row numbers are increasingly sorted.
!--------I------I-----I------------------------------------------------
! maxcol I  I   I --- I  maximal column index per row.
!        I      I     I                                array: maxcol(l)
!--------I------I-----I------------------------------------------------
! irow   I  I   I --- I  actual row number.
!--------I------I-----I------------------------------------------------
! norows I  I   I --- I  number of actual rows in <gbuf>.
!--------I------I-----I------------------------------------------------
! endgbufI  I   I --- I  pointer to the last active element in <gbuf>.
!--------I------I-----I------------------------------------------------
! ptrmat I      I --- I  pointer to the last element of <mat> already
!        I      I     I  loaded to <gbuf>.
!--------I------I-----I------------------------------------------------
! ptrlu  I      I --- I  pointer to the last element of the restored 
!        I      I     I  vector terms belonging to the eliminated
!        I      I     I  matrix in <mat>.
!--------I------I-----I------------------------------------------------
!


!
!
!**** START OF CALCULATION :
!     ---------------------
!
      ierr   = 0
      lout   = ilin(12)
      idoku  = ilin(13)
      eps    = LL8EPS(one * 1.d-10)

      thrshld2 = rlin(3)

      ia1buf = bw+1
      lbuf   = ia1buf*bw

      allocate(gbuf(lbuf),infbuf(ia1buf,ia2buf),stat=ierr)
      if (ierr > 0) then
	write(lout,1000)
	write(lout,1003) lbuf*ireal + ia1buf*ia2buf*iint
	write(lout,1020)
	goto 999
      endif

      norows = 0
      infbuf = 0
      endgbuf = 0

      il = 0
      ptrlu = 0

      if (idoku /= 0) then
        no2pct = NINT(REAL(l)/REAL(ififty))
        if (no2pct == 0) no2pct = 1
        cntpct = 0
        write(lout,1033) 
        write(lout,'(A2)',ADVANCE="NO") "  "
      endif

!
!**** Computation of maximal column index per row
!**** Rows must be sorted by row numbers
!
      maxcol(1) = index(infoL(1,1)+infoL(1,2))
      do i=2,nvt
        lvt  = infoL(i,2)
        indc = infoL(i,1)
	maxcol(i) = MAX(index(indc+lvt),maxcol(i-1))
      enddo

!
!**** Computation of minimal row index per column
!
      do i=1,maxcol(1)
	minrow(i) = 1
      enddo
      do j=2,l
        do i=maxcol(j-1)+1,maxcol(j)
	  minrow(i) = j
        enddo
      enddo

!
!**** Elimination process
!

      do irow=1,l     ! for all rows

!
!**** Load pivot row into buffer
!
        call LOADSYM(mat,lmat,index,lindex,infoL,ia1,gbuf,lbuf,infbuf,
     *               ia1buf,maxcol,l,endgbuf,irow,norows,ilin,ierr)
	if (ierr /= 0) goto 999

!
!**** Update <ptrmat>
!
        if (thrshld2 == one) then
	  ptrmat = infoL(irow,1)
	else
	  ptrmat = infoL(irow,1) + infoL(irow,2)
	endif

!
!**** Now follows the kernel of the Cholesky-algorithm
!
!
!**** Add the coefficients to be subtracted from the main diagonal element
!
        opcount = opcount + norows + 2
	rsub = zero
        include "norec.h"
        do i=2,norows
	  rsub = rsub + gbuf(infbuf(i,1)+1) * gbuf(infbuf(i,1)+1)
	enddo
	rdiag = gbuf(infbuf(1,1)+1) - rsub
        if (rdiag < eps) then
	  write(lout,1000)
	  write(lout,1007) irow,irow,eps
	  write(lout,1020)
	  ierr = 10000
	  goto 999
        endif
	rdiag = SQRT(rdiag)

	rinv  = one/rdiag
	gbuf(infbuf(1,1)+1) = rdiag
!
!**** Update the coefficients of the pivot row
!
        opcount = opcount + (infbuf(1,3)-1)*(2*(norows-1)+1)
        do j=2,infbuf(1,3)       ! for all coeff. of pivot row without main
				 ! diagonal element
          include "norec.h"
	  do i=2,irow-minrow(irow+j-1)+1  ! Subtract products of upper rows
	    gbuf(infbuf(1,1)+j) = gbuf(infbuf(1,1)+j) - 
     *                        gbuf(infbuf(i,1)+1) * gbuf(infbuf(i,1)+j)       
	  enddo
	  gbuf(infbuf(1,1)+j) = gbuf(infbuf(1,1)+j) * rinv
        enddo

!
!**** Update of the preconditioning matrix
!
!**** Matrix L is being stored rowwise. Indices are sorted increasingly.
!

        il = il + 1
	lvtbuf = 0
	if (thrshld2 == one) then
	  infoH(1) = infoL(irow,2)
	  infoH(2) = infoL(irow,1)
	endif
	infoL(il,1) = ptrlu
	infoL(il,3) = irow - 1
	if (thrshld2 == one) then
	  do ii=1,infoH(1)
	    Ltemp = gbuf(infbuf(1,1)+index(infoH(2)+ii)-irow+1)
	    if (ABS(Ltemp) > zero) then
	      lvtbuf = lvtbuf + 1
	      mat(infoL(il,1)+lvtbuf) = Ltemp
	      index(infoL(il,1)+lvtbuf) = index(infoH(2)+ii)
	    endif
	  enddo
	else
	  do ii=1,infbuf(1,3)
	    if (ABS(gbuf(infbuf(1,1)+ii)) > thrshld2) then
	      lvtbuf = lvtbuf + 1
	      mat(infoL(il,1)+lvtbuf) = gbuf(infbuf(1,1)+ii)
	      index(infoL(il,1)+lvtbuf) = irow + ii - 1
	    endif
	  enddo
	endif
	infoL(il,2) = lvtbuf
	ptrlu = ptrlu + lvtbuf
	if (ptrlu > ptrmat) then
	  write(lout,1000)
	  write(lout,1002) irow,1
	  write(lout,1010) 1
	  write(lout,1020)
	  ierr = 10001
	  goto 999
	endif

        ihelp = norows
	do i=ihelp,1,-1
	  if (infbuf(i,3) == 1) then
	    norows = norows - 1
	  else
	    exit
	  endif
	enddo

        do j=1,ia2buf
          include "norec.h"
          do i=norows,1,-1
            infbuf(i+1,j) = infbuf(i,j)
          enddo
        enddo

        include "norec.h"
        do i=2,norows+1
          infbuf(i,1) = infbuf(i,1) + 1
          infbuf(i,3) = infbuf(i,3) - 1
        enddo

	if ((idoku /= 0) .and. (MOD(irow-1,no2pct) == 0)) then
	  cntpct = cntpct + 1
	  if (MOD(cntpct,ifive) == 0) then
	    if (cntpct == ififty) then
	      write(lout,1032,ADVANCE="NO") cntpct*2
	      no2pct = l
	    else
	      write(lout,1031,ADVANCE="NO") cntpct*2
	    endif
	  else
	    write(lout,'(A1)',ADVANCE="NO") ">"
	  endif
	endif

      enddo  ! irow=1,l


      ilin(37) = ptrlu
      write(lout,1035)
      deallocate(gbuf,infbuf)

  999 continue
	    
 1000 format(//' *****************************************************'/
     &       ' ***  ERROR occured in CHOLSYM:                    ***')      
 1002 format(' ***    preconditioning matrix is too small;       ***'/
     &       ' ***    matrix overflow when writing the           ***'/
     &       ' ***   ',i6,'th row of L on processor ',i6,'!      ***')
 1003 format(' ***    allocation of elimination buffer           ***'/
     &       ' ***    (gbuf,infbuf) failed because of a too high ***'/
     &       ' ***    memory request of ',i10,' Bytes!        ***')
 1007 format(' ***    the pivot element l(',i6,',',i6,') is      ***'/
     &       ' ***    smaller than EPSILON(1.D0) (',g12.4,'), ***'/
     &       ' ***    i.e. the symmetric matrix is not SPD and   ***'/
     &       ' ***    the Cholesky-algorithm collapses!          ***')
 1010 format(' ***    the preconditioning matrix is too          ***'/
     &       ' ***    small on processor ',i6,' because of       ***'/
     &       ' ***    FILL IN! Inrease THRESHOLD values or       ***'/
     &       ' ***    increase LINSOL_GAUSS_FACTOR!              ***')
 1020 format(' *****************************************************')
!1031 format(/'  Allocating storage of size (in Bytes) ',i11,/
!    &       '  for the computation window  .....'/)
!1032 format(/'  Deallocating storage for the computation ',/
!    &       '  window  .....'/)
 1031 format(i2)
 1032 format(i3)
 1033 format(/'  Actual progress (char \">\" are 2 percent of',
     &       ' total work):')
 1035 format(/)
!
!**** END OF CALCULATION
!     ------------------
!
      r e t u r n
!-----END OF CHOLSYM---------------------------------------------------     
      e    n    d
