!**********************************************************************
!        1         2         3         4         5         6         7*
!**********************************************************************
!**                                                                 ***
!**                                                                 ***
      subroutine COLSORT(mat,lmat,index,lindex,info,ia1,perm,l,maxlvt,
     *                   lmatbk,ptrmbk,myproc,nproc,tids,nmsg)
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**   C O L S O R T  resorts the columns of <mat> - stored in       ***
!**                  packed rows - due to the vector perm.          ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**    Copyright (C) 1998 University Karlsruhe                      ***
!**    PROGRAMMER: H. Haefner                                       ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**     THE IMPLICIT NONE-INSTRUCTION HAS THE AIM,                  ***
!**     THAT ALL VARIABLES MUST BE DECLARED EXPLICITLY.             ***
!**                                                                 ***
      USE constants

      implicit none
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**      FORMAL PARAMETERS :                                        ***
!**      -----------------                                          ***
!**                                                                 ***
      integer  ia1,lmat,lindex,l,maxlvt,myproc,nproc,nmsg
      integer  lmatbk(nproc),ptrmbk(nproc+1),tids(nproc),info(ia1,ia2)
      integer  index(lindex),perm(l)
      double precision mat(lmat)
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**   LIST OF FORMAL PARAMETERS :                                   ***
!**   -------------------------                                     ***
!**                                                                 ***
!--------I------I-----I------------------------------------------------
! NAME   I TYPE I I/O I     MEANING
!--------I------I-----I------------------------------------------------
!--------I------I-----I------------------------------------------------
! perm   I  I   I in  I  permutation vector for the rows of the matrix
!        I      I     I  perm(i) = j: i is local and j is global param.
!--------I------I-----I------------------------------------------------
!**                                                                 ***
!**                                                                 ***
!**      LOCAL PARAMETERS :                                         ***
!**      -----------------                                          ***
!**                                                                 ***
      double precision matbuf(maxlvt)
      integer          permbuf(l),lix(maxlvt)
      integer          indbuf(lindex)
      integer          adda,i,indc,j,k,ierr,fix,lvt,mids,midr,from,to
      integer          msgcnt
!
!**** START OF CALCULATION :
!     ---------------------
!
      ierr = 0

      to = myproc-1
      if (to == 0) to = nproc
      to = tids(to)
      from = myproc+1
      if (from > nproc) from = 1
      from = tids(from)

      lix = 0
      msgcnt = 0

      do i=myproc,nproc
	if (nproc > 1) then
	  call MPRCVA(from,nmsg+msgcnt,l*iint,permbuf,midr,ierr)
	  call MPSNDA(to,nmsg+msgcnt,l*iint,perm,mids,ierr)
	  call MPSNDW(to,nmsg+msgcnt,l*iint,perm,mids,ierr)
	endif
	do k=1,lmatbk(myproc)   ! for all packed rows
	  lvt  = info(k,3)
	  indc = info(k,6)
	  j = lix(k) + 1
	  do while (j <= lvt .and. index(indc+j) <= ptrmbk(i))
	    j = j + 1
	  enddo
	  if (j <= lvt) then
	    fix = j
	    do while (j <= lvt .and. index(indc+j) <= ptrmbk(i+1))
	      j = j + 1
	    enddo
	    lix(k) = j - 1
	  else
	    fix = lix(k) + 1
	  endif
!
!*** Apply the vector <perm> to the k-th packed row
!
	  do j=fix,lix(k)
	    indbuf(indc+j) = perm(index(indc+j)-ptrmbk(i))
	  enddo
	enddo
	if (nproc > 1) then
	  call MPRCVW(from,nmsg+msgcnt,l*iint,permbuf,midr,ierr)
	  perm = permbuf
	endif
      enddo

      lix = 0

      do i=1,myproc-1
	call MPRCVA(from,nmsg+msgcnt,l*iint,permbuf,midr,ierr)
	call MPSNDA(to,nmsg+msgcnt,l*iint,perm,mids,ierr)
	call MPSNDW(to,nmsg+msgcnt,l*iint,perm,mids,ierr)
	do k=1,lmatbk(myproc)   ! for all packed rows
	  lvt  = info(k,3)
	  indc = info(k,6)
	  j = lix(k) + 1
	  do while (j <= lvt .and. index(indc+j) <= ptrmbk(i))
	    j = j + 1
	  enddo
	  if (j <= lvt) then
	    fix = j
	    do while (j <= lvt .and. index(indc+j) <= ptrmbk(i+1))
	      j = j + 1
	    enddo
	    lix(k) = j - 1
	  else
	    fix = lix(k) + 1
	  endif
!
!*** Apply the vector <perm> to the k-th packed row
!
	  do j=fix,lix(k)
	    indbuf(indc+j) = perm(index(indc+j)-ptrmbk(i))
	  enddo
	enddo

	call MPRCVW(from,nmsg+msgcnt,l*iint,permbuf,midr,ierr)
	perm = permbuf
      enddo

      nmsg  = nmsg + nproc + 1

      !It is safe now to assign the new indices
      index = indbuf

!
!*** Sort indices of matrixelements for all packed rows
!

      do k=1,lmatbk(myproc)
	adda = info(k,2)
	lvt  = info(k,3)
	indc = info(k,6)
        call HEAPSORTIX(index(indc+1),lix,lvt,.true.)
	do j=1,lvt
	  matbuf(j) = mat(adda+lix(j))
	enddo
	do j=1,lvt
	  mat(adda+j) = matbuf(j)
	enddo
      enddo


      R E T U R N
C------END OF COLSORT----------------------------------------------
      E N D
