!***********************************************************************
!        1         2         3         4         5         6         7 *
!***********************************************************************
!**                                                                  ***
!**                                                                  ***
      subroutine COMP_RHS_IGUESS(MAT,lmat,index,lindex,lsym,info,ia1,
     &                           rhs,llocal,lmatbk,ptrmbk,ptrinf,
     &                           ierr,tids,nproc,myproc,nmsg)
!**                                                                  ***
!**                                                                  ***
!***********************************************************************
!**                                                                  ***
!**    COMP_RHS_IGUESS  computes one right hand side and an initial  ***
!**                     guess for a distributed matrix               ***
!**                                                                  ***
!***********************************************************************
!**                                                                  ***
!**    Copyright (C) 1999 University Karlsruhe                       ***
!**    PROGRAMMER : H. Haefner                                       ***
!**                                                                  ***
!**    This file is part of LINSOL.                                  ***
!**    LINSOL is free software; you can redistribute it and/or       ***
!**    modify it under the terms of the GNU General Public License   ***
!**    as published by the Free Software Foundation; either          ***
!**    version 2 of the License, or (at your option) any later       ***
!**    version.                                                      ***
!**                                                                  ***
!**    LINSOL is distributed in the hope that it will be useful,     ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty   ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.       ***
!**    See the GNU General Public License for more details.          ***
!**                                                                  ***
!**    You should have received a copy of the GNU General Public     ***
!**    License along with LINSOL; if not, write to the Free          ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,       ***
!**    Boston, MA  02111-1307, USA.                                  ***
!**                                                                  ***
!***********************************************************************
!**                                                                  ***
      USE constants

      implicit none
!**                                                                  *** 
!***********************************************************************
!**                                                                  ***
!**    Formal Parameters :                                           ***
!**                                                                  ***
!***********************************************************************
      integer, intent(in) ::  llocal,lmat,lindex,ia1,myproc,nproc
      integer,intent(inout) ::  ierr,nmsg
      integer,intent(in) ::   lmatbk(nproc),ptrmbk(nproc+1)
      integer,intent(in) ::   ptrinf(ntyp+1,nproc)
      integer, intent(in) ::  info(ia1,ia2),index(lindex),tids(nproc)
      double precision, intent(in) ::  MAT(lmat)
      double precision, intent(out) ::  rhs(llocal)
      logical, intent(in) ::  lsym
!***********************************************************************
!**                                                                  ***
!**    List of Formal Parameters :                                   ***
!**                                                                  ***
!**                                                                  ***
!---------I------I-----I------------------------------------------------
! NAME    I TYPE I I/O I MEANING
!---------I------I-----I------------------------------------------------
! lmat    I int  I in  I number of matrix elements
!---------I------I-----I------------------------------------------------
! lindex  I int  I in  I number of indices
!---------I------I-----I------------------------------------------------
! info    I int  I in  I array with matrix-structure-information
!---------I------I-----I------------------------------------------------
! index   I int  I in  I array with indices
!---------I------I-----I------------------------------------------------
! MAT     I  dp  I in  I array with matrix-elements
!---------I------I-----I------------------------------------------------
! lsym    I l    I in  I indicates if matrix is symmetric
!---------I------I-----I------------------------------------------------
! ierr    I int  I i/o I error-indication
!---------I------I-----I------------------------------------------------
!**                                                                  ***
!**                                                                  ***
!**    Local Variables :                                             ***
!**                                                                  ***
      integer          i,j,from,midr
      integer          iw(2*nproc),jump(maxnpe),iermax,ihelp
      double precision dw(5*llocal),x(llocal)
      external         LL6AX
      double precision, parameter ::  zero = 0.0

!***********************************************************************
!**                                                                  ***
!**    List of Local Variables :                                     ***
!**                                                                  ***
!**                                                                  ***
!---------I------I-----I------------------------------------------------
! NAME    I TYPE I I/O I MEANING
!---------I------I-----I------------------------------------------------
!   dw    I  dp  I  -  I work array, array: dw(5*l)
!---------I------I-----I------------------------------------------------
!***********************************************************************


!***********************************************************************
!**    Start of Calculation                                          ***
!***********************************************************************

      ierr = 0000

!*****************************************
!   receive the solution from processor 1
!*****************************************

      from = tids(1)
      call MPRCVA(from,nmsg+myproc-1,lmatbk(myproc)*ireal,
     &            x(1),midr,ierr)
      call MPRCVW(from,nmsg+myproc-1,lmatbk(myproc)*ireal,
     &            x(1),midr,ierr)
      nmsg = nmsg+nproc

      call LL4INM(1,1,1,ierr,iermax,ihelp,myproc,nproc,tids,nmsg)
      if (iermax /= 0) then
        ierr = iermax
	goto 999
      endif

      call LL4JMP(jump,ptrinf,iw,2*nproc,lsym,myproc,nproc,tids,nmsg)

      if (lsym) then
        call LL3ASX(mat,x,rhs,dw,ptrmbk,lmatbk,ptrinf,
     #              llocal,nproc,tids,myproc,jump,.false.,
     #              lmat,lindex,ia1,info,index,nmsg)
      else
        call LL3AX(LL6AX,mat,x,rhs,dw,ptrmbk,lmatbk,ptrinf,
     #             llocal,nproc,tids,myproc,jump,.false.,
     #             lmat,lindex,ia1,info,index,nmsg)
      endif

!***********************************************************************
!**    End of Calculation                                            ***
!***********************************************************************

  999 r e t u r n
!-----END OF COMP_RHS_IGUESS-------------------------------------------
      e    n    d
