!**********************************************************************
!        1         2         3         4         5         6         7*
!**********************************************************************
!**                                                                 ***
!**                                                                 ***
      subroutine COPY(mat,lmat,index,lindex,l,info,infop,ia1,ia1LU,
     &                ilin,rlin,thrshld1,gaussfac,nvt,nvtLU,prec,lprec,
     &                lpmat,iprec,liprec,lpix,startmat,endpmat,lsym,
     &                myproc,nproc,tids,nmsg,ierr)
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**   C O P Y   copies the matrix <mat> into the preconditioning    ***
!**             matrix <prec> under the control of                  ***
!**             vector thrshld1.                                    ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**    Copyright (C) 1999 University Karlsruhe                      ***
!**    PROGRAMMER: H. Haefner                                       ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**     THE IMPLICIT NONE-INSTRUCTION HAS THE AIM,                  ***
!**     THAT ALL VARIABLES MUST BE DECLARED EXPLICITLY.             ***
!**                                                                 ***
      USE constants

      implicit none
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**      FORMAL PARAMETERS :                                        ***
!**      -----------------                                          ***
!**                                                                 ***
      integer,intent(in) ::  ia1,ia1LU,nproc,myproc,l,lmat,lindex,nvt
      integer,intent(in) ::  lprec,liprec
      integer,intent(inout) ::  iprec(liprec),nmsg
      integer,intent(out) ::  ierr,startmat,endpmat,lpmat,lpix,nvtLU
      integer,intent(in) ::  info(ia1,ia2),ilin(nilin),tids(nproc)
      integer,intent(in) ::  index(lindex)
      integer,intent(out) ::  infop(ia1LU,ia2LU)
      real,intent(in) ::  gaussfac
      double precision,intent(in) ::  mat(lmat),rlin(nrlin)
      double precision,intent(inout) ::  prec(lprec)
      double precision,intent(in)::  thrshld1(nvt)
      logical,intent(in) ::  lsym
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**   LIST OF FORMAL PARAMETERS :                                   ***
!**   -------------------------                                     ***
!**                                                                 ***
!**   (see Manual-Pages <lsolp>)                                    ***
!**                                                                 ***
!**                                                                 ***
!**      LOCAL PARAMETERS :                                         ***
!**      -----------------                                          ***
!**                                                                 ***
      integer    count,i,ihlp,k,gcount,endmat,gendmat,lglobal,lout
      integer    startind,endind,iermax,minlen
      integer    adda,addap,lvt,lvtp,indc,idoku
      real       thrscomp

!
!**** START OF CALCULATION :
!     ---------------------
!
      lout     = ilin(12)
      idoku    = ilin(13)
      gendmat  = ilin(30)
      endmat   = ilin(33)
      endind   = ilin(34)
      lglobal  = ilin(36)
      minlen   = MIN(lmat,lindex)

      ierr     = 0
      iermax   = 0

!
!***  This routine only works if holds: nvt = lmyblk
!
      iprec(7) = 5*l

      lpmat = lprec - iprec(7)

      if (nproc == 1) then
        startmat = lpmat - endmat
      else
        startmat = lpmat - minlen
      endif

      if (nproc == 1) then
        lpix     = INT(gaussfac*endind)
        startind = lpix - endind
        iprec(9) = 15 + l
      else
        lpix     = INT(gaussfac*minlen)
        startind = lpix - minlen
        iprec(9) = 15 + lglobal
      endif

      iprec(1)  = lpmat
      iprec(2)  = lpix
      iprec(3)  = ia1LU
      iprec(15) = 15

!***   Pointer to U    ***

      if (lsym) then
	iprec(10) = -999999999
      else
        if (nproc == 1) then
          iprec(10) = 15 + l + INT(gaussfac*endind)
        else
          iprec(10) = 15 + lglobal + INT(gaussfac*minlen)
        endif
      endif

      if (endmat /= endind) then
	ierr = 9999
	goto 999
      endif

      count = 0
      addap = startmat

!
!***  thrshld1 sorted due to info
!
      do k=1,nvt
        adda  = info(k,2)
        lvt   = info(k,3)
        indc  = info(k,6)
	lvtp = 0
        do i=1,lvt
          if (abs(mat(adda+i)) > thrshld1(k)) then  
            count = count + 1
	    lvtp  = lvtp + 1
            prec(iprec(7)+addap+lvtp) = mat(adda+i)
	    iprec(iprec(9)+addap+lvtp) = index(indc+i)
	  endif
	enddo
	infop(k,1) = addap
	infop(k,2) = lvtp
	infop(k,3) = info(k,5) 
	addap = addap + lvtp
	if (lvtp == 0) then
	  ierr = 8888
	  goto 999
	endif
      enddo

      nvtLU   = nvt
      endpmat = count

      call LL4ISM(1,1,1,count,gcount,ihlp,myproc,nproc,tids,nmsg)
      thrscomp = REAL(gcount)/REAL(gendmat)

      if ((idoku > 0) .and. (myproc == 1)) then
        write(lout,1000)
        write(lout,1001) thrscomp*100.0
        write(lout,1010)
      endif


  999 call LL4INM(1,1,1,ierr,iermax,ihlp,myproc,nproc,tids,nmsg)
      if (iermax > 0) then
	ierr = iermax
      endif

 1000 format('   -------------------------------------------------- '/
     &       '  ||  Copying matrix A into matrix A(P) in COPY:    ||')
 1001 format('  ||    percentage of entries of A                  ||'/
     &       '  ||    that are copied to A(P) ...... :',f7.2,'%    ||')
 1010 format('   -------------------------------------------------- '/)

      R E T U R N
!------END OF COPY-----------------------------------------------------
      E N D                                   
