!**********************************************************************
!        1         2         3         4         5         6         7*
!**********************************************************************
!**                                                                 ***
!**                                                                 ***
      subroutine CREATEP_BUF3(matbuf,indbuf,infbuf,matsize,indsize,
     *                        infsize,endpmat,nvtLU,ilin,myproc,
     *                        nproc,tids,nmsg,ierr)
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**   C R E A T E P _ B U F 3  creates 3 buffers of                 ***
!**                            LINSOL_BUFFER_SIZE (in Mbyte) for    ***
!**                            the matrix <mat>, the index array    ***
!**                            <index> and the information array    ***
!**                            <infoLU>.                            ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**    Copyright (C) 2000 University Karlsruhe                      ***
!**    PROGRAMMER: H. Haefner                                       ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**     THE IMPLICIT NONE-INSTRUCTION HAS THE AIM,                  ***
!**     THAT ALL VARIABLES MUST BE DECLARED EXPLICITLY.             ***
!**                                                                 ***
      USE constants

      implicit none
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**      FORMAL PARAMETERS :                                        ***
!**      -----------------                                          ***
!**                                                                 ***
      integer,intent(in) ::   endpmat,nvtLU,nproc,myproc
      integer,intent(inout) ::   nmsg
      integer,intent(out) ::  ierr
      integer,intent(in) ::   ilin(nilin),tids(nproc)
      double precision,pointer :: matbuf(:)
      integer,pointer :: indbuf(:)
      integer,pointer :: infbuf(:)
      integer,intent(out) ::   matsize,indsize,infsize
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**   LIST OF FORMAL PARAMETERS :                                   ***
!**   -------------------------                                     ***
!**                                                                 ***
!**   (see Manual-Pages <linsol>)                                   ***
!**                                                                 ***
!**                                                                 ***
!**      LOCAL PARAMETERS :                                         ***
!**      -----------------                                          ***
!**                                                                 ***
      integer    iermax,ihelp(6),idoku,lout,bufsize

!
!**** START OF CALCULATION :
!     ---------------------
!
      lout   = ilin(12)
      idoku  = ilin(13)
      bufsize = ilin(7)

      ierr  = 0
      if (bufsize < 1) then
	ierr = 1
	goto 999
      endif
	
      ihelp(1) = endpmat
      ihelp(2) = nvtLU
      call LL4INM(1,2,1,ihelp(1),ihelp(3),ihelp(5),myproc,nproc,
     *            tids,nmsg)
      matsize = min(ihelp(3),int(bufsize*1.e6/ireal))
      indsize = matsize
      infsize = min(ihelp(4),int(bufsize*1.e6/(iint*3*ia2LU)))
!     if (idoku > 0) then
!       if (myproc == 1) write(lout,1010) matsize*ireal
!     endif
!     if (idoku < 0) write(lout,1010) matsize*ireal 
      allocate (matbuf(matsize),stat=ierr)
!     if (idoku > 0) then
!       if (myproc == 1) write(lout,1011) indsize*iint
!     endif
!     if (idoku < 0) write(lout,1011) indsize*iint 
      allocate (indbuf(indsize),stat=ierr)
!     if (idoku > 0) then
!       if (myproc == 1) write(lout,1012) infsize*ia2LU*iint
!     endif
!     if (idoku < 0) write(lout,1012) infsize*ia2LU*iint 
      allocate (infbuf(infsize*ia2LU),stat=ierr)

  999 call LL4INM(1,1,1,ierr,iermax,ihelp(1),myproc,nproc,tids,nmsg)
      if (iermax > 0) then
        if (myproc == 1) then
          write(lout,1000)
          write(lout,1040)
          write(lout,1020)
        endif
      endif

 1000 format(' **************************************************'/
     &       ' ***  ERROR in routine CREATEP_BUF3:            ***')
 1040 format(' ***    Buffer allocation failed!!! Either the  ***'/
     &       ' ***    chosen buffer size is too large or the  ***'/
     &       ' ***    buffer size has not been set by the     ***'/
     &       ' ***    user! (>> set ilin(7))                  ***')
 1020 format(' **************************************************')
!1010 format('  Allocating storage of size (in Bytes) ',i9,/
!    &       '  for a matrix buffer  .....'/)
!1011 format('  Allocating storage of size (in Bytes) ',i9,/
!    &       '  for a index buffer  .....'/)
!1012 format('  Allocating storage of size (in Bytes) ',i9,/
!    &       '  for a information buffer  .....'/)

      R E T U R N
!------END OF CREATEP_BUF3------------------------------------------
      E N D                                   
