!:::::      ,,,,,ELIMINATIONX.....
!
!
!
!**********************************************************************
!        1         2         3         4         5         6         7*
!**********************************************************************
!**                                                                 ***
!**                                                                 ***
      subroutine ELIMINATIONX(mat,lmat,index,lindex,l,infoL,nvt,ia1,
     #                        colperm,infoU,bwlow,bwup,rlin,
     #                        ilin,opcount,ierr)
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**      E L I M I N A T I O N X  factorizes a 'packed row' matrix  ***
!**                               described by the triple           *** 
!**                               (mat,index,info) within a buffer  ***
!**                               described by (gbuf,infbuf) into   ***
!**                               the same matrix described by the  ***
!**                               triple (mat,index,infoL) for the  ***
!**                               matrix L  and (mat,index,infoU)   ***
!**                               for the matrix U by using an      ***
!**                               index vector in the computation   ***
!**                               window.                           ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**    Copyright (C) 2002 University Karlsruhe                      ***
!**    PROGRAMMER:    H. Haefner                                    ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
      USE constants

      implicit none
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**      FORMAL PARAMETERS :                                        ***
!**      -----------------                                          ***
!**                                                                 ***
      integer,intent(in)    ::  lmat,lindex,l,ia1,nvt
      integer,intent(in)    ::  bwlow,bwup
      integer,intent(out)   ::  ierr
      integer,intent(inout) ::  ilin(nilin),index(lindex)
      integer,intent(inout) ::  infoL(ia1,ia2LU)
      integer,intent(out)   ::  infoU(ia1,ia2LU)
      integer,intent(out)   ::  colperm(l)
      integer*8,intent(out) ::  opcount
      double precision,intent(in)    ::  rlin(nrlin)
      double precision,intent(inout) ::  mat(lmat)
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**   LIST OF FORMAL PARAMETERS :                                   ***
!**   -------------------------                                     ***
!**                                                                 ***
!**   see Manual-Pages <lsolp>                                      ***
!**                                                                 ***
!**                                                                 ***
!**                                                                 ***
!**                                                                 ***
!**        LOCAL PARAMETERS :                                       ***
!**        ----------------                                         ***
!**                                                                 ***
      integer ::  ia1buf  
      integer ::  lbuf
      integer    istart,highrow,i,idoku,ii,iibuf,il,iu,irow,j,jj,k,lout
      integer    iar1,ihelp,indc,lvt,lvtbuf,infoH(2),no2pct,cntpct
      integer    itix,ixcol,ixcolt,ixbuf,ixtemp,maxix,norows,norowsold
      integer    ilowbuf,irowbuf,endgbuf,ptrmat,ptrlu
      integer    ixprof(l),maxcol(l),bufperm(l),ixL(bwup+1),ixcnt
      integer, parameter ::  ifive = 5, ififty = 50
      integer,allocatable ::  infbuf(:,:)
      logical    lorigel,lwarn
      real              rhelp
      double precision  ldrop,thrshld2,pivot_el,xtemp,Ltemp,eps
      double precision,allocatable ::  gbuf(:)
      double precision, parameter ::  zero = 0.0, one=1.0
!
!**                                                                 ***
!**   LIST OF LOCAL PARAMETERS :                                    ***
!**   -------------------------                                     ***
!
!--------I------I-----I------------------------------------------------
! NAME   I TYPE I I/O I     MEANING
!--------I------I-----I------------------------------------------------
!--------I------I-----I------------------------------------------------
! gbuf   I  DP  I --- I  is a one-dimensional array with the actual
!        I      I     I  unpacked matrix rows.        array: gbuf(lbuf)
!--------I------I-----I------------------------------------------------
! infbuf I  I   I --- I  is a 2-dimensional information array for gbuf.
!        I      I     I  array: infbuf(ia1buf,ia2buf)
!        I      I     I  infbuf(...,1) - pointer to gbuf
!        I      I     I  infbuf(...,2) - global row number
!        I      I     I  infbuf(...,3) - length of vector in gbuf (lvt)
!        I      I     I  infbuf(1,...) is actual row number, further
!        I      I     I  row numbers are increasingly sorted.
!--------I------I-----I------------------------------------------------
! ixprof I  I   I --- I  index number of profile of high indices per
!        I      I     I  row.                          array: ixprof(l)
!--------I------I-----I------------------------------------------------
! maxcol I  I   I --- I  maximal row index per column.
!        I      I     I                                array: maxcol(l)
!--------I------I-----I------------------------------------------------
! irow   I  I   I --- I  actual row number.
!--------I------I-----I------------------------------------------------
! norows I  I   I --- I  number of actual rows in <gbuf>.
!--------I------I-----I------------------------------------------------
! endgbufI  I   I --- I  pointer to the last active element in <gbuf>.
!--------I------I-----I------------------------------------------------
! ptrmat I      I --- I  pointer to the last element of <mat> already
!        I      I     I  loaded to <gbuf>.
!--------I------I-----I------------------------------------------------
! ptrlu  I      I --- I  pointer to the last element of the restored 
!        I      I     I  vector terms belonging to the eliminated
!        I      I     I  matrix in <mat>.
!--------I------I-----I------------------------------------------------
! ilowbufI      I --- I  row number of row not yet stored in buffer  
!        I      I     I  (ilowbuf >= irow).
!--------I------I-----I------------------------------------------------
!


!
!
!**** START OF CALCULATION :
!     ---------------------
!
      ierr     = 0
      lout     = ilin(12)
      idoku    = ilin(13)
      thrshld2 = rlin(3)
      ldrop    = rlin(6)
!     eps      = LL8EPS(one * 1.d-10)
      eps      = TINY(one)

      opcount  = 0

      ia1buf = bwlow + bwup + 1
      lbuf   = ia1buf*(bwlow+2*bwup)

      allocate(gbuf(lbuf),infbuf(ia1buf,ia2buf),stat=ierr)
      if (ierr /= 0) then
	write(lout,1000)
	write(lout,1003) lbuf*ireal+ia1buf*ia2buf*iint
	write(lout,1020)
	goto 999
      endif

      do i=1,l
	colperm(i) = i
      enddo

      if (thrshld2 == one) bufperm = colperm

      norows = 0
      infbuf = 0
      endgbuf = 0

      il = 0
      iu = 0
      ptrlu = 0
      highrow = 0

      if (idoku /= 0) then
        no2pct = NINT(REAL(l)/REAL(ififty))
        if (no2pct == 0) no2pct = 1
        cntpct = 0
        write(lout,1033) 
        write(lout,'(A2)',ADVANCE="NO") "  "
      endif

!
!**** Computation of high index numbers per row
!
      ixprof(1) = index(infoL(1,1)+infoL(1,2))
      do i=2,l
	ixprof(i) = max(index(infoL(i,1)+infoL(i,2)),ixprof(i-1))
      enddo

!
!**** Computation of maximal row index per column
!
      maxcol = 0
      do i=1,nvt
        lvt  = infoL(i,2)
	iar1 = infoL(i,3) + 1
        indc = infoL(i,1)
        do k=1,lvt
	  maxcol(index(indc+k)) = MAX(iar1,maxcol(index(indc+k)))
        enddo
      enddo


!
!**** Elimination process
!

      do irow=1,l     ! for all rows

!
!**** Load pivot row into buffer (i.e. infbuf(1,..)), if norows == 0
!
	if (highrow < irow) then
	  irowbuf = 1
          call LOADROW(mat,lmat,index,lindex,infoL,ia1,gbuf,lbuf,infbuf,
     *                 ia1buf,colperm,ixprof,l,endgbuf,irow,irow,
     *                 irowbuf,norows,ilin,ierr)
          highrow = highrow + 1
	  if (ierr /= 0) goto 999
	endif

!
!**** Search maximal element in pivot row and exchange it with the first
!**** element in the row. Make entry in <colperm>.
!**** <ixcol> is global column index.
!**** <ixbuf> is local index of maximal element.
!
        pivot_el = gbuf(infbuf(1,1)+1)
	ixbuf = 1
        do i=2,infbuf(1,3)
	  if (ABS(gbuf(infbuf(1,1)+i)) > ABS(pivot_el)) then
	    pivot_el = gbuf(infbuf(1,1)+i)
	    ixbuf = i
	  endif
	enddo
	if (ixbuf /= 1) then
	  ixcolt = irow + ixbuf - 1
	  ixcol = colperm(ixcolt)
	  gbuf(infbuf(1,1)+ixbuf) = gbuf(infbuf(1,1)+1)
	  gbuf(infbuf(1,1)+1) = pivot_el
	else
	  ixcol = colperm(irow)
	endif

        if (ABS(pivot_el) < eps) then
          if (thrshld2 == one) then
	    write(lout,1030)
	    write(lout,1008) pivot_el,eps,irow
	    write(lout,1020)
	    lwarn = .true.
	  else
	    write(lout,1000)
	    write(lout,1007) pivot_el,eps,irow
	    write(lout,1020)
	    ierr = 10000
	    goto 999
          endif
        else
	  lwarn = .false.
        endif

!
!**** Update buffer <gbuf> and information array <infbuf>
!
        istart = highrow + 1
	do i=istart,maxcol(ixcol)
	  irowbuf = norows+1
          if (irowbuf > ia1buf) then
	    print*,' ERROR 2 in ELIMINATION'
	    ierr = irowbuf
	    goto 999
	  endif

          call LOADROW(mat,lmat,index,lindex,infoL,ia1,gbuf,lbuf,
     *                 infbuf,ia1buf,colperm,ixprof,l,endgbuf,
     *                 irow,i,irowbuf,norows,ilin,ierr)
          highrow = highrow + 1
	  if (ierr /= 0) goto 999
	enddo

!
!**** Exchange elements in buffer <gbuf>, i.e. exchange of elements in
!**** column with pivot element by elements of pivot column.
!
        do i=2,norows
	  if (ixbuf /= 1) then
	    xtemp = gbuf(infbuf(i,1)+ixbuf)
	    gbuf(infbuf(i,1)+ixbuf) = gbuf(infbuf(i,1)+1)
	    gbuf(infbuf(i,1)+1) = xtemp
	  endif
	enddo

!
!**** Update <colperm>
!

	if (ixbuf /= 1) then
	  ixcolt = irow + ixbuf - 1
	  ixtemp = colperm(ixcolt)
	  colperm(ixcolt) = colperm(irow)
	  colperm(irow) = ixtemp
	endif

!
!**** Update <bufperm>: pivot column indices are set to zero and
!****                   column indices < pivot index being accessed by
!****                   colperm are set to the new higher column index!
!

        if (thrshld2 == one) then
	  ixcolt = irow + ixbuf - 1
	  ixtemp = bufperm(ixcolt)
	  if (bufperm(irow) > 0) bufperm(irow) = ixtemp
	  if (colperm(ixcolt) < irow) bufperm(colperm(ixcolt)) = ixcolt
	  bufperm(colperm(irow)) = 0
	endif

!
!**** Kernel of the Gauss algorithm
!
        if (thrshld2 == one) then
	  ptrmat = infoL(irow,1)
	  maxix  = bufperm(index(ptrmat+1))
	  do i=2,infoL(irow,2)
	    if (bufperm(index(ptrmat+i)) > maxix) 
     *         maxix = bufperm(index(ptrmat+i))
	  enddo
	  if (lwarn) gbuf(infbuf(1,1)+1) = one
!
!**** Compute the elimination coefficients in row <irow>
!
	  Ltemp = one/gbuf(infbuf(1,1)+1) 
          include "norec.h"
          do i=2,infbuf(1,3)
	    gbuf(infbuf(1,1)+i) = gbuf(infbuf(1,1)+i) * Ltemp
	  enddo
          opcount = opcount + infbuf(1,3)
!
!**** Update the matrix stored in the buffer by applying elim. coeff.
!
          do j=2,norows        ! for all rows minus pivot row in buffer
	    if (ABS(gbuf(infbuf(j,1)+1)) > ldrop) then
              opcount = opcount + 2*infoL(infbuf(j,2),2)
              include "norec.h"
	      do i=1,infoL(infbuf(j,2),2) ! for all indices of row infbuf(j,2)
		itix = bufperm(index(infoL(infbuf(j,2),1)+i)) !permuted indices
		if (itix > irow .and. itix <= maxix) then
	          gbuf(infbuf(j,1)+itix-irow+1) =
     *               gbuf(infbuf(j,1)+itix-irow+1) -
     *               gbuf(infbuf(1,1)+itix-irow+1) * gbuf(infbuf(j,1)+1)
!                     elimination coeeficients    * elements of pivot column
	        endif
	      enddo          
	    endif
          enddo
	else
	  ptrmat = infoL(highrow,1) + infoL(highrow,2)
!
!**** Compute the elimination coefficients in row <irow>
!
          ixcnt = 0
	  Ltemp = one/gbuf(infbuf(1,1)+1) 
          include "norec.h"
          do i=2,infbuf(1,3)
	    gbuf(infbuf(1,1)+i) = gbuf(infbuf(1,1)+i) * Ltemp
	    if (ABS(gbuf(infbuf(1,1)+i)) > ldrop) then
	      ixcnt = ixcnt + 1
	      ixL(ixcnt) = i
	    endif
	  enddo
          opcount = opcount + infbuf(1,3)
!
!**** Update the matrix stored in the buffer by applying elim. coeff.
!
          do j=2,norows        ! for all rows minus pivot row in buffer
	    if (ABS(gbuf(infbuf(j,1)+1)) > ldrop) then
              opcount = opcount + 2*ixcnt
              include "norec.h"
	      do i=1,ixcnt     ! for all indices stored in ixL
	        gbuf(infbuf(j,1)+ixL(i)) = gbuf(infbuf(j,1)+ixL(i)) - 
     *                      gbuf(infbuf(1,1)+ixL(i))*gbuf(infbuf(j,1)+1)
	      enddo            ! Elim.-coeff.     * elem. of pivot col.
	    endif
          enddo
        endif

!
!**** Update of the preconditioning matrix
!
!**** Matrix U is being stored columnwise. Indices are increasingly sorted.
!**** Matrix L is being stored rowwise. Indices are not increasingly sorted.
!
        iu = iu + 1
	lvtbuf = 0
	infoU(iu,1) = ptrlu
	infoU(iu,3) = irow - 1
	if (thrshld2 == one) then
	  do ii=1,norows
! <infoL> must be sorted ascendingly due to row numbers;
! Vector terms of matrix <mat> must be sorted ascendingly regarding to
!    pointers in <infoL>;
! Vector terms of indices in <index> must be sorted ascendingly regarding to
!    pointers in <infoL>;
            infoH(1) = infbuf(ii,2)    ! global row number
	    do i=1,infoL(infoH(1),2) 
	      lorigel = .false.
	      if (index(infoL(infoH(1),1)+i) < colperm(irow)) then
		cycle
	      else
		if (index(infoL(infoH(1),1)+i) == colperm(irow))
     &             lorigel = .true.
		exit
	      endif
	    enddo
	    if (lorigel .and. (ABS(gbuf(infbuf(ii,1)+1)) > zero)) then
	      lvtbuf = lvtbuf + 1
	      mat(infoU(iu,1)+lvtbuf) = gbuf(infbuf(ii,1)+1)
	      index(infoU(iu,1)+lvtbuf) = infbuf(ii,2)
	    endif
	  enddo
	else
	  mat(infoU(iu,1)+1) = gbuf(infbuf(1,1)+1)
	  index(infoU(iu,1)+1) = infbuf(1,2)
	  lvtbuf = 1
	  do ii=2,norows
	    if (ABS(gbuf(infbuf(ii,1)+1)) > thrshld2) then
	      lvtbuf = lvtbuf + 1
	      mat(infoU(iu,1)+lvtbuf) = gbuf(infbuf(ii,1)+1)
	      index(infoU(iu,1)+lvtbuf) = infbuf(ii,2)
	    endif
	  enddo
	endif
	infoU(iu,2) = lvtbuf
	ptrlu = ptrlu + lvtbuf
	if (ptrlu > ptrmat) then
	  write(lout,1000)
	  write(lout,1001) irow,1
	  write(lout,1020)
	  ierr = 10001
	  goto 999
	endif

        il = il + 1
	lvtbuf = 0
	if (thrshld2 == one) then
	  infoH(1) = infoL(irow,2)
	  infoH(2) = infoL(irow,1)
	endif
	infoL(il,1) = ptrlu
	infoL(il,3) = irow - 1
	if (thrshld2 == one) then
	  do ii=1,infoH(1)     ! Main diagonal element is not stored in L
	    if (bufperm(index(infoH(2)+ii)) > irow) then
	      Ltemp =
     &           gbuf(infbuf(1,1)+bufperm(index(infoH(2)+ii))-irow+1)
	      if (ABS(Ltemp) > zero) then
	        lvtbuf = lvtbuf + 1
	        mat(infoL(il,1)+lvtbuf) = Ltemp
	        index(infoL(il,1)+lvtbuf) = index(infoH(2)+ii)
	      endif
	    endif
	  enddo
	else
	  do ii=2,infbuf(1,3)  ! Main diagonal element is not stored in L
	    if (ABS(gbuf(infbuf(1,1)+ii)) > thrshld2) then
	      lvtbuf = lvtbuf + 1
	      mat(infoL(il,1)+lvtbuf) = gbuf(infbuf(1,1)+ii)
	      index(infoL(il,1)+lvtbuf) = colperm(irow+ii-1)
	    endif
	  enddo
	endif
	infoL(il,2) = lvtbuf
	ptrlu = ptrlu + lvtbuf
	if (ptrlu > ptrmat) then
	  write(lout,1000)
	  write(lout,1002) irow,1
	  write(lout,1020)
	  ierr = 10001
	  goto 999
	endif

        do j=1,ia2buf
          include "norec.h"
          do i=2,norows
            infbuf(i-1,j) = infbuf(i,j)
          enddo
        enddo
        norows = norows - 1

        include "norec.h"
        do i=1,norows
          infbuf(i,1) = infbuf(i,1) + 1
          infbuf(i,3) = infbuf(i,3) - 1
        enddo

	if ((idoku /= 0) .and. (MOD(irow-1,no2pct) == 0)) then
	  cntpct = cntpct + 1
	  if (MOD(cntpct,ifive) == 0) then
	    if (cntpct == ififty) then
	      write(lout,1032,ADVANCE="NO") cntpct*2
	      no2pct = l
	    else
	      write(lout,1031,ADVANCE="NO") cntpct*2
	    endif
	  else
	    write(lout,'(A1)',ADVANCE="NO") ">"
	  endif
	endif

      enddo  ! irow=1,l


      ilin(37) = ptrlu
      write(lout,1035)
      deallocate(gbuf,infbuf)

  999 continue
	    
 1000 format(//' *****************************************************'/
     &       ' ***  ERROR occured in ELIMINATIONX:               ***')      
 1001 format(' ***    preconditioning matrix is too small;       ***'/
     &       ' ***    matrix overflow when writing the           ***'/
     &       ' ***   ',i6,'th row of U on processor ',i6,'!      ***')
 1002 format(' ***    preconditioning matrix is too small;       ***'/
     &       ' ***    matrix overflow when writing the           ***'/
     &       ' ***   ',i6,'th row of L on processor ',i6,'!      ***')
 1003 format(' ***    allocation of elimination buffer           ***'/
     &       ' ***    (gbuf,infbuf) failed because of a too high ***'/
     &       ' ***    memory request of ',i10,' Bytes!        ***')
 1007 format(' ***    the pivot element (',g12.4,') is        ***'/
     &       ' ***    smaller than TINY(1.D0) = ',g12.4,'     ***'/
     &       ' ***    in row ',i9,'!                          ***')
 1008 format(' ***    the pivot element (',g12.4,') is        ***'/
     &       ' ***    smaller than TINY(1.D0) = ',g12.4,'     ***'/
     &       ' ***    in row ',i9,'! We set it to 1.D0!!!     ***')
 1010 format(' ***    the preconditioning matrix is too          ***'/
     &       ' ***    small on processor ',i6,' because of       ***'/
     &       ' ***    FILL IN! Inrease THRESHOLD values or       ***'/
     &       ' ***    increase LINSOL_GAUSS_FACTOR!              ***')
 1020 format(' *****************************************************')
 1030 format(//' *****************************************************'/
     &       ' ***  WARNING occured in ELIMINATIONX:             ***')      
!1031 format(/'  Allocating storage of size (in Bytes) ',i11,/
!    &       '  for the computation window  .....'/)
!1032 format(/'  Deallocating storage for the computation ',/
!    &       '  window  .....'/)
 1031 format(i2)
 1032 format(i3)
 1033 format(/'  Actual progress (char \">\" are 2 percent of',
     &       ' total work):')
 1035 format(/)
!
!**** END OF CALCULATION
!     ------------------
!
      r e t u r n
!-----END OF ELIMINATIONX-----------------------------------------------     
      e    n    d
