!:::::      ,,,,,ELIMINATP.....
!
!
!
!**********************************************************************
!        1         2         3         4         5         6         7*
!**********************************************************************
!**                                                                 ***
!**                                                                 ***
      subroutine ELIMINATP(mat,lmat,index,lindex,l,lglobal,infoL,nvt,
     #                     ia1,lsym,colperm,infoU,bwlow,bwup,gbwlow,
     #                     gbwup,rlin,ilin,opcount,lmatbk,
     #                     myproc,nproc,tids,nmsg,ierr)
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**      E L I M I N A T P   eliminates a 'packed row' matrix       ***
!**                          described by the triple                *** 
!**                          (mat,index,info) within a buffer       ***
!**                          described by (gbuf,infbuf) into the    ***
!**                          same matrix described by the triple    ***
!**                          (mat,index,infoL) for the matrix L     ***
!**                          and (mat,index,infoU) for the          ***
!**                          matrix U on many processors.           ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**      Copyright (C) 1999 University Karlsruhe                    ***
!**      PROGRAMMER:    H. Haefner                                  ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
      USE constants

      implicit none
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**      FORMAL PARAMETERS :                                        ***
!**      -----------------                                          ***
!**                                                                 ***
      integer,intent(in)    ::  lmat,lindex,l,lglobal,ia1,nvt
      integer,intent(in)    ::  nproc,myproc
      integer,intent(in)    ::  bwlow,bwup,gbwlow,gbwup
      integer,intent(inout) ::  nmsg
      integer,intent(out)   ::  ierr
      integer,intent(in)    ::  tids(nproc)
      integer,intent(in)    ::  lmatbk(nproc)
      integer,intent(inout) ::  ilin(nilin),index(lindex)
      integer,intent(inout) ::  infoL(ia1,ia2LU)
      integer,intent(out)   ::  infoU(ia1,ia2LU)
      integer,intent(out)   ::  colperm(lglobal)
      integer*8,intent(out) ::  opcount
      double precision,intent(in) ::  rlin(nrlin)
      double precision,intent(inout) ::  mat(lmat)
      logical,intent(in) :: lsym
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**   LIST OF FORMAL PARAMETERS :                                   ***
!**   -------------------------                                     ***
!**                                                                 ***
!**   see Manual-Pages <lsolp>                                      ***
!**                                                                 ***
!**                                                                 ***
!**                                                                 ***
!**                                                                 ***
!**        LOCAL PARAMETERS :                                       ***
!**        ----------------                                         ***
!**                                                                 ***
      integer ::  ia1buf  
      integer ::  lbuf
      integer    i,idoku,ii,iibuf,il,iu,irow,j,jj,k,lmyblk,lout
      integer    iar1,ier,iermax,ihelp,indc,lvt,lvtbuf,mids,midr,lpivnr
      integer    highrow,ixhlp,maxix,no2pct,cntpct,cntirow
      integer    ilast,ixcol,ixcolt,ixbuf,ixtemp,jbuf,norows,norowsold
      integer    ilowbuf,irowbuf,endgbuf,ptrmat,ptrmath,ptrlu,infoH(2)
      integer    ixprof(l),maxcol(l),bufperm(lglobal)
      integer, parameter ::  iten = 10
      integer,allocatable ::  infbuf(:,:)
      real              rhelp
      logical           root,lorigel
      double precision  ldrop,thrshld2,pivot_el,xtemp,Ltemp,eps
      double precision,allocatable ::  gbuf(:)
      double precision  rcvbuf(gbwlow+2*gbwup)
      double precision, parameter ::  zero = 0.0, one = 1.0
!     real*8,parameter ::  inf_plus  = Z'7FF0000000000000'
!     real*8,parameter ::  inf_minus = Z'FFF0000000000000'
!
!**                                                                 ***
!**   LIST OF LOCAL PARAMETERS :                                    ***
!**   -------------------------                                     ***
!
!--------I------I-----I------------------------------------------------
! NAME   I TYPE I I/O I     MEANING
!--------I------I-----I------------------------------------------------
!--------I------I-----I------------------------------------------------
! gbuf   I  DP  I --- I  is a one-dimensional array with the actual
!        I      I     I  unpacked matrix rows.        array: gbuf(lbuf)
!--------I------I-----I------------------------------------------------
! infbuf I  I   I --- I  is a 2-dimensional information array for gbuf.
!        I      I     I  array: infbuf(ia1buf,ia2buf)
!        I      I     I  infbuf(...,1) - pointer to gbuf
!        I      I     I  infbuf(...,2) - global row number
!        I      I     I  infbuf(...,3) - length of vector in gbuf (lvt)
!        I      I     I  infbuf(1,...) is actual row number, further
!        I      I     I  row numbers are increasingly sorted.
!--------I------I-----I------------------------------------------------
! ixprof I  I   I --- I  index number of profile of high indices per
!        I      I     I  row.                          array: ixprof(l)
!--------I------I-----I------------------------------------------------
! maxcol I  I   I --- I  maximal row index per column.
!        I      I     I                                array: maxcol(l)
!--------I------I-----I------------------------------------------------
! irow   I  I   I --- I  actual row number.
!--------I------I-----I------------------------------------------------
! norows I  I   I --- I  number of actual rows in <gbuf>.
!--------I------I-----I------------------------------------------------
! endgbufI  I   I --- I  pointer to the last active element in <gbuf>.
!--------I------I-----I------------------------------------------------
! ptrmat I      I --- I  pointer to the last element of <mat> already
!        I      I     I  loaded to <gbuf>.
!--------I------I-----I------------------------------------------------
! ptrlu  I      I --- I  pointer to the last element of the restored 
!        I      I     I  vector terms belonging to the eliminated
!        I      I     I  matrix in <mat>.
!--------I------I-----I------------------------------------------------
! ilowbufI      I --- I  row number of row not yet stored in buffer  
!        I      I     I  (ilowbuf >= irow).
!--------I------I-----I------------------------------------------------
!


!
!
!**** START OF CALCULATION :
!     ---------------------
!
!**** <infoL> must be sorted ascendingly regarding to row numbers
!
      ierr     = 0
      lout     = ilin(12)
      idoku    = ilin(13)
      thrshld2 = rlin(3)
      ldrop    = rlin(6)
      lmyblk   = lmatbk(myproc)
!     eps      = LL8EPS(one * 1.d-10)
      eps      = TINY(one)

      opcount = 0

      ia1buf = (gbwlow + gbwup)/nproc + 1
      lbuf   = ia1buf*(bwlow+2*bwup)

      allocate(gbuf(lbuf),infbuf(ia1buf,ia2buf),stat=ierr)
      if (ierr /= 0) ierr = lbuf*ireal + ia1buf*ia2buf*iint
      call LL4INM(1,1,1,ierr,iermax,ihelp,myproc,nproc,tids,nmsg)
      if (iermax > 0) then
        ierr = iermax
        if (myproc == 1) then
          write(lout,1000)
          write(lout,1003) ierr
          write(lout,1020)
        endif
        goto 999
      endif

      norows = 0
      infbuf = 0
      endgbuf = 0

      il = 0
      iu = 0
      ptrmat  = 0
      ptrlu   = 0
      highrow = myproc - nproc

      if ((idoku > 0) .and. (myproc == 1)) then 
        no2pct  = FLOOR(REAL(lglobal)/REAL(iten))
        if (no2pct == 0) no2pct = 1
        cntirow = no2pct
        cntpct  = 0
        write(lout,1033)
!       write(lout,'(A2)',ADVANCE="NO") "  "
      endif
      if (idoku < 0) then 
        no2pct  = FLOOR(REAL(lglobal)/REAL(iten))
        if (no2pct == 0) no2pct = 1
        cntirow = no2pct
        cntpct  = 0
        write(lout,1033)
!       write(lout,'(A2)',ADVANCE="NO") "  "
      endif

!
!**** Computation of high index numbers per row
!
      if (myproc == 1) then
	do j=2,nproc
	  call MPRCVA(tids(j),nmsg+j,lmatbk(j)*iint,ixprof,midr,ierr)
	  call MPRCVW(tids(j),nmsg+j,lmatbk(j)*iint,ixprof,midr,ierr)
	  do i=1,lmatbk(j)
	    bufperm((i-1)*nproc+j) = ixprof(i)
	  enddo
	enddo
	do i=1,lmyblk
	  bufperm((i-1)*nproc+1) = index(infoL(i,1)+infoL(i,2))
	enddo
        do i=2,lglobal
	  bufperm(i) = MAX(bufperm(i),bufperm(i-1))
        enddo
	do j=2,nproc
	  do i=1,lmatbk(j)
	    ixprof(i) = bufperm((i-1)*nproc+j) 
	  enddo
	  call MPSNDA(tids(j),nmsg+j,lmatbk(j)*iint,ixprof,mids,ierr)
	  call MPSNDW(tids(j),nmsg+j,lmatbk(j)*iint,ixprof,mids,ierr)
	enddo
	do i=1,lmyblk
	  ixprof(i) = bufperm((i-1)*nproc+1)
	enddo
      else
	do i=1,lmyblk
	  ixprof(i) = index(infoL(i,1)+infoL(i,2))
	enddo
	call MPSNDA(tids(1),nmsg+myproc,lmyblk*iint,ixprof,mids,ierr)
	call MPSNDW(tids(1),nmsg+myproc,lmyblk*iint,ixprof,mids,ierr)
	call MPRCVA(tids(1),nmsg+myproc,lmyblk*iint,ixprof,midr,ierr)
	call MPRCVW(tids(1),nmsg+myproc,lmyblk*iint,ixprof,midr,ierr)
      endif
      nmsg = nmsg + nproc

!
!**** Computation of maximal row index per column
!**** (Assertion: the rows must be sorted ascendingly in Array <info>!)
!
      bufperm = 0

      do i=1,nvt
        lvt  = infoL(i,2)
	iar1 = infoL(i,3) + 1
        indc = infoL(i,1)
        do k=1,lvt
	  bufperm(index(indc+k)) = iar1
        enddo
      enddo
      if (myproc == 1) then
	do j=2,nproc
	  call MPRCVA(tids(j),nmsg+j,lglobal*iint,colperm,midr,ierr)
	  call MPRCVW(tids(j),nmsg+j,lglobal*iint,colperm,midr,ierr)
	  do i=1,lglobal
	    bufperm(i) = MAX(bufperm(i),colperm(i))
	  enddo
	enddo
	do j=2,nproc
	  do i=1,lmatbk(j)
	    maxcol(i) = bufperm((i-1)*nproc+j) 
	  enddo
	  call MPSNDA(tids(j),nmsg+j,lmatbk(j)*iint,maxcol,mids,ierr)
	  call MPSNDW(tids(j),nmsg+j,lmatbk(j)*iint,maxcol,mids,ierr)
	enddo
	do i=1,lmyblk
	  maxcol(i) = bufperm((i-1)*nproc+1)
	enddo
      else
	call MPSNDA(tids(1),nmsg+myproc,lglobal*iint,bufperm,mids,ierr)
	call MPSNDW(tids(1),nmsg+myproc,lglobal*iint,bufperm,mids,ierr)
	call MPRCVA(tids(1),nmsg+myproc,lmyblk*iint,maxcol,midr,ierr)
	call MPRCVW(tids(1),nmsg+myproc,lmyblk*iint,maxcol,midr,ierr)
      endif
      nmsg = nmsg + nproc

      do i=1,lglobal
	colperm(i) = i
      enddo

      if (thrshld2 == one) bufperm = colperm


!
!**** Elimination process
!

      do irow=1,lglobal     ! for all rows

       lpivnr = MOD(irow-1,nproc) + 1
       if (myproc == lpivnr) then

!
!**** If pivot row in buffer then copy infbuf(2,..) into infbuf(1,..)
!****                        else LOAD matrix row into buffer
!
	if (highrow < irow) then
	  irowbuf = 1
          call LOADROP(mat,lmat,index,lindex,infoL,ia1,gbuf,lbuf,infbuf,
     *               ia1buf,colperm,lglobal,ixprof,l,endgbuf,irow,irow,     
     *               irowbuf,norows,ilin,myproc,nproc,ierr)
	  highrow = highrow + nproc
	endif

!
!**** Search maximal element in pivot row and exchange it with the first
!**** element in the row. Make entry in <colperm>.
!**** <ixcol> is global column index.
!**** <ixbuf> is local index of maximal element.
!
        pivot_el = gbuf(infbuf(1,1)+1)
	ixbuf = 1
        do i=2,infbuf(1,3)
	  if (ABS(gbuf(infbuf(1,1)+i)) > ABS(pivot_el)) then
	    pivot_el = gbuf(infbuf(1,1)+i)
	    ixbuf = i
	  endif
	enddo

	if (ABS(pivot_el) < eps) then
	  ierr = maxnpe + myproc
	endif

	root = .true.
	call MPSCAT(ixbuf,iint,root,myproc,nproc,tids,nmsg,ier)
	if (ixbuf /= 1) then
	  ixcolt = irow + ixbuf - 1
	  ixcol = colperm(ixcolt)
	  gbuf(infbuf(1,1)+ixbuf) = gbuf(infbuf(1,1)+1)
	  gbuf(infbuf(1,1)+1) = pivot_el
	else
	  ixcol = colperm(irow)
	endif
       else
	root = .false.
	call MPSCAT(ixbuf,iint,root,myproc,nproc,tids,nmsg,ier)
	if (ixbuf /= 1) then
	  ixcolt = irow + ixbuf - 1
	  ixcol = colperm(ixcolt)
	else
	  ixcol = colperm(irow)
	endif
       endif

       call LL4INM(1,1,1,ierr,iermax,ihelp,myproc,nproc,tids,nmsg)
       if (iermax > 0) then
         ierr = iermax
         if (ierr <= maxnpe .and. myproc == 1) then
           write(lout,1000)
           write(lout,1004) ierr
           write(lout,1020)
         endif
         if (ierr > maxnpe .and. ierr <= 100000) then
	   if (lpivnr /= 1 .and. lpivnr == myproc) then
             call MPSNDA(tids(1),nmsg+1,ireal,pivot_el,mids,ierr)
	     call MPSNDW(tids(1),nmsg+1,ireal,pivot_el,mids,ierr)
	     nmsg = nmsg + 1
	   endif
	   if (lpivnr /= 1 .and. myproc == 1) then
             call MPRCVA(-1,nmsg+1,ireal,pivot_el,midr,ierr)
	     call MPRCVW(-1,nmsg+1,ireal,pivot_el,midr,ierr)
	     nmsg = nmsg + 1
	   endif
	   if (myproc == 1) then
             write(lout,1000)
             write(lout,1007) pivot_el,eps,irow,ierr-maxnpe
             write(lout,1020)
           endif
         endif
         if (ierr > 100000 .and. myproc == 1) then
           ier = mod(ierr,100000)
           write(lout,1000)
           write(lout,1010) ier
           write(lout,1020)
         endif
!        if (ierr > 10000000 .and. ierr <= 50000000 .and.
!    &       myproc == 1) then
!          ier = mod(ierr,10000000)
!          write(lout,1000)
!          write(lout,1006) irow-1,ier
!          write(lout,1020)
!        endif
!        if (ierr > 50000000 .and. myproc == 1) then
!          ier = mod(ierr,50000000)
!          write(lout,1000)
!          write(lout,1005) irow-1,ier
!          write(lout,1020)
!        endif
         goto 999
       endif

       if ((MOD(ixcol-1,nproc)+1) == myproc) then
         ixhlp = INT((ixcol-1)/nproc)+1
         maxix = maxcol(ixhlp)
         root = .true.
         call MPSCAT(maxix,iint,root,myproc,nproc,tids,nmsg,ierr)
       else
         root = .false.
         call MPSCAT(maxix,iint,root,myproc,nproc,tids,nmsg,ierr)
       endif


       do i=highrow+nproc,maxix,nproc
	 irowbuf = norows + 1
         if (irowbuf > ia1buf) then
	   print*,' ERROR 2 in ELIMINATION'
	   ierr = irowbuf
	   goto 999
	 endif

         call LOADROP(mat,lmat,index,lindex,infoL,ia1,gbuf,lbuf,
     *                infbuf,ia1buf,colperm,lglobal,ixprof,l,
     *                endgbuf,irow,i,irowbuf,norows,ilin,myproc,
     *                nproc,ierr)
	 highrow = highrow + nproc
       enddo

!
!**** Exchange elements in buffer <gbuf>, i.e. exchange of elements in
!**** column with pivot element by elements of pivot column.
!
       if (myproc == lpivnr) then
         ihelp = 2
       else
         ihelp = 1
       endif
	  
       do i=ihelp,norows
         if (ixbuf /= 1) then
	   xtemp = gbuf(infbuf(i,1)+ixbuf)
	   gbuf(infbuf(i,1)+ixbuf) = gbuf(infbuf(i,1)+1)
	   gbuf(infbuf(i,1)+1) = xtemp
	 endif
       enddo

!
!**** Update <colperm>
!
       if (ixbuf /= 1) then
         ixcolt = irow + ixbuf -1
	 ixtemp = colperm(ixcolt)
	 colperm(ixcolt) = colperm(irow)
	 colperm(irow) = ixtemp
       endif

!
!**** Update <bufperm>: pivot column indices are set to zero and
!****                   column indices < pivot index being accessed by
!****                   colperm are set to the new higher column index!
!

       if (thrshld2 == one) then
         ixcolt = irow + ixbuf -1
         ixtemp = bufperm(ixcolt)
         if (bufperm(irow) > 0) bufperm(irow) = ixtemp
         if (colperm(ixcolt) < irow) bufperm(colperm(ixcolt)) = ixcolt
         bufperm(colperm(irow)) = 0
       endif

!
!**** Update <ptrmat>
!
       if (thrshld2 == one) then
         rhelp   = REAL(irow)/REAL(nproc)
         ilast   = CEILING(rhelp)
         ptrmath = infoL(ilast,1)
	 if (ptrmath > ptrmat) ptrmat = ptrmath
       else
         rhelp  = REAL(highrow)/REAL(nproc)
         ilast  = CEILING(rhelp)
         if (ilast > 0) then
           ptrmat = infoL(ilast,1) + infoL(ilast,2)
         else
           ptrmat = infoL(1,1)
         endif
       endif

!
!**** Now follows the kernel of the GAUSS-algorithm
!
!
!**** Compute the elimination coefficients in row <irow>
!
       if (myproc == lpivnr) then
         opcount = opcount + infbuf(1,3)
         Ltemp = one/gbuf(infbuf(1,1)+1)
         include "norec.h"
         do i=2,infbuf(1,3)
	  gbuf(infbuf(1,1)+i) = gbuf(infbuf(1,1)+i) * Ltemp
	 enddo
	 root = .true.
	 ihelp = infbuf(1,3)-1
	 call MPSCAT(ihelp,iint,root,myproc,nproc,tids,nmsg,ierr)
	 call MPSCAT(gbuf(infbuf(1,1)+2),ihelp*ireal,root,myproc,nproc,
     *               tids,nmsg,ierr)
!
!**** Update the matrix stored in the buffer by applying elim. coeff.
!
         do j=2,norows         ! for all rows minus pivot row in buffer
	  if (ABS(gbuf(infbuf(j,1)+1)) > ldrop) then
            opcount = opcount + 2*(infbuf(1,3)-1)
            include "norec.h"
	    do i=2,infbuf(1,3) ! for all elements minus pivot element
	      gbuf(infbuf(j,1)+i) = gbuf(infbuf(j,1)+i) - 
     *                           gbuf(infbuf(1,1)+i)*gbuf(infbuf(j,1)+1)       
	    enddo              ! Elim.-coeff.     * elem. of pivot col.
	  endif
         enddo
       else
	 root = .false.
	 call MPSCAT(ihelp,iint,root,myproc,nproc,tids,nmsg,ierr)
	 call MPSCAT(rcvbuf,ihelp*ireal,root,myproc,nproc,
     *               tids,nmsg,ierr)
!
!**** Update the matrix stored in the buffer by applying elim. coeff.
!
         do j=1,norows
	  if (ABS(gbuf(infbuf(j,1)+1)) > ldrop) then
            opcount = opcount + 2*ihelp
            include "norec.h"
	    do i=2,ihelp+1     ! for all elements minus pivot element
	      gbuf(infbuf(j,1)+i) = gbuf(infbuf(j,1)+i) - 
     *                           rcvbuf(i-1)*gbuf(infbuf(j,1)+1)       
	    enddo              ! Elim.-coeff.     * elem. of pivot col.
	  endif
         enddo
       endif

!
!**** Update of the preconditioning matrix
!
!**** Matrix U is being stored columnwise. Indices are increasingly sorted.
!**** Matrix L is being stored rowwise. Indices are not increasingly sorted.
!
        iu = iu + 1
	lvtbuf = 0
	infoU(iu,1) = ptrlu
	infoU(iu,3) = irow - 1
        if (thrshld2 == one) then
	  do ii=1,norows
            rhelp  = REAL(infbuf(ii,2))/REAL(nproc)   ! local row number
            ilast  = CEILING(rhelp)
            do i=1,infoL(ilast,2)
              lorigel = .false.
              if (index(infoL(ilast,1)+i) < colperm(irow)) then
                cycle
              else
                if (index(infoL(ilast,1)+i) == colperm(irow))
     &             lorigel = .true.
                exit
              endif
            enddo
            if (lorigel .and. (ABS(gbuf(infbuf(ii,1)+1)) > zero)) then
              lvtbuf = lvtbuf + 1
              mat(infoU(iu,1)+lvtbuf) = gbuf(infbuf(ii,1)+1)
              index(infoU(iu,1)+lvtbuf) = infbuf(ii,2)
            endif
	  enddo
        else
          if (myproc == lpivnr) then
            mat(infoU(iu,1)+1) = gbuf(infbuf(1,1)+1)
            index(infoU(iu,1)+1) = infbuf(1,2)
            lvtbuf = 1
            do ii=2,norows
              if (ABS(gbuf(infbuf(ii,1)+1)) > thrshld2) then
                lvtbuf = lvtbuf + 1
                mat(infoU(iu,1)+lvtbuf) = gbuf(infbuf(ii,1)+1)
                index(infoU(iu,1)+lvtbuf) = infbuf(ii,2)
!                   if (.not. (mat(infoU(iu,1)+lvtbuf) == 
!    &                         mat(infoU(iu,1)+lvtbuf))) then
!                     ierr = 10000000 + myproc
!                     goto 888
!                   endif
!                   if ((mat(infoU(iu,1)+lvtbuf) == inf_plus) .or.
!    &                  (mat(infoU(iu,1)+lvtbuf) == inf_minus)) then
!                     ierr = 50000000 + myproc
!                     goto 888
!                   endif
              endif
            enddo
          else
	    do ii=1,norows
	      if (ABS(gbuf(infbuf(ii,1)+1)) > thrshld2) then
	        lvtbuf = lvtbuf + 1
	        mat(infoU(iu,1)+lvtbuf) = gbuf(infbuf(ii,1)+1)
	        index(infoU(iu,1)+lvtbuf) = infbuf(ii,2)
	      endif
	    enddo
          endif
	endif
	infoU(iu,2) = lvtbuf
	ptrlu = ptrlu + lvtbuf
	if (ptrlu > ptrmat) then
	  write(lout,1000)
	  write(lout,1001) irow,myproc
	  write(lout,1020)
	  ierr = 100000 + myproc
	  goto 888
	endif

	if (myproc == lpivnr) then
          il = il + 1
	  lvtbuf = 0
          if (thrshld2 == one) then
            rhelp  = REAL(irow)/REAL(nproc)
            ilast  = CEILING(rhelp)
            infoH(1) = infoL(ilast,2)
            infoH(2) = infoL(ilast,1)
          endif
	  infoL(il,1) = ptrlu
	  infoL(il,3) = irow - 1
          if (thrshld2 == one) then
            do ii=1,infoH(1)     ! Main diagonal element is not stored in L
              if (bufperm(index(infoH(2)+ii)) > irow) then
                Ltemp = 
     &             gbuf(infbuf(1,1)+bufperm(index(infoH(2)+ii))-irow+1)
                if (ABS(Ltemp) > zero) then
                  lvtbuf = lvtbuf + 1
                  mat(infoL(il,1)+lvtbuf) = Ltemp
                  index(infoL(il,1)+lvtbuf) = index(infoH(2)+ii)
                endif
              endif
            enddo
          else
	    do ii=2,infbuf(1,3)  ! Main diagonal element is not stored in L
	      if (ABS(gbuf(infbuf(1,1)+ii)) > thrshld2) then
	        lvtbuf = lvtbuf + 1
	        mat(infoL(il,1)+lvtbuf) = gbuf(infbuf(1,1)+ii)
	        index(infoL(il,1)+lvtbuf) = colperm(irow+ii-1)
!               if (.not. (mat(infoU(iu,1)+lvtbuf) == 
!    &                     mat(infoU(iu,1)+lvtbuf))) then
!                 ierr = 10000000 + myproc
!                 goto 888
!               endif
!               if ((mat(infoU(iu,1)+lvtbuf) == inf_plus) .or.
!    &              (mat(infoU(iu,1)+lvtbuf) == inf_minus)) then
!                 ierr = 50000000 + myproc
!                 goto 888
!               endif
	      endif
	    enddo
	  endif
	  infoL(il,2) = lvtbuf
	  ptrlu = ptrlu + lvtbuf
	  if (ptrlu > ptrmat) then
	    write(lout,1000)
	    write(lout,1002) irow,myproc
	    write(lout,1020)
	    ierr = 100000 + myproc
	    goto 888
	  endif

	  do j=1,ia2buf
	    include "norec.h"
	    do i=2,norows
	      infbuf(i-1,j) = infbuf(i,j)
	    enddo
	  enddo
	  norows = norows - 1
	endif

	include "norec.h"
	do i=1,norows
	  infbuf(i,1) = infbuf(i,1) + 1
	  infbuf(i,3) = infbuf(i,3) - 1
	enddo

        if ((idoku > 0).and.(myproc == 1).and.(irow >= cntirow)) then
          cntirow = cntirow + no2pct
          cntpct  = cntpct + 1
	  if (cntpct < iten) then
            write(lout,1034) cntpct*iten 
	  else
            write(lout,1035) cntpct*iten 
          endif
        endif
        if ((idoku < 0) .and. (irow >= cntirow)) then
          cntirow = cntirow + no2pct
          cntpct  = cntpct + 1
	  if (cntpct < iten) then
            write(lout,1034) cntpct*iten 
	  else
            write(lout,1035) cntpct*iten 
          endif
        endif

  888   continue

      enddo  ! irow=1,lglobal


      ilin(37) = ptrlu
      deallocate(gbuf,infbuf)
  999 continue
	    
 1000 format(' *******************************************************'/
     &       ' ***  ERROR occured in ELIMINATP:                    ***')
 1001 format(' ***    preconditioning matrix is too small;         ***'/
     &       ' ***    matrix overflow when writing the             ***'/
     &       ' ***   ',i6,'th row of U on processor ',i6,'!        ***')
 1002 format(' ***    preconditioning matrix is too small;         ***'/
     &       ' ***    matrix overflow when writing the             ***'/
     &       ' ***   ',i6,'th row of L on processor ',i6,'!        ***')
 1003 format(' ***    allocation of elimination buffer             ***'/
     &       ' ***    (gbuf,infbuf) failed because of a too high   ***'/
     &       ' ***    memory request of (max.) ',i10,' Bytes!   ***')
 1004 format(' ***    the buffer used within the                   ***'/
     &       ' ***    elimination process is too small             ***'/
     &       ' ***    on processor ',i6,' !                        ***')
!1005 format(' ***    matrix entries of U or L get too large in    ***'/
!    &       ' ***    row ',i9,' on processor ',i6,' !          ***')
!1006 format(' ***    matrix entries of U or L are no arithmetic   ***'/
!    &       ' ***    numbers in row ',i9,' on proc. ',i6,' !   ***')
 1007 format(' ***    the pivot element (',g12.4,') is smaller  ***'/
     &       ' ***    than TINY(1.D0) = ',g12.4,' in global     ***'/
     &       ' ***    row ',i9,' on processor ',i6,' !          ***')
 1010 format(' ***    preconditioning matrix is too small          ***'/
     &       ' ***    on processor ',i6,' because of               ***'/
     &       ' ***    FILL IN! Inrease THRESHOLD values or         ***'/
     &       ' ***    increase LINSOL_GAUSS_FACTOR!                ***')
 1020 format(' *******************************************************')
 1031 format(i2)
 1032 format(i3)
 1033 format(/'  Actual progress of total work in percent:'/)
 1034 format(3x,i4,' percent of work done ...')
 1035 format(3x,i4,' percent of work done (:>)'/)
!
!**** END OF CALCULATION
!     ------------------
!
      r e t u r n
!-----END OF ELIMINATP--------------------------------------------------     
      e    n    d
