C**********************************************************************
C        1         2         3         4         5         6         7*
C**********************************************************************
C**                                                                 ***
C**                                                                 ***
      subroutine FILL1_BUF(lmat,lindex,l,mat,index,info,ia1,nvt,lmatbk,
     *                 ptrmbk,myproc,nproc,rowbuf,rowcnt,typcnt,colind)
C**                                                                 ***
C**                                                                 ***
C**********************************************************************
C**                                                                 ***
C**   F I L L 1 _ B U F  fills a buffer with row entries, if only   ***	
C**                      one entry per row occurs.                  ***
C**                                                                 ***
C**********************************************************************
C**                                                                 ***
C**    Copyright (C) 1998 University Karlsruhe                      ***
C**    PROGRAMMER: H. Haefner                                       ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
C**                                                                 ***
C**********************************************************************
C**                                                                 ***
C**     THE IMPLICIT NONE-INSTRUCTION HAS THE AIM,                  ***
C**     THAT ALL VARIABLES MUST BE DECLARED EXPLICITLY.             ***
C**                                                                 ***
      USE constants

      implicit none
C**                                                                 ***
C**-----------------------------------------------------------------***
C**                                                                 ***
C**      FORMAL PARAMETERS :                                        ***
C**      -----------------                                          ***
C**                                                                 ***
      integer,intent(in) ::  nproc,myproc
      integer,intent(in) ::  lmat,lindex,l,ia1,nvt
      integer,intent(in) ::  index(lindex),info(ia1,ia2),lmatbk(nproc)
      integer,intent(in) ::  ptrmbk(nproc+1),rowcnt(l)
      integer,intent(inout)::  typcnt(ntyp),colind(l)
      double precision,intent(in):: mat(lmat)
      double precision,intent(inout):: rowbuf(l)
C**                                                                 ***
C**-----------------------------------------------------------------***
C**                                                                 ***
C**   LIST OF FORMAL PARAMETERS :                                   ***
C**   -------------------------                                     ***
C**                                                                 ***
C--------I------I-----I------------------------------------------------
C NAME   I TYPE I I/O I     MEANING
C--------I------I-----I------------------------------------------------
C--------I------I-----I------------------------------------------------
C rowbuf I  DP  I i/o I  stores the entries with only one entry per row
C        I      I     I  (Dirichlet entries)
C--------I------I-----I------------------------------------------------
C rowcnt I  I   I in  I  shows the rows with Dirichlet entries
C--------I------I-----I------------------------------------------------
C typcnt I  I   I i/o I  says in which data types the Dirichlet
C        I      I     I  entries occur          Array: typcnt(ntyp)
C--------I------I-----I------------------------------------------------
C colind I  I   I i/o I  stores the column indices of Dirichlet entries
C--------I------I-----I------------------------------------------------
C**                                                                 ***
C**                                                                 ***
C**      LOCAL PARAMETERS :                                         ***
C**      -----------------                                          ***
C**                                                                 ***
      integer    adda,ivt,iac,iar,iarloc,iarbeg,iarend,indr,indc,ilocal
      integer    k,lvt,lmyblk,ptrmyb
      double precision zero
      parameter (zero = 0.)

C
C**** START OF CALCULATION :
C     ---------------------
C
      lmyblk = lmatbk(myproc)
      ptrmyb = ptrmbk(myproc)

      do i = 1,lmyblk
	if (rowcnt(i) == 1) then
          do ivt = 1,nvt
	    if (info(ivt,1) == 1) then
	      adda = info(ivt,2)
	      if (mat(adda+i) /= zero) then
		typcnt(1) = typcnt(1) + 1
		goto 888
              endif
	    endif

	    if (info(ivt,1) == 2) then
	      adda   = info(ivt,2)
	      lvt    = info(ivt,3)
	      iac    = info(ivt,4)
	      iarloc = info(ivt,5) - ptrmyb
	      iarend = iarloc+lvt
	      if (i >= iarloc+1 .and. i <= iarend) then
		ilocal = i-iarloc
	        if (mat(adda+ilocal) /= zero) then
		  typcnt(2) = typcnt(2) + 1
		  rowbuf(i)   = mat(adda+ilocal)
		  colind(i) = iac+ilocal
		  goto 888
                endif
              endif
	    endif

	    if (info(ivt,1) == 3) then
	      adda   = info(ivt,2)
	      lvt    = info(ivt,3)
	      iac    = info(ivt,4)
	      iarloc = info(ivt,5) - ptrmyb
	      indc   = info(ivt,6)
	      iarbeg = iarloc+index(indc+1)
	      iarend = iarloc+index(indc+lvt)
	      if (i >= iarbeg .and. i <= iarend) then
	        k = 1
	        do while (iarloc+index(indc+k) <= i)
	          if ((iarloc+index(indc+k) == i) .and. 
     *                (mat(adda+k) /= zero)) then
		    typcnt(3) = typcnt(3) + 1
		    rowbuf(i)   = mat(adda+k)
		    colind(i) = iac+index(indc+k)
		    goto 888
                  endif
	          k = k+1
	        enddo
	      endif
	    endif

	    if (info(ivt,1) == 4) then
	      adda   = info(ivt,2)
	      lvt    = info(ivt,3)
	      iarloc = info(ivt,5) - ptrmyb
	      iarend = iarloc+lvt
	      indc   = info(ivt,6)
	      if (i >= iarloc+1 .and. i <= iarend) then
		ilocal = i-iarloc
	        if (mat(adda+ilocal) /= zero) then
		  typcnt(4) = typcnt(4) + 1
		  rowbuf(i)   = mat(adda+ilocal)
		  colind(i) = index(indc+ilocal)
		  goto 888
                endif
              endif
	    endif

	    if (info(ivt,1) == 5) then
	      adda   = info(ivt,2)
	      lvt    = info(ivt,3)
	      iac    = info(ivt,4)
	      indr   = info(ivt,7)
	      do k = 1,lvt
	        if ((index(indr+k)-ptrmyb == i) .and. 
     *              (mat(adda+k) /= zero)) then
		  typcnt(5) = typcnt(5) + 1
		  rowbuf(i)   = mat(adda+k)
		  colind(i) = iac+k
		  goto 888
                endif
	      enddo
	    endif

	    if (info(ivt,1) == 6) then
	      adda   = info(ivt,2)
	      lvt    = info(ivt,3)
	      indc   = info(ivt,6)
	      indr   = info(ivt,7)
	      do k = 1,lvt
	        if ((index(indr+k)-ptrmyb == i) .and. 
     *              (mat(adda+k) /= zero)) then
		  typcnt(6) = typcnt(6) + 1
		  rowbuf(i)   = mat(adda+k)
		  colind(i) = index(indc+k)
		  goto 888
                endif
	      enddo
	    endif

	    if (info(ivt,1) == 7) then
	      adda   = info(ivt,2)
	      lvt    = info(ivt,3)
	      iac    = info(ivt,4)
	      iarloc = info(ivt,5) - ptrmyb
	      if (i == iarloc+1) then
	        do k = 1,lvt
	          if (mat(adda+k) /= zero) then
		    typcnt(7) = typcnt(7) + 1
		    rowbuf(i)   = mat(adda+k)
		    colind(i) = iac+k
		    goto 888
                  endif
	        enddo
	      endif
	    endif

	    if (info(ivt,1) == 8) then
	      adda   = info(ivt,2)
	      lvt    = info(ivt,3)
	      iac    = info(ivt,4)
	      iarloc = info(ivt,5) - ptrmyb
	      iarend = iarloc+lvt
	      if (i >= iarloc+1 .and. i <= iarend) then
		ilocal = i-iarloc
	        if (mat(adda+ilocal) /= zero) then
		  typcnt(8) = typcnt(8) + 1
		  rowbuf(i)   = mat(adda+ilocal)
		  colind(i) = iac+1
		  goto 888
                endif
              endif
	    endif

	    if (info(ivt,1) == 9) then
	      adda   = info(ivt,2)
	      lvt    = info(ivt,3)
	      iarloc = info(ivt,5) - ptrmyb
	      indc   = info(ivt,6)
	      if (i == iarloc+1) then
	        do k = 1,lvt
	          if (mat(adda+k) /= zero) then
		    typcnt(9) = typcnt(9) + 1
		    rowbuf(i)   = mat(adda+k)
		    colind(i) = index(indc+k)
		    goto 888
                  endif
	        enddo
	      endif
	    endif

            if (info(ivt,1) == 11) then
	      adda   = info(ivt,2)
	      lvt    = info(ivt,3)
	      iac    = info(ivt,4)
	      indr   = info(ivt,7)
	      iarbeg = index(indr+1) - ptrmyb
	      iarend = index(indr+lvt) - ptrmyb
	      if (i >= iarbeg .and. i <= iarend) then
	        k = 1
	        do while (index(indr+k) - ptrmyb <= i)
	          if ((index(indr+k) - ptrmyb == i) .and. 
     *                (mat(adda+k) /= zero)) then
		    typcnt(11) = typcnt(11) + 1
		    rowbuf(i)   = mat(adda+k)
		    colind(i) = iac+1
		    goto 888
                  endif
	          k = k+1
	        enddo
	    endif
          enddo
        endif
      enddo

      R E T U R N
C------END OF FILL1_BUF--------------------------------------------
      E N D                                   
