!**********************************************************************
!        1         2         3         4         5         6         7*
!**********************************************************************
!**                                                                 ***
!**                                                                 ***
      subroutine GCBUF(gbuf,lbuf,infbuf,ia1buf,norows,endgbuf)
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**   G C B U F   performs a garbage collection on the buffer       ***
!**               <gbuf>; if infbuf(:,2) == 0 the corresponding     ***
!**               entries in the buffer <gbuf> are skipped and      ***
!**               the buffer is pushed together.                    ***  
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**    Copyright (C) 1999 University Karlsruhe                      ***
!**    PROGRAMMER: H. Haefner                                       ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**     THE IMPLICIT NONE-INSTRUCTION HAS THE AIM,                  ***
!**     THAT ALL VARIABLES MUST BE DECLARED EXPLICITLY.             ***
!**                                                                 ***
      USE constants

      implicit none
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**      FORMAL PARAMETERS :                                        ***
!**      -----------------                                          ***
!**                                                                 ***
      integer,intent(in) ::  ia1buf,lbuf,norows
      integer,intent(out) ::  endgbuf
      integer,intent(inout) ::  infbuf(ia1buf,ia2buf)
      double precision,intent(inout):: gbuf(lbuf)
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**   LIST OF FORMAL PARAMETERS :                                   ***
!**   -------------------------                                     ***
!**                                                                 ***
!--------I------I-----I------------------------------------------------
! NAME   I TYPE I I/O I     MEANING
!--------I------I-----I------------------------------------------------
!--------I------I-----I------------------------------------------------
! norows I  I   I in  I  number of actual rows in buffer
!--------I------I-----I------------------------------------------------
! lbuf   I  I   I in  I  length of buffer array
!--------I------I-----I------------------------------------------------
! ia1buf I  I   I in  I  first dimension of buffer information array
!--------I------I-----I------------------------------------------------
! endgbufI  I   I out I  pointer to the last entry within the buffer
!--------I------I-----I------------------------------------------------
!**                                                                 ***
!**                                                                 ***
!**      LOCAL PARAMETERS :                                         ***
!**      -----------------                                          ***
!**                                                                 ***
      integer    adda,i,j,lvt,ptrbuf
!     integer    ivt,infmin,ixmin,nivt
      integer    ptradd(norows),ixh(norows)
      logical    lmono,lident

!
!**** START OF CALCULATION :
!     ---------------------
!
!
!***  Create pointer array sorting the information array <infbuf>
!***  regarding to the pointers to the buffer <gbuf>
!
      lmono = .true.
      do i=2,norows
        if (infbuf(i,1) < infbuf(i-1,1)) then
          lmono = .false.
          exit
        endif
      enddo

      if (lmono) then
        do i=1,norows
          ptradd(i) = i
        enddo
      else
        do i=1,norows
          ixh(i) = infbuf(i,1)
        enddo
        lident = .true.
        call HEAPSORTIX(ixh,ptradd,norows,lident)
      endif

!     listed = 0
!     adda = 0
!     ivt  = 1
!     do i=1,norows
!nivt = ivt
!infmin = -1
!do
!  if ((infbuf(nivt,1) == adda) .and. (listed(nivt) == 0)) then
!    exit
!         else
!    if ((infbuf(nivt,1) > adda) .and. (infmin == -1) .and.
!    #          (listed(nivt) == 0)) then
!      infmin = infbuf(nivt,1)
!      ixmin  = nivt
!    endif
!    if ((infbuf(nivt,1) > adda) .and. (infbuf(nivt,1) < infmin)
!    #          .and. (listed(nivt) == 0)) then
!      infmin = infbuf(nivt,1)
!      ixmin  = nivt
!           endif
!         endif
!  nivt = nivt + 1
!  if (nivt > norows) then
!    nivt = ixmin
!    exit
!         endif
!       enddo
!ptradd(i) = nivt
!listed(nivt) = 1
!if (infbuf(nivt,2) /= 0) then
!         adda = infbuf(nivt,1) + infbuf(nivt,3)
!else
!         adda = infbuf(nivt,1)
!endif
!       if (ivt == nivt) ivt = ivt + 1
!     enddo

!
!***  Pack the information array pointers
!
      ptrbuf = 0
      do i=1,norows
	if (infbuf(ptradd(i),2) /= 0) then
	  adda = infbuf(ptradd(i),1)
	  lvt  = infbuf(ptradd(i),3)
	  do j=1,lvt
	    gbuf(ptrbuf+j) = gbuf(adda+j)
	  enddo
	  infbuf(ptradd(i),1) = ptrbuf
	  ptrbuf = ptrbuf + lvt
	endif
      enddo

      endgbuf = ptrbuf

      R E T U R N
!------END OF GCBUF--------------------------------------------------
      E N D                                   
