!**********************************************************************
!        1         2         3         4         5         6         7*
!**********************************************************************
!**                                                                 ***
!**                                                                 ***
      subroutine GCLU(pmat,pix,infoL,infoU,lpmat,lpix,ia1LU,nvtLU,
     *                lglobal,matmark,imark,endptr,myproc,nproc)
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**   G C L U   performs a garbage collection of the triple         ***
!**             (pmat,pix,infoL); the pointer of infoL and infoU to ***
!**             <pmat> and <pix> are adapted. The arrays <pmat> and ***
!**             <pix> are pushed together.                          ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**    Copyright (C) 1999 University Karlsruhe                      ***
!**    PROGRAMMER: H. Haefner                                       ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**     THE IMPLICIT NONE-INSTRUCTION HAS THE AIM,                  ***
!**     THAT ALL VARIABLES MUST BE DECLARED EXPLICITLY.             ***
!**                                                                 ***
      USE constants

      implicit none
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**      FORMAL PARAMETERS :                                        ***
!**      -----------------                                          ***
!**                                                                 ***
      integer,intent(in) ::  lpmat,lpix,ia1LU,imark,nvtLU,lglobal
      integer,intent(in) ::  myproc,nproc
      integer,intent(inout) ::  endptr
      integer,intent(inout) ::  pix(lpix),infoL(ia1LU,ia2LU)
      integer,intent(inout) ::  infoU(ia1LU,ia2LU)
      double precision,intent(in):: matmark
      double precision,intent(inout):: pmat(lpmat)
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**   LIST OF FORMAL PARAMETERS :                                   ***
!**   -------------------------                                     ***
!**                                                                 ***
!--------I------I-----I------------------------------------------------
! NAME   I TYPE I I/O I     MEANING
!--------I------I-----I------------------------------------------------
!--------I------I-----I------------------------------------------------
! endptr I  I   I i/o I  local size of matrix array and index array
!--------I------I-----I------------------------------------------------
! matmarkI  I   I in  I  marker for matrix elements to be removed
!--------I------I-----I------------------------------------------------
! imark  I  I   I in  I  marker for index elements to be removed
!--------I------I-----I------------------------------------------------
!**                                                                 ***
!**                                                                 ***
!**      LOCAL PARAMETERS :                                         ***
!**      -----------------                                          ***
!**                                                                 ***
      integer    i,ii,j,ihlp,inew,endind,endmat

!
!**** START OF CALCULATION :
!     ---------------------
!
!***  Pack the information array pointers to the matrix array <PMAT>
!
      infoL(1,1) = infoU(1,2)
      do j=2,myproc
	infoL(1,1) = infoL(1,1) + infoU(j,2)
      enddo

      do i=2,nvtLU
	infoL(i,1) = infoL(i-1,1) + infoL(i-1,2)
	do j=myproc+1,nproc+myproc
	  infoL(i,1) = infoL(i,1) + infoU((i-2)*nproc+j,2)
	enddo
      enddo

      if (nproc > 1) then
        j  = 1
        ii = myproc + 1
        do i=2,lglobal
	  if (i == ii) then
	    infoU(i,1) = infoL(j,1) + infoL(j,2)
	    j = j + 1
	    ii = ii + nproc
	  else
	    infoU(i,1) = infoU(i-1,1) + infoU(i-1,2)
	  endif
        enddo
      else
	do i=2,nvtLU
	  infoU(i,1) = infoL(i-1,1) + infoL(i-1,2)
	enddo
      endif

!**
!**    Garbage collection of marked indices of array <pix>
!**
      endind = endptr
      inew  = 1
      do i=1,endind
        pix(inew) = pix(i)
        if (pix(inew) /= imark) then
          inew = inew+1
        endif
      enddo
      inew = inew -1

!**
!**    Garbage collection of marked matrixelements of array <PMAT>
!**
      endmat = endptr
      inew  = 1
      do i=1,endmat
        pmat(inew) = pmat(i)
        if (pmat(inew) /= matmark) then
          inew = inew+1
        endif
      enddo
      inew = inew -1
      endptr = inew

      R E T U R N
!------END OF GCLU--------------------------------------------------
      E N D                                   
