!**********************************************************************
!        1         2         3         4         5         6         7*
!**********************************************************************
!**                                                                 ***
!**                                                                 ***
      subroutine GCOFFRO(mat,index,infoLU,lmat,lindex,ia1LU,nvtLU,
     *                   matmark,imark,endmat)
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**   G C O F F R O   performs a garbage collection of the triple   ***
!**                (mat,index,infoLU); the pointers to              ***
!**                <mat> and <index> are being adapted. The arrays  ***
!**                <mat> and <index> are pushed together.           ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**    Copyright (C) 2000 University Karlsruhe                      ***
!**    PROGRAMMER: H. Haefner                                       ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**     THE IMPLICIT NONE-INSTRUCTION HAS THE AIM,                  ***
!**     THAT ALL VARIABLES MUST BE DECLARED EXPLICITLY.             ***
!**                                                                 ***
      USE constants

      implicit none
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**      FORMAL PARAMETERS :                                        ***
!**      -----------------                                          ***
!**                                                                 ***
      integer,intent(in) ::  lmat,lindex,ia1LU,imark
      integer,intent(inout) ::  nvtLU,endmat
      integer,intent(inout) ::  index(lindex),infoLU(ia1LU,ia2LU)
      double precision,intent(in):: matmark
      double precision,intent(inout):: mat(lmat)
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**   LIST OF FORMAL PARAMETERS :                                   ***
!**   -------------------------                                     ***
!**                                                                 ***
!--------I------I-----I------------------------------------------------
! NAME   I TYPE I I/O I     MEANING
!--------I------I-----I------------------------------------------------
!--------I------I-----I------------------------------------------------
! endmat I  I   I i/o I  local size of matrix array
!--------I------I-----I------------------------------------------------
! matmarkI  I   I in  I  marker for matrix elements to be removed
!--------I------I-----I------------------------------------------------
! imark  I  I   I in  I  marker for index elements to be removed
!--------I------I-----I------------------------------------------------
!**                                                                 ***
!**                                                                 ***
!**      LOCAL PARAMETERS :                                         ***
!**      -----------------                                          ***
!**                                                                 ***
      integer    endind,i,inew,ivt,j
      integer    ixh(endmat),ptradd(nvtLU+1)
      logical    lmono,lident

!
!**** START OF CALCULATION :
!     ---------------------
!

!
!***  CREATE POINTER ARRAY SORTING THE INFORMATION ARRAY REGARDING TO
!***  POINTERS TO THE MATRIX ARRAY <MAT>
!
      lmono = .true.
      do i=2,nvtLU
        if (infoLU(i,1) < infoLU(i-1,1)) then
          lmono = .false.
          exit
        endif
      enddo

      if (lmono) then
        do i=1,nvtLU
          ptradd(i) = i
        enddo
      else
        do i=1,nvtLU
          ixh(i) = infoLU(i,1)
        enddo
        lident = .true.
        call HEAPSORTIX(ixh,ptradd,nvtLU,lident)
      endif

!
!***  PACK THE INFORMATION ARRAY POINTERS TO THE MATRIX ARRAY <MAT>
!
      if (nvtLU > 0) infoLU(ptradd(1),1) = 0
      do i=2,nvtLU
	infoLU(ptradd(i),1)=infoLU(ptradd(i-1),1)+infoLU(ptradd(i-1),2)
      enddo

!**
!**    GARBAGE COLLECTION OF MARKED INDICES OF ARRAY <INDEX>
!**

      endind = endmat

      inew = 0
      do i=1,endind
        if (index(i) /= imark) then
          inew = inew+1
          ixh(inew) = i
        endif
      enddo

      include "norec.h"
      do i=1,inew
        index(i) = index(ixh(i))
      enddo

!**
!**    GARBAGE COLLECTION OF MARKED MATRIXELEMENTS OF ARRAY <MAT>
!**

      inew = 0
      do i=1,endmat
        if (mat(i) /= matmark) then
          inew = inew + 1
          ixh(inew) = i
        endif
      enddo

      include "norec.h"
      do i=1,inew
        mat(i) = mat(ixh(i))
      enddo
      endmat = inew

      R E T U R N
!------END OF GCOFFRO------------------------------------------------
      E N D
