!**********************************************************************
!        1         2         3         4         5         6         7*
!**********************************************************************
!**                                                                 ***
!**                                                                 ***
      subroutine GCOPT(mat,index,info,lmat,lindex,ia1,nvt,matmark,
     *                 imark,endmat,endind,endmax,lsto9)
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**   G C O P T   performs a garbage collection of the triple       ***
!**         (mat,index,info); if info(:,3) == 0 the corresponding   ***
!**         vector term is skipped - the pointer to <mat> and       ***
!**         <index> are adapted. The arrays <mat> and <index> are   ***
!**         pushed together.                                        ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**    Copyright (C) 2000 University Karlsruhe                      ***
!**    PROGRAMMER: H. Haefner                                       ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**     THE IMPLICIT NONE-INSTRUCTION HAS THE AIM,                  ***
!**     THAT ALL VARIABLES MUST BE DECLARED EXPLICITLY.             ***
!**                                                                 ***
      USE constants

      implicit none
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**      FORMAL PARAMETERS :                                        ***
!**      -----------------                                          ***
!**                                                                 ***
      integer,intent(in) ::  endmax,lmat,lindex,ia1,imark
      integer,intent(inout) ::  nvt,endmat,endind
      integer,intent(inout) ::  index(lindex),info(ia1,ia2)
      double precision,intent(in):: matmark
      double precision,intent(inout):: mat(lmat)
      logical, intent(in) :: lsto9
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**   LIST OF FORMAL PARAMETERS :                                   ***
!**   -------------------------                                     ***
!**                                                                 ***
!--------I------I-----I------------------------------------------------
! NAME   I TYPE I I/O I     MEANING
!--------I------I-----I------------------------------------------------
!--------I------I-----I------------------------------------------------
! endmat I  I   I i/o I  local size of matrix array
!--------I------I-----I------------------------------------------------
! endind I  I   I i/o I  local size of index array
!--------I------I-----I------------------------------------------------
! matmarkI  I   I in  I  marker for matrix elements to be removed
!--------I------I-----I------------------------------------------------
! imark  I  I   I in  I  marker for index elements to be removed
!--------I------I-----I------------------------------------------------
!**                                                                 ***
!**                                                                 ***
!**      LOCAL PARAMETERS :                                         ***
!**      -----------------                                          ***
!**                                                                 ***
      integer    adda,i,ii,inew,j
      integer    ix,ixcnt,ixinf1,ixinf2,ixmin
      integer    ixh(endmax),ptradd(nvt+1),ptrind(nvt+1)
      logical    lmono,lident

!
!**** START OF CALCULATION :
!     ---------------------
!
!
!***  REMOVE VECTOR TERMS FROM ARRAY <INFO> IF VECTORLENGTH == 0
!
      inew = 0
      do i=1,nvt
	if (info(i,3) /= 0) then
	  inew = inew + 1
	  ptrind(inew) = i
	endif
      enddo
      do j=1,ia2
        include "norec.h"
        do i=1,inew
          info(i,j) = info(ptrind(i),j)
        enddo
      enddo
      nvt = inew

!     ivt = nvt
!     do while (ivt > 0)
!       if (info(ivt,3) == 0) then
!         do j=1,ia2
!           do i=ivt,nvt-1
!             info(i,j) = info(i+1,j)
!           enddo
!         enddo
!         nvt = nvt - 1
!       endif
!       ivt = ivt - 1             !  ivt <= nvt holds
!     enddo
!
!***  CREATE POINTER ARRAY SORTING THE INFORMATION ARRAY REGARDING TO
!***  POINTERS TO THE MATRIX ARRAY <MAT>
!
      lmono = .true.
      do i=2,nvt
	if (info(i,2) < info(i-1,2)) then
	  lmono = .false.
	  exit
	endif
      enddo

      if (lmono) then
        do i=1,nvt
	  ptradd(i) = i
        enddo
      else
	do i=1,nvt
	  ixh(i) = info(i,2)
	enddo
	lident = .true.
	call HEAPSORTIX(ixh,ptradd,nvt,lident)
      endif

!
!***  CREATE POINTER ARRAY SORTING THE INFORMATION ARRAY REGARDING TO
!***  POINTERS TO THE INDEX ARRAY <INDEX>
!
      if (lsto9) then
        lmono = .true.
        do i=2,nvt
	  if (info(i,6) < info(i-1,6)) then
	    lmono = .false.
	    exit
	  endif
        enddo

        if (lmono) then
          do i=1,nvt
	    ptrind(i) = i
          enddo
        else
	  do i=1,nvt
	    ixh(i) = info(i,6)
	  enddo
	  lident = .true.
	  call HEAPSORTIX(ixh,ptrind,nvt,lident)
        endif
      else
        ixcnt = 0
        do i=1,nvt
          if (info(i,1) == 3 .or. info(i,1) == 4 .or.
     *        info(i,1) == 9) then
	    ixcnt = ixcnt + 1
	    ixh(ixcnt) = info(i,6)
	    ptrind(ixcnt) = i
          else if (info(i,1) == 6) then
	    ixcnt = ixcnt + 1
	    ixh(ixcnt) = MIN(info(i,6),info(i,7))
	    ptrind(ixcnt) = i
          else if (info(i,1) == 5 .or. info(i,1) == 11) then
	    ixcnt = ixcnt + 1
	    ixh(ixcnt) = info(i,7)
	    ptrind(ixcnt) = i
	  endif
	enddo
	if (ixcnt == 0) goto 777
	lident = .false.
	call HEAPSORTIX(ixh,ptrind,ixcnt,lident)
      endif

!
!***  PACK THE INFORMATION ARRAY POINTERS TO THE MATRIX ARRAY <MAT>
!
  777 if (nvt > 0) info(ptradd(1),2) = 0
      do i=2,nvt
	info(ptradd(i),2) = info(ptradd(i-1),2) + info(ptradd(i-1),3)
      enddo

!
!***  PACK THE INFORMATION ARRAY POINTERS TO THE INDEX ARRAY <INDEX>
!
      if (lsto9) then
        if (nvt > 0) info(ptrind(1),6) = 0
        do i=2,nvt
	  info(ptrind(i),6) = info(ptrind(i-1),6) + info(ptrind(i-1),3)
        enddo
      else
        if (ixcnt == 0) goto 888
        if (info(ptrind(1),1) == 3 .or. info(ptrind(1),1) == 4 .or.
     *      info(ptrind(1),1) == 9) then
	  ixinf1 = 6
          info(ptrind(1),6) = 0
        else if (info(ptrind(1),1) == 6) then
	  if (info(ptrind(1),6) < info(ptrind(1),7)) then
	    ixinf1 = 7
            info(ptrind(1),6) = 0
            info(ptrind(1),7) = info(ptrind(1),3)
          else
	    ixinf1 = 6
            info(ptrind(1),7) = 0
            info(ptrind(1),6) = info(ptrind(1),3)
          endif
        else if (info(ptrind(1),1)==5 .or. info(ptrind(1),1)==11) then
	  ixinf1 = 7
          info(ptrind(1),7) = 0
        endif
        do i=2,ixcnt
          if (info(ptrind(i),1) == 3 .or. info(ptrind(i),1) == 4 .or.
     *        info(ptrind(i),1) == 9) then
           info(ptrind(i),6)=info(ptrind(i-1),ixinf1) +
     *                       info(ptrind(i-1),3)
	   ixinf1 = 6
          else if (info(ptrind(i),1) == 6) then
	    if (info(ptrind(i),6) < info(ptrind(i),7)) then
              info(ptrind(i),6) = info(ptrind(i-1),ixinf1) +
     *                            info(ptrind(i-1),3)
              info(ptrind(i),7) = info(ptrind(i),6) +
     *                            info(ptrind(i),3)
	      ixinf1 = 7
            else
              info(ptrind(i),7) = info(ptrind(i-1),ixinf1) +
     *                            info(ptrind(i-1),3)
              info(ptrind(i),6) = info(ptrind(i),7) +
     *                            info(ptrind(i),3)
	      ixinf1 = 6
            endif
          else if (info(ptrind(i),1) == 5 .or. info(ptrind(i),1) == 11)
     *         then
           info(ptrind(i),7)=info(ptrind(i-1),ixinf1) +
     *                       info(ptrind(i-1),3)
	   ixinf1 = 7
          endif
        enddo
      endif

  888 continue

!**
!**    GARBAGE COLLECTION OF MARKED INDICES OF ARRAY <INDEX>
!**
      inew = 0
      do i=1,endind
        if (index(i) /= imark) then
          inew = inew+1
	  ixh(inew) = i
        endif
      enddo

      include "norec.h"
      do i=1,inew
        index(i) = index(ixh(i))
      enddo
      endind = inew


!**
!**    GARBAGE COLLECTION OF MARKED MATRIXELEMENTS OF ARRAY <MAT>
!**
      inew = 0
      do i=1,endmat
	if (mat(i) /= matmark) then
	  inew = inew + 1
	  ixh(inew) = i
	endif
      enddo

      include "norec.h"
      do i=1,inew
        mat(i) = mat(ixh(i))
      enddo
      endmat = inew

      R E T U R N
!------END OF GCOPT-----------------------------------------------
      E N D
