!:::::      ,,,,,LL3ASXO.....
!
!
!
!**********************************************************************
!        1         2         3         4         5         6         7*
!**********************************************************************
!**                                                                 ***
!**                                                                 ***
      subroutine LL3ASXO(mat,x,b,buf,ptrmbk,lmatbk,
     #                   l,nproc,tids,myproc,jump,
     #                   ladd,nvt,lmat,lindex,ia1,info,index,nmsg)
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**      L L 3 A S X O  computes one matrix-vector-multiplication   ***
!**                     with symmetric matrix MAT                   ***
!**                     on a distributed memory system:             ***
!**                     b = b0 + MAT * x .                          ***
!**                     If ladd = .false.  :  b0 = 0 .              ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**    Copyright (C) 1995 University Karlsruhe                      ***
!**    PROGRAMMER :     H. Haefner                                  ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**   the implicit none-instruction has the aim,                    ***
!**   that all variables must be declared explicitly.               ***
!**                                                                 ***
      USE constants

      implicit none
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**      FORMAL PARAMETERS :                                        ***
!**      -----------------                                          ***
!**                                                                 ***
      integer           nvt,lmat,lindex,ia1,nproc,l,nmsg
      integer           info(ia1,ia2),index(lindex)
      integer           tids(nproc),myproc,jump(nproc)
      integer           ptrmbk(nproc+1),lmatbk(nproc)
      double precision  b(l), x(l), buf(5*l), mat(lmat)
      logical           ladd


!**                                                                 ***
!**                                                                 ***
!**        LOCAL PARAMETERS :                                       ***
!**        ----------------                                         ***
!**                                                                 ***
      integer           i,ierr,count,pbuf,pbufs,pbufr,sum
      integer           to,from,nrblk,mids,midr,njump
      integer           swap,lnbpid,rnbpid,ptsmyb
      integer           LL9MAP
      double precision, parameter ::  zero = 0.0
!**        ---------                                                ***
!**                                                                 ***
!**                                                                 ***
      pbufs = 1
      pbufr = 2*l+1
      pbuf  = 4*l+1
      nrblk = myproc
      ptsmyb = ptrmbk(myproc)
!**                                                                 ***
      if (.not. ladd) then
	do i=1,l
	  b(i) = zero
        enddo
      endif
!**                                                                 ***
!**   COPY INPUT- AND OUTPUT-VECTOR IN SEND BUFFER                  ***
!**                                                                 ***
      do i=1,l
        buf(pbufs+i-1) = x(i)
        buf(pbufs+l+i-1) = zero
      enddo
!**                                                                 ***
      sum = 0 
      njump = 0
   50 njump = njump+1
      sum = sum+jump(njump)
      if (sum < nproc) goto 50
!**                                                                 ***
!**   COMPUTE njump MATRIX BLOCKS WITH RECEIVING AND SENDING OF     ***
!**   VECTOR PARTS OF B                                             ***
!**                                                                 ***
      do count=1,njump
	if (nproc > 1) then
          lnbpid = LL9MAP(myproc-jump(count),nproc)
	  from = tids(lnbpid)
	  call MPRCVA(from,nmsg+count,2*l*ireal,buf(pbufr),
     #                midr,ierr)
        endif
!**                                                                 ***
!**   SEND ASYNCHRONOUSLY OUTPUT VECTOR                             ***
!**                                                                 ***
        if (nproc > 1) then
          rnbpid = LL9MAP(myproc+jump(count),nproc)
	  to = tids(rnbpid)
	  call MPSNDA(to,nmsg+count,2*l*ireal,buf(pbufs),mids,ierr)
	endif
!**                                                                 ***
!**   PERFORM SYMMETRIC MULTIPLICATION FOR MAIN DIAGONAL BLOCK      ***
!**   PERFORM NON-TRANSP.-PART-MULTIPLICATION FOR OTHER BLOCKS      ***
!**                                                                 ***
        if (count == 1) then
          call LL6ASXO(mat,x,b,ptsmyb,l,ptrmbk(nrblk),
     &                lmatbk(nrblk),nvt,lmat,lindex,ia1,info,index)
        else
          call LL6AXO(mat,lmat,index,lindex,info,ia1,nvt,b,buf(pbufs),
     &                l,ptsmyb,ptrmbk(nrblk),lmatbk(nrblk))
        endif

!**                                                                 ***
!**   PERFORM TRANSP.-PART-MULTIPLICATION FOR OTHER BLOCKS          ***
!**                                                                 ***
        if (nproc > 1) then
          if (count < njump) then
            nrblk = nrblk-jump(count)
            if (nrblk <= 0) nrblk = nrblk+nproc
            do i=pbuf,pbuf+l-1
              buf(i) = zero
            enddo
!**                                                                 ***
            call LL6ATXO(mat,lmat,index,lindex,info,ia1,nvt,x,
     &                   buf(pbuf),l,ptsmyb,ptrmbk(nrblk),
     &                   lmatbk(nrblk))
          endif
!**                                                                 ***
!**   WAIT RECEIVE                                                  ***
!**                                                                 ***
	  call MPRCVW(from,nmsg+count,2*l*ireal,buf(pbufr),
     #                midr,ierr)
!**                                                                 ***
!**   COMPUTE NEW PART OF VECTOR B IN RECEIVE BUFFER                ***
!**                                                                 ***
          if (count < njump) then
            do i=0,lmatbk(nrblk)-1
              buf(pbufr+l+i) = buf(pbufr+l+i)+buf(pbuf+i)
            enddo
          endif
!**                                                                 ***
!**   WAIT SEND                                                     ***
!**                                                                 ***
          call MPSNDW(to,nmsg+count,2*l*ireal,buf(pbufs),mids,ierr)
!**                                                                 ***
!**   SWAP RECEIVE AND SEND BUFFER                                  ***
!**                                                                 ***
          swap = pbufs
	  pbufs = pbufr
	  pbufr = swap
        endif

      enddo
	  
      nmsg = nmsg + count

      if (nproc > 1) then
        do i=1,lmatbk(myproc)
	  b(i) = b(i)+buf(pbufs+l+i-1)
        enddo
      endif
!**                                                                 ***
      r e t u r n
!-----END OF LL3ASXO---------------------------------------------------
      e    n    d
