!:::::      ,,,,,LL3AXO.....
!
!
!
!**********************************************************************
!        1         2         3         4         5         6         7*
!**********************************************************************
!**                                                                 ***
!**                                                                 ***
      subroutine LL3AXO(ROUTIN,mat,x,b,buf,ptrmbk,lmatbk,
     #                  l,nproc,tids,myproc,jump,ladd,nvtLU,
     #                  lmat,lindex,ia1LU,ia2h,infoLU,index,nmsg)
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**      L L 3 A X O   computes one matrix-vector-multiplication    ***
!**                    b = b0 + MAT * x                             ***
!**                    on a multiprocessor system.                  ***
!**                    if ladd = .false.  :  b0 = 0 .               ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**    Copyright (C) 2001 University Karlsruhe                      ***
!**    PROGRAMMER :     H. Haefner                                  ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**   the implicit none-instruction has the aim,                    ***
!**   that all variables must be declared explicitly.               ***
!**                                                                 ***
      USE constants

      implicit none
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**      FORMAL PARAMETERS :                                        ***
!**      -----------------                                          ***
!**                                                                 ***
      integer           nvtLU,lmat,lindex,ia1LU,ia2h,nproc,l,nmsg
      integer           infoLU(ia1LU,ia2h),index(lindex)
      integer           tids(nproc),myproc,jump(nproc)
      integer           ptrmbk(nproc+1),lmatbk(nproc)
      double precision  b(l), x(l), buf(2*l), mat(lmat)
      logical           ladd
      external          ROUTIN


!**                                                                 ***
!**                                                                 ***
!**        LOCAL PARAMETERS :                                       ***
!**        ----------------                                         ***
!**                                                                 ***
      integer           i,ierr,count,pbuf,pbufs,pbufr,sum
      integer           to,from,nrblk,mids,midr,njump
      integer           swap,lnbpid,rnbpid,ptsmyb
      integer           LL9MAP
!**                                                                 ***
!**        CONSTANTS:                                               ***
!**        ---------                                                ***
!**                                                                 ***
      double precision,parameter ::  zero = 0.0
!**                                                                 ***
      pbufs = 1
      pbufr = l+1
      nrblk = myproc
!**                                                                 ***
      if (.not. ladd) then
	do i=1,l
	  b(i) = zero
        enddo
      endif
!**                                                                 ***
!**   COPY INPUT VECTOR IN SEND BUFFER                              ***
!**                                                                 ***
      do i=1,l
        buf(pbufs+i-1) = x(i)
      enddo
!**                                                                 ***
      sum = 0 
      njump = 0
   50 njump = njump+1
      sum = sum+jump(njump)
      if (sum < nproc) goto 50
!**                                                                 ***
!**   COMPUTE njump MATRIX BLOCKS WITH RECEIVING AND SENDING OF     ***
!**   VECTOR PARTS OF X                                             ***
!**                                                                 ***
      do count=1,njump

!**   SEND ASYNCHRONOUSLY OUTPUT VECTOR                             ***

	if (nproc > 1) then
          lnbpid = LL9MAP(myproc-jump(count),nproc)
	  from = tids(lnbpid)
	  call MPRCVA(from,nmsg+count,l*ireal,buf(pbufr),
     &                midr,ierr)
          rnbpid = LL9MAP(myproc+jump(count),nproc)
	  to = tids(rnbpid)
	  call MPSNDA(to,nmsg+count,l*ireal,buf(pbufs),mids,ierr)
        endif

        ptsmyb = ptrmbk(myproc)
        call ROUTIN(mat,lmat,index,lindex,infoLU,ia1LU,nvtLU,b,
     &              buf(pbufs),l,ptsmyb,ptrmbk(nrblk),lmatbk(nrblk))
	nrblk = nrblk-jump(count)
        if (nrblk <= 0) nrblk = nrblk+nproc

!**   WAIT SEND AND WAIT RECEIVE                                    ***

        if (nproc > 1) then
          call MPSNDW(to,nmsg+count,l*ireal,buf(pbufs),mids,ierr)
	  call MPRCVW(from,nmsg+count,l*ireal,buf(pbufr),midr,ierr)
        endif

!**   SWAP RECEIVE AND SEND BUFFER                                  ***

        swap = pbufs
	pbufs = pbufr
	pbufr = swap
      enddo
	  
      nmsg = nmsg + count
!**                                                                 ***
      r e t u r n
!-----END OF LL3AXO----------------------------------------------------
      e    n    d
