!:::::      ,,,,,LL6AXO.....
!
!
!
!**********************************************************************
!        1         2         3         4         5         6         7*
!**********************************************************************
!**                                                                 ***
!**                                                                 ***
      subroutine LL6AXO(mat,lmat,index,lindex,info,ia1,nvt,b,x,l,
     &                  ptsmyb,ptsmbk,lmatbk)
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**      L L 6 A X O    is the kernel routine for the matrix-       ***
!**                     vector-multiplication for unpartioned       ***
!**                     row blocks:                                 ***
!**                     b = b + MAT * x .                           ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**    Copyright (C) 2001 University Karlsruhe                      ***
!**    PROGRAMMER : H.Haefner                                       ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**   the implicit none-instruction has the aim,                    ***
!**   that all variables must be declared explicitly.               ***
!**                                                                 ***
      USE constants

      implicit none
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**      FORMAL PARAMETERS :                                        ***
!**      -----------------                                          ***
!**                                                                 ***
      integer           nvt, lmat, lindex, ia1
      integer           info(ia1,ia2), index(lindex)
      integer           ptsmyb,ptsmbk,lmatbk,l
      double precision  b(l), x(l), mat(lmat)
!**                                                                 ***
!**        LOCAL PARAMETERS :                                       ***
!**        ----------------                                         ***
!**                                                                 ***
      integer           i, k, jc, jr, first
      integer           typ,adda,lvt,iac,iar,iac1,iar1,indc,indr
      double precision, parameter ::  zero = 0.0
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**** START OF CALCULATION :                                        ***
!**   ---------------------                                         ***
!**                                                                 ***
      do k=1,nvt
        typ   =info(k,1)
        adda  =info(k,2)
        lvt   =info(k,3)
        iac1  =info(k,4)-ptsmbk
        iar1  =info(k,5)-ptsmyb
        iac   =info(k,4)
        iar   =info(k,5)
        indc  =info(k,6)
        indr  =info(k,7)
!**                                                                 ***
        if ((typ == 1) .or. (typ == 2)) then
          do i=1,lvt
	    if ((iac1+i > 0) .and. (iac1+i <= lmatbk)) then
              b(i+iar1) = b(i+iar1) + mat(i+adda) * x(i+iac1)
            endif
          enddo
          cycle
        endif
!**                                                                 ***
        if (typ == 3) then
          include "norec.h"
          do i=1,lvt
            jc = iac1 + index(i+indc)
	    if ((jc > 0) .and. (jc <= lmatbk)) then
              b(jc-iac1+iar1) = b(jc-iac1+iar1) + mat(i+adda) * x(jc)
            endif
          enddo
          cycle
        endif
!**                                                                 ***
        if (typ == 4) then
          include "norec.h"
          do i=1,lvt
            jc = index(i+indc)-ptsmbk
	    if ((jc > 0) .and. (jc <= lmatbk)) then
              b(i+iar1) = b(i+iar1) + mat(i+adda) * x(jc)
            endif
          enddo
          cycle
        endif
!**                                                                 ***
        if (typ == 5) then
          do i=1,lvt
  	    jr = index(i+indr)-ptsmyb
	    if ((jr > 0) .and. (jr <= lmatbk)) then
              b(jr) = b(jr) + mat(i+adda) * x(i+iac1)
            endif
          enddo
          cycle
        endif
!**                                                                 ***
        if (typ == 6) then
          do i=1,lvt
            jr = index(i+indr)-ptsmyb
	    jc = index(i+indc)-ptsmbk
	    if ((jr > 0) .and. (jr <= lmatbk) .and.
     &          (jc > 0) .and. (jc <= lmatbk)) then
              b(jr) = b(jr) + mat(i+adda) * x(jc)
            endif
          enddo
          cycle
        endif
!**                                                                 ***
        if (typ == 7) then
	  do i=1,lvt
	    if ((iac1+i > 0) .and. (iac1+i <= lmatbk)) then
	      b(iar1+1) = b(iar1+1) + mat(i+adda) * x(iac1+i) 
            endif
          enddo
          cycle
        endif
!**                                                                 ***
        if (typ == 8) then
	  do i=1,lvt
	    if ((iar1+i > 0) .and. (iar1+i <= lmatbk)) then
	      b(iar1+i) = b(iar1+i) + mat(i+adda) * x(iac1+1)
            endif
          enddo
          cycle
        endif
!**                                                                 ***
        if (typ == 9) then
          include "norec.h"
          do i = 1,lvt
            jc = index(i+indc)-ptsmbk
            if ((jc > 0) .and. (jc <= lmatbk)) then
              b(iar1+1) = b(iar1+1) + mat(i+adda) * x(jc)
            endif
          enddo
          cycle
        endif
!**                                                                 ***
        if (typ == 11) then
          include "norec.h"
          do i=1,lvt
  	    jr = index(i+indr)-ptsmyb
	    if ((jr > 0) .and. (jr <= lmatbk)) then
              b(jr) = b(jr) + mat(i+adda) * x(iac1+1)
            endif
          enddo
        endif
      enddo
!
!**** END OF CALCULATION
!     ------------------
!
      r e t u r n
!-----END OF LL6AXO---------------------------------------------------
      e    n    d
