!**********************************************************************
!        1         2         3         4         5         6         7*
!**********************************************************************
!**                                                                 ***
!**                                                                 ***
      subroutine LOADROP(mat,lmat,index,lindex,info,ia1,gbuf,lbuf,
     *                   infbuf,ia1buf,colperm,lglobal,ixprof,l,
     *                   endgbuf,irow,nolrow,nolrowbuf,norows,ilin,
     *                   myproc,nproc,ierr)
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**   L O A D R O P  loads the row <nolrow> of the matrix <mat>     ***
!**                  into the buffer <gbuf> and the corresponding   ***
!**                  information into <infbuf(nolrowbuf,1:ia2buf)>  ***
!**                  on more than one processor.                    ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**    Copyright (C) 2000 University Karlsruhe                      ***
!**    PROGRAMMER: H. Haefner                                       ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**     THE IMPLICIT NONE-INSTRUCTION HAS THE AIM,                  ***
!**     THAT ALL VARIABLES MUST BE DECLARED EXPLICITLY.             ***
!**                                                                 ***
      USE constants

      implicit none
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**      FORMAL PARAMETERS :                                        ***
!**      -----------------                                          ***
!**                                                                 ***
      integer, intent(inout) ::  ierr,endgbuf,norows
      integer, intent(in) ::  ia1,ia1buf,irow,nolrow,nolrowbuf
      integer, intent(in) ::  l,lglobal,lbuf,lindex,lmat,nproc,myproc
      integer, intent(in) ::  colperm(lglobal),ixprof(l)
      integer, intent(in) ::  index(lindex),info(ia1,ia2LU),ilin(nilin)
      integer, intent(inout) ::  infbuf(ia1buf,ia2buf)
      double precision, intent(in) :: mat(lmat)
      double precision, intent(inout) :: gbuf(lbuf)
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**   LIST OF FORMAL PARAMETERS :                                   ***
!**   -------------------------                                     ***
!**                                                                 ***
!**   (see Manual-Pages <linsol>)                                   ***
!**                                                                 ***
!**                                                                 ***
!**      LOCAL PARAMETERS :                                         ***
!**      -----------------                                          ***
!**                                                                 ***
      integer    i,iermax,ihelp,inext,ixperm
      integer    adda,indc,lvt,lout,maxix
      real       rhelp
      double precision, parameter ::  zero = 0.0

!
!**** START OF CALCULATION :
!     ---------------------
!

!
!***  Information array <info> must be sorted increasingly! The pointers
!***  to the matrix array <mat> must be sorted increasingly, too!
!

      lout   = ilin(12)

!
!***  <inext> is the index number of <info> with row number <nolrow>.
!
      rhelp = REAL(nolrow)/REAL(nproc)
      inext = CEILING(rhelp)

      adda = info(inext,1)
      lvt  = info(inext,2)
      indc = adda

      maxix = colperm(index(indc+1)) - irow + 1
      do i=2,lvt
	ixperm = colperm(index(indc+i)) - irow + 1
	if (ixperm > maxix) maxix = ixperm
      enddo

!
!***  Load matrix row <nolrowbuf> into buffer <gbuf> at location
!**   <endgbuf> +1..<infbuf(nolrowbuf,3)>
!
      infbuf(nolrowbuf,1) = endgbuf
      infbuf(nolrowbuf,2) = 0
      infbuf(nolrowbuf,3) = max(maxix,ixprof(inext) - irow + 1)
      norows = norows + 1
      if (endgbuf+infbuf(nolrowbuf,3) > lbuf) then
	call GCBUF(gbuf,lbuf,infbuf,ia1buf,norows,endgbuf)
	if (endgbuf+infbuf(nolrowbuf,3) > lbuf) then
	  ierr = myproc
          write(lout,1000)
          write(lout,1010) myproc
          write(lout,1020)
	  goto 999
	else
	  infbuf(nolrowbuf,1) = endgbuf
          infbuf(nolrowbuf,2) = nolrow
	  gbuf(endgbuf+1:endgbuf+infbuf(nolrowbuf,3)) = zero
	  do i=1,lvt
	    gbuf(endgbuf+colperm(index(indc+i))-irow+1) = mat(adda+i)       
	  enddo
	  endgbuf = endgbuf + infbuf(nolrowbuf,3)
	endif
      else       
        infbuf(nolrowbuf,2) = nolrow
	gbuf(endgbuf+1:endgbuf+infbuf(nolrowbuf,3)) = zero
	do i=1,lvt
	  gbuf(endgbuf+colperm(index(indc+i))-irow+1) = mat(adda+i)       
	enddo
	endgbuf = endgbuf + infbuf(nolrowbuf,3)
      endif


  999 continue

 1000 format(/' **************************************************'/
     &       ' ***  FATAL ERROR occured in LOADROP:           ***')
 1010 format(' ***     The buffer used within the             ***'/
     &       ' ***     elimination process is too small       ***'/
     &       ' ***     on processor ',i6,' !                  ***')
 1020 format(' **************************************************')

      R E T U R N
!------END OF LOADROP----------------------------------------------
      E N D                                   
