!**********************************************************************
!        1         2         3         4         5         6         7*
!**********************************************************************
!**                                                                 ***
!**                                                                 ***
      subroutine LOADSYM(mat,lmat,index,lindex,info,ia1,gbuf,lbuf,
     *                   infbuf,ia1buf,maxcol,l,endgbuf,irow,
     *                   norows,ilin,ierr)
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**   L O A D S Y M  loads the row <irow> of the matrix <mat>       ***
!**                  into the buffer <gbuf> and the corresponding   ***
!**                  information into <infbuf(1,1:ia2buf)>  on      ***
!**                  1 processor.                                   ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**    Copyright (C) 2001 University Karlsruhe                      ***
!**    PROGRAMMER: H. Haefner                                       ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**     THE IMPLICIT NONE-INSTRUCTION HAS THE AIM,                  ***
!**     THAT ALL VARIABLES MUST BE DECLARED EXPLICITLY.             ***
!**                                                                 ***
      USE constants

      implicit none
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**      FORMAL PARAMETERS :                                        ***
!**      -----------------                                          ***
!**                                                                 ***
      integer, intent(inout) ::  endgbuf,norows
      integer, intent(in) ::  ia1,ia1buf,irow
      integer, intent(in) ::  l,lbuf,lindex,lmat
      integer, intent(out) ::  ierr
      integer, intent(in) ::  maxcol(l),ilin(nilin)
      integer, intent(in) ::  index(lindex),info(ia1,ia2LU)
      integer, intent(inout) ::  infbuf(ia1buf,ia2buf)
      double precision, intent(in) :: mat(lmat)
      double precision, intent(inout) :: gbuf(lbuf)
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**   LIST OF FORMAL PARAMETERS :                                   ***
!**   -------------------------                                     ***
!**                                                                 ***
!**   (see Manual-Pages <linsol>)                                   ***
!**                                                                 ***
!**                                                                 ***
!**      LOCAL PARAMETERS :                                         ***
!**      -----------------                                          ***
!**                                                                 ***
      integer    i,iermax,ihelp,inext,ixperm
      integer    adda,indc,lvt,lout,maxix
      integer, parameter ::  ione = 1
      double precision, parameter ::  zero = 0.0

!
!**** START OF CALCULATION :
!     ---------------------
!

!
!***  Information array <info> must be sorted increasingly! The pointers
!***  to the matrix array <mat> must be sorted increasingly, too!
!

      ierr  = 0

      lout   = ilin(12)

!
!***  <inext> is the index number of <info> with row number <nolrow>.
!
      inext = irow

      adda = info(inext,1)
      lvt  = info(inext,2)
      indc = adda

!
!***  Load matrix row <irow> into buffer <gbuf> at location
!**   <endgbuf> +1..<infbuf(1,3)>
!
      infbuf(1,1) = endgbuf
      infbuf(1,2) = 0
      infbuf(1,3) = maxcol(irow) - irow + 1
      norows = norows + 1
      if (endgbuf+infbuf(1,3) > lbuf) then
	call GCBUF(gbuf,lbuf,infbuf,ia1buf,norows,endgbuf)
	if (endgbuf+infbuf(1,3) > lbuf) then
	  ierr = 9001
          write(lout,1000)
          write(lout,1010) ione
          write(lout,1020)
	  goto 999
	else
	  infbuf(1,1) = endgbuf
          infbuf(1,2) = irow
	  gbuf(endgbuf+1:endgbuf+infbuf(1,3)) = zero
	  do i=1,lvt
	    gbuf(endgbuf+index(indc+i)-irow+1) = mat(adda+i)       
	  enddo
	  endgbuf = endgbuf + infbuf(1,3)
	endif
      else       
        infbuf(1,2) = irow
	gbuf(endgbuf+1:endgbuf+infbuf(1,3)) = zero
	do i=1,lvt
	  gbuf(endgbuf+index(indc+i)-irow+1) = mat(adda+i)       
	enddo
	endgbuf = endgbuf + infbuf(1,3)
      endif


  999 continue

 1000 format(/' **************************************************'/
     &       ' ***  FATAL ERROR occured in LOADSYM:           ***')
 1010 format(' ***    the buffer used within the              ***'/
     &       ' ***    elimination process is too small        ***'/
     &       ' ***    on processor ',i6,' !                   ***')
 1020 format(' ********************************************i*****')

      R E T U R N
!------END OF LOADROW----------------------------------------------
      E N D                                   
