!:::      ,,,,,LSOLP.....
!
!
!
!**********************************************************************
!        1         2         3         4         5         6         7*
!**********************************************************************
!**                                                                 ***
!**                                                                 ***
      subroutine LSOLP(lmat,lindex,l,nproc,lsym,ia1,info,mat,x,
     &                 b,index,rlin,ilin,lmatbk,tids,iconv,ierr)
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**      L S O L P   performs the logical distribution of matrix    ***
!**                  <MAT>, sorts the array <info> and computes     ***
!**                  the array <ptrinf> and then calls LSOLPP.      ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**    Copyright (C) 1997 University Karlsruhe                      ***
!**    PROGRAMMER: H. Haefner                                       ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
      USE constants
      USE bando
      USE sspbo
      USE rcmck
      USE rcmcks
      USE precsave
      USE scriptparms
      USE randperm

      implicit none
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**      FORMAL PARAMETERS :                                        ***
!**      -----------------                                          ***
!**                                                                 ***
      integer,intent(in) ::  lmat,lindex,l,nproc
      integer,intent(inout) ::  ia1
      integer,intent(out) ::  iconv,ierr
      integer,intent(inout) ::  lmatbk(nproc),tids(maxnpe)
      integer,intent(inout) ::  index(lindex),info(ia1,ia2),ilin(nilin)
      logical,intent(in) ::  lsym
      double precision,intent(inout) :: mat(lmat),x(l),b(l),rlin(nrlin)
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**   LIST OF FORMAL PARAMETERS :                                   ***
!**   -------------------------                                     ***
!**                                                                 ***
!**   (see Manual-Pages <linsol>)                                   ***
!**                                                                 ***
!**                                                                 ***
!**      LOCAL PARAMETERS :                                         ***
!**      -----------------                                          ***
!**                                                                 ***
      integer            i,j,lout,nvt,myproc,liw,ldw,lprec,iertag
      integer            idoku,ityp,ivt,liprec,nmsg,iermax,ier
      integer            startmat,endmat,endind,endpmat
      integer            csepfc,isprec,maxnvp,nvtold,optim,bwoalg
      integer            lmatnew,lindnew,ia1new,ihelp(6*nproc)
      integer            lmath,lindh,minlen
      integer            to,from,mids,midr,nrhs
      integer            lglobal,lpmat,lpix,mxlvt,MAXLVT,MXLVTLU
      integer            bwlow,bwup,bwh,gbwlow,gbwup,lg,lrest
      integer            lenU,lenL,nvtLU,nvtL,nvtU
      integer            ia1LU,label,lmyblk,ierr1,ierr2
      integer            lmatwrp(nproc),ptrmbk(nproc+1)
      real               sparsefac,gaussfac,pfac,rmax
      double precision   thrshld1(l)
      double precision   y(l)
      logical            lhelp,lmain,lmix,lfel,lbwopt,ltry,lopen,root
      logical            lonly9,lalloc
!
!**                                                                 ***
!**   LIST OF LOCAL PARAMETERS :                                    ***
!**   -------------------------                                     ***
!
!--------I------I-----I------------------------------------------------
! NAME   I TYPE I I/O I     MEANING
!--------I------I-----I------------------------------------------------
!--------I------I-----I------------------------------------------------
! ptrinf I  R   I out I  points to the array info. The array info
!        I      I     I  must be sorted.   array: ptrinf(ntyp+1,nproc)
!--------I------I-----I------------------------------------------------
!
      integer,          allocatable :: ptrinf(:,:),iw(:)
      double precision, allocatable :: dw(:)
!     integer,pointer :: indnew(:)
!     integer,dimension(:,:),pointer :: infnew
!     double precision,pointer :: matnew(:)
      integer, parameter ::  ione = 1
      real, parameter ::     rone = 1.0
      double precision, parameter ::  dotone = 0.1,one = 1.0,zero = 0.0
      integer          :: lgraph
      integer          :: ibufsize,runacc
      integer*8        :: opcount,gopcnt,ihlp8
      double precision :: memopt,threshold
      double precision :: tlsolp,telsolp,topt,teopt,tbwo,tebwo,tbwop
      double precision :: tebwop,telim,teelim,thlp,tehlp
      double precision :: tpelim,tepelim,tdata,tedata
      integer,allocatable,dimension(:) :: graph,vertix
      character*72     :: tmpdir

!     interface
!      subroutine LSPCVT(mat,lmat,index,lindex,l,info,nvt,ia1,lsym,
!    &                   matnew,lmatnew,indnew,lindnew,infnew,ia1new,
!    &                   lmatbk,ptrmbk,myproc,nproc,tids,ilin,nmsg,ierr)
!       implicit none
!       integer,intent(in) :: lmat,lindex,l,ia1,nvt,nproc,myproc
!       integer,intent(inout) :: ierr,nmsg
!       integer,intent(out) :: lmatnew,lindnew,ia1new
!       integer,intent(in) :: info(ia1,ia2),index(lindex),tids(nproc)
!       integer,intent(in) :: ilin(nilin)
!       integer,intent(in) :: lmatbk(nproc),ptrmbk(nproc+1)
!       double precision,intent(in) :: mat(lmat)
!       integer,pointer :: indnew(:)
!       integer,dimension(:,:),pointer :: infnew
!       double precision,pointer :: matnew(:)
!       logical,intent(in) :: lsym
!      end subroutine
!     end interface
!
!
!**** START OF CALCULATION :
!     ---------------------
!
      if (nproc == 1) then
	ilin(17) = 1
	lmatbk(1) = l
	tids(1) = 9999
      endif

      nmsg   = ilin(3)
      nrhs   = ilin(4)
      nvt    = ilin(5)
      isprec = ilin(6)
      lout   = ilin(12)
      idoku  = ilin(13)
      myproc = ilin(17)
      optim  = ilin(20)
      runacc = ilin(21)
      bwoalg = ilin(22)

      lmyblk = lmatbk(myproc)

      ierr = 0

      if ((MOD(optim,10) == 1) .and. (myproc == 1)) then
        call SECONDS(tlsolp,telsolp)
      endif

      if ((idoku > 0) .and. (myproc == 1)) then
        write(lout,1000)
        write(lout,1001)
        include 'nproc.h'
        write(lout,1002)
      endif
      if (idoku < 0) then
        write(lout,1000)
        write(lout,1001)
        include 'nproc.h'
        write(lout,1002)
      endif

      ptrmbk(1) = 0
      do i=1,nproc
        ptrmbk(i+1) = ptrmbk(i)+lmatbk(i)
      enddo
!***  ilin(36) is the global dimension of the lin. system (lglobal) ***
      lglobal  = ptrmbk(nproc+1)
      ilin(36) = lglobal

!**
!***  Check of parameters concerning matrix and number of proc.
!**
      call LSPCHK(lmat,lindex,l,nproc,nvt,info,ia1,ilin,ierr,tids,
     &            lmatbk,lmain,lsym)
!     if (ierr /= 0 .and. ierr < 8095) then
      if (ierr /= 0) then
	iconv = 99
        if (myproc == 1) then
          write(lout,1020)
          write(lout,1021)
          write(lout,1030)
	endif
        goto 999
      endif

!**
!***  Check storage patterns 1 and 9 of matrix
!**
      call LSPCHK2(mat,lmat,index,lindex,info,ia1,nvt,ilin,l,
     &             lmatbk(myproc),ptrmbk(myproc),lmain,lmix,lonly9,
     &             tids,nproc,ierr)
      if (ABS(ierr) == 1) then
        if (myproc == 1) then
          write(lout,1040)
          if (ierr == -1) write(lout,1219)
          if (ierr == 1) write(lout,1319)
          write(lout,1030)
	endif
	ierr = 0
      endif
      if (ABS(ierr) > 1) then
	iconv = 99
        if (myproc == 1) then
          write(lout,1020)
          if (ierr < 8020) write(lout,1019)
          if (ierr == 8020) write(lout,1119)
          write(lout,1030)
	endif
        goto 999
      endif

      if (nrhs == -1) then
        call SET_RHS_IGUESS(MAT,lmat,index,lindex,lsym,info,ia1,nvt,
     &                      b,l,lmatbk,ptrmbk,
     &                      ilin,tids,nproc,myproc,nmsg)
!       else
!         call COMP_RHS_IGUESS(MAT,lmat,index,lindex,lsym,info,ia1,
!    &                         b,l,lmatbk,ptrmbk,ptrinf,
!    &                         ierr,tids,nproc,myproc,nmsg)
!       endif
      endif


      if (MOD(isprec,100) >= 10) then
        if  (.not. ALLOCATED(prec) .or. .not. ALLOCATED(iprec)) then
	  ierr = 7777
	  iconv = 99
          if (myproc == 1) then
            write(lout,1020)
            write(lout,1037)
            write(lout,1030)
	  endif
          goto 999
	endif
      endif


      if (((ilin(8) >= 63) .and. (MOD(isprec,100) < 10)) .or.
     &    ((MOD(optim,200000) >= 100000) .and. (MOD(isprec,1000) < 100))
     &    .or.((MOD(optim,1000000) >= 200000) .and.
     &         ((MOD(isprec,10000) < 1000) .or.
     &          (MOD(isprec,10000) >= 2000)))) then
        if (MOD(optim,10000) < 1000) then
          optim = optim + 1000
          if (idoku > 0) then
            if (myproc == 1) write(lout,1060)
          endif
          if (idoku < 0) write(lout,1060)
	endif
        if (MOD(optim,100000) < 10000) then
          optim = optim + 10000
          if (idoku > 0) then
            if (myproc == 1) write(lout,1061)
          endif
          if (idoku < 0) write(lout,1061)
	endif
!       if (MOD(optim,200000) < 100000) then
!         optim = optim + 100000
!         if (idoku > 0) then
!           if (myproc == 1) write(lout,1062)
!         endif
!         if (idoku < 0) write(lout,1062)
!       endif
      endif

      if (ilin(8) < 63) then
        lprec  = 5*l
        liprec = 1

        ierr1 = 0
        ierr2 = 0

        if (ALLOCATED(iprec)) then
          deallocate(iprec)
          allocate(iprec(liprec),stat=ierr2)
        endif
        if (.not. ALLOCATED(iprec)) then
          allocate(iprec(liprec),stat=ierr2)
        endif

        if (MOD(isprec,10) == 0) then
          if (ALLOCATED(prec)) deallocate(prec)
          allocate(prec(lprec),stat=ierr1)
        endif
        if (MOD(isprec,10) == 1) then
          if (.not. ALLOCATED(prec)) then
	    lalloc = .false.
	    allocate(prec(lprec),stat=ierr1)
	  else
	    lalloc = .true.
          endif
        endif
        if (ierr1 /= 0) ierr1 = lprec
	ihelp(1) = ierr1
        if (ierr2 /= 0) ierr2 = liprec
	ihelp(2) = ierr2
        call LL4INM(1,2,1,ihelp(1),ihelp(3),ihelp(5),myproc,nproc,
     &              tids,nmsg)
	ierr1 = ihelp(3)
	ierr2 = ihelp(4)
        if (ierr1 /= 0) then
	  ierr  = ierr1
	  iconv = 99
          if (myproc == 1) then
            write(lout,1020)
            write(lout,1065) ierr1*ireal
            write(lout,1030)
	  endif
        endif
        if (ierr2 /= 0) then
	  ierr  = ierr2
	  iconv = 99
          if (myproc == 1) then
            write(lout,1020)
            write(lout,1066) ierr2*iint
            write(lout,1030)
	  endif
        endif
        if (ierr /= 0) goto 999
      endif

      ierr   = 0
      iertag = 0
      iconv  = 0

!***********************************************************************
!**    Allocation of work array iw                                   ***
!***********************************************************************

      csepfc = int(3.5*l/cusfac)
      maxnvp = csepfc/cachsz
      if (MOD(csepfc,cachsz) /= 0) maxnvp = maxnvp+1
      liw = MAX(2*nproc,lglobal)  ! <-- VTSORT
!     liw = 2*nproc
      liw = MAX(liw,ia1*(ia2+1))
      if (lmix) liw = MAX(liw,l)
      if (MOD(optim,1000) >= 100) then
	liw = MAX(liw,maxnvp*nvt*ia2)
        if (optim == 200) then
	  liw = MAX(liw,4*l)
        endif
      endif
!     if (MOD(optim,1000000) >= 100000 .or. lmix) then
!       liw = MAX(liw,ia1*(ia2+1))
!     endif
      if (ilin(8) >= 63) then
	if (MOD(isprec,100) < 10) then
	  liw = MAX(liw,ia1*(ia2+1))
	  if (nproc == 1) liw = MAX(liw,l*2)
	  if (nproc > 1) liw = MAX(liw,lglobal*MAX(ia2LU+1,2))
          if (MOD(optim,100) >= 10) liw = MAX(liw,3*lglobal)
        endif
      endif
!     if (idoku > 0) then
!       if (myproc == 1) write(lout,1014) liw*iint
!     endif
!     if (idoku < 0) write(lout,1014) liw*iint
      allocate (iw(liw),stat=ierr)

!***********************************************************************
!**    Allocation of work array dw                                   ***
!***********************************************************************

      if ((ilin(1) == 1) .or. (ilin(1) == 200) .or.
     &    (ilin(1) == 11) .or. (ilin(1) == 201)) ldw = 22*l
      if ((ilin(1) == 2) .or. (ilin(1) == 4) .or. (ilin(1) == 6)
     &    .or. ((ilin(1) == 101) .and. lsym) .or.
     &    ((ilin(1) == 220) .and. lsym) .or. ((ilin(1) == 221)
     &    .and. lsym)) ldw = 18*l
      if (ilin(1) == 3) ldw = 18*l
      if (ilin(1) == 5) ldw = 19*l
      if (ilin(1) == 7) ldw = 22*l
      if ((ilin(1) == 9) .or. (ilin(1) == 100) .and. lsym) ldw = 17*l
      if (ilin(1) == 100 .and. .not. lsym) then
       ldw = 22*l
       if (idoku > 0) then
         if (myproc == 1) then
	   write(lout,1040)
	   write(lout,1041)
	   write(lout,1042)
	   write(lout,1050)
         endif
       endif
       if (idoku < 0) then
	 write(lout,1040)
	 write(lout,1041)
	 write(lout,1042)
	 write(lout,1050)
       endif
      endif
      if ((ilin(1) == 101 .and. .not. lsym) .or. (ilin(1) == 220
     &    .and. .not. lsym) .or. (ilin(1) == 221 .and. .not.
     &    lsym)) then
       ldw = 24*l
       if (idoku > 0) then
         if (myproc == 1) then
	   write(lout,1040)
	   write(lout,1041)
	   write(lout,1043)
	   write(lout,1050)
         endif
       endif
       if (idoku < 0) then
	 write(lout,1040)
	 write(lout,1041)
	 write(lout,1043)
	 write(lout,1050)
       endif
      endif
      if (ilin(1) == 15 .or. ilin(1) == 202 .or. ilin(1) == 203
     &    .or. ilin(1) == 210 .or. ilin(1) == 211) ldw = 24*l
      if ((ilin(1) == 16) .or. (ilin(1) == 204) .or.
     &    (ilin(1) == 212) .or. (ilin(1) == 999)) ldw = 54*l
!     ldw = MAX(ldw,lglobal)  <-- VTSORT

!     if (idoku > 0) then
!       if (myproc == 1) write(lout,1015) ldw*ireal
!     endif
!     if (idoku < 0) write(lout,1015) ldw*ireal
      allocate (dw(ldw),stat=ierr)
      ihelp(1) = ierr
      call LL4INM(1,1,1,ihelp(1),ihelp(2),ihelp(3),myproc,nproc,
     &            tids,nmsg)
      ierr = ihelp(2)
      if (ierr /= 0) then
        iconv = 99
        if (myproc == 1) then
          write(lout,1020)
          write(lout,1072)
          write(lout,1030)
	endif
        goto 999
      endif

!**
!***  Eliminate zero entries in matrix
!**

      if (MOD(optim,1000000) >= 1000) then
        if ((MOD(optim,10) == 1) .and. (myproc == 1)) then
          call SECONDS(topt,teopt)
        endif
      endif

      if (MOD(optim,10000) >= 1000 .and.
     &    (MOD(isprec,10000) < 10 .or. MOD(isprec,10000) >= 2000)) then
        memopt = zero
        lhelp=.false.
        call PACK_MAT(mat,lmat,index,lindex,info,ia1,ilin,memopt,
     &                lhelp,lmatbk,ptrmbk,nproc,tids,ierr)
	nvt = ilin(5)
        if (ierr /= 0) then
	  iconv = 99
          if (myproc == 1) then
            write(lout,1020)
            write(lout,1051)
            write(lout,1030)
	  endif
          goto 999
        endif
      endif

!**
!***  Restore matrix <mat> in storage pattern 'packed row'
!**

      if (MOD(optim,100000) >= 10000 .and.
     &    (MOD(isprec,10000) < 10 .or. MOD(isprec,10000) >= 2000)) then
	threshold = zero
	lmain = .false.
        call LSPCVT_INPL(mat,lmat,index,lindex,lglobal,info,ia1,
     &                   lsym,ilin,lmatbk(myproc),ptrmbk(myproc),
     &                   myproc,nproc,tids,lmain,threshold,ierr)
        if (ierr /= 0) then
	  iconv = 99
          if (myproc == 1) then
            write(lout,1020)
            write(lout,1029) lmatbk(myproc),ia1
            write(lout,1030)
	  endif
          goto 999
        endif
      endif

!**
!***  Sort indices of matrixelements for all packed rows
!**

      if (lonly9) then
        call VTSORT(mat,lmat,index,lindex,info,ia1,dw,iw,lglobal,nvt)

        do i=2,nvt
          if (info(i,5) < info(i-1,5)) then
            call INFOSORT(info,ia1,nvt,iw,lout,myproc,nproc,
     &                    tids,nmsg,ierr)
            exit
          endif
        enddo
      endif

!**
!***  Eliminate dirichlet boundary values in a forward
!***  elimination
!**

      if (MOD(optim,200000) >= 100000 .or. MOD(isprec,1000) >= 100) then

	if ((ilin(14) == 1) .and. (MOD(isprec,1000) < 100)) then
	  ier = 0
	  call getKeyword('TMPDIR',tmpdir,ier)
	  if (ier == 0) then
	    label = 0
	    goto 666
	  endif
	  ier = 0
  661     call getKeyword('TMP',tmpdir,ier)
	  if (ier == 0) then
	    label = 1
	    goto 666
	  endif
	  ier = 0
  662     call getKeyword('TEMP',tmpdir,ier)
	  if (ier == 0) then
	    label = 2
	    goto 666
	  endif
	  ier = 0
  663     call getKeyword('TEMP_LOCAL',tmpdir,ier)
	  if (ier == 0) then
	    label = 3
	    goto 666
	  endif
	  ier = 0
  664     call getKeyword('WORK',tmpdir,ier)
	  if (ier == 0) then
	    label = 4
	    goto 666
	  endif

  665     call getKeyword('PWD',tmpdir,ier)
          label = 5

  666     write(prefile,2000) myproc
 2000     format('/prefel.dat_',i4.4)
          prefile = TRIM(tmpdir) // TRIM(prefile)
		 
	  if (idoku > 0) then
	    if (myproc == 1) then
	      write(lout,1300) prefile,ilin(45)
	    endif
	  endif
	  if (idoku < 0) write(lout,1300) prefile,ilin(45)

          open(ilin(45),FILE=prefile,IOSTAT=ierr,STATUS='REPLACE',
     &         FORM='UNFORMATTED')

          ihelp(1) = ierr
          call LL4INM(1,1,1,ihelp(1),ihelp(2),ihelp(3),
     &                myproc,nproc,tids,nmsg)
	  iermax = ihelp(2)
	  if (iermax > 0) then
	    if (label < 5) goto (661,662,663,664,665) label+1
	    ierr  = iermax
	    iconv = 99
            if (myproc == 1) then
              write(lout,1020)
              write(lout,1081)
              write(lout,1030)
	    endif
            goto 999
	  endif
	endif

	if (MOD(isprec,1000) >= 100) then
	  if (idoku > 0) then
	    if (myproc == 1) then
              write(lout,1301) prefile,ilin(45)
	    endif
	  endif
	  if (idoku < 0) write(lout,1301) prefile,ilin(45)

          open(ilin(45),FILE=prefile,IOSTAT=ierr,STATUS='OLD',
     &         FORM='UNFORMATTED')

          ihelp(1) = ierr
          call LL4INM(1,1,1,ihelp(1),ihelp(2),ihelp(3),
     &                myproc,nproc,tids,nmsg)
	  iermax = ihelp(2)
	  if (iermax > 0) then
	    ierr  = iermax
	    iconv = 99
            if (myproc == 1) then
              write(lout,1020)
              write(lout,1081)
              write(lout,1030)
	    endif
            goto 999
	  endif
	endif

        call PREFEL(b,mat,lmat,index,lindex,info,ia1,dw,l,iw,liw,
     &              lsym,ilin,lmatbk,ptrmbk,
     &              nproc,tids,iconv,ierr)
        if (iconv == 99) then
          if (myproc == 1) then
            write(lout,1020)
            write(lout,1052)
            write(lout,1030)
	  endif
          goto 999
        endif
        if (ierr /= 0) then
          if (ierr == 6002) then
            iconv = 4
          else
            iconv = 99
          endif
          if (myproc == 1) then
            if (iconv == 4) then
              write(lout,1020)
              write(lout,1031)
              write(lout,1030)
            else
              write(lout,1020)
              write(lout,1032)
              write(lout,1030)
            endif
          endif
          goto 999
        endif
	INQUIRE(ilin(45),OPENED=lopen)
	if (lopen) close(ilin(45))
      endif
      nvt = ilin(5)   ! this is a precaution

      if (MOD(optim,1000000) >= 1000) then
        if ((MOD(optim,10) == 1) .and. (myproc == 1)) then
          call SECONDS(thlp,tehlp)
          topt  = thlp - topt
          teopt = tehlp - teopt
          write(lout,1100)
          write(lout,1103) topt
          write(lout,1104) teopt
        endif
      endif

      if ((MOD(optim,100) >= 10) .or. (ilin(8) >= 63)) then
        lbwopt = .false.
        call BANDWIDTH(index,lindex,info,ia1,nvt,ilin,tids,nproc,
     &                 bwlow,bwup,gbwlow,gbwup,lbwopt,lsym)
      endif

      if (MOD(optim,1000000) >= 200000 .and.
     &    ALLOCATED(perm) .and. (MOD(isprec,10000) >= 1000)) then

        if (idoku > 0) then
          if (myproc == 1) then
            write(lout,1200)
            write(lout,1225)
            write(lout,1200)
          endif
        endif
        if (idoku < 0) then
          write(lout,1200)
          write(lout,1225)
          write(lout,1200)
        endif

        if ((MOD(optim,10) == 1) .and. (myproc == 1)) then
          call SECONDS(tbwop,tebwop)
        endif

        call RHSIXSRT(b,perm,l,lmatbk,ptrmbk,myproc,nproc,tids,nmsg)
	if (ilin(16) == 4711) then
          call RHSIXSRT(x,perm,l,lmatbk,ptrmbk,myproc,nproc,tids,nmsg)
	endif

        if (MOD(isprec,10000) >= 2000) then
          call ROWSORT(mat,lmat,index,lindex,info,ia1,nvt,perm,l,
     &                 iw,ilin,ptrmbk,myproc,nproc,tids,nmsg,iconv,ierr)       
          if (iconv == 99) then
            if (myproc == 1) then
              write(lout,1020)
              write(lout,1052)
              write(lout,1030)
            endif
            goto 999
          endif
          ibufsize=MAX(MAXLVT(info, ia1, nvt),l)
          call COLSORT(mat,lmat,index,lindex,info,ia1,perm,l,ibufsize,
     &                 lmatbk,ptrmbk,myproc,nproc,tids,nmsg)
        endif

        if ((MOD(optim,10) == 1) .and. (myproc == 1)) then
          call SECONDS(thlp,tehlp)
          tbwop  = thlp - tbwop
          tebwop = tehlp - tebwop
          write(lout,1100)
          write(lout,1105) tbwop
          write(lout,1106) tebwop
        endif

        lbwopt = .true.
        if ((MOD(optim,100) >= 10) .or. (ilin(8) >= 63)) then
          call BANDWIDTH(index,lindex,info,ia1,nvt,ilin,tids,nproc,
     &                   bwlow,bwup,gbwlow,gbwup,lbwopt,lsym)
        endif

        if (idoku > 0) then
          if (myproc == 1) then
            write(lout,1200)
            write(lout,1214)
            write(lout,1250)
          endif
        endif
        if (idoku < 0) then
          write(lout,1200)
          write(lout,1214)
          write(lout,1250)
        endif
      endif

      if (MOD(optim,1000000) >= 200000 .and.
     &    (.not. ALLOCATED(perm) .or. (MOD(isprec,10000) < 1000))) then

!***
!***  GPS is the default!
!***
        if (bwoalg == 0) bwoalg = 1

        if (idoku > 0) then
          if (myproc == 1) then
            write(lout,1200)
            if (bwoalg == 1) write(lout,1215)
            if (bwoalg == 2) write(lout,1216)
            if (bwoalg == 3) write(lout,1217)
            if (bwoalg == 4) write(lout,1218)
            write(lout,1200)
          endif
        endif
        if (idoku < 0) then
          write(lout,1200)
          if (bwoalg == 1) write(lout,1215)
          if (bwoalg == 2) write(lout,1216)
          if (bwoalg == 3) write(lout,1217)
          if (bwoalg == 4) write(lout,1218)
          write(lout,1200)
        endif

        if ((MOD(optim,10) == 1) .and. (myproc == 1)) then
          call SECONDS(tbwop,tebwop)
        endif

        if (.not. ALLOCATED(perm)) then
           allocate(perm(l),stat=ierr)
           ihelp(1) = ierr
           call LL4INM(1,1,1,ihelp(1),ihelp(2),ihelp(3),myproc,nproc,
     &          tids,nmsg)
           ierr = ihelp(2)
           if (ierr /= 0) then
              iconv = 99
              if (myproc == 1) then
                 write(lout,1020)
                 write(lout,1075)
                 write(lout,1030)
              endif
              goto 999
           endif
        endif

        if (bwoalg == 1) then
!***  Gibbs-Poole-Stockmeyer (GPS) algorithm

           lgraph = 2*lindex
           allocate(graph(lgraph),vertix(l+1),stat=ierr)
           ihelp(1) = ierr
           call LL4INM(1,1,1,ihelp(1),ihelp(2),ihelp(3),myproc,nproc,
     &          tids,nmsg)
           ierr = ihelp(2)
           if (ierr /= 0) then
              iconv = 99
              if (myproc == 1) then
                 write(lout,1020)
                 write(lout,1073)
                 write(lout,1030)
              endif
              goto 999
           endif

           call PREPRO(index,lindex,info,ia1,graph,lgraph,vertix,l,
     &                 lglobal,ilin,ptrmbk,lmatbk,tids,nproc,ierr)
           if (ierr /= 0) then
              iconv = 99
              if (myproc == 1) then
                 write(lout,1020)
                 write(lout,1074)
                 write(lout,1030)
              endif
              goto 999
           endif


           if ((MOD(optim,10) == 1) .and. (myproc == 1)) then
              call SECONDS(tbwo,tebwo)
           endif

           call BANDOPTIM(graph,vertix,lmatbk,perm,idoku,lout,
     &                    myproc,nproc,tids,nmsg)


           if ((MOD(optim,10) == 1) .and. (myproc == 1)) then
              call SECONDS(thlp,tehlp)
              tbwo  = thlp - tbwo
              tebwo = tehlp - tebwo
              write(lout,1100)
              write(lout,1107) tbwo
              write(lout,1108) tebwo
           endif

           deallocate(graph,vertix)

           call RHSIXSRTEST(b,perm,l,lmatbk,ptrmbk,myproc,nproc,tids,
     &                      nmsg,lout,ierr)
           if (ierr /= 0) then
              if (myproc == 1) then
                 write(lout,1020)
                 write(lout,1055)
                 write(lout,1030)
              endif
              goto 999
           endif

           if (ilin(16) == 4711) then
              call RHSIXSRT(x,perm,l,lmatbk,ptrmbk,myproc,nproc,tids,
     &             nmsg)
           endif

           call ROWSORT(mat,lmat,index,lindex,info,ia1,nvt,perm,l,
     &          iw,ilin,ptrmbk,myproc,nproc,tids,nmsg,iconv,ierr)
           if (iconv == 99) then
              if (myproc == 1) then
                 write(lout,1020)
                 write(lout,1052)
                 write(lout,1030)
              endif
              goto 999
           endif

           ibufsize=MAX(MAXLVT(info, ia1, nvt),l)
           call COLSORT(mat,lmat,index,lindex,info,ia1,perm,l,ibufsize,
     &          lmatbk,ptrmbk,myproc,nproc,tids,nmsg)


        elseif (bwoalg == 2) then
!***
!***  Single pass optimizer
!***
           call SSPBOPTIM(mat,lmat,b,perm,index,lindex,info,ia1,lsym,l,
     &          ilin,lmatbk,myproc,nproc,tids,nmsg,iconv,ierr)
           if (iconv == 99) then
              if (myproc == 1) then
                 write(lout,1020)
                 write(lout,1054)
                 write(lout,1030)
              endif
              goto 999
           endif

           call RHSIXSRT(b,perm,l,lmatbk,ptrmbk,myproc,nproc,tids,
     &          nmsg)
           if (ilin(16) == 4711) then
              call RHSIXSRT(x,perm,l,lmatbk,ptrmbk,myproc,nproc,tids,
     &             nmsg)
           endif

           call ROWSORT(mat,lmat,index,lindex,info,ia1,nvt,perm,l,
     &          iw,ilin,ptrmbk,myproc,nproc,tids,nmsg,iconv,ierr)
           if (iconv == 99) then
              if (myproc == 1) then
                 write(lout,1020)
                 write(lout,1052)
                 write(lout,1030)
              endif
              goto 999
           endif

!	  iunit = 25
!         call LINOUT(mat,lmat,index,lindex,info,ia1,nvt,
!    &                l,lsym,ilin,iunit,ierr)

           ibufsize=MAX(MAXLVT(info, ia1, nvt),l)
           call COLSORT(mat,lmat,index,lindex,info,ia1,perm,l,ibufsize,
     &          lmatbk,ptrmbk,myproc,nproc,tids,nmsg)


        elseif (bwoalg == 3) then
!***
!***  Reverse Cuthill McKee algorithm
!***
           call RCMCKOPTIM(mat,lmat,b,perm,index,lindex,info,ia1,lsym,
     &          l,ilin,lmatbk,myproc,nproc,tids,nmsg)


        elseif (bwoalg == 4) then
!***
!***  Reverse Cuthill McKee algorithm (without iteration)
!***
           call RCMCKOPTIMS(mat,lmat,b,perm,index,lindex,info,ia1,lsym,
     &          l,ilin,lmatbk,myproc,nproc,tids,nmsg)


        elseif (bwoalg == 5) then
!***
!***  Random permutation for test purposes
!***
           call RANDPERMUT(graph, vertix, lmatbk, perm,
     &          myproc, nproc, tids, nmsg)
           call RHSIXSRT(b,perm,l,lmatbk,ptrmbk,myproc,nproc,tids,
     &          nmsg)
           if (ilin(16) == 4711) then
              call RHSIXSRT(x,perm,l,lmatbk,ptrmbk,myproc,nproc,tids,
     &             nmsg)
           endif
           call ROWSORT(mat,lmat,index,lindex,info,ia1,nvt,perm,l,
     &          ilin,ptrmbk,myproc,nproc,tids,nmsg,iertag,iconv,ierr)
           if (ierr /= 0) then
              if (myproc == 1) then
                 write(lout,1020)
                 write(lout,1052)
                 write(lout,1030)
              endif
              goto 999
           endif
           ibufsize=max(MAXLVT(info, ia1, nvt),l)
           call COLSORT(mat,lmat,index,lindex,info,ia1,perm,l,ibufsize,
     &          lmatbk,ptrmbk,myproc,nproc,tids,nmsg)

        endif  ! Bandwidth optimization algorithm selection

        if ((MOD(optim,10) == 1) .and. (myproc == 1)) then
          call SECONDS(thlp,tehlp)
          tbwop  = thlp - tbwop
          tebwop = tehlp - tebwop
          write(lout,1100)
          write(lout,1105) tbwop
          write(lout,1106) tebwop
        endif

        lbwopt = .true.
        if ((MOD(optim,100) >= 10) .or. (ilin(8) >= 63)) then
          call BANDWIDTH(index,lindex,info,ia1,nvt,ilin,tids,nproc,
     &                   bwlow,bwup,gbwlow,gbwup,lbwopt,lsym)
        endif

        if (idoku > 0) then
          if (myproc == 1) then
            write(lout,1200)
            write(lout,1214)
            write(lout,1250)
          endif
        endif
        if (idoku < 0) then
          write(lout,1200)
          write(lout,1214)
          write(lout,1250)
        endif

      endif   ! Bandwidth optimization


!**                                                                 ***
!***  Printing of vector terms on unit <ilin(41)>                   ***
!**                                                                 ***

! 777 continue

      if ((MOD(ilin(19),10) == 1) .or. (MOD(ilin(19),10) == 3)) then
        if (ilin(13) > 0) then
          if (myproc == 1) write(lout,1120) ilin(41)
        endif
        if (ilin(13) < 0) then
          write(lout,1120) ilin(41)
        endif
        call SHOWVT(MAT,lmat,index,lindex,info,ia1,nvt,l,lsym,
     &              myproc,nproc,lmatbk,ilin(41),ierr)
      endif

!**                                                                 ***
!***  Store the matrix <MAT> in LINSOL-format on unit <ilin(43)>    ***
!**                                                                 ***
      ihelp(1) = MOD(ilin(19),1000)
      if (ihelp(1) >= 100 .and. ihelp(1) < 200) then
        if (ilin(13) > 0) then
          if (myproc == 1) write(lout,1121) ilin(43)
        endif
        if (ilin(13) < 0) then
          write(lout,1121) ilin(43)
        endif
        call LINOUT(mat,lmat,index,lindex,info,ia1,nvt,lmyblk,lsym,
     &              ilin,ilin(43),ierr)
      endif
      if (ihelp(1) >= 200 .and. ihelp(1) < 300) then
        if (ilin(13) > 0) then
          if (myproc == 1) write(lout,1122) ilin(43)
        endif
        if (ilin(13) < 0) then
          write(lout,1122) ilin(43)
        endif
        call LINBOUT(mat,lmat,index,lindex,info,ia1,nvt,lmyblk,lsym,
     &               ilin,ilin(43),ierr)
      endif

!***********************************************************************
!**    Gauss algorithm as preconditioning                            ***
!***********************************************************************

      if (ilin(8) >= 63) then
        endmat = ilin(33)
        endind = ilin(34)

        if ((rlin(2) == 0) .and. (rlin(3) == 0) .and. (rlin(4) >= 0))
     &  then
          ltry = .true.
        else
          ltry = .false.
        endif
!**    Try first a safe Gauss-factor                                 ***
  111   if (ltry) then
	  if (lsym) then
            sparsefac = REAL(ilin(30))/
     &                  REAL((MAX(gbwlow,gbwup)+1)*lglobal)
	  else
            sparsefac = REAL(ilin(30))/REAL((gbwlow+gbwup+1)*lglobal)
	  endif
          gaussfac  = rone/sparsefac
          if (idoku > 0) then
            if (myproc == 1) then
              write(lout,1140) sparsefac
              write(lout,1141) gaussfac
            endif
          endif
          if (idoku < 0) then
            write(lout,1140) sparsefac
            write(lout,1141) gaussfac
          endif
        else
	  gaussfac = REAL(ABS(rlin(4)))
          if (gaussfac < REAL(dotone)) then
	    gaussfac = REAL(dotone)
            if (idoku > 0) then
              if (myproc == 1) then
                write(lout,1040)
                write(lout,1044) dotone
                write(lout,1050)
              endif
            endif
            if (idoku < 0) then
              write(lout,1040)
              write(lout,1044) dotone
              write(lout,1050)
            endif
          endif
        endif

        minlen = MIN(lmat,lindex)

	if (nproc == 1) then
          lprec = 5*l + INT(gaussfac*endmat)
        else
          lprec = 5*l + INT(gaussfac*minlen)
	endif
	if (lsym) then
	  if (nproc == 1) then
	    ia1LU  = l
            liprec = 15 + l + INT(gaussfac*endind) + ia1LU*ia2LU
          else
	    ia1LU  = lglobal
            liprec = 15 + lglobal + INT(gaussfac*minlen) + ia1LU*ia2LU
	  endif
	else
	  if (nproc == 1) then
	    ia1LU  = l
            liprec = 15 + l + INT(gaussfac*endind) + 2*ia1LU*ia2LU
!!!???      liprec = 15 + l + INT(gaussfac*endind) + 2*(ia1LU+10)*ia2LU
          else
	    ia1LU  = lglobal
            liprec = 15 + lglobal + INT(gaussfac*minlen) +
     &               2*ia1LU*ia2LU
	  endif
	endif

        ierr1 = 0
        ierr2 = 0

        if ((MOD(isprec,100) < 10) .and. ALLOCATED(iprec)) then
          deallocate(iprec)
          allocate(iprec(liprec),stat=ierr2)
        endif
        if ((MOD(isprec,100) < 10) .and. .not. ALLOCATED(iprec)) then
          allocate(iprec(liprec),stat=ierr2)
        endif

        if ((MOD(isprec,100) < 10) .and. (MOD(isprec,10) == 0)) then
          if (ALLOCATED(prec)) deallocate(prec)
          allocate(prec(lprec),stat=ierr1)
        endif
        if ((MOD(isprec,100) < 10) .and. (MOD(isprec,10) == 1)) then
          if (.not. ALLOCATED(prec)) then
	    lalloc = .false.
	    allocate(prec(lprec),stat=ierr1)
	  else
	    lalloc = .true.
          endif
        endif

        if (ierr1 /= 0) ierr1 = lprec
	ihelp(1) = ierr1
        if (ierr2 /= 0) ierr2 = liprec
	ihelp(2) = ierr2
        call LL4INM(1,2,1,ihelp(1),ihelp(3),ihelp(5),myproc,nproc,
     &              tids,nmsg)
	ierr1 = ihelp(3)
	ierr2 = ihelp(4)
        if ((ierr1 /= 0) .or. (ierr2 /= 0)) then
          if (ltry) then
            if (myproc == 1) then
              write(lout,1040)
              if (ierr1 /= 0) write(lout,1063) ierr1*ireal
              if (ierr2 /= 0) write(lout,1064) ierr2*iint
              write(lout,1050)
	    endif
            ltry = .false.
            goto 111
          else
	    ierr  = MAX(ierr1,ierr2)
	    iconv = 99
            if ((ierr1 /= 0) .and. (myproc == 1)) then
              write(lout,1020)
              write(lout,1065) ierr1*ireal
              write(lout,1030)
	    endif
            if ((ierr2 /= 0) .and. (myproc == 1)) then
              write(lout,1020)
              write(lout,1066) ierr2*iint
              write(lout,1030)
	    endif
            goto 999
          endif
        else
	  rlin(4) = DBLE(gaussfac)
        endif


!***                                                         ***
!***   Don't compute factorization, if already done in a     ***
!***   previous call of the routine LSOLP!!!                 ***
!***                                                         ***

        if (MOD(isprec,100) >= 10) goto 888


        if ((MOD(optim,10) == 1) .and. (myproc == 1)) then
          call SECONDS(telim,teelim)
        endif

!***   Sort array <info> so that the row numbers    ***
!***   are equivalent to the first index of <info>  ***

        call INFOSORT(info,ia1,nvt,iw,lout,myproc,nproc,
     &                tids,nmsg,ierr)
      
	if (lsym) then
	  call MATSORT(mat,lmat,info,ia1,nvt,ilin,
     &                 myproc,nproc,tids,nmsg,ierr,iconv)

	  call IXSORT(index,lindex,info,ia1,nvt,ilin,
     &                myproc,nproc,tids,nmsg,ierr,iconv)

	  call SYMMAT(mat,lmat,index,lindex,info,ia1,l,lglobal,
     &                ilin,ptrmbk,lmatbk,tids,nproc,iconv,ierr)
	  if (iconv == 99) goto 999
	endif


!***   Compute first threshold array!               ***

        call THRES1(mat,lmat,info,ia1,ilin,rlin,thrshld1,nvt,
     &              myproc,nproc,tids,nmsg)


!***   Copy matrix <mat> into the preconditioning matrix prec;   ***
!***   the preconditioning matrix has the offset                 ***
!***   (lprec-MIN(lmat,lindex)); also index array <index> must   ***
!***   be copied into the array <iprec> with offset (15+lglobal+ ***
!***    +INT(gaussfac*MIN(lmat,lindex))-MIN(lmat,lindex));       ***
!***   also information array <info> must be copied into the     ***
!***   array <iprec> with offset <iprec(13)>.                    ***

!***   Pointer to L    ***

        if (lsym) then
          if (nproc == 1) then
            iprec(13) = 15 + l + INT(gaussfac*endind)
          else
            iprec(13) = 15 + lglobal + INT(gaussfac*minlen)
          endif
	else
          if (nproc == 1) then
            iprec(13) = 15 + l + INT(gaussfac*endind) + ia1LU*ia2LU
!!!???      iprec(13) = 15 + l + INT(gaussfac*endind) + (ia1LU+10)*ia2LU
          else
            iprec(13) = 15 + lglobal + INT(gaussfac*minlen)
     &                     + ia1LU*ia2LU
          endif
        endif

        call COPY(mat,lmat,index,lindex,l,info,iprec(iprec(13)+1),ia1,
     &            ia1LU,ilin,rlin,thrshld1,gaussfac,nvt,nvtLU,prec,
     &            lprec,lpmat,iprec,liprec,lpix,startmat,endpmat,lsym,
     &            myproc,nproc,tids,nmsg,ierr)
	if (ierr == 8888) then
	  iconv = 99
	  write(lout,1020)
	  write(lout,1033)
	  write(lout,1030)
	  goto 999
	endif
	if (ierr == 9999) then
	  iconv = 99
	  write(lout,1020)
	  write(lout,1034)
	  write(lout,1030)
	  goto 999
	endif

!***   Normalization of the elimination matrix      ***

        do i=1,nproc
	  ihelp(i) = 1
	enddo

        call LSPNORM(mat,lmat,index,lindex,info,ia1,l,
     &               prec(iprec(7)+1),lpmat,iprec(iprec(9)+1),lpix,
     &               iprec(iprec(13)+1),ia1LU,nvtLU,dw(1),dw(l+1),
     &               ldw-l,lsym,ilin,lmatbk,ptrmbk,ihelp,
     &               tids,myproc,nproc,nmsg,iconv,ierr)
        if (iconv == 4) then
          if (myproc == 1) then
            write(lout,1020)
            write(lout,1035)
            write(lout,1030)
          endif
          goto 999
        endif

!***   Computation of the second threshold value    ***

        if (rlin(3) > zero .and. rlin(3) < one) then
          call THRES2(prec(iprec(7)+1),lpmat,iprec(iprec(13)+1),ia1LU,
     &                ilin,rlin,nvtLU,myproc,nproc,tids,nmsg)
	endif

!***   Gauss algorithm   (A=U*P*P*L = U*L)                            ***
!***   Prerequisite: the array <info> must be sorted ascendingly and  ***
!***                 the array <mat> must be sorted ascendingly!      ***
!***                 Variable <startmat> refers to the array <prec>!  ***

        if (nproc == 1) then

!***   Computation of the bandwidths    ***

          if (MOD(optim,1000000) < 200000) then
	    lbwopt = .false.
            call BNDWTHLU(iprec(iprec(9)+1),lpix,iprec(iprec(13)+1),
     &                    ia1LU,nvtLU,ilin,tids,nproc,bwlow,bwup,gbwlow,
     &                    gbwup,lbwopt)
	  endif

          if (idoku /= 0) then
            write(lout,1200)
            if (ilin(8) >= 63 .and. ilin(8) <= 69) write(lout,1201)
            if (ilin(8) >= 72 .and. ilin(8) <= 78) write(lout,1401)
            write(lout,1200)
	  endif

          if ((MOD(optim,10) == 1) .and. (myproc == 1)) then
            call SECONDS(tpelim,tepelim)
          endif

          if (lsym) then
	    bwh = MAX(bwlow,bwup)
            call CHOLSYM(prec(iprec(7)+1),lpmat,iprec(iprec(9)+1),
     &                   lpix,l,iprec(iprec(13)+1),nvtLU,ia1LU,
     &                   bwh,rlin,ilin,opcount,ierr)
	  else
            if ((idoku > 0) .and. (myproc == 1)) then
              write(lout,1226) rlin(6)
            endif
            if (idoku < 0) then
              write(lout,1226) rlin(6)
            endif
	    if (ilin(8) >= 63 .and. ilin(8) <= 69) then
              call ELIMINATION(prec(iprec(7)+1),lpmat,iprec(iprec(9)+1),
     &                         lpix,l,iprec(iprec(13)+1),nvtLU,ia1LU,
     &                         iprec(iprec(15)+1),iprec(iprec(10)+1),
     &                         bwlow,bwup,rlin,ilin,opcount,ierr)
	    endif
	    if (ilin(8) >= 72 .and. ilin(8) <= 78) then
              call ELIMINATIONX(prec(iprec(7)+1),lpmat,
     &                          iprec(iprec(9)+1),
     &                          lpix,l,iprec(iprec(13)+1),nvtLU,ia1LU,
     &                          iprec(iprec(15)+1),iprec(iprec(10)+1),
     &                          bwlow,bwup,rlin,ilin,opcount,ierr)
	    endif
	  endif
	  if (ierr == 0) then
            if (idoku /= 0) then
              write(lout,1200)
              if (ilin(8) >= 63 .and. ilin(8) <= 69) write(lout,1202)
              if (ilin(8) >= 72 .and. ilin(8) <= 78) write(lout,1402)
              write(lout,1200)
	    endif
	  else
	    iconv = 99
	    goto 999
	  endif

          if ((MOD(optim,10) == 1) .and. (myproc == 1)) then
            call SECONDS(thlp,tehlp)
            tpelim  = thlp - tpelim
            tepelim = tehlp - tepelim
            write(lout,1100)
            write(lout,1109) tpelim
            write(lout,1110) tepelim
          endif

          if (MOD(optim,100) >= 10) then
            if (idoku /= 0) then
              write(lout,1130) opcount
              write(lout,1132) DBLE(opcount)*1.d-6/tpelim
            endif
          endif

          if (MOD(optim,100) >= 10) then
            call ULNORM(prec(iprec(7)+1),lpmat,iprec(iprec(9)+1),lpix,
     &                  iprec(iprec(13)+1),ia1LU,iw,l,'L',
     &                  ilin,myproc,nproc,tids,nmsg)
	    if (.not. lsym) then
              call ULNORM(prec(iprec(7)+1),lpmat,iprec(iprec(9)+1),lpix,
     &                    iprec(iprec(10)+1),ia1LU,iw,l,'U',
     &                    ilin,myproc,nproc,tids,nmsg)
            endif
          endif

	  if (lsym) goto 888


!***   Sort indices of L ascendingly                          ***

          if ((MOD(optim,10) == 1) .and. (myproc == 1)) then
            call SECONDS(tdata,tedata)
          endif

          mxlvt = MXLVTLU(iprec(iprec(13)+1),ia1LU,nvtLU)
          call LUSORT(prec(iprec(7)+1),lpmat,iprec(iprec(9)+1),lpix,
     &                iprec(iprec(13)+1),ia1LU,mxlvt,nvtLU)

!***   Resorting of L into storage patterns "packed column"   ***

          if (idoku /= 0) then
            write(lout,1200)
            write(lout,1203)
            write(lout,1200)
	  endif

          call SHIFTL(prec(iprec(7)+1),lpmat,iprec(iprec(9)+1),lpix,
     &                iprec(iprec(13)+1),iprec(iprec(10)+1),ia1LU,nvtLU,
     &                nvtL,iw,l,ilin,myproc,nproc,ierr)
	  if (ierr == 0) then
            if (idoku /= 0) then
              write(lout,1200)
              write(lout,1204)
              write(lout,1200)
	    endif
	  else
	    iconv = 99
	    goto 999
	  endif

          if ((MOD(optim,10) == 1) .and. (myproc == 1)) then
            call SECONDS(thlp,tehlp)
            tdata  = thlp - tdata
            tedata = tehlp - tedata
            write(lout,1100)
            write(lout,1111) tdata
            write(lout,1112) tedata
          endif

!         if (MOD(optim,100) >= 10) then
!           call ULNORM(prec(iprec(7)+1),lpmat,iprec(iprec(9)+1),lpix,
!    &                  iprec(iprec(13)+1),ia1LU,iw,nvtL,'L',
!    &                  ilin,myproc,nproc,tids,nmsg)
!           call ULNORM(prec(iprec(7)+1),lpmat,iprec(iprec(9)+1),lpix,
!    &                  iprec(iprec(10)+1),ia1LU,iw,l,'U',
!    &                  ilin,myproc,nproc,tids,nmsg)
!         endif

        else     ! >>>>>>> if (nproc > 1)

          if (idoku > 0) then
            if (myproc == 1) then
              write(lout,1200)
              write(lout,1205)
              write(lout,1200)
	    endif
	  endif
          if (idoku < 0) then
            write(lout,1200)
            write(lout,1205)
            write(lout,1200)
	  endif

          if ((MOD(optim,10) == 1) .and. (myproc == 1)) then
            call SECONDS(tdata,tedata)
          endif

          do i=1,nvtLU
            iprec(iprec(13)+i) = iprec(iprec(13)+i) - startmat
          enddo
          call ROWSRT_WRAP(prec(iprec(7)+startmat+1),lpmat-startmat,
     &                     iprec(iprec(9)+startmat+1),lpix-startmat,
     &                     iprec(iprec(13)+1),ia1LU,nvtLU,l,
     &                     iw,ilin,ptrmbk,
     &                     endpmat,myproc,nproc,tids,nmsg,iconv,ierr)
          if (iconv == 99) then
            if (myproc == 1) then
              write(lout,1020)
              write(lout,1052)
              write(lout,1030)
	    endif
            goto 999
          endif

          if (idoku > 0) then
            if (myproc == 1) then
              write(lout,1200)
              write(lout,1206)
              write(lout,1200)
	    endif
	  endif
          if (idoku < 0) then
            write(lout,1200)
            write(lout,1206)
            write(lout,1200)
	  endif

!***   Scatter the array <lmatwrp> onto all processors                ***

          lg = lglobal/nproc
	  lrest = MOD(lglobal,nproc)
	  do i=1,nproc
	    lmatwrp(i) = lg
	  enddo
	  do i=1,lrest
	    lmatwrp(i) = lmatwrp(i) + 1
	  enddo

          if (myproc == 1) then
	    root = .true.
	    call MPSCAT(lmatwrp,nproc*iint,root,myproc,nproc,tids,
     &                  nmsg,ierr)
	  else
	    root = .false.
	    call MPSCAT(lmatwrp,nproc*iint,root,myproc,nproc,tids,
     &                  nmsg,ierr)
	  endif


          call INFOSRT_WRAP(iprec(iprec(13)+1),ia1LU,nvtLU,iw,lout,
     &                      myproc,nproc,tids,nmsg,ierr)
          if (ierr /= 0) goto 999

	  do i=1,ia1LU*ia2LU
	    iw(i) = iprec(iprec(13)+i)
	  enddo

          call MATSORTLU(prec(iprec(7)+startmat+1),lpmat-startmat,
     &                   iw,ia1LU,
     &                   nvtLU,ilin,endpmat,myproc,
     &                   nproc,tids,nmsg,ierr,iconv)
          if (ierr /= 0) goto 999

          call IXSORTLU(iprec(iprec(9)+startmat+1),lpix-startmat,
     &                  iprec(iprec(13)+1),ia1LU,
     &                  nvtLU,ilin,endpmat,myproc,
     &                  nproc,tids,nmsg,ierr,iconv)
          if (ierr /= 0) goto 999

          do i=1,nvtLU
            iprec(iprec(13)+i) = iprec(iprec(13)+i) + startmat
          enddo

!***   Computation of the bandwidths    ***

	  lbwopt = .false.
          call BNDWTHLU(iprec(iprec(9)+1),lpix,iprec(iprec(13)+1),
     &                  ia1LU,nvtLU,ilin,tids,nproc,bwlow,bwup,gbwlow,
     &                  gbwup,lbwopt)


!***   <infoLU> has <nvtLU> entries                    ***

          if (idoku > 0) then
            if (myproc == 1) then
              write(lout,1200)
              write(lout,1201)
              write(lout,1200)
	    endif
	  endif
          if (idoku < 0) then
            write(lout,1200)
            write(lout,1201)
            write(lout,1200)
	  endif

          if ((MOD(optim,10) == 1) .and. (myproc == 1)) then
            call SECONDS(tpelim,tepelim)
          endif


          if ((idoku > 0) .and. (myproc == 1)) then
            write(lout,1226) rlin(6)
          endif
          if (idoku < 0) then
            write(lout,1226) rlin(6)
          endif
          if (ilin(8) >= 63 .and. ilin(8) <= 69) then
            call ELIMINATP(prec(iprec(7)+1),lpmat,iprec(iprec(9)+1),
     &                   lpix,l,lglobal,iprec(iprec(13)+1),nvtLU,ia1LU,
     &                   lsym,iprec(iprec(15)+1),iprec(iprec(10)+1),
     &                   bwlow,bwup,gbwlow,gbwup,rlin,ilin,opcount,      
     &                   lmatwrp,myproc,nproc,tids,nmsg,ierr)
	  endif
          if (ilin(8) >= 72 .and. ilin(8) <= 78) then
            call ELIMINATPX(prec(iprec(7)+1),lpmat,iprec(iprec(9)+1),
     &                    lpix,l,lglobal,iprec(iprec(13)+1),nvtLU,ia1LU,
     &                    lsym,iprec(iprec(15)+1),iprec(iprec(10)+1),
     &                    bwlow,bwup,gbwlow,gbwup,rlin,ilin,opcount,
     &                    lmatwrp,myproc,nproc,tids,nmsg,ierr)
	  endif
	  if (ierr == 0) then
            if (idoku > 0) then
              if (myproc == 1) then
                write(lout,1200)
                write(lout,1202)
                write(lout,1200)
	      endif
	    endif
            if (idoku < 0) then
              write(lout,1200)
              write(lout,1202)
              write(lout,1200)
	    endif
	  else
	    iconv = 99
	    goto 999
	  endif

          if ((MOD(optim,10) == 1) .and. (myproc == 1)) then
            call SECONDS(thlp,tehlp)
            tpelim  = thlp - tpelim
            tepelim = tehlp - tepelim
            write(lout,1100)
            write(lout,1109) tpelim
            write(lout,1110) tepelim
          endif

          if (MOD(optim,100) >= 10) then
            call LL4ISM8(1,1,1,opcount,gopcnt,ihlp8,
     *                   myproc,nproc,tids,nmsg)
            if ((myproc == 1) .and. (idoku > 0)) then
              write(lout,1130) gopcnt
              write(lout,1132) gopcnt*1.d-6/tpelim
            endif
            if (idoku < 0) then
              write(lout,1130) gopcnt
              write(lout,1132) gopcnt*1.d-6/tpelim
            endif
          endif

          nvtL = lmatwrp(myproc)

!***   <infoU> has <lglobal> entries                   ***
!***   <infoL> has <lmatwrp(myproc)> = <nvtL> entries  ***

          if (MOD(optim,100) >= 10) then
            call ULNORM(prec(iprec(7)+1),lpmat,iprec(iprec(9)+1),lpix,
     &                  iprec(iprec(13)+1),ia1LU,iw,nvtL,'L',
     &                  ilin,myproc,nproc,tids,nmsg)
            call ULNORM(prec(iprec(7)+1),lpmat,iprec(iprec(9)+1),lpix,
     &                  iprec(iprec(10)+1),ia1LU,iw,lglobal,'U',
     &                  ilin,myproc,nproc,tids,nmsg)
          endif

!***   Sort indices of L ascendingly                          ***

          mxlvt = MXLVTLU(iprec(iprec(13)+1),ia1LU,nvtL)
          call LUSORT(prec(iprec(7)+1),lpmat,iprec(iprec(9)+1),lpix,
     &                iprec(iprec(13)+1),ia1LU,mxlvt,nvtL)

!***   Resort 'packed columns' of U into 'packed rows'        ***

          if (idoku > 0) then
            if (myproc == 1) then
              write(lout,1200)
              write(lout,1207)
              write(lout,1200)
	    endif
	  endif
          if (idoku < 0) then
            write(lout,1200)
            write(lout,1207)
            write(lout,1200)
	  endif

!***   Indices of U are sorted in ascending order              ***
!***   Column numbers of <infoU> are sorted in ascending order ***

	  call SHIFTU_ROW(prec(iprec(7)+1),lpmat,iprec(iprec(9)+1),
     &                    lpix,iprec(iprec(13)+1),iprec(iprec(10)+1),
     &                    ia1LU,lmatwrp(myproc),iw,lglobal,ilin,myproc,
     &                    nproc,tids,nmsg,ierr)

!***   Thus column indices of U are sorted in ascending order  ***
!***   Row numbers of <infoU> are sorted in ascending order    ***

          if (ierr == 0) then
            if (idoku > 0) then
              if (myproc == 1) then
                write(lout,1200)
                write(lout,1208)
                write(lout,1200)
	      endif
	    endif
            if (idoku < 0) then
              write(lout,1200)
              write(lout,1208)
              write(lout,1200)
	    endif
	  else
	    iconv = 99
            if (myproc == 1) then
              write(lout,1020)
              write(lout,1038)
              write(lout,1030)
	    endif
            goto 999
          endif

          nvtU = lmatwrp(myproc)
!***   <infoU> has <lmatwrp(myproc)> = <nvtU> entries  ***

!***   Shift U to the right and resort wrapped rows of L blockwise  ***

          if (idoku > 0) then
            if (myproc == 1) then
              write(lout,1200)
              write(lout,1210)
              write(lout,1200)
	    endif
	  endif
          if (idoku < 0) then
            write(lout,1200)
            write(lout,1210)
            write(lout,1200)
	  endif

          call SHIFTU_END(prec(iprec(7)+1),lpmat,iprec(iprec(9)+1),
     &                    lpix,iprec(iprec(10)+1),iprec(iprec(13)+1),
     &                    ia1LU,nvtU,nvtL,ilin,lenU,
     &                    lenL,lmat,rlin,myproc,nproc,tids,nmsg,ierr)
          if (ierr > 0) then
            goto 999
          endif
          call ROWSRTP_NOP(prec(iprec(7)+1),lpmat-lenU,
     &                     iprec(iprec(9)+1),lpix-lenU,
     &                     iprec(iprec(13)+1),ia1LU,nvtL,l,lprec,
     &                     iw,ilin,ptrmbk,iprec(7),lenL,myproc,
     &                     nproc,tids,nmsg,iconv,ierr)
          if (iconv == 99) then
            if (myproc == 1) then
              write(lout,1020)
              write(lout,1052)
              write(lout,1030)
	    endif
            goto 999
          endif

          if (idoku > 0) then
            if (myproc == 1) then
              write(lout,1200)
              write(lout,1212)
              write(lout,1200)
	    endif
	  endif
          if (idoku < 0) then
            write(lout,1200)
            write(lout,1212)
            write(lout,1200)
	  endif

!***   Shift U to the left and resort wrapped rows of U blockwise   ***

          if (idoku > 0) then
            if (myproc == 1) then
              write(lout,1200)
              write(lout,1211)
              write(lout,1200)
	    endif
	  endif
          if (idoku < 0) then
            write(lout,1200)
            write(lout,1211)
            write(lout,1200)
	  endif

          call SHIFTU_BEG(prec(iprec(7)+1),lpmat,iprec(iprec(9)+1),
     &                    lpix,iprec(iprec(10)+1),ia1LU,nvtU,ilin,
     &                    lenU,lenL,ierr)
          ihelp(1) = ierr
          call LL4INM(1,1,1,ihelp(1),ihelp(2),ihelp(3),myproc,nproc,
     &                tids,nmsg)
          iermax = ihelp(2)
          if (iermax > 0) then
            if (myproc == 1) then
              write(lout,1020)
              write(lout,1053)
              write(lout,1030)
	    endif
            goto 999
          endif

          do i=1,nvtU
            iprec(iprec(10)+i) = iprec(iprec(10)+i) - lenL
          enddo
          call ROWSRTP_NOP(prec(iprec(7)+lenL+1),lpmat-lenL,
     &                     iprec(iprec(9)+lenL+1),lpix-lenL,
     &                     iprec(iprec(10)+1),ia1LU,nvtU,l,lprec,
     &                     iw,ilin,ptrmbk,iprec(7),lenU,myproc,
     &                     nproc,tids,nmsg,iconv,ierr)
          if (iconv == 99) then
            if (myproc == 1) then
              write(lout,1020)
              write(lout,1052)
              write(lout,1030)
	    endif
            goto 999
          endif

          if (idoku > 0) then
            if (myproc == 1) then
              write(lout,1200)
              write(lout,1212)
              write(lout,1200)
	    endif
	  endif
          if (idoku < 0) then
            write(lout,1200)
            write(lout,1212)
            write(lout,1200)
	  endif

!***   Resorting of U into storage patterns "packed column"   ***

          if (idoku > 0) then
            if (myproc == 1) then
              write(lout,1200)
              write(lout,1209)
              write(lout,1200)
	    endif
	  endif
          if (idoku < 0) then
            write(lout,1200)
            write(lout,1209)
            write(lout,1200)
	  endif

!***   Sorting of row numbers of <infoU> in ascending order   ***

          call INFOSRT_WRAP(iprec(iprec(10)+1),ia1LU,nvtU,iw,lout,
     &                      myproc,nproc,tids,nmsg,ierr)

!***   Resorting of U into storage patterns "packed column"   ***
!***   Column indices of U are sorted in ascending order      ***
!***   Row numbers of <infoU> are sorted in ascending order   ***

          call SHIFTLU(prec(iprec(7)+lenL+1),lpmat-lenL,
     &                 iprec(iprec(9)+lenL+1),lpix-lenL,
     &                 iprec(iprec(10)+1),ia1LU,nvtU,lenU,iw,lglobal,
     &                 ilin,myproc,nproc,tids,nmsg,ierr)

!***   Thus row indices of U are sorted in ascending order    ***

          if (ierr == 0) then
            if (idoku > 0) then
              if (myproc == 1) then
                write(lout,1200)
                write(lout,1208)
                write(lout,1200)
	      endif
	    endif
            if (idoku < 0) then
              write(lout,1200)
              write(lout,1208)
              write(lout,1200)
	    endif
	  else
	    iconv = 99
            if (myproc == 1) then
              write(lout,1020)
              write(lout,1039)
              write(lout,1030)
	    endif
            goto 999
          endif

          nvtU = lglobal
!***   <infoU> has <lglobal> = <nvtU> entries  ***
          do i=1,nvtU
            iprec(iprec(10)+i) = iprec(iprec(10)+i) + lenL
          enddo

!***   Sort indices of U ascendingly                          ***

!         mxlvt = MXLVTLU(iprec(iprec(10)+1),ia1LU,nvtU)
!         call LUSORT(prec(iprec(7)+1),lpmat,iprec(iprec(9)+1),lpix,
!    &                iprec(iprec(10)+1),ia1LU,mxlvt,nvtU)

!         if (MOD(optim,100) >= 10) then
!           call ULNORM(prec(iprec(7)+1),lpmat,iprec(iprec(9)+1),lpix,
!    &                  iprec(iprec(10)+1),ia1LU,iw,nvtU,'U',
!    &                  ilin,myproc,nproc,tids,nmsg)
!         endif

!***   Resorting of L into storage patterns "packed column"   ***

          if (idoku > 0) then
            if (myproc == 1) then
              write(lout,1200)
              write(lout,1203)
              write(lout,1200)
	    endif
	  endif
          if (idoku < 0) then
            write(lout,1200)
            write(lout,1203)
            write(lout,1200)
	  endif

!***   Sorting of row numbers of <infoL> in ascending order   ***

          call INFOSRT_WRAP(iprec(iprec(13)+1),ia1LU,nvtL,iw,lout,
     &                      myproc,nproc,tids,nmsg,ierr)

!***   Sorting of L into storage patterns "packed column"     ***
!***   Column indices of L are sorted in ascending order      ***
!***   Row numbers of <infoL> are sorted in ascending order   ***

          call SHIFTLU(prec(iprec(7)+1),lpmat,iprec(iprec(9)+1),lpix,
     &                 iprec(iprec(13)+1),ia1LU,nvtL,lenL,iw,lglobal,
     &                 ilin,myproc,nproc,tids,nmsg,ierr)

!***   Thus row indices of L are sorted in ascending order    ***

          if (ierr == 0) then
            if (idoku > 0) then
              if (myproc == 1) then
                write(lout,1200)
                write(lout,1204)
                write(lout,1200)
	      endif
	    endif
            if (idoku < 0) then
              write(lout,1200)
              write(lout,1204)
              write(lout,1200)
	    endif
	  else
	    iconv = 99
            if (myproc == 1) then
              write(lout,1020)
              write(lout,1139)
              write(lout,1030)
	    endif
            goto 999
          endif

          nvtL = lglobal
!***   <infoL> has <lglobal> = <nvtL> entries  ***

!***   Sort indices of L ascendingly                          ***

!         mxlvt = MXLVTLU(iprec(iprec(13)+1),ia1LU,nvtL)
!         call LUSORT(prec(iprec(7)+1),lpmat,iprec(iprec(9)+1),lpix,
!    &                iprec(iprec(13)+1),ia1LU,mxlvt,nvtL)

!         if (MOD(optim,100) >= 10) then
!           call ULNORM(prec(iprec(7)+1),lpmat,iprec(iprec(9)+1),lpix,
!    &                  iprec(iprec(13)+1),ia1LU,iw,nvtL,'L',
!    &                  ilin,myproc,nproc,tids,nmsg)
!           call ULNORM(prec(iprec(7)+1),lpmat,iprec(iprec(9)+1),lpix,
!    &                  iprec(iprec(10)+1),ia1LU,iw,nvtU,'U',
!    &                  ilin,myproc,nproc,tids,nmsg)
!         endif

          if ((MOD(optim,10) == 1) .and. (myproc == 1)) then
            call SECONDS(thlp,tehlp)
            tdata  = thlp - tdata - tpelim
            tedata = tehlp - tedata - tepelim
            write(lout,1100)
            write(lout,1111) tdata
            write(lout,1112) tedata
          endif

	endif     ! if (nproc > 1)


        if ((MOD(optim,10) == 1) .and. (myproc == 1)) then
          call SECONDS(thlp,tehlp)
          telim  = thlp - telim
          teelim = tehlp - teelim
          write(lout,1100)
          write(lout,1113) telim
          write(lout,1114) teelim
        endif

      endif   ! Elimination process

  888 continue


!***********************************************************************
!**    Resorting of scrambled "packed rows" over the processors      ***
!***********************************************************************

      if (lmix) then
        if (nvt /= lmatbk(myproc)) ierr = 1
        ihelp(1)=ierr
        call LL4INM(1,1,1,ihelp(1),ihelp(2),ihelp(3),myproc,nproc,
     &              tids,nmsg)
        iermax=ihelp(2)
        if (iermax == 1) then
          write(lout,1099) iermax
          goto 999
        endif
	startmat = 0
	endmat = ilin(33)
	endind = ilin(34)
        if (endmat /= endind) ierr = 2
        ihelp(1)=ierr
        call LL4INM(1,1,1,ihelp(1),ihelp(2),ihelp(3),myproc,nproc,
     &              tids,nmsg)
        iermax=ihelp(2)
        if (iermax == 2) then
          write(lout,1099) iermax
          goto 999
        endif
        call ROWSRT_NOP(mat,lmat,index,lindex,info,ia1,nvt,l,
     &                  iw,ilin,ptrmbk,endmat,myproc,
     &                  nproc,tids,nmsg,iconv,ierr)
        if (iconv == 99) then
          if (myproc == 1) then
            write(lout,1020)
            write(lout,1052)
            write(lout,1030)
	  endif
          goto 999
        endif
	ilin(33) = endmat
	ilin(34) = endmat
        do i=1,lmatbk(myproc)
          iw(i) = info(i,5) + 1
        enddo
        call RHSIXSRTEST(b,iw,l,lmatbk,ptrmbk,myproc,nproc,tids,nmsg,
     &                   lout,ierr)
        if (ierr /= 0) then
          if (myproc == 1) then
            write(lout,1020)
            write(lout,1056)
            write(lout,1030)
	  endif
          goto 999
        endif
      endif

!***********************************************************************
!***  Optimization of storage patterns from packed row               ***
!***  to packed diag. or full diag. if faster                        ***
!***********************************************************************

      if (MOD(optim,10000000) >= 1000000) then
        call STORAGEOPT(mat,lmat,index,lindex,info,nvt,ia1,l,lsym,
     &			ilin,lmatbk,ptrmbk,myproc,nproc,ierr)
      endif

!***********************************************************************
!**    Create main diagonal for symmetric matrices if not there      ***
!***********************************************************************

      if (lsym .and. .not. lmain) then
        memopt = zero
	lmain = .true.
        call PACK_MAT(mat,lmat,index,lindex,info,ia1,ilin,memopt,
     &                lmain,lmatbk,ptrmbk,nproc,tids,ierr)
	nvt = ilin(5)
        if (ierr /= 0) then
	  iconv = 99
          if (myproc == 1) then
            write(lout,1020)
            write(lout,1051)
            write(lout,1030)
	  endif
          goto 999
        endif
      endif

!***********************************************************************
!**    Logical separation of all storage patterns                    ***
!***********************************************************************

      if (nproc > 1) then
       iertag = ilin(39)
       endind = ilin(34)
       nvtold = nvt
       do ivt=1,nvtold
	if (info(ivt,1) == 2 .or. info(ivt,1) == 20) then
          call LSPSC2(mat,l,lmat,info,nvt,ivt,ia1,lmatbk,ptrmbk,
     #                myproc,nproc,tids,nmsg,lout,ierr,iertag)
        endif
	if (info(ivt,1) == 3 .or. info(ivt,1) == 30) then
          call LSPSC3(mat,index,l,lmat,lindex,info,nvt,ivt,ia1,lmatbk,
     #                ptrmbk,myproc,nproc,tids,nmsg,lout,ierr,iertag)
        endif
	if (info(ivt,1) == 4 .or. info(ivt,1) == 40) then
          call LSPSC4(mat,index,dw,iw,l,lmat,lindex,endind,info,nvt,
     #                ivt,ia1,lmatbk,ptrmbk,myproc,nproc,tids,
     #                nmsg,lout,ierr,iertag)
        endif
	if (info(ivt,1) == 5 .or. info(ivt,1) == 50) then
          call LSPSC5(mat,index,l,lmat,lindex,info,nvt,ivt,ia1,lmatbk,
     #                ptrmbk,myproc,nproc,tids,nmsg,lout,ierr,iertag)
        endif
	if (info(ivt,1) == 6 .or. info(ivt,1) == 60) then
          call LSPSC6(mat,index,dw,iw,l,lmat,lindex,info,nvt,ivt,ia1,
     #                lmatbk,ptrmbk,myproc,nproc,tids,nmsg,lout,
     #                ierr,iertag)
        endif
	if (info(ivt,1) == 7 .or. info(ivt,1) == 70) then
          call LSPSC7(mat,l,lmat,info,nvt,ivt,ia1,lmatbk,ptrmbk,
     #                myproc,nproc,tids,nmsg,lout,ierr,iertag)
        endif
	if (info(ivt,1) == 9 .or. info(ivt,1) == 90) then
          call LSPSC9(index,lindex,info,nvt,ivt,ia1,lmatbk,ptrmbk,
     #                myproc,nproc,tids,nmsg,lout,ierr,iertag)
        endif
       enddo
       ihelp(1)=iertag
       call LL4INM(1,1,1,ihelp(1), ihelp(2), ihelp(3),
     #      myproc,nproc,tids,nmsg)
       iermax=ihelp(2)
       if (iermax /= 0) then
 	 ierr = iermax
	 if (myproc == 1) then
           do i=2,nproc
	    from = tids(i)
	    call MPRCVA(from,nmsg+i,6*iint,ihelp(1+6*(i-1)),midr,ier)
	    call MPRCVW(from,nmsg+i,6*iint,ihelp(1+6*(i-1)),midr,ier)
	   enddo
	   ihelp(1) = nvt
	   ihelp(2) = ia1
	   ihelp(3) = ilin(33)
	   ihelp(4) = lmat
	   ihelp(5) = endind
	   ihelp(6) = lindex
	 else
	   to = tids(1)
	   ihelp(1) = nvt
	   ihelp(2) = ia1
	   ihelp(3) = ilin(33)
	   ihelp(4) = lmat
	   ihelp(5) = endind
	   ihelp(6) = lindex
	   call MPSNDA(to,nmsg+myproc,6*iint,ihelp,mids,ier)
	   call MPSNDW(to,nmsg+myproc,6*iint,ihelp,mids,ier)
	 endif
	 nmsg = nmsg+nproc-1
	 if (myproc == 1) then
	   write(lout,1020)
	   write(lout,1022)
	   rmax = 0.0
	   ier  = 0
           call getKeyword('LINSOL_INFO_FACTOR',pfac,ier)
	   do i=1,nproc
	     nvt = ihelp(1+6*(i-1))
	     ia1 = ihelp(2+6*(i-1))
	     if (nvt > ia1) then
	       write(lout,1023) ia1,nvt,i
	       if (ier == 0) then
	         if (REAL(nvt)*pfac/REAL(ia1) > rmax)
     &               rmax = REAL(nvt)*pfac/REAL(ia1)
               else
	         if (REAL(nvt)/REAL(ia1) > rmax)
     &               rmax = REAL(nvt)/REAL(ia1)
	       endif
	     endif
	   enddo
	   if (rmax > 0.0) write(lout,1024) rmax

	   rmax = 0.0
	   ier  = 0
           call getKeyword('LINSOL_MATRIX_FACTOR',pfac,ier)
	   do i=1,nproc
	     endmat = ihelp(3+6*(i-1))
	     lmath  = ihelp(4+6*(i-1))
	     if (endmat > lmath) then
	       write(lout,1025) lmath,endmat,i
	       if (ier == 0) then
	         if (REAL(endmat)*pfac/REAL(lmath) > rmax)
     &               rmax = REAL(endmat)*pfac/REAL(lmath)
               else
	         if (REAL(endmat)/REAL(lmath) > rmax)
     &               rmax = REAL(endmat)/REAL(lmath)
	       endif
	     endif
	   enddo
	   if (rmax > 0.0) write(lout,1026) rmax

	   rmax = 0.0
	   ier  = 0
           call getKeyword('LINSOL_INDEX_FACTOR',pfac,ier)
	   do i=1,nproc
	     endind = ihelp(5+6*(i-1))
	     lindh  = ihelp(6+6*(i-1))
	     if (endind > lindh) then
	       write(lout,1027) lindh,endind,i
	       if (ier == 0) then
	         if (REAL(endind)*pfac/REAL(lindh) > rmax)
     &               rmax = REAL(endind)*pfac/REAL(lindh)
               else
	         if (REAL(endind)/REAL(lindh) > rmax)
     &               rmax = REAL(endind)/REAL(lindh)
	       endif
	     endif
	   enddo
	   if (rmax > 0.0) write(lout,1028) rmax

	   write(lout,1030)
	 endif
         iconv = 99
	 goto 999
       endif

       if (idoku > 0) then
         if (myproc == 1) then
	   write(lout,1010) nproc
	   write(lout,1012) nvt-nvtold
         endif
       endif
       if (idoku < 0) then
         write(lout,1010) nproc
         write(lout,1012) nvt-nvtold
       endif
       ilin(5)  = nvt
       ilin(34) = endind
       ilin(35) = nvt
      endif

!     if (idoku > 0) then
!       if (myproc == 1) write(lout,1016) (ntyp+1)*nproc*iint
!     endif
!     if (idoku < 0) write(lout,1016) (ntyp+1)*nproc*iint
      allocate (ptrinf(ntyp+1,nproc),stat=ierr)
      ihelp(1) = ierr
      call LL4INM(1,1,1,ihelp(1),ihelp(2),ihelp(3),myproc,nproc,
     &            tids,nmsg)
      ierr = ihelp(2)
      if (ierr /= 0) then
	iconv = 99
        if (myproc == 1) then
          write(lout,1020)
          write(lout,1076)
          write(lout,1030)
	endif
        goto 999
      endif

      call LSPSRT(index,lindex,nvt,ia1,ptrmbk,info,ptrinf,
     &            lout,myproc,nproc,tids,nmsg)


      if (ALLOCATED(norm) .and. ALLOCATED(prec)) then
        do i=1,l
          prec(i) = norm(i)
        enddo
      endif
      if (.not. ALLOCATED(norm) .and. MOD(isprec,100) == 1) then
        if (.not. lalloc) then
	  ierr = 7777
	  iconv = 99
          if (myproc == 1) then
            write(lout,1020)
            write(lout,1036)
            write(lout,1030)
	  endif
          goto 999
        endif
      endif

      ilin(3) = nmsg


      call LSOLPP(lmat,lprec,liprec,lindex,l,ldw,liw,nproc,
     &            lsym,ia1,info,mat,prec,x,b,dw,iprec,
     &            index,iw,rlin,ilin,lmatbk,ptrinf,tids,
     &            iconv,ierr)
      if (ierr /= 0) goto 999


      if (MOD(optim,1000000) >= 200000 .and. ALLOCATED(perm)) then
        call RHSSRTIX(x,perm,l,lmatbk,ptrmbk,myproc,nproc,tids,nmsg)
      endif

      if (nrhs == -1) then
        call WRITE_ERROR(x,l,lmatbk,ptrmbk,lout,tids,nproc,myproc,nmsg)
      endif

      if (MOD(ilin(6),10000) >= 2000) ilin(6) = ilin(6) - 2000      

      if (ilin(14) == 0) then
	if ((MOD(isprec,10) == 0) .and. (MOD(ilin(6),10) == 1)) then
	  if (.not. ALLOCATED(norm)) allocate(norm(l),stat=ierr)
	  ihelp(1) = ierr
          call LL4INM(1,1,1,ihelp(1),ihelp(2),ihelp(3),myproc,nproc,
     &                tids,nmsg)
	  ierr = ihelp(2)
          if (ierr /= 0) then
	    iconv = 99
            if (myproc == 1) then
              write(lout,1020)
              write(lout,1077)
              write(lout,1030)
	    endif
            goto 999
          endif
          do i=1,l
            norm(i) = prec(i)
          enddo
        endif
        if (ALLOCATED(iw)) deallocate(iw)
        if (ALLOCATED(dw)) deallocate(dw)
        if (ALLOCATED(ptrinf)) deallocate(ptrinf)
        if (ALLOCATED(prec)) deallocate(prec)
        if (ALLOCATED(iprec)) deallocate(iprec)
!       deallocate(iw,dw,ptrinf,prec,iprec)
        if (ALLOCATED(perm)) deallocate(perm)
      endif
      if (ilin(14) == 1) then
	deallocate(iw,dw,ptrinf)
	if (ilin(8) >= 63 .and. MOD(ilin(6),100) < 10)
     &    ilin(6) = ilin(6) + 10      
	if (ALLOCATED(perm) .and. MOD(ilin(6),10000) < 1000)
     &    ilin(6) = ilin(6) + 1000      
      endif
      if (ilin(14) == 2) then
	deallocate(iw,dw,ptrinf)
        if (ALLOCATED(prec)) deallocate(prec)
        if (ALLOCATED(iprec)) deallocate(iprec)
      endif

  999 if ((MOD(optim,10) == 1) .and. (myproc == 1)) then
        call SECONDS(thlp,tehlp)
        tlsolp  = thlp - tlsolp
        telsolp = tehlp - telsolp
        write(lout,1100)
        write(lout,1101) tlsolp
        write(lout,1102) telsolp
      endif

!
!
!**** END OF CALCULATION
!     ------------------
!
 1000 format('  ---------------------------------------------------- '/
     &       ' | -------------------------------------------------- |')
 1001 format(' ||      L S O L P - Version 1.0  (January 2002)     ||')
 1003 format(' ||      installed on a NON-PARALLEL computer        ||')
 1004 format(' ||      installed on a PARALLEL computer            ||')
 1002 format(' | -------------------------------------------------- |'/
     &       '  ---------------------------------------------------- '/)
 1010 format(/'  Matrix is subdivided logically into ',i4,/
     &       '  column blocks')
 1012 format('  --->',i6,' additional vector terms are created'/)
!1014 format('  Allocating storage of size (in Bytes) ',i9,/
!    &       '  for integer work array <iw> ...................'/)
!1015 format('  Allocating storage of size (in Bytes) ',i9,/
!    &       '  for real work array <dw> ......................'/)
!1016 format('  Allocating storage of size (in Bytes) ',i9,/
!    &       '  for pointer array <ptrinf> ....................'/)
!1017 format('  Allocating storage of size (in Bytes) ',i9,/
!    &       '  for integer preconditioning array <iprec> .....'/)
!1018 format('  Allocating storage of size (in Bytes) ',i9,/
!    &       '  for real preconditioning array <prec> .........'/)
 1019 format(' ***   information array (info) for storage     ***'/
     &       ' ***   pattern 1 or 9 is wrong!                 ***')
 1119 format(' ***   enlarge LINSOL_BUFFER_SIZE and <ilin(7)> ***'/
     &       ' ***   respectively!                            ***')
 1219 format(' ***   The matrix is probably singular!!!       ***')
 1319 format(' ***   The matrix is probably numerically       ***'/
     &       ' ***   instable!!!                              ***')
 1020 format(' **************************************************'/
     &       ' ***  ERROR occured in LSOLP:                   ***')
 1021 format(' ***   check input parameters!                  ***')
 1022 format(' ***   one of the following parameters is too   ***'/
     &       ' ***   small: ia1,lmat,lindex!                  ***')
 1023 format(' ***   <ia1> is ',i6,', but must be greater     ***'/
     &       ' ***   than ',i6,' on process ',i6,' for        ***'/
     &       ' ***   for sectioning!                          ***')
 1024 format(' ***   try <LINSOL_INFO_FACTOR> > ',f8.2,'      ***'/
     &       ' ***   or enlarge <ia1>!                        ***')
 1025 format(' ***   <lmat> is ',i9,', but must be greater ***'/
     &       ' ***   than ',i9,' on process ',i6,' !       ***')
 1026 format(' ***   try <LINSOL_MATRIX_FACTOR> >',f8.2,'     ***'/
     &       ' ***   or enlarge <lmat>!                       ***')
 1027 format(' ***   <lindex> is ',i9,', but must be       ***'/
     &       ' ***   greater than ',i9,' on process ',i5,'! ***')
 1028 format(' ***   try <LINSOL_INDEX_FACTOR> > ',f8.2,'     ***'/
     &       ' ***   or enlarge <lindex>!                     ***')
 1029 format(' ***   First dimension of array <info> is too   ***'/
     &       ' ***   small and could not be reallocated!      ***'/
     &       ' ***   Routine LSPCVT_INPL needs                ***'/
     &       ' ***   value ',i9,' but has only ',i9,'   ***'/
     &       ' ***   as first dimension!                      ***')
 1030 format(' **************************************************')
 1031 format(' ***   there is a row with no nonzero entry!    ***')
 1032 format(' ***   internal error?!                         ***')
 1033 format(' ***   row without nonzero entries in the       ***'/
     &       ' ***   preconditioning matrix is not allowed!   ***'/
     &       ' ***   Reduce <rlin(2)> or LINSOL_GAUSS_THRES1! ***')
 1034 format(' ***   the size of the array <mat> and <index>  ***'/
     &       ' ***   must not be different! Check the sizes   ***'/
     &       ' ***   of the above mentioned arrays!           ***')
 1035 format(' ***   normalization failed!                    ***')
 1036 format(' ***   matrix is normalized due to <ilin(6)>,   ***'/
     &       ' ***   but neither the normalization vector nor ***'/
     &       ' ***   the preconditioning array is available!  ***')
 1037 format(' ***   matrix is preconditioned due to          ***'/
     &       ' ***   <ilin(6)>, but the needed arrays for     ***'/
     &       ' ***   the preconditioning (<prec> and <iprec>) ***'/
     &       ' ***   are not available!                       ***')
 1038 format(' ***   resorting packed columns of U into       ***'/
     &       ' ***   packed rows failed!                      ***')
 1039 format(' ***   resorting packed rows of U into packed   ***'/
     &       ' ***   columns failed!                          ***')
 1139 format(' ***   resorting packed rows of L into packed   ***'/
     &       ' ***   columns failed!                          ***')
 1040 format(' **************************************************'/
     &       ' ***  WARNING:                                  ***')
 1041 format(' ***   matrix <mat> is nonsymmetric!            ***')
 1042 format(' ***   automatic method switch from             ***'/
     &       ' ***   CG to polyalgorithm (ilin(1)=10)!        ***')
 1043 format(' ***   automatic method switch to a             ***'/
     &       ' ***   polyalgorithm (ilin(1)=20,60,80)         ***')
 1044 format(' ***   <rlin(4)> or the environment variable    ***'/
     &       ' ***   LINSOL_GAUSS_FACTOR is set < ',f8.2,',   ***'/
     &       ' ***   thus the size of the preconditioning     ***'/
     &       ' ***   matrix is set to the size of the         ***'/
     &       ' ***   original matrix <mat>!                   ***')
 1050 format(' **************************************************'/)
 1051 format(' ***   packing failed!                          ***')
 1052 format(' ***   sorting of rows failed!                  ***')
 1053 format(' ***   <LINSOL_GAUSS_FACTOR> or <rlin(4)> is    ***'/
     &       ' ***   too small!                               ***')
 1054 format(' ***   SSPBO algorithm failed!                  ***')
 1055 format(' ***   GPS algorithm failed!                    ***')
 1056 format(' ***   scrambled rows are missing!              ***')
 1060 format(/'  INFO: value 1000 is added to parameter <optim>,'/
     &       '        i.e. zero entries are removed in <mat> !')
 1061 format(/'  INFO: value 10000 is added to parameter <optim>,'/
     &       '        i.e. <mat> is restored in packed rows!')
 1062 format(/'  INFO: value 100000 is added to parameter <optim>,'/
     &       '        i.e. single entries in the rows of <mat>'/
     &       '        are moved to main diagonal!')
 1063 format(' ***   safe Gauss-factor has been used to       ***'/
     &       ' ***   allocate array <prec>; the resultant     ***'/
     &       ' ***   allocation size of (maximum) ',i10,'  ***'/
     &       ' ***   Bytes is too high! Now we will try YOUR  ***'/
     &       ' ***   GAUSS-factor stored in <rlin(4)>!!!      ***')
 1064 format(' ***   safe Gauss-factor has been used to       ***'/
     &       ' ***   allocate array <iprec>; the resultant    ***'/
     &       ' ***   allocation size of (maximum) ',i10,'  ***'/
     &       ' ***   Bytes is too high! Now we will try YOUR  ***'/
     &       ' ***   GAUSS-factor stored in <rlin(4)>!!!      ***')
 1065 format(' ***   allocation of array <prec> failed!       ***'/
     &       ' ***   The allocation size of (maximum)         ***'/
     &       ' ***   ',i10,' Bytes is too high because of  ***'/
     &       ' ***   a too large Gauss-factor!                ***')
 1066 format(' ***   allocation of array <iprec> failed!      ***'/
     &       ' ***   The allocation size of (maximum)         ***'/
     &       ' ***   ',i10,' Bytes is too high because of  ***'/
     &       ' ***   a too large Gauss-factor!                ***')
 1072 format(' ***   allocation of array <dw> or <iw> failed! ***')
 1073 format(' ***   allocation of arrays <graph> and         ***'/
     &       ' ***   <vertix> failed!                         ***')
 1074 format(' ***   allocation of array <hix> or <hixr>      ***'/
     &       ' ***   in routine PREPRO failed!                ***')
 1075 format(' ***   allocation of array <perm> failed!       ***')
 1076 format(' ***   allocation of array <ptrinf> failed!     ***')
 1077 format(' ***   allocation of array <norm> failed!       ***')
 1078 format(' ***   allocation of array <iprec> failed!      ***')
 1081 format(' ***   opening of the file failed!              ***')
 1099 format('   ERROR: Permutation of Right_Hand_Side not possible'/
     &       '          ierr = ',i4)
 1100 format(/'  TIME STATISTICS: ')
 1101 format('   CPU-Time     for LSOLP            (in seconds):',f8.2)
 1102 format('   Elapsed Time for LSOLP            (in seconds):',f8.2/)
 1103 format('   CPU-Time     for optimization     (in seconds):',f8.2)
 1104 format('   Elapsed Time for optimization     (in seconds):',f8.2/)
 1105 format('   CPU-Time     for BW-opt. process  (in seconds):',f8.2)
 1106 format('   Elapsed Time for BW-opt. process  (in seconds):',f8.2/)
 1107 format('   CPU-Time     for pure BW-optim.   (in seconds):',f8.2)
 1108 format('   Elapsed Time for pure BW-optim.   (in seconds):',f8.2/)
 1109 format('   CPU-Time     for pure factorization  (in secs):',f8.2)
 1110 format('   Elapsed Time for pure factorization  (in secs):',f8.2/)
 1111 format('   CPU-Time     for datatransfer ops    (in secs):',f8.2)
 1112 format('   Elapsed Time for datatransfer ops    (in secs):',f8.2/)
 1113 format('   CPU-Time     for factorization process  (secs):',f8.2)
 1114 format('   Elapsed Time for factorization process  (secs):',f8.2/)
 1120 format('  Vector terms are checked and printed on unit ',i3/)
 1121 format('  Matrix(part) is printed on unit ',i3,' in ASCII-format')
 1122 format('  Matrix(part) is printed on unit ',i3,/
     &       '  in binary format (single precision)')
 1130 format('  Global number of floating-point operations is:',i15)
!1131 format('  Global number of floating-point operations is:',i15)
 1132 format('  Mflop/s rate for the pure factorization is   :',f15.2/)
 1140 format('  Sparsity factor of <MAT> within bandwidth is :',f12.5)
 1141 format('  Thus the used safe Gauss-factor is ......... :',f12.5/)
 1200 format('   ------------------------------------------------ ')
 1201 format('  |  Factorization starts                          |')
 1202 format('  |  Factorization ended sucessfully               |')
 1401 format('  |  Indexed factorization starts                  |')
 1402 format('  |  Indexed factorization ended sucessfully       |')
 1203 format('  |  Resorting of L into packed columns starts     |')
 1204 format('  |  Resorting of matrix L ended sucessfully       |')
 1205 format('  |  Wrapping around of the rows of the pre-       |'/
     &       '  |  conditioning matrix onto processors starts    |')
 1206 format('  |  Wrapping around ended sucessfully             |')
 1207 format('  |  Resorting of U into packed rows starts        |')
 1208 format('  |  Resorting of matrix U ended sucessfully       |')
 1209 format('  |  Resorting of U into packed columns starts     |')
 1210 format('  |  Blockwise distribution of the rows of the     |'/
     &       '  |  matrix L onto processors starts               |')
 1211 format('  |  Blockwise distribution of the rows of the     |'/
     &       '  |  matrix U onto processors starts               |')
 1212 format('  |  BLockwise distribution ended sucessfully      |')
 1214 format('  |  Bandwidth optimization ended sucessfully      |')
 1215 format('  |  Bandwidth optimization starts with            |'/
     &       '  |  Gibbs-Poole-Stockmeyer (GPS) algorithm        |')
 1216 format('  |  Bandwidth optimization starts with            |'/
     &       '  |  Single Pass Bandwidth Optimization algorithm  |')
 1217 format('  |  Bandwidth optimization starts with            |'/
     &       '  |  Reverse Cuthill McKee algorithm               |')
 1218 format('  |  Bandwidth optimization starts with            |'/
     &       '  |  Reverse Cuthill McKee (no iteration) alg.     |')
 1225 format('  |  Permutation vector (from a former BWO-process)|'/
     &       '  |  is applied to rhs (and initial solution) and  |'/
     &       '  |  in case of a new matrix also to the matrix!   |')
 1226 format(/'  LINSOL_LDROP_FACTOR (rlin(6)) :',g12.5)
 1250 format('   ------------------------------------------------ '/)
 1300 format(/'  Opening file to store pre-forward elimination data'/
     &        '  in file'/
     &        '  ',a72,/
     &        '  on unit ',i3,' ...'/)
 1301 format(/'  Opening file to read pre-forward elimination data '/
     &        '  from file '/
     &        '  ',a72,/
     &        '  on unit ',i3,' ...'/)

      r e t u r n
!-----END OF LSOLP----------------------------------------------------
      e    n    d
