!:::::      ,,,,,LSPCHK2.....
!
!
!
!**********************************************************************
!        1         2         3         4         5         6         7*
!**********************************************************************
!**                                                                 ***
!**                                                                 ***
      subroutine LSPCHK2(mat,lmat,index,lindex,info,ia1,nvt,ilin,l,
     *                 lmyblk,ptrmyb,lmain,lmix,lonly9,tids,nproc,ierr)
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**     L S P C H K 2 :  Checks vector terms of matrix and          ***
!**                      determines if matrix rows distributed on   ***
!**                      more than one processor are scrambled      ***
!**                      (works only for packed rows)!!!            ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**    Copyright (C) 1999 University Karlsruhe                      ***
!**    PROGRAMMER: H. Haefner                                       ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**     THE IMPLICIT NONE-INSTRUCTION HAS THE AIM,                  ***
!**     THAT ALL VARIABLES MUST BE DECLARED EXPLICITLY.             ***
!**                                                                 ***
      USE constants

      implicit none
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**      FORMAL PARAMETERS :                                        ***
!**      -----------------                                          ***
!**                                                                 ***
      integer, intent(in) ::  ia1,l,lmat,lindex,lmyblk,nproc,nvt,ptrmyb
      integer, intent(out) ::  ierr
      integer, intent(in) ::  index(lindex),info(ia1,ia2),tids(nproc)
      integer, intent(inout) ::  ilin(nilin)
      double precision, intent(in) :: mat(lmat)
      logical, intent(in) ::  lmain
      logical, intent(out) ::  lmix, lonly9
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**   LIST OF FORMAL PARAMETERS :                                   ***
!**   -------------------------                                     ***
!**                                                                 ***
!**   (see Manual-Pages <lsolp>)                                    ***
!**                                                                 ***
!**                                                                 ***
!**      LOCAL PARAMETERS :                                         ***
!**      -----------------                                          ***
!**                                                                 ***
      integer    i,k,ix,ixl,iarl
      integer    idoku,ixmain,myproc,nmsg,imax,imix,lglobal,lout
      integer    ibufsz,ibufszr,iermax,ihlp,optim
      double precision ::  maxrow(l), minnrm, maxnrm, rhlp
      integer, parameter ::  ione = 1
      double precision, parameter ::  minnum = TINY(1.d0)
      double precision, parameter ::  maxnum = HUGE(1.d0), zero = 0.0
!
!
!**** START OF CALCULATION :
!     ---------------------
!
      nmsg   = ilin(3)
      ibufsz = ilin(7)
      lout   = ilin(12)
      idoku  = ilin(13)
      myproc = ilin(17)
      optim  = ilin(20)
      lglobal= ilin(36)
      ierr   = 0

!***
!***  compute minimum and maximum over all rows 
!***
      maxrow = zero

      do i=1,nvt
	if (info(i,1) == 1) then
	  do k=1,lmyblk
	    maxrow(k) = MAX(maxrow(k),ABS(mat(info(i,2)+k)))
	  enddo
	endif
	if (info(i,1) == 2) then
	  iarl = info(i,5) - ptrmyb
	  do k=1,info(i,3)
	    maxrow(iarl+k) = MAX(maxrow(iarl+k),ABS(mat(info(i,2)+k)))
	  enddo
	endif
	if (info(i,1) == 3) then
	  iarl = info(i,5) - ptrmyb
	  do k=1,info(i,3)
	    ix   = index(info(i,6)+k)
	    maxrow(iarl+ix) = MAX(maxrow(iarl+ix),ABS(mat(info(i,2)+k)))
	  enddo
	endif
	if (info(i,1) == 6) then
	  do k=1,info(i,3)
	    ixl  = index(info(i,7)+k) - ptrmyb
	    maxrow(ixl) = MAX(maxrow(ixl),ABS(mat(info(i,2)+k)))
	  enddo
	endif
	if ((info(i,1) == 7) .or. (info(i,1) == 9)) then
	  iarl = info(i,5) - ptrmyb
	  do k=1,info(i,3)
	    maxrow(iarl+1) = MAX(maxrow(iarl+1),ABS(mat(info(i,2)+k)))
	  enddo
	endif
	if (info(i,1) == 8) then
	  iarl = info(i,5) - ptrmyb
	  do k=1,info(i,3)
	    maxrow(iarl+k) = MAX(maxrow(iarl+k),ABS(mat(info(i,2)+k)))
	  enddo
	endif
	if (info(i,1) == 11) then
	  do k=1,info(i,3)
	    ixl  = index(info(i,7)+k) - ptrmyb
	    maxrow(ixl) = MAX(maxrow(ixl),ABS(mat(info(i,2)+k)))
	  enddo
	endif
      enddo

      call LL4RMIN(lmyblk,1,l,maxrow,minnrm,rhlp,myproc,nproc,
     &             tids,nmsg)
      if (minnrm < minnum) then
	do i=1,lmyblk
	  ierr = -1
	  if (maxrow(i) < minnum) then
            write(lout,1010)
            write(lout,1015) i,myproc
            write(lout,1020)
          endif
        enddo
      endif
      call LL4RNM(lmyblk,1,l,maxrow,maxnrm,rhlp,myproc,nproc,tids,nmsg)
      if (maxnrm > maxnum) then
	ierr = 1
	do i=1,lmyblk
	  if (maxrow(i) > maxnum) then
            write(lout,1010)
            write(lout,1016) i,myproc
            write(lout,1020)
          endif
        enddo
      endif

      if ((MOD(optim,100) >= 10) .and. (idoku > 0)) then
	if (myproc == 1) then
          write(lout,1021) minnrm
          write(lout,1022) maxnrm
	endif
      endif
      if ((MOD(optim,100) >= 10) .and. (idoku < 0)) then
        write(lout,1021) minnrm
        write(lout,1022) maxnrm
      endif


!***
!***  check main diagonal
!***
      if (lmain) then
        do i=1,nvt 
	  if (info(i,1) == 1) then
	    ixmain = i
	    exit
	  endif
        enddo
	if ((info(ixmain,4) /= ptrmyb) .or. (info(ixmain,5) /= ptrmyb)
     *      .or. (info(ixmain,3) /= lmyblk) .or. (info(ixmain,2) < 0)
     *      .or. (info(ixmain,2) > lmat-lmyblk)) then
          ierr = 8011
          write(lout,1010)
          write(lout,1012) myproc
          write(lout,1020)
          goto 999
        endif
      endif

      imix = 0
      lonly9 = .true.
      do i=1,nvt
	if (info(i,1) /= 9) then
	  lonly9 = .false.
	  exit
	endif
      enddo
      if (lonly9) then
	do i=1,nvt
          if ((info(i,5) < 0) .or.
     *        (info(i,5) > lglobal) .or. (info(i,3) > lglobal)
     *        .or. (info(i,3) < 0) .or. (info(i,2) < 0) .or.
     *        (info(i,2) > lmat-info(i,3)) .or. (info(i,6) < 0) .or.
     *        (info(i,6) > lindex-info(i,3))) then
            ierr  = 8019
	    write(lout,1010)
	    write(lout,1013) info(i,5),myproc
	    write(lout,1020)
            goto 999
          endif
          if ((info(i,5) < ptrmyb) .or.
     *        (info(i,5) > ptrmyb+lmyblk)) then
	    imix = 1
	  endif
        enddo
      else
	do i=1,nvt
          if ((info(i,1) == 9) .and. ((info(i,5) < ptrmyb) .or.
     *        (info(i,5) > ptrmyb+lmyblk) .or. (info(i,3) > lglobal)
     *        .or. (info(i,3) < 0) .or. (info(i,2) < 0) .or.
     *        (info(i,2) > lmat-info(i,3)) .or. (info(i,6) < 0) .or.
     *        (info(i,6) > lindex-info(i,3)))) then
            ierr  = 8019
	    write(lout,1010)
	    write(lout,1013) info(i,5),myproc
	    write(lout,1020)
            goto 999
          endif
        enddo
      endif

      call LL4INM(ione,ione,ione,imix,imax,ihlp,myproc,nproc,
     *            tids,nmsg)
      imix = imax
      if (imix == 1) then
	lmix = .true.
	if (myproc == 1) then
	  write(lout,1011)
	  write(lout,1020)
	endif
      else
	lmix = .false.
      endif

      ibufszr = INT(ibufsz*1.e6/ireal)
      if ((ilin(8) >= 63 .and. ibufszr < lglobal) .or. 
     &    (MOD(optim,200000) >= 100000 .and. ilin(14) == 1 .and.
     &     ibufsz < 1)) then
        ierr  = 8020
        if (myproc == 1) then
          write(lout,1010)
          write(lout,1014) ibufsz
          write(lout,1020)
        endif
        goto 999
      endif

  999 call LL4INM(ione,ione,ione,ierr,iermax,ihlp,myproc,nproc,
     *            tids,nmsg)
      ierr = iermax
      ilin(3) = nmsg

 1000 format(' **************************************************'/
     &       ' ***  WARNING occured in LSOLP (LSPCHK2):       ***')
 1010 format(' **************************************************'/
     &       ' ***  ERROR occured in LSOLP (LSPCHK2):         ***')
 1011 format(' **************************************************'/
     &       ' ***  REMARK:  packed rows are not distributed  ***'/
     &       ' ***    blockwise onto the processors; thus     ***'/
     &       ' ***    they will be reassembled blockwise!     ***')
 1012 format(' ***   Information array (info) of the main     ***'/
     &       ' ***   diagonal on processor ',i6,' is wrong!   ***')
 1013 format(' ***   Information array (info) of the packed   ***'/
     &       ' ***   row with row number ',i9,' on         ***'/
     &       ' ***   processor ',i6,' is wrong!               ***')
 1014 format(' ***   Buffer size ',i6,' (in MByte) is too     ***'/
     &       ' ***   small! Enlarge ilin(7)!                  ***')
 1015 format(' ***   the maximum element of row ',i9,' of  ***'/
     &       ' ***   array <MAT> is smaller than the lowest   ***'/
     &       ' ***   machine number on processor ',i6,'!      ***')
 1016 format(' ***   the maximum element of row ',i9,' of  ***'/
     &       ' ***   array <MAT> is larger than the largest   ***'/
     &       ' ***   machine number on processor ',i6,'!      ***')
 1020 format(' **************************************************'/)
 1021 format(/'  MIN(MAX(a(i,j),j=1,lglobal), i=1,lglobal) ... : ',
     &       g12.5)
 1022 format('  MAX(MAX(a(i,j),j=1,lglobal), i=1,lglobal) . . : ',
     &       g12.5/)
!    &       '  corresponds to MINIMUM (MAXIMUM) of the MAXIMA '/
!    &       '  of all (<lglobal>) rows of the matrix <MAT>! '/)

      r e t u r n
!-----END OF LSPCHK2--------------------------------------------------      
      e n d
