C:::::      ,,,,,LSPCVT.....
C
C
C
C**********************************************************************
C        1         2         3         4         5         6         7*
C**********************************************************************
C**                                                                 ***
C**                                                                 ***
      subroutine LSPCVT(mat,lmat,index,lindex,l,info,nvt,ia1,lsym,
     #                  matnew,lmatnew,indnew,lindnew,infnew,ia1new,
     #                  lmatbk,ptrmbk,myproc,nproc,tids,ilin,nmsg,ierr)
C**                                                                 ***
C**                                                                 ***
C**********************************************************************
C**                                                                 ***
C**      L S P C V T   converts a general matrix in LINSOL-Format   ***
C**                    described by the triple (mat,index,info)     ***
C**                    into a matrix in 'packed row'-Format         ***
C**                    described by the triple                      ***
C**                    (matnew,indnew,infnew).                      ***
C**                                                                 ***
C**********************************************************************
C**                                                                 ***
!**    Copyright (C) 1998 University Karlsruhe                      ***
!**    PROGRAMMER:    H. Haefner                                    ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
C**                                                                 ***
C**********************************************************************
C**                                                                 ***
      USE constants

      implicit none
C**                                                                 ***
C**-----------------------------------------------------------------***
C**                                                                 ***
C**      FORMAL PARAMETERS :                                        ***
C**      -----------------                                          ***
C**                                                                 ***
      integer,intent(in) :: lmat,lindex,l,ia1,nvt,nproc,myproc
      integer,intent(inout) :: ierr,nmsg
      integer,intent(out) :: lmatnew,lindnew,ia1new
      integer,intent(in) :: info(ia1,ia2),index(lindex),tids(nproc)
      integer,intent(in) :: ilin(nilin)
      integer,intent(in) :: lmatbk(nproc),ptrmbk(nproc+1)
      double precision,intent(in) :: mat(lmat)
      integer,pointer :: indnew(:)
      integer,dimension(:,:),pointer :: infnew
      double precision,pointer :: matnew(:)
      logical,intent(in) :: lsym
C**                                                                 ***
C**-----------------------------------------------------------------***
C**                                                                 ***
C**   LIST OF FORMAL PARAMETERS :                                   ***
C**   -------------------------                                     ***
C**                                                                 ***
C**   see Manual-Pages <lsolp>                                      ***
C**                                                                 ***
C**                                                                 ***
C**                                                                 ***
C**                                                                 ***
C**        LOCAL PARAMETERS :                                       ***
C**        ----------------                                         ***
C**                                                                 ***
      integer    bufsize,i,idoku,ivt,js,lout,mxvmat
      integer    lmyblk,nofrows,ptrmat1,ptrmat9,ptrind9,ptrinfo
      integer    runind(ntyp)
      double precision,dimension(:,:),pointer :: rbuf
      double precision  threshld1
      logical    lmain,lformat,dtyp(ntyp),setmd

!***********************************************************************
!**    Interface of external subroutines                             ***
!***********************************************************************
      interface
       subroutine CREATE_BUF(rbuf,nofcols,bufsize,nofrows,ilin,lmatbk,
     *                       myproc,nproc,tids,nmsg)
        implicit none
        USE constants
        integer,intent(in) ::   nofcols,bufsize,nproc,myproc
        integer,intent(inout) ::   nmsg
        integer,intent(in) ::   ilin(nilin),tids(nproc),lmatbk(nproc)
        double precision,dimension(:,:),pointer :: rbuf
        integer,intent(out)::   nofrows
       end subroutine
      end interface
C**                                                                 ***
C----------------------------------------------------------------------
C
C
C**** START OF CALCULATION :
C     ---------------------
C
      lout   = ilin(12)
      idoku  = ilin(13)
      lmyblk = lmatbk(myproc)

      bufsize   = 5
      threshld1 = 0.0

      mxvmat = 0.0
      do i=1,lmat
	if (abs(mat(i)) .gt. mxvmat) then
	  mxvmat = abs(mat(i))
	endif
      enddo

      lmatnew = 0
      do i=1,lmat
	if (abs(mat(i)) .gt. threshld1) lmatnew = lmatnew + 1
      enddo

      do i=1,ntyp
        dtyp(i) = .false.
      enddo

      do ivt=1,nvt
       if (info(ivt,1) == 1) dtyp(1) = .true.
       if (info(ivt,1) == 2) dtyp(2) = .true.
       if (info(ivt,1) == 3) dtyp(3) = .true.
       if (info(ivt,1) == 4) dtyp(4) = .true.
       if (info(ivt,1) == 5) dtyp(5) = .true.
       if (info(ivt,1) == 6) dtyp(6) = .true.
       if (info(ivt,1) == 7) dtyp(7) = .true.
       if (info(ivt,1) == 8) dtyp(8) = .true.
       if (info(ivt,1) == 9) dtyp(9) = .true.
       if (info(ivt,1) == 11) dtyp(11) = .true.
      enddo

!
! >>> lmain = .true. means that a explicit main diagonal has to be
! >>>                inserted into the new matrix
!
!     if (lsym) then
!       lmain = .true.
!     else
!       if (lusemd) then
!         lmain = .true.
!       else
!         lmain = .false.
!       endif
!     endif

!
! >>> lformat = .true. means that a explicit main diagonal has to created
!
!     if (.not. lmain) then
!       lformat = .false.
!     endif
!     if (lmain .and. dtyp(1)) then
!       lformat = .false.
!     endif
!     if (lmain .and. .not. dtyp(1)) then
!       lformat = .true.
!     endif


!
! >>> Is condition .true. then the matrix is already in 'packed row'-Format
!
      if (.not. dtyp(1) .and. .not. dtyp(2) .and. .not. dtyp(3)
     *    .and. .not. dtyp(4) .and. .not. dtyp(5) 
     *    .and. .not. dtyp(6) .and. .not. dtyp(7)
     *    .and. .not. dtyp(8) .and. .not. dtyp(11)
     *  .and. (threshld1 .eq. 0)) then
        if (idoku .gt. 0) then
	  if (myproc .eq. 1) write(lout,1021) lmat*ireal
	endif
	if (idoku .lt. 0) write(lout,1021) lmat*ireal
	allocate(matnew(lmat))

        if (idoku .gt. 0) then
	  if (myproc .eq. 1) write(lout,1022) lindex*iint
	endif
	if (idoku .lt. 0) write(lout,1022) lindex*iint
	allocate(indnew(lindex))

        if (idoku .gt. 0) then
	  if (myproc .eq. 1) write(lout,1023) ia1*ia2*iint
	endif
	if (idoku .lt. 0) write(lout,1023) ia1*ia2*iint
	allocate(infnew(ia1,ia2))

        if (idoku .gt. 0) then
	  if (myproc .eq. 1) then
	    write(lout,1031)
          endif
        endif
        if (idoku .lt. 0) then
          write(lout,1031)
        endif

        lmatnew = lmat
	lindnew = lindex
	ia1new  = ia1

	matnew = mat
	indnew = index
	infnew = info
      else
        if (idoku .gt. 0) then
	  if (myproc .eq. 1) write(lout,1021) lmatnew*ireal
	endif
	if (idoku .lt. 0) write(lout,1021) lmatnew*ireal
	allocate(matnew(lmatnew))

        lindnew = lmatnew
        if (idoku .gt. 0) then
	  if (myproc .eq. 1) write(lout,1022) lindnew*iint
	endif
	if (idoku .lt. 0) write(lout,1022) lindnew*iint
	allocate(indnew(lindnew))

        ia1new = lmyblk 
        if (idoku .gt. 0) then
	  if (myproc .eq. 1) write(lout,1023) ia1new*ia2*iint
	endif
	if (idoku .lt. 0) write(lout,1023) ia1new*ia2*iint
	allocate(infnew(ia1new,ia2))
      endif

      call CREATE_BUF(rbuf,l,bufsize,nofrows,ilin,lmatbk,
     *                myproc,nproc,tids,nmsg)

      setmd = .false.
      ptrmat9 = 0
      ptrind9 = 0
      ptrinfo = 0
      runind  = 1

      do js=1,lmyblk,nofrows
	rbuf = 0.0
        call FILL_BUF(lmat,lindex,mat,index,info,ia1,lmatbk,ptrmbk,
     *                myproc,nproc,rbuf,nofrows,l,runind,nvt,js)

        call STO19_BUF(lmatnew,lindnew,matnew,indnew,
     *                 infnew,ia1new,
     *                 rbuf,nofrows,l,js,ptrmat1,ptrmat9,
     *                 ptrind9,ptrinfo,threshld1,setmd)
      enddo

	    
c1000 format(' **********************************************'/
c    &       ' ***  Error occured in LSOLP (LSPCVT):      ***')
c1010 format(' ***   LSPCVT: Implementation error         ***'/
c    &       ' ***           on processor ',i6,'          ***')
c1014 format('       Allocating storage of size (in Bytes) ',i9,/
c     &       '       for integer work array <iw> ...................'/)
c1020 format(' **********************************************')
 1021 format('  Allocating storage of size (in Bytes) ',i11,/
     &       '  for real preconditioning array <matnew> .........'/)
 1022 format('  Allocating storage of size (in Bytes) ',i11,/
     &       '  for integer preconditioning array <indnew> .........'/)
 1023 format('  Allocating storage of size (in Bytes) ',i11,/
     &       '  for new matrix information array <infnew> .........'/)
 1031 format('  Matrix <MAT> is already in `packed row`-Format',/
     &       '  --> MAT is copied into the new matrix'/)
C
C**** END OF CALCULATION
C     ------------------
C
      r e t u r n
C-----END OF LSPCVT---------------------------------------------------
      e    n    d
