!:::::      ,,,,,LSPCVT_INPL.....
!
!
!
!**********************************************************************
!        1         2         3         4         5         6         7*
!**********************************************************************
!**                                                                 ***
!**                                                                 ***
      subroutine LSPCVT_INPL(mat,lmat,index,lindex,l,info,ia1,lsym,
     #                       ilin,lmyblk,ptrmyb,myproc,nproc,tids,
     #                       cr_md,threshold,ierr)
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**      L S P C V T _ I N P L   converts a general matrix in       ***
!**                              LINSOL-Format described by the     ***
!**                              triple (mat,index,info) into a     ***
!**                              matrix in 'packed row'-Format      ***
!**                              in place.                          ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**    Copyright (C) 1998 University Karlsruhe                      ***
!**    PROGRAMMER:    H. Haefner                                    ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!     USE scriptparms
      USE constants

      implicit none
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**      FORMAL PARAMETERS :                                        ***
!**      -----------------                                          ***
!**                                                                 ***
      integer,intent(in) :: lmat,lindex,l,nproc,myproc
      integer,intent(in) :: lmyblk,ptrmyb
      integer,intent(inout) :: ia1
      integer,intent(out) :: ierr
      integer,intent(inout) :: info(ia1,ia2),index(lindex)
      integer,intent(in) :: tids(nproc)
      integer,intent(inout) :: ilin(nilin)
      double precision,intent(in) :: threshold
      double precision,intent(inout) :: mat(lmat)
      logical,intent(in) :: cr_md,lsym
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**   LIST OF FORMAL PARAMETERS :                                   ***
!**   -------------------------                                     ***
!**                                                                 ***
!**   see Manual-Pages <lsolp>                                      ***
!**                                                                 ***
!**                                                                 ***
!**                                                                 ***
!**                                                                 ***
!**        LOCAL PARAMETERS :                                       ***
!**        ----------------                                         ***
!**                                                                 ***
      integer    i,idoku,ier,iermax,ihelp(3),istat,ivt,j,js,lout
      integer    nmsg,nvt,runacc,endmax
      integer    endind,endindo,endindon,endmat,endmato,endmaton
      integer    inddiff,matdiff
      integer    nofrows,ptrmat1,ptrmat9,ptrind9,ptrinfo
      integer    infnew(lmyblk,ia2)
      real       pinfofac

      double precision,dimension(:,:),pointer :: rbuf
      double precision, parameter ::  matmark = TINY(1.d0), zero = 0.0
      integer, parameter ::  ixmark = -999
      logical    lmain,dtyp9

!***********************************************************************
!**    Interface of external subroutines                             ***
!***********************************************************************
      interface
       subroutine CREATE_BUF(rbuf,nofcols,nofrows,ilin,lmyblk,
     *                       myproc,nproc,tids,nmsg,ierr)
        USE constants
        implicit none
        integer,intent(in) ::   nofcols,nproc,myproc,lmyblk
        integer,intent(inout) ::   ierr,nmsg
        integer,intent(in) ::   ilin(nilin),tids(nproc)
        double precision,dimension(:,:),pointer :: rbuf
        integer,intent(out)::   nofrows
       end subroutine
      end interface
!**                                                                 ***
!----------------------------------------------------------------------
!
!
!**** START OF CALCULATION :
!     ---------------------
!
      ierr   = 0

      nmsg   = ilin(3)
      nvt    = ilin(5)
      lout   = ilin(12)
      idoku  = ilin(13)
      runacc = ilin(21)

      dtyp9 = .true.

      do ivt=1,nvt
        if (info(ivt,1) /= 9) then
	  dtyp9 = .false.
	  exit
        endif
      enddo

      if (idoku > 0) then
        if (myproc == 1) write(lout,1000)
      endif
      if (idoku < 0) write(lout,1000)

!
! >>> Is <lrestore> .true. then the matrix is already in 'packed row'-Format
!
      if (dtyp9) then
        if (idoku > 0) then
	  if (myproc == 1) then
	    write(lout,1001)
	    write(lout,1003)
	    write(lout,1010)
	  endif
	endif
	if (idoku < 0) then 
	  write(lout,1001)
	  write(lout,1003)
	  write(lout,1010)
	endif
	goto 999
      else
        if (idoku > 0) then
	  if (myproc == 1) then
	    write(lout,1002)
	    write(lout,1004)
	    write(lout,1010)
	  endif
	endif
	if (idoku < 0) then 
	  write(lout,1002)
	  write(lout,1004)
	  write(lout,1010)
	endif
      endif

      endmat = ilin(33)
      endind = ilin(34)


      call CREATE_BUF(rbuf,l,nofrows,ilin,lmyblk,myproc,nproc,tids,
     &                nmsg,ierr)
      if (ierr /= 0) goto 999

      ptrinfo = 0
      endmaton = endmat
      endindon = endind
!     print*,' nofrows, endmat, endind:', nofrows,endmat,endind

      do js=1,lmyblk,nofrows
	rbuf = zero
        call FILL_BUF_INPL(mat,lmat,index,lindex,info,nvt,ia1,
     *                     lmyblk,ptrmyb,rbuf,nofrows,l,js,matmark,
     *                     ixmark,myproc,nproc)

	endmato = endmaton
	endindo = endindon

        if (runacc == 0) then
          call GC(mat,index,info,lmat,lindex,ia1,nvt,matmark,ixmark,
     *            endmaton,endindon,dtyp9)
        else
          endmax = MAX(endmaton,endindon)
          call GCOPT(mat,index,info,lmat,lindex,ia1,nvt,matmark,ixmark,
     *               endmaton,endindon,endmax,dtyp9)
        endif

        if (js == 1) then
          ptrmat9 = endmaton
          ptrind9 = endindon
        else
          matdiff = endmato - endmaton
          inddiff = endindo - endindon
!         print*,' ptrinfo,matdiff,inddiff:', ptrinfo,matdiff,inddiff
	  do i=1,ptrinfo
	    infnew(i,2) = infnew(i,2) - matdiff
	    infnew(i,6) = infnew(i,6) - inddiff
          enddo
	  do i=endmato+1,ptrmat9
	    mat(i-matdiff) = mat(i)
	  enddo
	  do i=endindo+1,ptrind9
	    index(i-inddiff) = index(i)
	  enddo
	  ptrmat9 = ptrmat9 - matdiff
	  ptrind9 = ptrind9 - inddiff
        endif

        call STO19_BUF(lmat,lindex,mat,index,
     *                 infnew,lmyblk,
     *                 rbuf,nofrows,l,js,ptrmyb,ptrmat1,ptrmat9,
     *                 ptrind9,ptrinfo,threshold,cr_md)
      enddo

      deallocate(rbuf)

      endmat = ptrmat9
      endind = ptrind9
      ilin(33) = endmat
      ilin(34) = endind

      if (lmyblk > ia1) then
	ierr = 1
      endif
      call LL4INM(1,1,1,ierr,iermax,ihelp,myproc,nproc,tids,nmsg)
      if (iermax /= 0) then
!       deallocate(info,STAT=istat)
!       if (istat /= 0) then
	  ierr = 6001
	  goto 999
!       else
!         call getKeyword('LINSOL_INFO_PARFAC',pinfofac,ier)
!         if (ier /= 0) then
!           pinfofac = REAL(nproc)
!         endif
!         ia1 = int(lmyblk*pinfofac)
!         allocate(info(ia1,ia2))
!         if (idoku > 0) then
!           if (myproc == 1) write(lout,1023) ia1
!         endif
!         if (idoku < 0) write(lout,1023) ia1
!         ierr = 0
!       endif
      endif

      do j=1,ia2
        do i=1,lmyblk
	  info(i,j) = infnew(i,j)
        enddo
      enddo

      nvt      = lmyblk
      ilin(35) = nvt

!***  Globalization of the local variables ilin(33),ilin(34) and ilin(35)
      call LL4ISM(1,3,1,ilin(33),ilin(30),ihelp,myproc,nproc,
     *            tids,nmsg)

      ilin(3)  = nmsg
      ilin(5)  = nvt

  999 continue
	    
 1000 format(/'   ------------------------------------------------ '/
     &       '  ||  Convertion in routine LSPCVT_INPL:          ||')
 1001 format('  ||    matrix <MAT> is completely stored in      ||'/
     &       '  ||    storage pattern "packed row"!             ||')
 1002 format('  ||    matrix <MAT> is NOT completely stored in  ||'/
     &       '  ||    storage pattern "packed row"!             ||')
 1003 format('  ||    There is NOTHING to do!                   ||')
 1004 format('  ||    matrix <MAT> will be restored in place    ||'/
     &       '  ||    into storage pattern "packed row"!        ||')
 1010 format('   ------------------------------------------------ '/)
!1023 format('  Caution: array <info> has been reallocated with'/
!    &       '           value ',i8,' as first dimension <ia1>'/)
!
!**** END OF CALCULATION
!     ------------------
!
      r e t u r n
!-----END OF LSPCVT_INPL----------------------------------------------
      e    n    d
