!**********************************************************************
!        1         2         3         4         5         6         7*
!**********************************************************************
!**                                                                 ***
!**                                                                 ***
      subroutine MATSORTLU(mat,lmat,infoLU,ia1LU,nvtLU,ilin,endmat,
     *                     myproc,nproc,tids,nmsg,ierr,iconv)
!**                                                                 ***
!**                                                                 ***
!**    State of paramet.: i/o in   i/o in  in   i/o   in    in      ***
!**                       in  i/o   out  out                        ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**   M A T S O R T L U  sorts the array  <mat> increasingly        ***
!**                      according to the information               ***
!**                      array <infoLU>.                            ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**    Copyright (C) 2000 University Karlsruhe                      ***
!**    PROGRAMMER: H. Haefner                                       ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**     THE IMPLICIT NONE-INSTRUCTION HAS THE AIM,                  ***
!**     THAT ALL VARIABLES MUST BE DECLARED EXPLICITLY.             ***
!**                                                                 ***
      USE constants

      implicit none
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**      FORMAL PARAMETERS :                                        ***
!**      -----------------                                          ***
!**                                                                 ***
      integer, intent(inout) ::  nmsg
      integer, intent(in) ::  ia1LU,nvtLU,nproc,myproc,lmat,endmat
      integer, intent(out) ::  ierr,iconv
      integer, intent(inout) ::  infoLU(ia1LU,ia2LU),ilin(nilin)
      integer, intent(in) ::  tids(nproc)
      double precision, intent(inout) :: mat(lmat)
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**   LIST OF FORMAL PARAMETERS :                                   ***
!**   -------------------------                                     ***
!**                                                                 ***
!**   (see Manual-Pages <linsol>)                                   ***
!**                                                                 ***
!**                                                                 ***
!**      LOCAL PARAMETERS :                                         ***
!**      -----------------                                          ***
!**                                                                 ***
      integer    bufsize
      double precision,dimension(:),pointer :: buf
      integer    i,ii,j,k,iermax,lastelem
      integer    buf_fill,ihelp,lout,offset,ptrfill,vtfirst,vtlast
      integer    ptradd(nvtLU),ptraddr(nvtLU)
      logical    lfill,lmove,lmono,lident

!***********************************************************************
!**    Interface of external subroutines                             ***
!***********************************************************************
      interface
       subroutine CREATE_BUFR(buf,bufsize,maxbuf,ilin,myproc,nproc,
     *                        tids,nmsg,ierr)
	USE constants
        implicit none
        integer,intent(in) ::   nproc,myproc,maxbuf
        integer,intent(inout) ::  nmsg
        integer,intent(out) ::  ierr
        integer,intent(in) ::   ilin(nilin),tids(nproc)
        double precision,dimension(:),pointer ::  buf
        integer,intent(out)::   bufsize
       end subroutine
      end interface

!
!**** START OF CALCULATION :
!     ---------------------
!

      ierr  = 0

      lout   = ilin(12)

!
!***  Create pointer array sorting the information array regarding to
!***  pointers to the matrix array <mat>
!
      lmono = .true.
      do i=2,nvtLU
        if (infoLU(i,1) < infoLU(i-1,1)) then
          lmono = .false.
          exit
        endif
      enddo

      if (lmono) then
        do i=1,nvtLU
          ptradd(i) = i
          ptraddr(i) = i
        enddo
      else
        do i=1,nvtLU
          ptraddr(i) = infoLU(i,1)
        enddo
        lident = .true.
        call HEAPSORTIX(ptraddr,ptradd,nvtLU,lident)
      endif

      if (.not. lmono) then
        lident = .true.
        call HEAPSORTIX(ptradd,ptraddr,nvtLU,lident)
      endif

!**
!**   Allocate buffer for array <mat>
!**
      call CREATE_BUFR(buf,bufsize,endmat,ilin,myproc,nproc,
     *                 tids,nmsg,ierr)
      if (ierr /= 0) then
        iconv = 99
        goto 9999
      endif

!**
!**   Restore matrix increasingly according to information array <infoLU>
!**

      ptrfill = 0
      vtfirst = 1
      vtlast  = 0

      do while (vtfirst <= nvtLU)
!**
!**   Compute last index of array <vtlast> to be filled into buffer
!**
        buf_fill = 0
	lfill = .false.
	do i=vtfirst,nvtLU
	  buf_fill = buf_fill + infoLU(i,2)
	  if (buf_fill > bufsize) exit
	  lfill = .true.
	  vtlast = vtlast + 1
	enddo

	if (.not. lfill) then
	  ierr = 8200 + myproc
	  iconv = 99
	  goto 999
	endif

!**
!**   Fill buffer up to bufsize
!**
	offset = 0
	do i=vtfirst,vtlast
	  buf(offset+1:offset+infoLU(i,2)) = 
     &                 mat(infoLU(i,1)+1:infoLU(i,1)+infoLU(i,2))
	  offset = offset + infoLU(i,2)
	enddo

!**
!**   Push together the matrix array up to the end of the array
!**
        lastelem = endmat
	i = nvtLU
	k = nvtLU
	do while (k > vtlast)
	  if (ptraddr(i) > vtlast) then
	    do j=infoLU(ptraddr(i),2),1,-1
	      mat(lastelem) = mat(infoLU(ptraddr(i),1)+j)
	      lastelem = lastelem - 1
	    enddo
	    infoLU(ptraddr(i),1) = lastelem
	    k = k-1
	  endif
	  i = i-1
        enddo

!**
!**   Restore the buffer starting from the pointer <ptrfill>
!**
        offset = 0
        do i=vtfirst,vtlast
	  infoLU(i,1) = ptrfill
	  mat(ptrfill+1:ptrfill+infoLU(i,2)) = 
     &                  buf(offset+1:offset+infoLU(i,2))
	  offset = offset + infoLU(i,2)
	  ptrfill = ptrfill + infoLU(i,2)
        enddo

        vtfirst = vtlast + 1
      enddo ! while-loop


      deallocate(buf)


  999 call LL4INM(1,1,1,ierr,iermax,ihelp,myproc,nproc,tids,nmsg)
      if (iermax > 0) then
        ierr = iermax
        if (myproc == 1) then
          write(lout,1000)
          write(lout,1010) iermax-8200
          write(lout,1020)
        endif
      endif
 1000 format(' **********************************************'/
     &       ' ***  FATAL ERROR occured in MATSORTLU:     ***')
 1010 format(' ***     The number of elements of a single ***'/
     &       ' ***     row is greater than the size of    ***'/
     &       ' ***     the buffer on processor ',i6,' !   ***')
 1020 format(' **********************************************')

 9999 R E T U R N
!------END OF MATSORTLU--------------------------------------------
      E N D                                   
