!**********************************************************************
!        1         2         3         4         5         6         7*
!**********************************************************************
!**                                                                 ***
!**                                                                 ***
      subroutine PACK_MAT(mat,lmat,index,lindex,info,ia1,ilin,
     *                    memopt,lmain,lmatbk,ptrmbk,nproc,tids,ierr)
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**   P A C K _ M A T  packs the storage patterns 2,7 and 8, if     ***
!**                    less than 66 percent are not equal to zero.  ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**    Copyright (C) 1998 University Karlsruhe                      ***
!**    PROGRAMMER: H. Haefner                                       ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**     THE IMPLICIT NONE-INSTRUCTION HAS THE AIM,                  ***
!**     THAT ALL VARIABLES MUST BE DECLARED EXPLICITLY.             ***
!**                                                                 ***
      USE constants

      implicit none
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**      FORMAL PARAMETERS :                                        ***
!**      -----------------                                          ***
!**                                                                 ***
      integer,intent(in) ::  lmat,lindex,ia1,nproc
      integer,intent(in) ::  lmatbk(nproc),ptrmbk(nproc+1),tids(nproc)
      integer,intent(inout) ::  index(lindex),info(ia1,ia2),ilin(nilin)
      integer,intent(out) :: ierr
      double precision,intent(inout):: memopt
      double precision,intent(inout):: mat(lmat)
      logical,intent(in) ::  lmain
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**   LIST OF FORMAL PARAMETERS :                                   ***
!**   -------------------------                                     ***
!**                                                                 ***
!--------I------I-----I------------------------------------------------
! NAME   I TYPE I I/O I     MEANING
!--------I------I-----I------------------------------------------------
!--------I------I-----I------------------------------------------------
! lmain  I  L   I in  I  true : a main diagonal will be created
!--------I------I-----I------------------------------------------------
! memopt I  DP  I in  I  if the percentage of nonzero elements of a full
!        I      I     I  storage pattern is less than <memopt> then
!        I      I     I  the full storage pattern will be packed!
!        I      I     I  (0 <= memopt <=1) e.g. memopt == 1  -->
!        I      I     I  full storage patterns will always be packed!
!XXXXXXXXIXXXXXXIXXXXXI  if (memopt == -1.) take environment-variable
!XXXXXXXXIXXXXXXIXXXXXI  LINSOL_MEMORY_OPT
!--------I------I-----I------------------------------------------------
!**                                                                 ***
!**                                                                 ***
!**      LOCAL PARAMETERS :                                         ***
!**      -----------------                                          ***
!**                                                                 ***
      integer    myproc, nmsg, idoku, lout, nvt
      integer    adda,addan,cnt,i,ivt,iac,iar,indc,indcn,indr,indrn,k
      integer    endind,endmat,gendind,gendmat,ishift,ixmin,ixmark
      integer    knew,lvt,lmyblk,ptrmyb,runacc,endmax
      integer    cnt2,cnt7,cnt8,ihelp,isum,madda,nivt
      double precision mone,zero,matmark
      real       memfac,pcnt,one
      integer    jc, jr, iermax, indmax, iarloc, iacloc, nlvt
      logical    lsto9
      parameter (zero = 0., one = 1., mone = -1.)

!
!**** START OF CALCULATION :
!     ---------------------
!
      nmsg   = ilin(3)
      nvt    = ilin(5)
      lout   = ilin(12)
      idoku  = ilin(13)
      myproc = ilin(17)
      runacc = ilin(21)

      ierr   = 0

      lmyblk = lmatbk(myproc)
      ptrmyb = ptrmbk(myproc)
      matmark= TINY(1.d0)
      ixmark = -999

!     ier = 0
!     if (memopt == mone) then
!       call getKeyword('LINSOL_MEMORY_FACTOR',memfac,ier)
!       if (ier == 0) then
!         if (memfac < zero) memfac = zero
!         if (memfac > one)  memfac = one
!         memopt = memfac
!       else
!         if (idoku > 0) then
!           if (myproc == 1) then
!             write(lout,1001)
!             write(lout,1012)
!             write(lout,1020)
!           endif
!         endif
!         if (idoku < 0) then
!           write(lout,1001)
!           write(lout,1012)
!           write(lout,1020)
!         endif
!         memopt = zero
!       endif
!     endif
!
!***  SET VARIABLES POINTING TO THE END OF THE MATRIX- AND INDEX-ARRAY
!
      endmat = ilin(33)
      endind = ilin(34)

!
!***  CREATE ENTRY IN INFORMATION ARRAY FOR MAIN DIAGONAL IF <LMAIN> SET
!
      if (lmain) then
	nvt = nvt + 1
	info(nvt,1) = 1
	info(nvt,2) = endmat
	info(nvt,3) = lmyblk
	info(nvt,4) = ptrmyb
	info(nvt,5) = ptrmyb
	info(nvt,6) = 0
	info(nvt,7) = 0
	do i=1,lmyblk
	  mat(endmat+i) = zero
        enddo
	madda = endmat
        endmat=endmat+lmyblk
      else
	do ivt=1,nvt
	  if (info(ivt,1) == 1) madda = info(ivt,2)
        enddo
      endif

!
!***  PACK THE STORAGE PATTERNS 2, 3, 6, 7, 8, 9 AND 11;
!***  EXTRACT THE MAIN DIAGONAL IF NOT PRESENT
!
      cnt2 = 0
      cnt7 = 0
      cnt8 = 0
      do ivt = 1,nvt

	if (info(ivt,1) == 2) then
	  adda   = info(ivt,2)
	  lvt    = info(ivt,3)
	  iac    = info(ivt,4)
	  iar    = info(ivt,5)
	  if (iac == iar) then
	    ierr = 0501
	    goto 888
	  endif
	  cnt    = 0
	  do k=1,lvt
	    if (mat(adda+k) /= zero) cnt = cnt+1
	  enddo
	  pcnt = REAL(cnt)/REAL(lvt)
	  if (pcnt < memopt) then
	    addan = adda
	    indcn = endind
	    knew  = 0
	    do k=1,lvt
	      if (mat(adda+k) /= zero) then
		knew = knew + 1
		ixmin = min(indcn+knew,lindex)
		mat(addan+knew) = mat(adda+k)
		index(ixmin) = iac+k
              endif
	    enddo
	    endind      = endind + knew
	    cnt2        = cnt2 + 1
	    info(ivt,1) = 3
	    info(ivt,3) = knew
	    if (endind <= lindex) then
	      ishift = index(indcn+1) - 1
	      do k=1,knew
	        index(indcn+k) = index(indcn+k) - ishift
	      enddo
	      info(ivt,4) = iac + ishift
	      info(ivt,5) = iar + ishift
	      info(ivt,6) = indcn
	    endif
	    do k=knew+1,lvt
	      mat(addan+k) = matmark
	    enddo
	  endif

	  if (pcnt >= memopt .and. pcnt < one) then
	    k = 1
	    do while (mat(adda+k) == zero .and. lvt > 0)
	      lvt = lvt - 1
	      iac = iac + 1
	      iar = iar + 1
	      mat(adda+k) = matmark
	      k = k + 1
	    enddo
	    k = info(ivt,3)
	    do while (mat(adda+k) == zero .and. lvt > 0)
	      lvt = lvt - 1
	      mat(adda+k) = matmark
	      k = k - 1
	    enddo
	    info(ivt,3) = lvt
	    info(ivt,4) = iac
	    info(ivt,5) = iar
	  endif
	endif

	if (info(ivt,1) == 3) then
	  adda   = info(ivt,2)
	  lvt    = info(ivt,3)
	  iac    = info(ivt,4)
	  iar    = info(ivt,5)
	  if (iac == iar) then
	    ierr = 0501
	    goto 888
	  endif
	  indc   = info(ivt,6)
	  nlvt   = lvt
	  do k=1,lvt
	    if (mat(adda+k) == zero) then
	      mat(adda+k) = matmark
	      index(indc+k) = ixmark
	      nlvt = nlvt - 1
            endif
	  enddo
	  info(ivt,3) = nlvt
	endif

	if (info(ivt,1) == 4 .and. lmain) then
	  adda   = info(ivt,2)
	  lvt    = info(ivt,3)
	  iar    = info(ivt,5)
	  iarloc = iar - ptrmyb
	  indc   = info(ivt,6)
	  do k=1,lvt
	    if (index(indc+k) == iar+k) then
	      mat(madda+iarloc+k) = mat(madda+iarloc+k) + mat(adda+k)
	      mat(adda+k) = zero
            endif
	  enddo
	endif

	if (info(ivt,1) == 5 .and. lmain) then
	  adda   = info(ivt,2)
	  lvt    = info(ivt,3)
	  iac    = info(ivt,4)
	  iacloc = iac - ptrmyb
	  indr   = info(ivt,7)
	  do k=1,lvt
	    if (index(indr+k) == iac+k) then
	      mat(madda+iacloc+k) = mat(madda+iacloc+k) + mat(adda+k)
	      mat(adda+k) = zero
            endif
	  enddo
	endif

	if (info(ivt,1) == 6) then
	  adda   = info(ivt,2)
	  lvt    = info(ivt,3)
	  indc   = info(ivt,6)
	  indr   = info(ivt,7)
	  if (lmain) then
	    do k=1,lvt
	      jc = index(indc+k)
	      jr = index(indr+k)
	      if (jr == jc) then
	        mat(madda+jr-ptrmyb) = mat(madda+jr-ptrmyb)+mat(adda+k)
	        mat(adda+k) = zero
              endif
	    enddo
	  endif
	  nlvt   = lvt
	  do k = 1,lvt
	    if (mat(adda+k) == zero) then
	      mat(adda+k) = matmark
	      index(indc+k) = ixmark
	      index(indr+k) = ixmark
	      nlvt = nlvt - 1
            endif
	  enddo
	  info(ivt,3) = nlvt
	endif

	if (info(ivt,1) == 7) then
	  adda   = info(ivt,2)
	  lvt    = info(ivt,3)
	  iac    = info(ivt,4)
	  iar    = info(ivt,5)
	  iarloc = iar - ptrmyb
          if ((iar < iac).or.(iar+1 > iac+lvt).or..not.lmain) goto 10
	  do k=1,lvt
	    if (iac+k == iar+1) then
	      mat(madda+iarloc+1) = mat(madda+iarloc+1) + mat(adda+k)
	      mat(adda+k) = zero
            endif
	  enddo
   10     cnt    = 0
	  do k=1,lvt
	    if (mat(adda+k) /= zero) cnt = cnt+1
	  enddo
	  pcnt = REAL(cnt)/REAL(lvt)
	  if (pcnt < memopt) then
	    addan = adda
	    indcn = endind
	    knew  = 0
	    do k=1,lvt
	      if (mat(adda+k) /= zero) then
		knew = knew + 1
		ixmin = min(indcn+knew,lindex)
		mat(addan+knew) = mat(adda+k)
		index(ixmin) = iac+k
              endif
	    enddo
	    endind      = endind + knew
	    cnt7        = cnt7 + 1
	    info(ivt,1) = 9
	    info(ivt,3) = knew
	    info(ivt,4) = 0
	    info(ivt,6) = indcn
	    do k=knew+1,lvt
	      mat(addan+k) = matmark
	    enddo
	  endif

	  if (pcnt >= memopt .and. pcnt < one) then
	    k = 1
	    do while (mat(adda+k) == zero .and. lvt > 0)
	      lvt = lvt - 1
	      iac = iac + 1
	      mat(adda+k) = matmark
	      k = k + 1
	    enddo
	    k = info(ivt,3)
	    do while (mat(adda+k) == zero .and. lvt > 0)
	      lvt = lvt - 1
	      mat(adda+k) = matmark
	      k = k - 1
	    enddo
	    info(ivt,3) = lvt
	    info(ivt,4) = iac
	  endif
	endif

	if (info(ivt,1) == 8) then
	  adda   = info(ivt,2)
	  lvt    = info(ivt,3)
	  iac    = info(ivt,4)
	  iacloc = iac - ptrmyb
	  iar    = info(ivt,5)
          if ((iac < iar).or.(iac+1 > iar+lvt).or..not.lmain) goto 20
	  do k=1,lvt
	    if (iar+k == iac+1) then
	      mat(madda+iacloc+1) = mat(madda+iacloc+1) + mat(adda+k)
	      mat(adda+k) = zero
            endif
	  enddo
   20     cnt    = 0
	  do k=1,lvt
	    if (mat(adda+k) /= zero) cnt = cnt+1
	  enddo
	  pcnt = REAL(cnt)/REAL(lvt)
	  if (pcnt < memopt) then
	    addan = adda
	    indrn = endind
	    knew  = 0
	    do k=1,lvt
	      if (mat(adda+k) /= zero) then
		knew = knew + 1
		ixmin = min(indrn+knew,lindex)
		mat(addan+knew) = mat(adda+k)
		index(ixmin) = iar+k
              endif
	    enddo
	    endind      = endind + knew
	    cnt8        = cnt8 + 1
	    info(ivt,1) = 11
	    info(ivt,3) = knew
	    info(ivt,5) = 0
	    info(ivt,7) = indrn
	    do k=knew+1,lvt
	      mat(addan+k) = matmark
	    enddo
	  endif

	  if (pcnt >= memopt .and. pcnt < one) then
	    k = 1
	    do while (mat(adda+k) == zero .and. lvt > 0)
	      lvt = lvt - 1
	      iar = iar + 1
	      mat(adda+k) = matmark
	      k = k + 1
	    enddo
	    k = info(ivt,3)
	    do while (mat(adda+k) == zero .and. lvt > 0)
	      lvt = lvt - 1
	      mat(adda+k) = matmark
	      k = k - 1
	    enddo
	    info(ivt,3) = lvt
	    info(ivt,5) = iar
	  endif
	endif

	if (info(ivt,1) == 9) then
	  adda   = info(ivt,2)
	  lvt    = info(ivt,3)
	  iar    = info(ivt,5)
	  iarloc = iar - ptrmyb
	  indc   = info(ivt,6)
	  if (lmain) then
	    do k=1,lvt
	      jc = index(indc+k)
	      if (iar+1 == jc) then
	        mat(madda+iarloc+1) = mat(madda+iarloc+1) + mat(adda+k)
	        mat(adda+k) = zero
              endif
	    enddo
	  endif
	  nlvt   = lvt
	  do k=1,lvt
	    if (mat(adda+k) == zero) then
	      mat(adda+k) = matmark
	      index(indc+k) = ixmark
	      nlvt = nlvt - 1
            endif
	  enddo
	  info(ivt,3) = nlvt
	endif

	if (info(ivt,1) == 11) then
	  adda   = info(ivt,2)
	  lvt    = info(ivt,3)
	  iac    = info(ivt,4)
	  iacloc = iac - ptrmyb
	  indr   = info(ivt,7)
	  if (lmain) then
	    do k=1,lvt
	      jr = index(indr+k)
	      if (iac+1 == jr) then
	        mat(madda+iacloc+1) = mat(madda+iacloc+1) + mat(adda+k)
	        mat(adda+k) = zero
              endif
	    enddo
	  endif
	  nlvt   = lvt
	  do k=1,lvt
	    if (mat(adda+k) == zero) then
	      mat(adda+k) = matmark
	      index(indr+k) = ixmark
	      nlvt = nlvt - 1
            endif
	  enddo
	  info(ivt,3) = nlvt
	endif
      enddo

      if (endind > lindex) then
	ierr = 0401
      endif
  888 call LL4INM(1,1,1,ierr,iermax,ihelp,myproc,nproc,tids,nmsg)
      if (iermax > 0) goto 999

      lsto9 = .true.
      do ivt=1,nvt
        if (info(ivt,1) /= 9) then
	  lsto9 = .false.
	  exit
	endif
      enddo

      if (runacc == 0) then
        call GC(mat,index,info,lmat,lindex,ia1,nvt,matmark,ixmark,
     *          endmat,endind,lsto9)
      else
	endmax = MAX(endmat,endind)
        call GCOPT(mat,index,info,lmat,lindex,ia1,nvt,matmark,ixmark,
     *             endmat,endind,endmax,lsto9)
      endif

      if (ilin(13) < 0) then
        write(lout,1030)
        if (lmain) write(lout,1031)
      endif
      if (ilin(13) > 0) then
        if (myproc == 1) then
          write(lout,1030)
          if (lmain) write(lout,1031)
        endif
      endif
!
!***  PRINT THE PERCENTAGE OF NONZERO MAIN DIAGONAL ELEMENTS
!
      if (lmain .and. ((mod(ilin(6),10) /= 0) .or. ilin(15) == 0)) then
        cnt = 0
        do i=1,lmyblk
          if (mat(madda+i) /= zero) cnt = cnt + 1
        enddo
        call LL4ISM(1,1,1,cnt,isum,ihelp,myproc,nproc,tids,nmsg)
        pcnt = REAL(isum)/REAL(ilin(36))
        pcnt = pcnt*100
        if (ilin(13) < 0) then
          write(lout,1032) pcnt
        endif
        if (ilin(13) > 0) then
          if (myproc == 1) then
            write(lout,1032) pcnt
          endif
        endif
      endif

!
!***  PRINT THE NUMBER OF NEW CREATED PACKED STORAGE PATTERNS
!
      if (ilin(13) < 0) then
        write(lout,1033) cnt2
        write(lout,1034) cnt7
        write(lout,1035) cnt8
      endif
      if (ilin(13) > 0) then
        if (myproc == 1) then
          write(lout,1033) cnt2
          write(lout,1034) cnt7
          write(lout,1035) cnt8
        endif
      endif

      call LL4ISM(1,1,1,endmat,gendmat,ihelp,myproc,nproc,
     *            tids,nmsg)
      if (ilin(13) < 0) then
        write(lout,1036) ilin(30)
        write(lout,1037) gendmat
      endif
      if (ilin(13) > 0) then
        if (myproc == 1) then
          write(lout,1036) ilin(30)
          write(lout,1037) gendmat
        endif
      endif
      ilin(30) = gendmat
      ilin(33) = endmat

      call LL4ISM(1,1,1,endind,gendind,ihelp,myproc,nproc,
     *            tids,nmsg)
      if (ilin(13) < 0) then
        write(lout,1038) ilin(31)
        write(lout,1039) gendind
        write(lout,1040)
      endif
      if (ilin(13) > 0) then
        if (myproc == 1) then
          write(lout,1038) ilin(31)
          write(lout,1039) gendind
          write(lout,1040)
        endif
      endif
      ilin(31) = gendind
      ilin(34) = endind


  999 if (iermax > 0) then
	ierr = iermax
	if (ierr == 401) then
         call LL4INM(1,1,1,endind,indmax,ihelp,myproc,nproc,tids,nmsg)
	 if (myproc == 1) then
          write(lout,1000)
          write(lout,1010) indmax
          write(lout,1020)
         endif
        endif
	if (ierr == 501) then
	 if (myproc == 1) then
          write(lout,1000)
          write(lout,1011)
          write(lout,1020)
         endif
        endif
      endif

      ilin(3) = nmsg
      ilin(5) = nvt
      ilin(35)= nvt

 1000 format(' **************************************************'/
     &       ' ***  ERROR occured in routine PACK_MAT:        ***')
!1001 format(' **************************************************'/
!    &       ' ***  WARNING from routine PACK_MAT:            ***')
 1010 format(' ***   index array <index> too small! Set       ***'/
     &       ' ***   <lindex> to ',i8,' on each proc.!      ***')
 1011 format(' ***   full or packed diagonal must not be      ***'/
     &       ' ***   defined as main diagonal!                ***')
!1012 format(' ***   the environment-variable                 ***'/
!    &       ' ***   LINSOL_MEMORY_FACTOR is not set! This    ***'/
!    &       ' ***   means that vector terms of types {2,7,8} ***'/
!    &       ' ***   will not be packed.                      ***')
 1020 format(' **************************************************'/)
 1030 format(/'   ------------------------------------------------ '/
     &       '  ||  Optimizations in routine PACK_MAT:          ||')
 1031 format('  ||    a Main Diagonal (MD) has been created!!   ||')
 1032 format('  ||    ',f6.1,' percent of the MD are being        ||'/
     &       '  ||             filled with non-zero elements!   ||')
 1033 format('  ||    ',i6,' full diagonals have been packed    ||'/
     &       '  ||           into packed diagonals,             ||')
 1034 format('  ||    ',i6,' full rows have been packed         ||'/
     &       '  ||           into packed rows,                  ||')
 1035 format('  ||    ',i6,' full columns have been packed      ||'/
     &       '  ||           into packed columns!               ||')
 1036 format('  ||    The size of the original (global)         ||'/
     &       '  ||    matrix <MAT> is ',i12,' >>>          ||')
 1037 format('  ||    The size of the packed (global)           ||'/
     &       '  ||    matrix <MAT> is ',i12,' !            ||')
 1038 format('  ||    The size of the original (global) index   ||'/
     &       '  ||    array <index> is ',i12,' >>>         ||')
 1039 format('  ||    The size of the packed (global) index     ||'/
     &       '  ||    array <index> is ',i12,' !           ||')
 1040 format('   ------------------------------------------------ '/)

      R E T U R N
!------END OF PACK_MAT--------------------------------------------
      E N D
