!**********************************************************************
!        1         2         3         4         5         6         7*
!**********************************************************************
!**                                                                 ***
!**                                                                 ***
      subroutine permchain(perm,rperm,l,lmatbk,ptrmbk,myproc,nproc,
     *                     tids,nmsg)
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**   P E R M C H A I N   closes an open chain of permutations of   ***
!**                       the permutation vector <perm>.            ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**    Copyright (C) 2001 University Karlsruhe                      ***
!**    PROGRAMMER: H. Haefner                                       ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**     THE IMPLICIT NONE-INSTRUCTION HAS THE AIM,                  ***
!**     THAT ALL VARIABLES MUST BE DECLARED EXPLICITLY.             ***
!**                                                                 ***
      USE constants

      implicit none
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**      FORMAL PARAMETERS :                                        ***
!**      -----------------                                          ***
!**                                                                 ***
      integer,intent(in) ::  l,myproc,nproc
      integer,intent(inout) ::  nmsg
      integer,intent(in) ::  lmatbk(nproc),ptrmbk(nproc+1),tids(nproc)
      integer,intent(inout) ::  perm(l),rperm(l)
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**   LIST OF FORMAL PARAMETERS :                                   ***
!**   -------------------------                                     ***
!**                                                                 ***
!--------I------I-----I------------------------------------------------
! NAME   I TYPE I I/O I     MEANING
!--------I------I-----I------------------------------------------------
!--------I------I-----I------------------------------------------------
! perm   I  I   I io  I  permutation vector
!        I      I     I  perm(i) = j: i is local and j is global param.
!--------I------I-----I------------------------------------------------
! rperm  I  I   I io  I  reverse permutation vector
!        I      I     I  rperm(i) = j: i is local and j is global param.
!--------I------I-----I------------------------------------------------
!**                                                                 ***
!**                                                                 ***
!**      LOCAL PARAMETERS :                                         ***
!**      -----------------                                          ***
!**                                                                 ***
      integer  permbufs(l),permbufr(l),chain(l)
      integer  i,ip,j,jloc,k,ierr,iglob,mids,midr,from,to,lmyblk
      integer  chloc,ihlp,newit,newitmax,nrblk
!
!**** START OF CALCULATION :
!     ---------------------
!
      ierr   = 0
      lmyblk = lmatbk(myproc)

      to = myproc+1
      if (to > nproc) to = 1
      to = tids(to)
      from = myproc-1
      if (from == 0) from = nproc
      from = tids(from)

      do i=1,lmyblk
        chain(i) = i + ptrmbk(myproc)
      enddo

      newit = 1

      do while (newit > 0)
        permbufs = rperm

        do ip=1,nproc
  	  if (nproc > 1) then
	    call MPRCVA(from,nmsg+ip,l*iint,permbufr,midr,ierr)
	    call MPSNDA(to,nmsg+ip,l*iint,permbufs,mids,ierr)
	  endif

	  nrblk = myproc + nproc - ip + 1
	  if (nrblk > nproc) nrblk = nrblk - nproc

	  do i=1,lmyblk
	    iglob = i + ptrmbk(myproc)
	    chloc = chain(i) - ptrmbk(nrblk)
	    if ((chloc > 0) .and. (chloc <= lmyblk)) then
	      if (perm(i) == iglob) then
                if (permbufs(chloc) /= chain(i)) then
	          j = permbufs(chloc)
	          jloc = j - ptrmbk(nrblk)
	          chain(i) = j
	          do while (jloc > 0 .and. jloc <= lmatbk(nrblk))
	            if (permbufs(jloc) /= j) then
		      j = permbufs(jloc)
	              jloc = j - ptrmbk(nrblk)
		      chain(i) = j
	            else
		      perm(i) = j
		      permbufs(jloc) = iglob
		      chain(i) = -1
		      exit
	            endif
	          enddo
	        else
		  perm(i) = chain(i)
		  permbufs(chloc) = iglob
		  chain(i) = -1
		endif
	      else
	        chain(i) = -1
	      endif
	    endif
	  enddo

	  if (nproc > 1) then
	    call MPSNDW(to,nmsg+ip,l*iint,permbufs,mids,ierr)
	    call MPRCVW(from,nmsg+ip,l*iint,permbufr,midr,ierr)
	    permbufs = permbufr
	  endif
        enddo

        rperm = permbufs
        nmsg = nmsg + nproc

	newit = 0
	do i=1,lmyblk
	  if (chain(i) /= -1) then
	    newit = 1
	    exit
	  endif
	enddo
        call LL4INM(1,1,1,newit,newitmax,ihlp,myproc,nproc,tids,nmsg)
	newit = newitmax

      enddo


      R E T U R N
!------END OF PERMCHAIN---------------------------------------------
      E N D                                   
