!:::::      ,,,,,PREFEL.....
!
!
!
!**********************************************************************
!        1         2         3         4         5         6         7*
!**********************************************************************
!**                                                                 ***
!**                                                                 ***
      subroutine PREFEL(b,mat,lmat,index,lindex,info,ia1,dw,l,iw,liw,
     #                  lsym,ilin,lmatbk,ptrmbk,nproc,tids,iconv,ierr)
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**      program description of PREFEL                              ***
!**                                                                 ***
!**      Finds single entries in rows and permutes the matrix so    ***
!**      they end up on the main diagonal. Then the corresponding   ***
!**      columns are eliminated.                                    ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**    Copyright (C) 1998 University Karlsruhe                      ***
!**    PROGRAMMER : Hartmut Haefner                                 ***
!**                 Detlev Zundel                                   ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
      USE constants
      USE precsave

      implicit none
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**      FORMAL PARAMETERS :                                        ***
!**      -----------------                                          ***
!**                                                                 ***
      integer :: l,lmat
      double precision mat(lmat),b(l),dw(2*l)
      integer, intent(out) ::  iconv
      integer :: ia1,ierr,lindex,liw
      integer :: nproc,ptrmbk(nproc+1),ilin(nilin),iw(liw)
      integer :: info(ia1,ia2),index(lindex)
      integer :: tids(nproc), lmatbk(nproc)
      logical :: lsym
      intent(inout) :: mat, index, b, info, ilin, lsym
      intent(in) :: lindex, ia1, l, ptrmbk, lmatbk, tids, nproc
      intent(out) :: ierr

      integer :: LL9MAP
!**********************************************************************
!**                                                                 ***
!**    List of Formal Parameters :                                  ***
!**                                                                 ***
!**                                                                 ***
!---------I------I-----I-----------------------------------------------
! NAME    I TYPE I I/O I MEANING
!---------I------I-----I-----------------------------------------------
! info    I int  I     I array with matrix-structure-information
!---------I------I-----I-----------------------------------------------
! index   I int  I     I array with indices
!---------I------I-----I-----------------------------------------------

!FIXME: Describe all parameters

      external LL6AX9

!**********************************************************************
!**                                                                 ***
!**                                                                 ***
!**        LOCAL PARAMETERS :                                       ***
!**        ----------------                                         ***
!**                                                                 ***
      integer :: myproc, nmsg, endmat, endind, endmato, endindo
      integer :: gendmat, gendmato, geldiff, gendmorg, endmorg, endiorg
      integer :: rowperm(l), col_done(l), commbuf1(l+1), commbuf2(l+1)
      integer :: rperm(l), touched(l)
      integer :: i, j, j0, jj0, k, ix, p, irow, ierrh, nvt, endmax
      integer :: iermax, jump(nproc)
      integer :: elimlist(l), elimllen, eldiff, lmyblk, lvt, runacc
      double precision :: xtemp(l), g(l), dmax
      integer :: frtid, lfrtid, totid, ltotid, nrblk, mids, midr
      integer :: iloop, ibuf(6), idoku, isave, lout, noloop, permloc
      integer :: infcnt, matcnt, gmatcnt, matsum, gmatsum, lvtcnt
      integer :: ia1red, ierr1, ierr2, irowloc, irowlocp, lmatred
      integer :: bufsize, ibufsize, infcntold, matcntold, unit
      logical :: lfirst

      integer, allocatable          :: infored(:,:), ixred(:)
      double precision, allocatable :: matred(:)

      integer, parameter :: ixmark = -999
      double precision, parameter ::  zero = 0.0, matmark = zero

      ierr   = 0
      noloop = 0

      nmsg   = ilin(3)
      nvt    = ilin(5)
      bufsize= ilin(7)
      lout   = ilin(12)
      idoku  = ilin(13)
      isave  = ilin(14)
      myproc = ilin(17)
      runacc = ilin(21)
      gendmat= ilin(30)
      endmat = ilin(33)
      endind = ilin(34)
      unit   = ilin(45)

      lmyblk = lmatbk(myproc)

      endmorg = endmat
      gendmorg= gendmat
      endiorg = endind

      ! Initialize parameters for the ringshift
      lfrtid=LL9MAP(myproc-1,nproc)
      frtid=tids(lfrtid)
      ltotid=LL9MAP(myproc+1, nproc)
      totid=tids(ltotid)

      ! Initialize the jump vector
      jump = 1

      if (idoku > 0) then
        if (myproc == 1) then
          write(lout,1000)
        endif
      endif
      if (idoku < 0) then
        write(lout,1000)
      endif

      col_done = 0
      matsum   = 0
      infcntold = 0
      matcntold = 0
      lfirst = .true.

!**   Forward pre-elimination loop                                  ***

 111  noloop = noloop + 1

      if (MOD(ilin(6),1000) < 100) then

!**   Determine the permutation vector                              ***

        do i=1,lmyblk
          rowperm(i)=i+ptrmbk(myproc)
	  touched(i) = 0
        enddo

        do i=1,lmyblk
          if (info(i,3) == 1) then
            rowperm(i) = index(info(i,6)+1)
	    if (rowperm(i) /= i+ptrmbk(myproc)) touched(i) = 1
          endif
        enddo

!**   Compute the touched rows by a ring shift                       ***

!     commbuf1(1:l)=rowperm
!     do p=1,nproc
!       if (nproc > 1) then
!         call MPRCVA(frtid, nmsg+p, l*iint, commbuf2, midr, ierr)
!         call MPSNDA(totid, nmsg+p, l*iint, commbuf1, mids, ierr)
!       endif

!**     fast version of nrblk=(((myproc-p+1)-1) mod nproc)+1         ***
!       nrblk=myproc+nproc-p+1
!       if (nrblk>nproc) nrblk=nrblk-nproc

!**     Compute the touched elements                                 ***
!       do i=1,lmatbk(nrblk)
!         permloc = commbuf1(i) - ptrmbk(myproc)
!         if ((permloc > 0) .and. (permloc <= lmyblk) .and.
!    &        (commbuf1(i) /= i+ptrmbk(nrblk))) then
!           touched(permloc) = 1
!         endif
!       enddo

!       if (nproc > 1) then
!         call MPSNDW(totid, nmsg+p, l*iint, commbuf1, mids, ierr)
!         call MPRCVW(frtid, nmsg+p, l*iint, commbuf2, midr, ierr)
!         commbuf1(1:l)=commbuf2(1:l)
!       endif
!     enddo
!     nmsg=nmsg+nproc

        call REVPERM(rowperm,rperm,touched,l,lmatbk,ptrmbk,
     &               myproc,nproc,tids,nmsg)

        call PERMCHAIN(rowperm,rperm,l,lmatbk,ptrmbk,
     &                 myproc,nproc,tids,nmsg)

!**   Permute the matrix and the right hand side                    ***
        ilin(33) = endmat
        ilin(34) = endind
        call ROWSORT(mat,lmat,index,lindex,info,ia1,nvt,rowperm,l,
     #               iw,ilin,ptrmbk,myproc,nproc,
     #               tids,nmsg,iconv,ierr)
        if (iconv == 99) then
          if (myproc == 1) then
            write(lout,1010)
            write(lout,1012)
            write(lout,1020)
          endif
          goto 999
        endif
        endmat = ilin(33)
        endind = ilin(34)
	if (isave == 1) write(unit) (rowperm(j),j=1,l)
      endif

      if (MOD(ilin(6),1000) >= 100) read(unit) (rowperm(j),j=1,l)
      
      call RHSIXSRT(b,rowperm,l,lmatbk,ptrmbk,myproc,nproc,tids,nmsg)

!**   Compute the number of permuted rows                           ***
      ibuf = 0
      do i=1,nvt
        irow = info(i,5) + 1
        if ((info(i,3) == 1) .and. (irow == index(info(i,6)+1))) 
     &    ibuf(1) = ibuf(1) + 1
      enddo
      do i=1,lmyblk
        if (rowperm(i) /= i+ptrmbk(myproc)) ibuf(2) = ibuf(2) + 1
      enddo
      call LL4ISM(1,2,1,ibuf(1),ibuf(3),ibuf(5),myproc,nproc,tids,nmsg)
      if (idoku > 0 .and. noloop < 4) then
        if (myproc == 1) then
          write(lout,1101) noloop
          write(lout,1102) ibuf(3)
          write(lout,1103) ibuf(4)
        endif
      endif
      if (idoku < 0 .and. noloop < 4) then
        write(lout,1101) noloop
        write(lout,1102) ibuf(3)
        write(lout,1103) ibuf(4)
      endif
      

!**   There is nothing else to do if the matrix is symmetric        ***
      if (lsym) goto 999
!----------------------------------------------------------------------

      do i=2,nvt
        if (info(i,5) < info(i-1,5)) then
          call INFOSORT(info,ia1,nvt,iw,lout,myproc,nproc,
     &                  tids,nmsg,ierr)
          exit
        endif
      enddo

!**   Determine dirichlet coefficients (xtemp)                      ***

      if (MOD(ilin(6),1000) < 100) then
        xtemp = zero
        elimllen = 0
        do i=1,nvt 
          irow = info(i,5) + 1
          if ((info(i,3) == 1) .and.
     #        (irow == index(info(i,6)+1)) .and.
     #        (col_done(irow-ptrmbk(myproc)) == 0)) then
            xtemp(irow-ptrmbk(myproc))=b(irow-ptrmbk(myproc))/
     #                                 mat(info(i,2)+1)
            elimllen=elimllen+1
            elimlist(elimllen)=irow
          endif
        enddo
	if (isave == 1) then
          write(unit) elimllen
	  if (elimllen > 0) write(unit) (elimlist(j),j=1,elimllen)
	endif
      endif

      if (MOD(ilin(6),1000) >= 100) then
        xtemp = zero
	read(unit) elimllen
	if (elimllen > 0) read(unit) (elimlist(j),j=1,elimllen)

	if (ALLOCATED(perm) .and. MOD(ilin(6),10000) >= 1000 .and.
     #      ALLOCATED(prec) .and. MOD(ilin(6),10) == 1) then
	  if (nproc == 1) then
	    do i=1,elimllen
	      irowloc = elimlist(i)
	      irowlocp = perm(elimlist(i))
	      xtemp(irowloc) = b(irowloc)*prec(irowlocp)/
     #                         mat(info(irowlocp,2)+1)
	    enddo
	  else
	    dw = zero
	    do i=1,nvt
	      if ((info(i,7) == 10000*myproc) .and.
     #            (info(i,3) == 1)) then
	        irowloc = info(i,5) - ptrmbk(myproc) + 1
	        dw(irowloc) = mat(info(i,2)+1)
	        dw(l+irowloc) = prec(irowloc)
	      endif
	    enddo
	    call RHSSRTIX(dw,perm,l,lmatbk,ptrmbk,myproc,nproc,tids,
     #                    nmsg)
	    call RHSSRTIX(dw(l+1),perm,l,lmatbk,ptrmbk,myproc,nproc,
     #                    tids,nmsg)
	    do i=1,elimllen
	      irowloc = elimlist(i) - ptrmbk(myproc)
	      xtemp(irowloc) = b(irowloc)*dw(l+irowloc)/dw(irowloc)
	    enddo
	  endif
	endif
	if (ALLOCATED(perm) .and. MOD(ilin(6),10000) >= 1000 .and.
     #      (.not. ALLOCATED(prec) .or. MOD(ilin(6),10) == 0)) then
	  if (nproc == 1) then
	    do i=1,elimllen
	      irowloc = elimlist(i)
	      irowlocp = perm(elimlist(i))
	      xtemp(irowloc) = b(irowloc)/mat(info(irowlocp,2)+1)
	    enddo
	  else
	    dw(1:l) = zero
	    do i=1,nvt
	      if ((info(i,7) == 10000*myproc) .and.
     #            (info(i,3) == 1)) then
	        irowloc = info(i,5) - ptrmbk(myproc) + 1
	        dw(irowloc) = mat(info(i,2)+1)
	      endif
	    enddo
	    call RHSSRTIX(dw,perm,l,lmatbk,ptrmbk,myproc,nproc,tids,
     #                    nmsg)
	    do i=1,elimllen
	      irowloc = elimlist(i) - ptrmbk(myproc)
	      xtemp(irowloc) = b(irowloc)/dw(irowloc)
	    enddo
	  endif
	endif
	if ((.not. ALLOCATED(perm)) .and.
     #      (ALLOCATED(prec) .and. MOD(ilin(6),10) == 1)) then
	  if (nproc == 1) then
	    do i=1,elimllen
	      irowloc = elimlist(i)
	      xtemp(irowloc) = b(irowloc)*prec(irowloc)/
     #                         mat(info(irowloc,2)+1)
	    enddo
	  else
	    dw(1:l) = zero
	    do i=1,nvt
	      if ((info(i,7) == 10000*myproc) .and.
     #            (info(i,3) == 1)) then
	        irowloc = info(i,5) - ptrmbk(myproc) + 1
	        dw(irowloc) = mat(info(i,2)+1)
	      endif
	    enddo
	    do i=1,elimllen
	      irowloc = elimlist(i) - ptrmbk(myproc)
	      xtemp(irowloc) = b(irowloc)*prec(irowloc)/dw(irowloc)
	    enddo
	  endif
	endif
	if ((.not. ALLOCATED(perm)) .and.
     #      (.not. ALLOCATED(prec) .or. MOD(ilin(6),10) == 0)) then
	  if (nproc == 1) then
	    do i=1,elimllen
	      irowloc = elimlist(i)
	      xtemp(irowloc) = b(irowloc)/mat(info(irowloc,2)+1)
	    enddo
	  else
	    dw(1:l) = zero
	    do i=1,nvt
	      if ((info(i,7) == 10000*myproc) .and.
     #            (info(i,3) == 1)) then
	        irowloc = info(i,5) - ptrmbk(myproc) + 1
	        dw(irowloc) = mat(info(i,2)+1)
	      endif
	    enddo
	    do i=1,elimllen
	      irowloc = elimlist(i) - ptrmbk(myproc)
	      xtemp(irowloc) = b(irowloc)/dw(irowloc)
	    enddo
	  endif
	endif
      endif

      call LL4INM(1,1,1,elimllen,ibuf(1),ibuf(2),
     #            myproc,nproc,tids,nmsg)

!**   There is nothing else to do if there are no single entries    ***
      if (ibuf(1) == 0) goto 999
!----------------------------------------------------------------------

!**   Sort elimlist                                                 ***
!     do i=1,elimllen-1
!       do j=1,elimllen-i
!         if (elimlist(j) > elimlist(j+1)) then
!           k=elimlist(j)
!           elimlist(j)=elimlist(j+1)
!           elimlist(j+1)=k
!         endif
!       enddo
!     enddo

!**   Determine g=A*xtemp                                           ***

      if (MOD(ilin(6),1000) < 100) then
        call LL3AXO(LL6AX9,mat,xtemp,g,dw,ptrmbk,lmatbk,
     #              l,nproc,tids,myproc,jump,
     #              .false.,nvt,lmat,lindex,ia1,ia2,info,index,nmsg)
      else
	read(unit) matcnt, infcnt

        call LL4ISM(1,1,1,matcnt,gmatcnt,ibuf(1),myproc,nproc,tids,nmsg)

        if (gmatcnt > 0) then
	  if (lfirst) then
	    allocate(infored(infcnt,ia2),ixred(matcnt),stat=ierr1)
	    allocate(matred(matcnt),stat=ierr2)
	    ia1red  = infcnt
	    lmatred = matcnt
	  endif
	  if (infcnt > infcntold .or. matcnt > matcntold) then
	    deallocate(infored,ixred,matred)
	    allocate(infored(infcnt,ia2),ixred(matcnt),stat=ierr1)
	    allocate(matred(matcnt),stat=ierr2)
	    ia1red  = infcnt
	    lmatred = matcnt
	  endif

          ibuf(1) = ierr1
          ibuf(2) = ierr2
          call LL4INM(1,2,1,ibuf(1),ibuf(3),ibuf(5),myproc,nproc,
     &                tids,nmsg)
          if (ibuf(3) > 0 .or. ibuf(4) > 0) then
            if (myproc == 1 .and. ibuf(3) > 0) then
              write(lout,1010)
              write(lout,1013)
              write(lout,1020)
            endif
            if (myproc == 1 .and. ibuf(4) > 0) then
              write(lout,1010)
              write(lout,1014)
              write(lout,1020)
            endif
            ierr = MAX(ierr1,ierr2)
            goto 999
          endif

	  infcntold = infcnt
	  matcntold = matcnt
	  lfirst = .false.

	  infored(1:infcnt,1) = 9
          if (matcnt > 0) then
	    read(unit) (infored(i,2), i=1,infcnt)
	    read(unit) (infored(i,3), i=1,infcnt)
	    read(unit) (infored(i,5), i=1,infcnt)
	    read(unit) (ixred(i), i=1,matcnt)
  	    read(unit) (matred(i), i=1,matcnt)
	  endif
	  infored(1:infcnt,6) = infored(1:infcnt,2)

!  if (ALLOCATED(perm) .and. MOD(ilin(6),10000) >= 1000) then
!           call ROWSORT(matred,lmatred,ixred,lmatred,infored,ia1red,
!    &                   infcnt,perm,l,iw,ilin,ptrmbk,myproc,nproc,
!    &                   tids,nmsg,iconv,ierr)
!           if (iconv == 99) then
!             if (myproc == 1) then
!               write(lout,1010)
!               write(lout,1012)
!               write(lout,1020)
!             endif
!             goto 999
!           endif
!           ibufsize=MAX(MAXLVT(infored, ia1red, infcnt),l)
!           call COLSORT(matred,lmatred,ixred,lmatred,infored,ia1red,
!    &                   perm,l,ibufsize,lmatbk,ptrmbk,
!    &                   myproc,nproc,tids,nmsg)
!         endif

!***  Use the matrix (MATRED)bwo,not_normalized !!
!         if (ALLOCATED(prec) .and. MOD(ilin(6),10) == 1) then
!           do i=1,lmyblk
!             dw(i) = 1.d0/prec(i)
!           enddo
!           call LSPPA9(matred,lmatred,ixred,lmatred,infored,ia1red,
!    &                  dw,l,infcnt,ptrmbk(myproc))
!         endif

          call LL3AXO(LL6AX9,matred,xtemp,g,dw,ptrmbk,lmatbk,
     #                l,nproc,tids,myproc,jump,.false.,infcnt,
     #                lmatred,lmatred,ia1red,ia2,infored,ixred,nmsg)
	  matsum = matsum + matcnt
	else
	  g = zero
        endif  ! fi (gmatcnt > 0)
      endif

!**   Blank out g for the diagonal elements
      do i=1,elimllen
        g(elimlist(i)-ptrmbk(myproc)) = zero
        col_done(elimlist(i)-ptrmbk(myproc)) = 1
      enddo

!**   Adjust right hand side as b=b-g                               ***

      do i=1,lmyblk
        b(i) = b(i) - g(i)
      enddo

!**   Mark eliminated matrix entries and indices                    ***
!**   Note that elimlist and the column indices have to be sorted!  ***

      if (MOD(ilin(6),1000) < 100) then
        if (isave == 1) then
          if (lfirst) then
	    bufsize = INT(bufsize*1.e6/3.)
	    ia1red  = INT(bufsize/(ia2*iint))
	    lmatred = INT(bufsize/ireal)
	    allocate(infored(ia1red,ia2),ixred(lmatred),stat=ierr1)
	    allocate(matred(lmatred),stat=ierr2)

            ibuf(1) = ierr1
            ibuf(2) = ierr2
            call LL4INM(1,2,1,ibuf(1),ibuf(3),ibuf(5),myproc,nproc,
     &                  tids,nmsg)
            if (ibuf(3) > 0 .or. ibuf(4) > 0) then
              if (myproc == 1 .and. ibuf(3) > 0) then
                write(lout,1010)
                write(lout,1013)
                write(lout,1020)
              endif
              if (myproc == 1 .and. ibuf(4) > 0) then
                write(lout,1010)
                write(lout,1014)
                write(lout,1020)
              endif
              ierr = MAX(ierr1,ierr2)
              goto 999
            endif
	    lfirst = .false.
          endif

	  matcnt = 0
	  infcnt = 0
	endif

        ierr = 0
        if (nproc > 1) then
          commbuf1(1:l) = elimlist
          commbuf1(l+1) = elimllen
	  touched(1:lmyblk) = info(1:lmyblk,3)
          do p=1,nproc
            call MPRCVA(frtid,nmsg+p,(l+1)*iint,commbuf2,midr,ierrh)
            call MPSNDA(totid,nmsg+p,(l+1)*iint,commbuf1,mids,ierrh)

            do i=1,lmyblk
	      lvt = info(i,3)
	      if (lvt == 1) cycle
	      j0 = 1
              if (isave == 1) lvtcnt = 0
              do k=1,commbuf1(l+1)
	        jj0 = j0
                do j=jj0,lvt
	  	  ix = index(info(i,6)+j)
                  if (ix < commbuf1(k)) then
		    j0 = j0 + 1
		    cycle
		  endif
                  if (ix > commbuf1(k)) exit
                  if (ix == commbuf1(k)) then
		    matcnt = matcnt + 1
                    if (isave == 1) then
		      if (matcnt > lmatred) then
		        matcnt = matcnt - 1
		        ierr = 9999
		      endif
		      ixred(matcnt) = index(info(i,6)+j)
		      matred(matcnt) = mat(info(i,2)+j)
		      lvtcnt = lvtcnt + 1
		    endif
                    index(info(i,6)+j) = ixmark
                    mat(info(i,2)+j) = matmark
                    touched(i) = touched(i) - 1
                    if (touched(i) == 0) ierr = 7777
                  endif
                enddo
              enddo
              if (isave == 1 .and. lvtcnt > 0) then
		infcnt = infcnt + 1
		if (infcnt > ia1red) then
		  infcnt = infcnt - 1
		  ierr = 8888
		endif
		infored(infcnt,2) = matcnt - lvtcnt
		infored(infcnt,3) = lvtcnt
		infored(infcnt,5) = i - 1 + ptrmbk(myproc)
	      endif
            enddo

            call MPSNDW(totid,nmsg+p,(l+1)*iint,commbuf1,mids,ierrh)
            call MPRCVW(frtid,nmsg+p,(l+1)*iint,commbuf2,midr,ierrh)
            commbuf1(1:l+1) = commbuf2(1:l+1)
          enddo
	  info(1:lmyblk,3) = touched(1:lmyblk)
          nmsg=nmsg+nproc
        else
          do i=1,lmyblk
	    lvt = info(i,3)
	    if (lvt == 1) cycle
	    j0 = 1
            if (isave == 1) lvtcnt = 0
            do k=1,elimllen
	      jj0 = j0
              do j=jj0,lvt
	        ix = index(info(i,6)+j)
                if (ix < elimlist(k)) then
	          j0 = j0 + 1
	          cycle
	        endif
                if (ix > elimlist(k)) exit
                if (ix == elimlist(k)) then
		  matcnt = matcnt + 1
                  if (isave == 1) then
		    if (matcnt > lmatred) then
		      matcnt = matcnt - 1
		      ierr = 9999
		    endif
		    ixred(matcnt) = index(info(i,6)+j)
		    matred(matcnt) = mat(info(i,2)+j)
		    lvtcnt = lvtcnt + 1
		  endif
                  index(info(i,6)+j) = ixmark
                  mat(info(i,2)+j) = matmark
                  info(i,3) = info(i,3) - 1
                  if (info(i,3) == 0) ierr = 7777
                endif
              enddo
            enddo
            if (isave == 1 .and. lvtcnt > 0) then
	      infcnt = infcnt + 1
	      if (infcnt > ia1red) then
	        infcnt = infcnt - 1
	        ierr = 8888
	      endif
	      infored(infcnt,2) = matcnt - lvtcnt
	      infored(infcnt,3) = lvtcnt
	      infored(infcnt,5) = i - 1
	    endif
          enddo
        endif

        call LL4INM(1,1,1,ierr,iermax,ibuf(1),myproc,nproc,tids,nmsg)

        if (iermax > 0) then
	  ierr = iermax
          if ((myproc == 1) .and. (ierr == 7777)) then
            write(lout,1010)
            write(lout,1011)
            write(lout,1020)
          endif
          if ((myproc == 1) .and. (ierr == 8888)) then
            write(lout,1010)
            write(lout,1015)
            write(lout,1020)
          endif
          if ((myproc == 1) .and. (ierr == 9999)) then
            write(lout,1010)
            write(lout,1016)
            write(lout,1020)
          endif
          goto 999
        endif

        if (isave == 1) then
	  write(unit) matcnt, infcnt
	  if (matcnt > 0) then
	    write(unit) (infored(i,2), i=1,infcnt)
	    write(unit) (infored(i,3), i=1,infcnt)
	    write(unit) (infored(i,5), i=1,infcnt)
	    write(unit) (ixred(i), i=1,matcnt)
	    write(unit) (matred(i), i=1,matcnt)
          endif
        endif

!**   Pack the new matrix                                           ***
        endmato = endmat
        gendmato= gendmat
        endindo = endind

        if (runacc == 0) then
          call GC(mat,index,info,lmat,lindex,ia1,nvt,matmark,ixmark,
     *            endmat,endind,.true.)
        else
          endmax = MAX(endmat,endind)
          call GCOPT(mat,index,info,lmat,lindex,ia1,nvt,matmark,ixmark,
     *               endmat,endind,endmax,.true.)
        endif

        ibuf(1) = endmat
	ibuf(2) = matcnt
        call LL4ISM(1,2,1,ibuf(1),ibuf(3),ibuf(5),
     &              myproc,nproc,tids,nmsg)
	gendmat = ibuf(3)
	gmatcnt = ibuf(4)
  
        eldiff = endmato - endmat
        geldiff= gendmato - gendmat
        if (eldiff /= endindo - endind) ierr = 1
        call LL4INM(1,1,1,ierr,ibuf(1),ibuf(2),myproc,nproc,tids,nmsg)
        if (ibuf(1) > 0) then
          if (myproc == 1) then
            write(lout,1010)
            write(lout,1017)
            write(lout,1020)
	  endif
          ierr = 6003
          goto 999
        endif
      endif     !fi (MOD(ilin(6),1000) < 100)


      if (MOD(ilin(6),1000) < 100) then
        if (idoku > 0 .and. noloop < 4) then
          if (myproc == 1) then
            write(lout,1002) geldiff
          endif
        endif
        if (idoku < 0 .and. noloop < 4) then
          write(lout,1002) geldiff
        endif
      else
        if (idoku > 0 .and. noloop < 4) then
          if (myproc == 1) then
            write(lout,1002) gmatcnt
          endif
        endif
        if (idoku < 0 .and. noloop < 4) then
          write(lout,1002) gmatcnt
        endif
      endif

!     dmax = zero
!     iloop = 0
!     do i=1,lmatbk(myproc)
!       if (abs(g(i)) > dmax) dmax = ABS(g(i))
!     enddo
!     if (dmax /= zero) iloop=1
!     call LL4INM(1,1,1,iloop,ibuf(1),ibuf(2),myproc,nproc,tids,nmsg)
!     if (ibuf(1) > 0) goto 111

      if (gmatcnt > 0) goto 111

  999 if (ALLOCATED(infored)) deallocate(infored)
      if (ALLOCATED(ixred)) deallocate(ixred)
      if (ALLOCATED(matred)) deallocate(matred)

      if (idoku > 0) then
        if (myproc == 1) then
          write(lout,1003) noloop
        endif
      endif
      if (idoku < 0) then
        write(lout,1003) noloop
      endif

      if (MOD(ilin(6),1000) < 100) then
        if (idoku > 0) then
          if (myproc == 1) then
            write(lout,1004) gendmorg - gendmat
          endif
        endif
        if (idoku < 0) then
          write(lout,1004) gendmorg - gendmat
        endif
      else
        call LL4ISM(1,1,1,matsum,gmatsum,ibuf(1),
     &              myproc,nproc,tids,nmsg)
        if (idoku > 0) then
          if (myproc == 1) then
            write(lout,1004) gmatsum
          endif
        endif
        if (idoku < 0) then
          write(lout,1004) gmatsum
        endif
      endif

      if (MOD(ilin(6),1000) < 100) then
        call LL4ISM(1,1,1,endind,ilin(31),ibuf(1),
     &              myproc,nproc,tids,nmsg)
      endif

!**   end of elimination                                            ***
      ilin(3)  = nmsg
      if (MOD(ilin(6),1000) < 100) then
        ilin(5)  = nvt
        ilin(30) = gendmat
        ilin(33) = endmat
        ilin(34) = endind
      endif

      if ((isave == 1) .and. (MOD(ilin(6),1000) < 100))
     &  ilin(6) = ilin(6) + 100

!**                                                                 ***
!**                                                                 ***
 1000 format(/'   ------------------------------------------------ '/
     &       '  ||  Optimizations in routine PREFEL             ||'/
     &       '  ||  (forward pre-elimination is performed):     ||')
 1101 format('  ||    forward elimination step - - - :',i9,' ||')
 1102 format('  ||    no. of rows with one entry ... :',i9,' ||')
 1103 format('  ||    permuted rows with one entry . :',i9,' ||')
 1002 format('  ||    entries shifted to rhs . . . . :',i9,' ||')
 1003 format('  ||                                              ||'/
     &       '  ||    Summary  - - - - - - - - - - - :          ||'/
     &       '  ||    forward elimination steps .... :',i9,' ||')
 1004 format('  ||    entries shifted to rhs . . . . :',i9,' ||'/
     &       '   ------------------------------------------------ '/)
 1010 format(' **************************************************'/
     &       ' ***  ERROR occured in routine PREFEL:          ***')
 1011 format(' ***   matrix is singular!                      ***')
 1012 format(' ***   sorting of rows failed!                  ***')
 1013 format(' ***   allocation of integer-arrays for the     ***'/
     &       ' ***   matrix reduction failed!                 ***')
 1014 format(' ***   allocation of double precision-arrays    ***'/
     &       ' ***   for the matrix reduction failed!         ***')
 1015 format(' ***   the allocated information buffer is too  ***'/
     &       ' ***   small! Enlarge LINSOL_BUFFER_SIZE and    ***'/
     &       ' ***   <ilin(7)> respectively!                  ***')
 1016 format(' ***   the allocated element buffer is too      ***'/
     &       ' ***   small! Enlarge LINSOL_BUFFER_SIZE and    ***'/
     &       ' ***   <ilin(7)> respectively!                  ***')
 1017 format(' ***   internal error in garbage collection?!   ***')
 1020 format(' **************************************************'/)
      r e t u r n
!-----END OF PREFEL----------------------------------------------------
      e    n    d
