!:::::      ,,,,,BANDO.....  -*- f90 -*-
!
!
!
!**********************************************************************
!        1         2         3         4         5         6         7*
!**********************************************************************
!**                                                                 ***
!**                                                                 ***
module randperm
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**      program description of BANDO                               ***
!**                                                                 ***
!**      Returns a random permutation vector                        ***
!**      This is a drop-in replacement for the `bando' routine      ***
!**      to generate test cases. (This is a serial routine!)        ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**    Copyright (C) 1998, 1999 University Karlsruhe                ***
!**    AUTHOR : Detlev Zundel                                       ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                 ***
!**********************************************************************

  use constants

  implicit none

  contains

    ! Subroutine : randpermut
    ! Purpose    : Main subroutine
    !
    !
    ! List of Formal Parameters :
    !
    !---------I------I-----I------------------------------------------
    ! NAME    I TYPE I I/O I MEANING
    !---------I------I-----I------------------------------------------
    ! graph   I int  I in  I array with indices
    !---------I------I-----I------------------------------------------
    ! vertix  I int  I in  I array with matrix-structure-information
    !---------I------I-----I------------------------------------------
    ! lmatbk  I int  I in  I Array with nr. of matrix lines per proc.
    !---------I------I-----I------------------------------------------
    ! permut  I int  I out I Permutation vector
    !---------I------I-----I------------------------------------------
    ! myproc  I int  I in  I Logical number of this proc. (1-nproc)
    !---------I------I-----I------------------------------------------
    ! nproc   I int  I in  I Number of total processors
    !---------I------I-----I------------------------------------------
    ! tids    I int  I in  I Array of physical processor IDs
    !---------I------I-----I------------------------------------------
    ! nmsg    I int  I i/o I Monotonously increasing Message nr.
    !---------I------I-----I------------------------------------------

    subroutine RANDPERMUT(graph, vertix, lmatbk, permut, &
         myproc, nproc, tids, nmsg)

      ! Formal parameters
      integer, dimension (:) :: graph, vertix, permut
      integer, dimension (:) :: lmatbk, tids
      integer :: myproc, nproc, nmsg
      intent (in) :: graph, vertix, lmatbk, myproc, nproc, tids
      intent (inout) :: nmsg
      intent (out) :: permut

      ! Local variables for all steps
      integer :: i, tmp, idx
      double precision :: rnd

      call INIT_RAND()

      if (nproc>1) then
        write (*,*) 'RANDPERM: This routine does NOT run on more than 1 processor!'
        stop
      endif

      ! Initialize perm vector
      do i=1, lmatbk(1)
        permut(i)=i
      end do

      ! Permute the vector
      do i=1, lmatbk(1)-1
        CALL RANDOM_NUMBER(rnd)
        idx=floor(rnd*(lmatbk(1)-i))+i
        tmp=permut(idx)
        permut(idx)=permut(i)
        permut(i)=tmp
      enddo

!      do i=1, lmatbk(1)
!        write (*,*) i,' => ',permut(i)
!      enddo

    end subroutine RANDPERMUT

    subroutine INIT_RAND()
      integer size
      integer, dimension(:), allocatable ::  seeds

      call RANDOM_SEED(size=size)
      allocate(seeds(size))
      call RANDOM_SEED(get=seeds)
      call SYSTEM_CLOCK(COUNT=seeds(1))
      call RANDOM_SEED(put=seeds)

    end subroutine INIT_RAND

end module randperm
