!:::::      ,,,,,RCMCK.....  -*- f90 -*-
!
!
!
!**********************************************************************
!        1         2         3         4         5         6         7*
!**********************************************************************
!**                                                                 ***
!**                                                                 ***
module rcmcks
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**      program description of RCMCK                               ***
!**                                                                 ***
!**      Band optimizes a matrix using the                          ***
!**      Reverse Cuthill McKee                                      ***
!**      algorithm.                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**    Copyright (C) 1998, 1999 University Karlsruhe                ***
!**    AUTHOR : Detlev Zundel                                       ***
!**                                                                 ***
!**    $Id: rcmcks.f90,v 1.5 2002/08/19 15:19:29 ne12 Exp $         ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                 ***
!**********************************************************************

  use constants
  use comm_ifc

  implicit none


  ! Declarations of the used subroutines


  contains

    ! Subroutine : rcmckoptim
    ! Purpose    : Main subroutine for the band optimizer
    !
    ! Assertions : This routine works only with packed rows
    !              i.e. vector terms of the LINSOL type 9
    !
    ! Notes      : This implementation is derived from the
    !              implementation of the single pass optimizer
    !              contained in rcmck.f90
    subroutine RCMCKOPTIMS(mat,lmat,b,perm,index,lindex,info,ia1,lsym,l,&
         ilin,lmatbk,myproc,nproc,tids,nmsg)

      ! Formal parameters
      integer :: lindex, ia1, l, lmat
      integer :: info(:,:), myproc, nproc, nmsg
      integer :: ilin(:), lmatbk(nproc), tids(nproc)
      logical :: lsym
      integer, dimension(:) :: index, perm
      double precision, dimension(:) :: mat
      double precision, dimension(:) :: b
      intent (in) :: ia1, l, lmatbk, myproc
      intent (in) :: nproc, lsym, lindex, lmat
      intent (inout) :: ilin, nmsg
      intent (inout) :: mat, b, perm

      integer :: msgsize, headersize, datasize
      parameter(msgsize=100, headersize=3, datasize=msgsize-headersize)

      integer :: symbfactor
      parameter(symbfactor=3)

      ! Local variables
      integer :: lfrtid, frtid, ltotid, totid
      integer :: mids, midr, ierr
      integer :: mids2, midr2, ierr2
      integer :: ptrmbk(nproc+1), ifirstrow(nproc), ilastrow(nproc)
      integer :: endmat, endind
      integer :: mmsin(2), mmsout(2), mmsbuf(2)
      integer :: i, j, k, p
      integer :: itact, ibasketprocess, ibasketrec
      integer :: maxstoredeg

      ! Communication variables
      integer ::  msgbuf1(msgsize), msgbuf2(msgsize)

      ! Local variables (before STEP II)
      integer :: mindeg, maxdeg, mediandeg, maxcanddeg
      integer :: startcandnum(nproc)
      integer :: iterstart, itercount, maxbandw, maxlocalbandw

      ! Local variables for symmetrization step (Step II)
      integer, allocatable, dimension(:,:) :: deglist
      integer, allocatable, dimension(:,:) :: mindlist
      integer, allocatable, dimension(:) :: mncon, mnumb
      integer, allocatable, dimension(:,:) :: sendbuf, recbuf
      logical :: exist, diagonal
      integer :: colindex, irow, intermed

      ! Local variables for step III
      integer :: imin, numbmin, noofproc
      integer :: allocsize

      ! Local variables for step IV
      logical :: active
      integer :: iglobnewlast, numbexcess
      integer :: iloc, nodcount, isum, isumcount
      integer :: nlano, m
      integer,allocatable,dimension(:) :: iglobnew, iold
      integer :: nunod(nproc)
      logical, allocatable, dimension(:) :: used
      integer, allocatable, dimension(:) ::ilano, numb, ncon, par, numbnew
      integer, allocatable, dimension(:,:) :: indlist
      integer :: isend, isec, irest, iseccount, ipar, icount
      integer :: parsize, numloc
      integer :: sendproc, key, number

      ! Local variables used in the ranking process
      integer :: iil, is, i2, i3, ic
      integer :: istack1
      integer, allocatable, dimension(:) :: nstack1
      integer, allocatable, dimension(:) :: ncond
      integer :: icomp
      integer, allocatable, dimension(:) :: allnew

      ! Local variables for the resorting
      integer, allocatable, dimension(:) :: permbuf, ioldbuf, inewsort
      integer, allocatable, dimension(:,:) :: idatasend, idatarec
      double precision, allocatable, dimension(:,:) :: rdatasend, rdatarec
      integer, allocatable, dimension(:) :: mnumbnew, mnconnew
      integer, allocatable, dimension(:,:) :: mindlistnew
      integer, allocatable, dimension(:,:) :: mindlistcorrected
      double precision, allocatable, dimension(:,:) :: realelnew
      integer :: idx
      integer, allocatable, dimension(:) :: inewsortbuf
      integer :: elemcount

      ! Local variables for final resorting
      integer, allocatable, dimension(:) :: lix
      double precision, allocatable, dimension(:) :: matbuf

      ! Initialize parameters for the ringshift
      lfrtid=LL9MAP(myproc-1,nproc)
      frtid=tids(lfrtid)
      ltotid=LL9MAP(myproc+1, nproc)
      totid=tids(ltotid)

      ! Initialize ptrmbk, ifirstrow, ilastrow
      ptrmbk(1)=0
      do p=1, nproc
        ptrmbk(p+1)=ptrmbk(p)+lmatbk(p)
        ifirstrow(p)=ptrmbk(p)+1
        ilastrow(p)=ptrmbk(p+1)
      enddo

      ! First we build the mindlist and mncon fields
      ! So we have to determine allocation limits for those

      ! Determine an upper bound for maxdegree
      maxdeg=0
      mindeg=ilastrow(nproc)
      do i=1, lmatbk(myproc)
        if (info(i,3)>maxdeg) maxdeg=info(i,3)
      enddo
      mmsin(1)=maxdeg
      call LL4INM(1, 1, 1, mmsin, mmsout, mmsbuf, &
           myproc, nproc, tids, nmsg)
      maxstoredeg=mmsout(1)
      if (lsym) then
        maxdeg=maxstoredeg
      else
        ! Approximate upper bound (probably needs to be changed)
        maxdeg=symbfactor*maxstoredeg
      endif

      if (l<3) then
        write(*,*) 'rcmck: Sorry, (# of lines on proc.)<3 you loose.'
        stop
      endif

      ! allocate the fields
      allocate(mindlist(l, maxdeg))
      allocate(mncon(l), mnumb(l))

      ! Generate the index list
      do i=1, lmatbk(myproc)
        mncon(i)=info(i,3)
        do j=1, mncon(i)
          mindlist(i,j)=index(info(i,6)+j)
        enddo
      enddo

      ! Step I:
      ! Delete the main diagonal
      do i=1, lmatbk(myproc)
        mnumb(i)=i+ifirstrow(myproc)-1
        diagonal=.false.
        do j=1, mncon(i)
          if (mindlist(i,j)==mnumb(i)) then
            diagonal=.true.
            mncon(i)=mncon(i)-1
          endif
          if (diagonal .and. (j<=mncon(i))) then
            mindlist(i,j)=mindlist(i,j+1)
          endif
        enddo
      enddo

      ! Step II:
      ! Symmetrize the connection lists
      ! for LINSOL we have to do this for lsym=.true. AND lsym=.false.
      ! because lsym=.true. implies the storage of only half the matrix!
      if (.true.) then
        ! allocate the comm buffers
        allocate(sendbuf(l,(maxdeg+1)),recbuf(l,(maxdeg+1)))

        ! Prepare the send buffer
        do i=1, lmatbk(myproc)
          sendbuf(i,1)=mncon(i)
          do j=1, mncon(i)
            sendbuf(i,j+1)=mindlist(i,j)
          enddo
        enddo

        ! Symmetrize the index list by ringshift
        do itact=1, nproc
          ibasketprocess=modulo(myproc-itact+nproc,nproc)+1
          ibasketrec=modulo(myproc-itact+nproc-1,p)+1
          if (nproc>1) then
            call MPSNDA(totid, nmsg+itact, (maxdeg+1)*l*iint, &
                 sendbuf, mids, ierr)
            call MPRCVA(frtid, nmsg+itact, (maxdeg+1)*l*iint, &
                 recbuf, midr, ierr)
          endif

          ! Process passing basket
          do i=1, lmatbk(ibasketprocess)
            do j=1, sendbuf(i,1)
              exist=.false.
              colindex=sendbuf(i,j+1)
              if ((colindex>=ifirstrow(myproc)) .and. &
                   (colindex<=ilastrow(myproc))) then
                irow=colindex-ifirstrow(myproc)+1
                do k=1, mncon(irow)
                  if (mindlist(irow,k)==i+ifirstrow(ibasketprocess)-1) then
                    exist=.true.
                    exit
                  endif
                enddo           ! k
                if (.not. exist) then
                  mncon(irow)=mncon(irow)+1
                  if (mncon(irow)>maxdeg) then
                    write(*,*) 'sspbo: Sorry, ', &
                         'matrix storage exceeded during symmetrization!'
                    write(*,*) '       Change symbfactor and recompile.'
                    stop
                  endif

                  mindlist(irow,mncon(irow))=i+ifirstrow(ibasketprocess)-1
                endif
              endif
            enddo               ! j
          enddo                 ! i

          if (nproc>1) then
            call MPSNDW(totid, nmsg+itact, (maxdeg+1)*l*iint, &
                 sendbuf, mids, ierr)
            call MPRCVW(frtid, nmsg+itact, (maxdeg+1)*l*iint, &
                 recbuf, midr, ierr)
            sendbuf=recbuf
          endif

        enddo                   ! itact
        if (nproc>1) nmsg=nmsg+nproc

        deallocate(sendbuf,recbuf)

        ! Sort the index lists
        do i=1, lmatbk(myproc)
          do j=1, mncon(i)-1
            do k=1, mncon(i)-j
              if (mindlist(i,k)>mindlist(i,k+1)) then
                intermed=mindlist(i,k)
                mindlist(i,k)=mindlist(i,k+1)
                mindlist(i,k+1)=intermed
              endif
            enddo
          enddo
        enddo

      endif                     ! Symmetrization of connection lists

      ! Assertion:
      ! We now have the correct information in mncon(:) and
      ! mindlist(:) for the symmetric and unsymmetric case


      ! Determine the median of the degrees
      ! (we use the field mindlist for efficiency)
      maxdeg=maxval(mncon(1:lmatbk(myproc)))
      mindeg=minval(mncon(1:lmatbk(myproc)))
      mmsin(1)=maxdeg
      call LL4INM(1, 1, 1, mmsin, mmsout, mmsbuf, &
           myproc, nproc, tids, nmsg)
      maxdeg=mmsout(1)
      mmsin(1)=mindeg
      call LL4MIN(1, 1, 1, mmsin, mmsout, mmsbuf, &
           myproc, nproc, tids, nmsg)
      mindeg=mmsout(1)

!       allocate(deglist(maxdeg,3))
!       deglist(:,1)=0
!       do i=1, lmatbk(myproc)
!         deglist(mncon(i),1)=deglist(mncon(i),1)+1
!       enddo
!       call LL4ISM(1, maxdeg, 1, deglist(:,1), deglist(:,2), &
!            deglist(:,3), myproc, nproc, tids, nmsg)
!       j=0
!       i=1
!       do while (j<(ilastrow(nproc)/2))
!         j=j+deglist(i,2)
!         i=i+1
!       enddo
!       mediandeg=i-1
!       deallocate(deglist)
!       maxcanddeg=max(min((maxdeg+mindeg)/2,mediandeg-1),mindeg)
      maxcanddeg=mindeg

      ! Determine number of possible start nodes per proc
      startcandnum=0
      do i=1, lmatbk(myproc)
        if (mncon(i)<=maxcanddeg) then
          startcandnum(myproc)=startcandnum(myproc)+1
        endif
      enddo
      allocate(sendbuf(nproc,2))
      call LL4ISM(1, nproc, 1, startcandnum, sendbuf(:,1), &
           sendbuf(:,2), myproc, nproc, tids, nmsg)
      startcandnum=sendbuf(:,1)
      deallocate(sendbuf)
      ! Transform list into a "sum" list
      do i=2, nproc
        startcandnum(i)=startcandnum(i-1)+startcandnum(i)
      enddo

      ! Allocate fields
      ! FIXME: This has to be better!
      allocsize=int(l*1.5)
      allocate(ilano(allocsize), numb(allocsize), numbnew(allocsize))
      allocate(ncon(allocsize),used(allocsize))
      allocate(indlist(allocsize,maxdeg))
      allocate(iold(l+allocsize))
      allocate(iglobnew(l+allocsize))


      ! Step III:
      ! Iterate over all possible start nodes
      itercount=0
      iterstart=0
      noofproc=1
      maxbandw=ilastrow(nproc)
      maxlocalbandw=0

360   continue

      itercount=itercount+1
      do while ((noofproc<=nproc).and.(itercount>startcandnum(noofproc)))
        noofproc=noofproc+1
      end do
!      if (noofproc>nproc) goto 3090
      if ((noofproc>nproc).or.(itercount==2)) goto 3090

      if (myproc==noofproc) then
        ! We have the next startnode on the local proc.
        iterstart=iterstart+1
        do while (mncon(iterstart)>maxcanddeg)
          iterstart=iterstart+1
        enddo

        if (myproc/=1) then
          ! Send the node information to processor 1
          mmsin(1)=mncon(iterstart)
          mmsin(2)=iterstart+ifirstrow(myproc)-1
          call MPSNDA(tids(1), 1, 2*iint, mmsin, mids, ierr)
          call MPSNDW(tids(1), 1, 2*iint, mmsin, mids, ierr)
        else
          imin=mncon(iterstart)
          numbmin=iterstart
        end if
      else                      ! myproc==noofproc
        if (myproc==1) then
          ! Receive start node information
          call MPRCVA(tids(noofproc), 1, 2*iint, mmsin, midr, ierr)
          call MPRCVW(tids(noofproc), 1, 2*iint, mmsin, midr, ierr)
          imin=mmsin(1)
          numbmin=mmsin(2)
        endif
      endif

      ! Assumption:
      ! Now we have the next iteration node. It has the number
      ! `numbmin' on processor `noofproc' and the degree `imin'.

      ! Step IV:
      ! Number the nodes
      iglobnewlast=1
      if (myproc==1) then
        active=.true.
      else
        active=.false.
      endif
      d1: do                        ! d1 - loops forever
        if ((myproc==1) .and. (active)) then
          ! This is a one-time setup to get things rolling
          iloc=1
          iglobnew(1)=iglobnewlast
          iold(1)=numbmin
          nlano=1
          ilano(1)=iold(1)
          numbexcess=0
          m=imin
          if (noofproc==1) then
            ! Start node is on processor 1
            do i=1,m
              numb(i)=mindlist(numbmin-ifirstrow(myproc)+1, i)
            enddo
          else                  ! noofproc==1
            ! Get neighbour list from `noofproc'
            msgbuf1(1)=myproc   ! procid
            msgbuf1(2)=1        ! msgtype
            msgbuf1(3)=1        ! len
            msgbuf1(4)=numbmin  ! data(1)
            call MPSNDA(tids(noofproc), 0, msgsize*iint, msgbuf1, &
                 mids, ierr)
            allocate(par(m))
            call MPSNDW(tids(noofproc), 0, msgsize*iint, msgbuf1, &
                 mids, ierr)
            call MPRCVA(tids(noofproc), 1, m*iint, par, &
                 midr, ierr)
            call MPRCVW(tids(noofproc), 1, m*iint, par, &
                 midr, ierr)
            do i=1,m
              numb(i)=par(i)
            enddo
            deallocate(par)
          endif                 ! noofproc==1

          ! Now everything should be setup for the standard cycle

        endif                   ! (myproc==1).and.(active)


        if (active) then
          cycles: do            ! endless loop over all levels for one proc
            ! Here comes the code for the active processor
            ! He does the numbering and requests msgs from
            ! other procs
            do i=1, m
              used(i)=.false.
            enddo
            ! Group new nodes by proc
            do p=1, nproc
              nunod(p)=0
            enddo
            i=1
            d2: do p=1, nproc
              d3: do
                if (numb(i)<=ilastrow(p)) then
                  nunod(p)=nunod(p)+1
                  i=i+1
                else
                  exit d3
                endif
                if (i==(m+1)) then
                  exit d2
                endif
              enddo d3
            enddo d2
            ! Processor `p' now holds `nunod(p)' new nodes
            ! Now request the relevant info from each in turn
            i=0
            do p=1, nproc
              if (nunod(p)==0) cycle
              if (myproc==p) then
                ! The active processor can fill in the info right away
                do nodcount=1, nunod(p)
                  i=i+1
                  ncon(i)=mncon(numb(i)-ifirstrow(myproc)+1)
                  do j=1, ncon(i)
                    indlist(i,j)=mindlist(numb(i)-ifirstrow(myproc)+1,j)
                  enddo
                enddo
              else                ! myproc==p

                ! Request info from processor `p'
                !
                ! At first we request the `ncon' data
                !
                isend=i
                isec=int(nunod(p)/datasize)
                irest=mod(nunod(p), datasize)
                ! Process the "full" messages
                do iseccount=1, isec
                  msgbuf1(1)=myproc     ! procid
                  msgbuf1(2)=2          ! msgtype
                  msgbuf1(3)=datasize   ! len

                  ! Fill in the `numb' data
                  do k=1, datasize
                    msgbuf1(headersize+k)=numb(isend+k)
                  enddo

                  call MPSNDA(tids(p), 0, msgsize*iint, msgbuf1, &
                       mids, ierr)
                  call MPSNDW(tids(p), 0, msgsize*iint, msgbuf1, &
                       mids, ierr)
                  call MPRCVA(tids(p), 1, datasize*iint, msgbuf1, &
                       midr, ierr)
                  call MPRCVW(tids(p), 1, datasize*iint, msgbuf1, &
                       midr, ierr)

                  ! Store away the received data
                  do k=1, datasize
                    ncon(isend+k)=msgbuf1(k)
                  enddo
                  isend=isend+datasize
                enddo
                ! Process the "rest"
                if (irest/=0) then
                  msgbuf1(1)=myproc     ! procid
                  msgbuf1(2)=2          ! msgtype
                  msgbuf1(3)=irest      ! len

                  ! Fill in the `numb' data
                  do k=1, irest
                    msgbuf1(headersize+k)=numb(isend+k)
                  enddo

                  call MPSNDA(tids(p), 0, msgsize*iint, msgbuf1, &
                       mids, ierr)
                  call MPSNDW(tids(p), 0, msgsize*iint, msgbuf1, &
                       mids, ierr)
                  call MPRCVA(tids(p), 1, m*iint, msgbuf1, &
                       midr, ierr)
                  call MPRCVW(tids(p), 1, m*iint, msgbuf1, &
                       midr, ierr)

                  ! Store away the received data
                  do k=1, irest
                    ncon(isend+k)=msgbuf1(k)
                  enddo
                  isend=isend+irest
                endif             ! irest/=0

                !
                ! Now we can request the `indlist' data
                !
                isend=i
                ! Process the "full" messages
                do iseccount=1, isec
                  msgbuf1(1)=myproc     ! procid
                  msgbuf1(2)=1          ! msgtype
                  msgbuf1(3)=datasize   ! len

                  ! Fill in the `numb' data
                  do k=1, datasize
                    msgbuf1(headersize+k)=numb(isend+k)
                  enddo

                  call MPSNDA(tids(p), 0, msgsize*iint, msgbuf1, &
                       mids, ierr)

                  ! Calculate the size of the receive message
                  isum=0
                  do isumcount=isend+1, isend+datasize
                    isum=isum+ncon(isumcount)
                  enddo
                  isend=isend+datasize

                  allocate(par(isum))

                  call MPSNDW(tids(p), 0, msgsize*iint, msgbuf1, &
                       mids, ierr)
                  call MPRCVA(tids(p), 1, isum*iint, par, &
                       midr, ierr)
                  call MPRCVW(tids(p), 1, isum*iint, par, &
                       midr, ierr)

                  ! Store away the received data
                  ipar=0
                  do icount=1, datasize
                    i=i+1
                    do j=1, ncon(i)
                      ipar=ipar+1
                      indlist(i,j)=par(ipar)
                    enddo
                  enddo
                  deallocate(par)
                enddo             ! all "full" sections

                ! Process the "rest"
                if (irest/=0) then
                  msgbuf1(1)=myproc     ! procid
                  msgbuf1(2)=1          ! msgtype
                  msgbuf1(3)=irest      ! len

                  ! Fill in the `numb' data
                  do k=1, irest
                    msgbuf1(headersize+k)=numb(isend+k)
                  enddo

                  call MPSNDA(tids(p), 0, msgsize*iint, msgbuf1, &
                       mids, ierr)

                  ! Calculate the size of the receive message
                  isum=0
                  do isumcount=isend+1, isend+irest
                    isum=isum+ncon(isumcount)
                  enddo
                  allocate(par(isum))
                  call MPSNDW(tids(p), 0, msgsize*iint, msgbuf1, &
                       mids, ierr)
                  call MPRCVA(tids(p), 1, isum*iint, par, &
                       midr, ierr)
                  call MPRCVW(tids(p), 1, isum*iint, par, &
                       midr, ierr)

                  ! Store away the received data
                  ipar=0
                  do icount=1, irest
                    i=i+1
                    do j=1, ncon(i)
                      ipar=ipar+1
                      indlist(i,j)=par(ipar)
                    enddo
                  enddo
                  deallocate(par)
                  isend=isend+irest
                endif             ! irest/=0

              endif               ! myproc==p
            enddo                 ! p

            !
            ! Now follows the numbering of the nodes
            !
            allocate(ncond(m))
            allocate(nstack1(m))

            do iil=1, nlano       ! d5
              istack1=0
              do i=1,m
                if (.not. used(i)) then
                  do j=1, ncon(i)
                    if (ilano(iil)==indlist(i,j)) then
                      istack1=istack1+1
                      used(i)=.true.
                      nstack1(istack1)=i
                      exit
                    endif
                  enddo           ! j
                endif             ! not used(i)
              enddo               ! i
              if (istack1==1) then
                ! Only one candidate in stack1
                iloc=iloc+1
                iglobnew(iloc)=iglobnew(iloc-1)+1
                iold(iloc)=numb(nstack1(1))
                numbnew(nstack1(1))=iglobnew(iloc)
              endif
              if (istack1>1) then
                ! Sort the candidates in stack1 according to their
                ! degree
                ! Build ncond table
                do i=1, istack1
                  ncond(i)=ncon(nstack1(i))
                enddo
                do i=1, istack1-1
                  do j=1,istack1-i
                    if (ncon(j)>ncon(j+1)) then
                      intermed=nstack1(j+1)
                      nstack1(j+1)=nstack1(j)
                      nstack1(j)=intermed
                      intermed=ncond(j+1)
                      ncond(j+1)=ncond(j)
                      ncond(j)=intermed
                    endif
                  enddo           ! j
                enddo             ! i
                do i=1,istack1
                  iloc=iloc+1
                  iglobnew(iloc)=iglobnew(iloc-1)+1
                  iold(iloc)=numb(nstack1(i))
                  numbnew(nstack1(i))=iglobnew(iloc)
                enddo
              endif               ! (istack1>1)
            enddo                 ! d5

            ! This check verifies the assumption that every node
            ! has been "used" in the numbering
            do i=1, m
              if (.not. used(i)) then
                write(*,*) 'rcmck: Error in processing of nodes. Node ',&
                     i,' still unused.'
                stop
              endif
            enddo

            deallocate(nstack1)
            deallocate(ncond)

            ! Check whether we already exceed the best bandwidth
            ! by calculating the `current' bandwidth
            do i=1, m
              searchj: do j=1, ncon(i)
                do k=1, nlano
                  if (indlist(i,j)==ilano(k)) then
                    if ((numbnew(i)-(iglobnew(iloc)-m-nlano+k)) > &
                      maxlocalbandw) then
                      maxlocalbandw=numbnew(i)-(iglobnew(iloc)-m-nlano+k)
                    endif
                    cycle searchj
                  endif
                enddo
              enddo searchj
            enddo

!            if (maxlocalbandw>maxbandw) then
!              write(*,*) 'Ok - we can skip this round right now.'
!            endif


            ! The active proc has numbered all of his nodes (or more)
            ! or we succeeded a previous best permutation
            ! We can stop here if we are the last proc
            if ((maxlocalbandw>maxbandw).or. &
                 ((iglobnew(iloc)>=ilastrow(myproc)).and.(myproc==nproc))) then
              ! Send exit signal to all waiting procs
              do p=1, nproc
                if (p/=myproc) then
                  msgbuf1(1)=myproc ! procid
                  msgbuf1(2)=4      ! msgtype
                  call MPSNDA(tids(p), 0, msgsize*iint, msgbuf1, &
                       mids, ierr)
                  call MPSNDW(tids(p), 0, msgsize*iint, msgbuf1, &
                       mids, ierr)
                endif
              enddo
              exit d1
            endif

            ! Now prepare the list for the next step
            ! Calculate size for the allnew field
            icount=0
            do i=1,m
              icount=icount+ncon(i)
            enddo
            allocate(allnew(icount))
            icount=0
            do i=1,m
              do ic=1, ncon(i)
                icount=icount+1
                allnew(icount)=indlist(i,ic)
              enddo
            enddo                 ! i

            ! Eliminate backward connections
            do ic=1, nlano
              do i=1, icount
                if (allnew(i)==ilano(ic)) then
                  allnew(i)=0
                endif
              enddo
            enddo                 ! ic

            ! Eliminate cross connections
            do ic=1, m
              do i=1, icount
                if (allnew(i)==numb(ic)) then
                  allnew(i)=0
                endif
              enddo
            enddo                 ! ic

            ! Eliminate duplicate neighbours
            do i=1, icount-1
              if (allnew(i)/=0) then
                do j=i+1, icount
                  if (allnew(j)==allnew(i)) then
                    allnew(j)=0
                  endif
                enddo
              endif
            enddo                 ! i

            ! Compress allnew inplace (dzu)
            icomp=0
            do i=1, icount
              if (allnew(i)==0) then
                icomp=icomp+1
              elseif (icomp/=0) then
                allnew(i-icomp)=allnew(i)
              endif
            enddo
            icount=icount-icomp

            ! Sort the allnew field
            do i=1, icount-1
              do j=1, icount-i
                if (allnew(j)>allnew(j+1)) then
                  intermed=allnew(j+1)
                  allnew(j+1)=allnew(j)
                  allnew(j)=intermed
                endif
              enddo
            enddo                 ! i

            ! Setup the new tables for the next cycle
            nlano=m
            do i=1, m
              ilano(i)=iold(iloc-m+i)
            enddo
            m=icount
            ! Check if we overflow our fields
            if (m>allocsize) then
              write(*,*) 'rcmck: Overflow of the allocated fields. ',&
                   'Change source-code!'
              stop
            endif
            do i=1, m
              numb(i)=allnew(i)
            enddo
            deallocate(allnew)

            ! If we have numbered all (or more) of our nodes then
            ! we pass on control
            if (iglobnew(iloc)>=ilastrow(myproc)) then
              iglobnewlast=iglobnew(iloc)
              numbexcess=iglobnewlast-ilastrow(myproc)
              msgbuf1(1)=myproc   ! procid
              msgbuf1(2)=3        ! msgtype
              msgbuf1(3)=4        ! msglen
              msgbuf1(4)=nlano
              msgbuf1(5)=iglobnewlast
              msgbuf1(6)=numbexcess
              msgbuf1(7)=m
              call MPSNDA(tids(myproc+1), 0, msgsize*iint, msgbuf1, &
                   mids, ierr)
              call MPSNDW(tids(myproc+1), 0, msgsize*iint, msgbuf1, &
                   mids, ierr)

              parsize=nlano+numbexcess+m
              allocate(par(parsize))
              ipar=0
              do i=1, nlano
                ipar=ipar+1
                par(ipar)=ilano(i)
              enddo
              do i=1, numbexcess
                ipar=ipar+1
                par(ipar)=iold(lmatbk(myproc)+i)
              enddo
              do i=1,m
                ipar=ipar+1
                par(ipar)=numb(i)
              enddo
              call MPSNDA(tids(myproc+1), 2, parsize*iint, par, &
                   mids, ierr)
              call MPSNDW(tids(myproc+1), 2, parsize*iint, par, &
                   mids, ierr)
              deallocate(par)

              active=.false.
              exit cycles
            endif               ! myproc<nproc
          enddo cycles
        else                    ! active

          ! This is the code for the passive procs.
          ! We wait in an endless loop until we either get control
          ! or the calculation is finished and answer to requests
          ! in the meantime
          do                    ! d4 (endless loop)
            ! Do an open receive
            call MPRCVA(-1, 0, msgsize*iint, msgbuf1, &
                 midr, ierr)
            call MPRCVW(-1, 0, msgsize*iint, msgbuf1, &
                 midr, ierr)

            sendproc=msgbuf1(1)
            key=msgbuf1(2)
            number=msgbuf1(3)

!            write (*,*) 'rcmck: ', myproc,' received msg with key ',key,&
!                 ' from ', sendproc
            ! According to`key' we have to fulfill different requests
            msgtype: select case (key)

            case(1)
              ! Send list of conections for the requested nodes
              isum=0
              do i=1, number
                isum=isum+mncon(msgbuf1(headersize+i)-ifirstrow(myproc)+1)
              enddo
              allocate(par(isum))
              ipar=0
              do i=1, number
                numloc=msgbuf1(headersize+i)-ifirstrow(myproc)+1
                do j=1, mncon(numloc)
                  ipar=ipar+1
                  par(ipar)=mindlist(numloc,j)
                enddo
              enddo
              call MPSNDA(tids(sendproc), 1, isum*iint, par, &
                   mids, ierr)
              call MPSNDW(tids(sendproc), 1, isum*iint, par, &
                   mids, ierr)
              deallocate(par)

            case (2)
              ! Send number of connections for the requested nodes
              allocate(par(number))
              ipar=0
              do i=1, number
                ipar=ipar+1
                par(ipar)=mncon(msgbuf1(headersize+i)-ifirstrow(myproc)+1)
              enddo
              call MPSNDA(tids(sendproc), 1, number*iint, par, &
                   mids, ierr)
              call MPSNDW(tids(sendproc), 1, number*iint, par, &
                   mids, ierr)
              deallocate(par)

            case (3)
              ! We receive control
              nlano=msgbuf1(headersize+1)
              iglobnewlast=msgbuf1(headersize+2)
              numbexcess=msgbuf1(headersize+3)
              m=msgbuf1(headersize+4)
              parsize=nlano+numbexcess+m
              allocate(par(parsize))
              call MPRCVA(tids(sendproc), 2, parsize*iint, par, &
                   midr, ierr)
              call MPRCVW(tids(sendproc), 2, parsize*iint, par, &
                   midr, ierr)
              ipar=0
              do i=1, nlano
                ipar=ipar+1
                ilano(i)=par(ipar)
              enddo
              iloc=0
              do i=1, numbexcess
                ipar=ipar+1
                iloc=iloc+1
                iglobnew(iloc)=iglobnewlast-numbexcess+i
                iold(iloc)=par(ipar)
              enddo
              do i=1,m
                ipar=ipar+1
                numb(i)=par(ipar)
              enddo
              deallocate(par)

              if (numbexcess>=lmatbk(myproc)) then
                numbexcess=numbexcess-lmatbk(myproc)
                msgbuf1(1)=myproc ! procid
                msgbuf1(2)=3      ! msgtype
                msgbuf1(3)=4      ! msglen
                msgbuf1(4)=nlano
                msgbuf1(5)=iglobnewlast
                msgbuf1(6)=numbexcess
                msgbuf1(7)=m
                call MPSNDA(tids(myproc+1), 0, msgsize*iint, msgbuf1, &
                 mids, ierr)
                call MPSNDW(tids(myproc+1), 0, msgsize*iint, msgbuf1, &
                     mids, ierr)

                parsize=nlano+numbexcess+m
                allocate(par(parsize))
                ipar=0
                do i=1, nlano
                  ipar=ipar+1
                  par(ipar)=ilano(i)
                enddo
                do i=1, numbexcess
                  ipar=ipar+1
                  par(ipar)=numb(i)
                enddo
                do i=1,m
                  ipar=ipar+1
                  par(ipar)=numb(i)
                enddo
                call MPSNDA(tids(myproc+1), 2, parsize*iint, par, &
                     mids, ierr)
                call MPSNDW(tids(myproc+1), 2, parsize*iint, par, &
                     mids, ierr)
                deallocate(par)

              else              ! numbexcess>=lmatbk(myproc)
                ! We gain control so end receive loop
                active=.true.
                cycle d1
              endif             ! numbexcess>=lmatbk(myproc)

            case (4)
              ! We are finished
              exit d1

            endselect msgtype
          enddo                 ! d4
        endif                   ! active
      enddo d1

      ! Was this the best permutation so far?
      ! (but globalize it first)
      mmsin(1)=maxlocalbandw
      call LL4INM(1, 1, 1, mmsin, mmsout, mmsbuf, &
           myproc, nproc, tids, nmsg)
      maxlocalbandw=mmsout(1)

      ! Yes indeed - so remember it
      if (maxlocalbandw<maxbandw) then
        ! Copy the permutation vector
        do i=1,lmatbk(myproc)
          perm(i)=iold(i)
        enddo
        maxbandw=maxlocalbandw
      endif

      goto 360
3090  continue                  ! We have finished the iterations

      deallocate(iglobnew)
      deallocate(indlist)
      deallocate(ncon, used)
      deallocate(ilano, numb, numbnew)

      ! We use the reversed best permutation
      ! After all this is the /reverse/ Cuthill McKee algorithm
      ! A simple flip is not possible because the "opposing" processor
      ! might have less or more nodes than we have so we resort to
      ! another ring shift
      allocate(permbuf(l))
      do itact=1, nproc
        ibasketprocess=modulo(myproc-itact+nproc,nproc)+1
        ibasketrec=modulo(myproc-itact+nproc-1,p)+1
        if (nproc>1) then
          call MPSNDA(totid, nmsg+itact, l*iint, &
               perm, mids, ierr)
          call MPRCVA(frtid, nmsg+itact, l*iint, &
               permbuf, midr, ierr)
        endif

        ! Process passing basket
        do i=1, lmatbk(ibasketprocess)
          j=ilastrow(nproc)+1-(ifirstrow(ibasketprocess)+i-1)
          if ((j>=ifirstrow(myproc)).and.&
               (j<=ilastrow(myproc))) then
            iold(j-ifirstrow(myproc)+1)=perm(i)
          endif
        enddo                 ! i

        if (nproc>1) then
          call MPSNDW(totid, nmsg+itact, l*iint, &
               perm, mids, ierr)
          call MPRCVW(frtid, nmsg+itact, l*iint, &
               permbuf, midr, ierr)

          ! Copy the received data
          perm(1:l)=permbuf(1:l)
        endif
      enddo                   ! itact
      if (nproc>1) nmsg=nmsg+nproc
      deallocate(permbuf)
      perm(1:l)=iold(1:l)

      ! Sorting of nodes for old numbers by sending
      ! iold around the processors once
      allocate(inewsort(l))
      allocate(ioldbuf(l))

      do itact=1, nproc
        ibasketprocess=modulo(myproc-itact+nproc,nproc)+1
        ibasketrec=modulo(myproc-itact+nproc-1,p)+1
        if (nproc>1) then
          call MPSNDA(totid, nmsg+itact, l*iint, &
               iold, mids, ierr)
          call MPRCVA(frtid, nmsg+itact, l*iint, &
               ioldbuf, midr, ierr)
        endif

        ! Process passing basket
        do i=1, lmatbk(ibasketprocess)
          if ((iold(i)>=ifirstrow(myproc)).and.&
               (iold(i)<=ilastrow(myproc))) then
            inewsort(iold(i)-ifirstrow(myproc)+1)= &
                 i+ifirstrow(ibasketprocess)-1
          endif
        enddo                 ! i

        if (nproc>1) then
          call MPSNDW(totid, nmsg+itact, l*iint, &
               iold, mids, ierr)
          call MPRCVW(frtid, nmsg+itact, l*iint, &
               ioldbuf, midr, ierr)

          ! Copy the received data
          iold(1:l)=ioldbuf(1:l)
        endif
      enddo                   ! itact
      if (nproc>1) nmsg=nmsg+nproc

      deallocate(ioldbuf)

      ! Permute the matrix and the right hand side
      allocate(mnumbnew(l),mnconnew(l),mindlistnew(l, maxstoredeg))
      allocate(realelnew(l,maxstoredeg))
      allocate(idatasend(l,(maxstoredeg+2)), rdatasend(l,(maxstoredeg+1)))
      allocate(idatarec(l,(maxstoredeg+2)), rdatarec(l,(maxstoredeg+1)))

      ! Distribute the matrix according to the new numbers
      do i=1, lmatbk(myproc)
        idatasend(i,1)=ifirstrow(myproc)+i-1
        idatasend(i,2)=info(i,3)
        do j=1, info(i,3)
          idatasend(i,j+2)=index(info(i,6)+j)
          rdatasend(i,j)=mat(info(i,2)+j)
        enddo
        rdatasend(i,maxstoredeg+1)=b(i)
      enddo

      do itact=1, nproc
        ibasketprocess=modulo(myproc-itact+nproc,nproc)+1
        ibasketrec=modulo(myproc-itact+nproc-1,p)+1
        if (nproc>1) then
          call MPSNDA(totid, nmsg+itact, l*(maxstoredeg+2)*iint, &
               idatasend, mids, ierr)
          call MPSNDA(totid, nmsg+itact, l*(maxstoredeg+1)*ireal, &
               rdatasend, mids2, ierr2)
          call MPRCVA(frtid, nmsg+itact, l*(maxstoredeg+2)*iint, &
               idatarec, midr, ierr)
          call MPRCVA(frtid, nmsg+itact, l*(maxstoredeg+1)*ireal, &
               rdatarec, midr2, ierr2)
        endif

        ! Process passing basket
        do i=1, lmatbk(myproc)
          if ((iold(i)>=ifirstrow(ibasketprocess)).and.&
               (iold(i)<=ilastrow(ibasketprocess))) then
            idx=iold(i)-ifirstrow(ibasketprocess)+1
            mnumbnew(i)=i+ifirstrow(myproc)-1
            mnconnew(i)=idatasend(idx,2)
            b(i)=rdatasend(idx, maxstoredeg+1)
            do k=1, mnconnew(i)
              mindlistnew(i,k)=idatasend(idx,k+2)
              realelnew(i,k)=rdatasend(idx,k)
            enddo
          endif
        enddo                 ! i

        if (nproc>1) then
          call MPSNDW(totid, nmsg+itact, l*(maxstoredeg+2)*iint, &
               idatasend, mids, ierr)
          call MPSNDW(totid, nmsg+itact, l*(maxstoredeg+1)*ireal, &
               rdatasend, mids2, ierr2)
          call MPRCVW(frtid, nmsg+itact, l*(maxstoredeg+2)*iint, &
               idatarec, midr, ierr)
          call MPRCVW(frtid, nmsg+itact, l*(maxstoredeg+1)*ireal, &
               rdatarec, midr2, ierr2)

          ! Copy the received data
          idatasend=idatarec
          rdatasend=rdatarec
        endif
      enddo                   ! itact
      if (nproc>1) nmsg=nmsg+nproc

      ! Copy the permutation vector
      do i=1, lmatbk(myproc)
        perm(i)=iold(i)
      enddo

      deallocate(idatarec, rdatarec)
      deallocate(idatasend, rdatasend)
      deallocate(iold)

      ! Now we have to renumber the index lists through another ring shift
      ! of the new numbers
      allocate(inewsortbuf(l))
      allocate(mindlistcorrected(l, maxstoredeg))

      do itact=1, nproc
        ibasketprocess=modulo(myproc-itact+nproc,nproc)+1
        ibasketrec=modulo(myproc-itact+nproc-1,p)+1
        if (nproc>1) then
          call MPSNDA(totid, nmsg+itact, l*iint, &
               inewsort, mids, ierr)
          call MPRCVA(frtid, nmsg+itact, l*iint, &
               inewsortbuf, midr, ierr)
        endif

        ! Process passing basket
        do i=1, lmatbk(myproc)
          do j=1, mnconnew(i)
            if ((mindlistnew(i,j)>=ifirstrow(ibasketprocess)).and.&
                 (mindlistnew(i,j)<=ilastrow(ibasketprocess))) then
              k=mindlistnew(i,j)-ifirstrow(ibasketprocess)+1
              mindlistcorrected(i,j)=inewsort(k)
            endif
          enddo                 ! j
        enddo                   ! i

        if (nproc>1) then
          call MPSNDW(totid, nmsg+itact, l*iint, &
               inewsort, mids, ierr)
          call MPRCVW(frtid, nmsg+itact, l*iint, &
               inewsortbuf, midr, ierr)

          ! Copy the received data
          inewsort=inewsortbuf
        endif
      enddo                   ! itact
      if (nproc>1) nmsg=nmsg+nproc

      deallocate(inewsortbuf)

      ! Now we have to transform the 2d arrays into LINSOL
      ! storage scheme
      elemcount=0
      do i=1, lmatbk(myproc)
        elemcount=elemcount+mnconnew(i)
      enddo

      ! This should be done but it is impossible right now
      !lindex=elemcount
      !lmat=elemcount
      !deallocate(mat,index)
      !allocate(mat(lmat), index(lindex))

      ! Because we cannot reallocate storage we simply check and die
      ! in case...
      if ((elemcount>lmat).or.(elemcount>lindex)) then
        write(*,*) 'on processor ',myproc,' lmat=',lmat, &
             ' lindex=',lindex,' elemcount=',elemcount
        mmsin=1
      else
        mmsin=0
      endif
      call LL4INM(1, 1, 1, mmsin, mmsout, mmsbuf, &
           myproc, nproc, tids, nmsg)
      if (mmsout(1)==1) then
        write(*,*) 'rcmck: Sorry, you loose. ', &
             'Matrix storage exceeded during permutation!'
        stop
      endif

      idx=0
      do i=1, lmatbk(myproc)
        info(i,2)=idx
        info(i,3)=mnconnew(i)
        info(i,5)=mnumbnew(i)-1 ! These numbers are 0-based
        info(i,6)=idx
        do j=1, mnconnew(i)
          mat(idx+j)=realelnew(i,j)
          index(idx+j)=mindlistcorrected(i,j)
        enddo
        idx=idx+mnconnew(i)
      enddo

      deallocate(mindlistcorrected)
      deallocate(realelnew)
      deallocate(mnumbnew,mnconnew,mindlistnew)

      deallocate(inewsort)

      allocate(lix(ilastrow(nproc)), matbuf(ilastrow(nproc)))
      ! Sort the indices because LINSOL needs this for
      ! the subsequent processing
      do i=1,lmatbk(myproc)
        idx  = info(i,2)
        k    = info(i,3)
        call HEAPSORTIX(index(info(i,6)+1),lix,k,.true.)
        do j=1,k
          matbuf(j) = mat(idx+lix(j))
        enddo
        do j=1,k
          mat(idx+j) = matbuf(j)
        enddo
      enddo
      deallocate(lix, matbuf)

      ! Copy back the changed global variables
      ilin(33) = elemcount      ! endmat
      ilin(34) = elemcount      ! endind

    end subroutine RCMCKOPTIMS

end module rcmcks
