!**********************************************************************
!        1         2         3         4         5         6         7*
!**********************************************************************
!**                                                                 ***
!**                                                                 ***
      subroutine REVPERM(perm,rperm,touched,l,lmatbk,ptrmbk,
     &                   myproc,nproc,tids,nmsg)
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**   R E V P E R M   computes the reverse permutation of the       ***
!**                   permutation vector <perm>.                    ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**    Copyright (C) 2001 University Karlsruhe                      ***
!**    PROGRAMMER: H. Haefner                                       ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**     THE IMPLICIT NONE-INSTRUCTION HAS THE AIM,                  ***
!**     THAT ALL VARIABLES MUST BE DECLARED EXPLICITLY.             ***
!**                                                                 ***
      USE constants

      implicit none
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**      FORMAL PARAMETERS :                                        ***
!**      -----------------                                          ***
!**                                                                 ***
      integer,intent(in) ::  l,myproc,nproc
      integer,intent(inout) ::  nmsg
      integer,intent(in) ::  lmatbk(nproc),ptrmbk(nproc+1),tids(nproc)
      integer,intent(in) ::  perm(l)
      integer,intent(inout) ::  touched(l)
      integer,intent(out) ::  rperm(l)
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**   LIST OF FORMAL PARAMETERS :                                   ***
!**   -------------------------                                     ***
!**                                                                 ***
!--------I------I-----I------------------------------------------------
! NAME   I TYPE I I/O I     MEANING
!--------I------I-----I------------------------------------------------
!--------I------I-----I------------------------------------------------
! perm   I  I   I in  I  permutation vector
!        I      I     I  perm(i) = j: i is local and j is global param.
!--------I------I-----I------------------------------------------------
! rperm  I  I   I out I  reverse permutation vector
!        I      I     I  rperm(i) = j: i is local and j is global param.
!--------I------I-----I------------------------------------------------
!**                                                                 ***
!**                                                                 ***
!**      LOCAL PARAMETERS :                                         ***
!**      -----------------                                          ***
!**                                                                 ***
      integer  rpermbuf(l)
      integer  i,ip,k,ierr,mids,midr,from,to,lmyblk,nrblk
!
!**** START OF CALCULATION :
!     ---------------------
!
      ierr   = 0
      lmyblk = lmatbk(myproc)

      to = myproc+1
      if (to > nproc) to = 1
      to = tids(to)
      from = myproc-1
      if (from == 0) from = nproc
      from = tids(from)

      do i=1,lmyblk
	rperm(i) = i + ptrmbk(myproc)
      enddo

      do ip=1,nproc
	if (nproc > 1) then
	  call MPRCVA(from,nmsg+ip,l*iint,rpermbuf,midr,ierr)
	endif

	nrblk = myproc + nproc - ip + 1
	if (nrblk > nproc) nrblk = nrblk - nproc

	do i=1,lmyblk
	  if (touched(i) == 0) cycle
	  k = perm(i) - ptrmbk(nrblk)
	  if (k>0 .and. k <= lmatbk(nrblk)) then
	    rperm(k) = i + ptrmbk(myproc)
	    touched(i) = 0
	  endif
        enddo

	if (nproc > 1) then
	  call MPSNDA(to,nmsg+ip,l*iint,rperm,mids,ierr)
	  call MPSNDW(to,nmsg+ip,l*iint,rperm,mids,ierr)
	  call MPRCVW(from,nmsg+ip,l*iint,rpermbuf,midr,ierr)
	  rperm(1:l) = rpermbuf(1:l)
	endif
      enddo

      nmsg = nmsg + nproc

      R E T U R N
!------END OF REVPERM-----------------------------------------------
      E N D                                   
