!**********************************************************************
!        1         2         3         4         5         6         7*
!**********************************************************************
!**                                                                 ***
!**                                                                 ***
      subroutine RHSIXSRTEST(rhs,perm,l,lmatbk,ptrmbk,myproc,nproc,
     &                       tids,nmsg,lout,ierr)
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**   R H S I X S R T E S T  resorts the right hand side <rhs> due  ***
!**                        to the vector perm (rhs(perm) = rhs) and ***
!**                        tests, if perm is a permutation vector.  ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**    Copyright (C) 2002 University Karlsruhe                      ***
!**    PROGRAMMER: H. Haefner                                       ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**     THE IMPLICIT NONE-INSTRUCTION HAS THE AIM,                  ***
!**     THAT ALL VARIABLES MUST BE DECLARED EXPLICITLY.             ***
!**                                                                 ***
      USE constants

      implicit none
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**      FORMAL PARAMETERS :                                        ***
!**      -----------------                                          ***
!**                                                                 ***
      integer,intent(in) ::  l,lout,myproc,nproc
      integer,intent(inout) ::  nmsg
      integer,intent(out) ::  ierr
      integer,intent(in) ::  lmatbk(nproc),ptrmbk(nproc+1),tids(nproc)
      integer,intent(in) ::  perm(l)
      double precision,intent(inout) ::  rhs(l)
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**   LIST OF FORMAL PARAMETERS :                                   ***
!**   -------------------------                                     ***
!**                                                                 ***
!--------I------I-----I------------------------------------------------
! NAME   I TYPE I I/O I     MEANING
!--------I------I-----I------------------------------------------------
!--------I------I-----I------------------------------------------------
! perm   I  I   I in  I  permutation vector for the rows of the matrix
!        I      I     I  perm(i) = j: i is local and j is global param.
!--------I------I-----I------------------------------------------------
!**                                                                 ***
!**                                                                 ***
!**      LOCAL PARAMETERS :                                         ***
!**      -----------------                                          ***
!**                                                                 ***
      double precision rhsupd(l),rhsbuf(l)
      integer          itest(l), itestbuf(l), ih(6)
      integer          i,j,k,mids,midr,from,to,msgcnt,lmyblk
!
!**** START OF CALCULATION :
!     ---------------------
!
      ierr   = 0
      msgcnt = 0
      lmyblk = lmatbk(myproc)

      itest = 0

      to = myproc-1
      if (to == 0) to = nproc
      to = tids(to)
      from = myproc+1
      if (from > nproc) from = 1
      from = tids(from)

      do i=myproc,nproc
	do j=1,lmyblk
	  if (perm(j) > ptrmbk(i) .and.
     &        perm(j) <= ptrmbk(i+1)) then
            rhsupd(perm(j)-ptrmbk(i)) = rhs(j)
	    itest(perm(j)-ptrmbk(i))  = 1
	  endif
	enddo
	if (nproc > 1) then
	  msgcnt = msgcnt + 1
	  call MPRCVA(from,nmsg+msgcnt,l*ireal,rhsbuf,midr,ierr)
	  call MPRCVA(from,nmsg+msgcnt,l*iint,itestbuf,midr,ierr)
	  call MPSNDA(to,nmsg+msgcnt,l*ireal,rhsupd,mids,ierr)
	  call MPSNDW(to,nmsg+msgcnt,l*ireal,rhsupd,mids,ierr)
	  call MPSNDA(to,nmsg+msgcnt,l*iint,itest,mids,ierr)
	  call MPSNDW(to,nmsg+msgcnt,l*iint,itest,mids,ierr)
	  call MPRCVW(from,nmsg+msgcnt,l*ireal,rhsbuf,midr,ierr)
	  call MPRCVW(from,nmsg+msgcnt,l*iint,itestbuf,midr,ierr)
	  do j=1,l
	    rhsupd(j)  = rhsbuf(j)
	    itest(j)   = itestbuf(j)
	  enddo
	endif
      enddo

      do i=1,myproc-1
	do j=1,lmyblk
	  if (perm(j) > ptrmbk(i) .and.
     &        perm(j) <= ptrmbk(i+1)) then
            rhsupd(perm(j)-ptrmbk(i)) = rhs(j)
	    itest(perm(j)-ptrmbk(i))  = 1
	  endif
	enddo
	msgcnt = msgcnt + 1
	call MPRCVA(from,nmsg+msgcnt,l*ireal,rhsbuf,midr,ierr)
	call MPRCVA(from,nmsg+msgcnt,l*iint,itestbuf,midr,ierr)
	call MPSNDA(to,nmsg+msgcnt,l*ireal,rhsupd,mids,ierr)
	call MPSNDW(to,nmsg+msgcnt,l*ireal,rhsupd,mids,ierr)
	call MPSNDA(to,nmsg+msgcnt,l*iint,itest,mids,ierr)
	call MPSNDW(to,nmsg+msgcnt,l*iint,itest,mids,ierr)
	call MPRCVW(from,nmsg+msgcnt,l*ireal,rhsbuf,midr,ierr)
	call MPRCVW(from,nmsg+msgcnt,l*iint,itestbuf,midr,ierr)
	do j=1,l
	  rhsupd(j)  = rhsbuf(j)
	  itest(j)   = itestbuf(j)
	enddo
      enddo

      nmsg = nmsg + msgcnt

      do j=1,lmyblk
        rhs(j) = rhsupd(j)
      enddo

      ierr   = 0
      do j=1,lmyblk
        if (itest(j) /= 1) then
          ierr = ierr + 1
        endif
      enddo

      ih = 0

      ih(1) = ierr
      if (ierr > 0) ih(2) = 1
      call LL4ISM(1,2,1,ih(1),ih(3),ih(5),myproc,nproc,tids,nmsg)
      ierr = ih(3)
      if (ierr > 0) then
        if (myproc == 1) then
          write(lout,1000)
          write(lout,1001) ierr
          write(lout,1002) ih(4)
          write(lout,1010)
        endif
      endif

 1000 format(/' **********************************************'/
     &       ' ***  Error occured in RHSIXSRTEST:         ***')
 1001 format(' ***   <perm> is no permutation vector      ***'/
     &       ' ***   because of ',i6,' missing entries!   ***')
 1002 format(' ***   <perm> is corrupt on ',i6,' procs!   ***')
 1010 format(' **********************************************'/)

      R E T U R N
!------END OF RHSIXSRTEST----------------------------------------------
      E N D                                   
