!**********************************************************************
!        1         2         3         4         5         6         7*
!**********************************************************************
!**                                                                 ***
!**                                                                 ***
      subroutine RHSSRTIX(rhs,perm,l,lmatbk,ptrmbk,myproc,nproc,tids,
     *                    nmsg)
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**   R H S S R T I X   resorts the right hand side <rhs> due to    ***
!**                     the vector perm ( rhs = rhs(perm) ).        ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**    Copyright (C) 1999 University Karlsruhe                      ***
!**    PROGRAMMER: H. Haefner                                       ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**     THE IMPLICIT NONE-INSTRUCTION HAS THE AIM,                  ***
!**     THAT ALL VARIABLES MUST BE DECLARED EXPLICITLY.             ***
!**                                                                 ***
      USE constants

      implicit none
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**      FORMAL PARAMETERS :                                        ***
!**      -----------------                                          ***
!**                                                                 ***
      integer, intent(in) ::  l,myproc,nproc
      integer, intent(inout) ::  nmsg
      integer, intent(in) ::  lmatbk(nproc),ptrmbk(nproc+1),tids(nproc)
      integer, intent(in) ::  perm(l)
      double precision, intent(inout) ::  rhs(l)
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**   LIST OF FORMAL PARAMETERS :                                   ***
!**   -------------------------                                     ***
!**                                                                 ***
!--------I------I-----I------------------------------------------------
! NAME   I TYPE I I/O I     MEANING
!--------I------I-----I------------------------------------------------
!--------I------I-----I------------------------------------------------
! perm   I  I   I in  I  permutation vector for the rows of the matrix
!        I      I     I  perm(i) = j: i is local and j is global param.
!--------I------I-----I------------------------------------------------
!**                                                                 ***
!**                                                                 ***
!**      LOCAL PARAMETERS :                                         ***
!**      -----------------                                          ***
!**                                                                 ***
      double precision rhsupd(l),rhsbuf(l)
      integer          i,j,ierr,mids,midr,from,to,msgcnt,lmyblk
!
!**** START OF CALCULATION :
!     ---------------------
!
      ierr   = 0
      msgcnt = 0
      lmyblk = lmatbk(myproc)

      to = myproc-1
      if (to == 0) to = nproc
      to = tids(to)
      from = myproc+1
      if (from > nproc) from = 1
      from = tids(from)

      do i=myproc,nproc
	if (nproc > 1) then
	  msgcnt = msgcnt + 1
	  call MPRCVA(from,nmsg+msgcnt,l*ireal,rhsbuf,midr,ierr)
	  call MPSNDA(to,nmsg+msgcnt,l*ireal,rhs,mids,ierr)
	  call MPSNDW(to,nmsg+msgcnt,l*ireal,rhs,mids,ierr)
	endif
	do j=1,lmyblk
	  if (perm(j) > ptrmbk(i) .and. perm(j) <= ptrmbk(i+1)) then
            rhsupd(j) = rhs(perm(j)-ptrmbk(i))
	  endif
	enddo
	if (nproc > 1) then
	  call MPRCVW(from,nmsg+msgcnt,l*ireal,rhsbuf,midr,ierr)
	  do j=1,l
	    rhs(j) = rhsbuf(j)
	  enddo
	endif
      enddo

      do i=1,myproc-1
	msgcnt = msgcnt + 1
	call MPRCVA(from,nmsg+msgcnt,l*ireal,rhsbuf,midr,ierr)
	call MPSNDA(to,nmsg+msgcnt,l*ireal,rhs,mids,ierr)
	call MPSNDW(to,nmsg+msgcnt,l*ireal,rhs,mids,ierr)
	do j=1,lmyblk
	  if (perm(j) > ptrmbk(i) .and. perm(j) <= ptrmbk(i+1)) then
            rhsupd(j) = rhs(perm(j)-ptrmbk(i))
	  endif
	enddo
	call MPRCVW(from,nmsg+msgcnt,l*ireal,rhsbuf,midr,ierr)
	do j=1,l
	  rhs(j) = rhsbuf(j)
        enddo
      enddo

      nmsg = nmsg + msgcnt

      do j=1,lmyblk
        rhs(j) = rhsupd(j)
      enddo

      R E T U R N
C------END OF RHSSRTIX----------------------------------------------
      E N D                                   
