!***********************************************************************
!        1         2         3         4         5         6         7 *
!***********************************************************************
!**                                                                  ***
!**                                                                  ***
      subroutine SET_RHS_IGUESS(MAT,lmat,index,lindex,lsym,info,ia1,
     &                          nvt,rhs,l,lmatbk,ptrmbk,
     &                          ilin,tids,nproc,myproc,nmsg)
!**                                                                  ***
!**                                                                  ***
!***********************************************************************
!**                                                                  ***
!**    SET_RHS_IGUESS   sets right hand sides and one initial        ***
!**                     guess for a distributed matrix               ***
!**                                                                  ***
!***********************************************************************
!**                                                                  ***
!**    Copyright (C) 1999 University Karlsruhe                       ***
!**    PROGRAMMER : H. Haefner                                       ***
!**                                                                  ***
!**    This file is part of LINSOL.                                  ***
!**    LINSOL is free software; you can redistribute it and/or       ***
!**    modify it under the terms of the GNU General Public License   ***
!**    as published by the Free Software Foundation; either          ***
!**    version 2 of the License, or (at your option) any later       ***
!**    version.                                                      ***
!**                                                                  ***
!**    LINSOL is distributed in the hope that it will be useful,     ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty   ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.       ***
!**    See the GNU General Public License for more details.          ***
!**                                                                  ***
!**    You should have received a copy of the GNU General Public     ***
!**    License along with LINSOL; if not, write to the Free          ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,       ***
!**    Boston, MA  02111-1307, USA.                                  ***
!**                                                                  ***
!***********************************************************************
!**                                                                  ***
      USE solex
      USE constants

      implicit none
!**                                                                  *** 
!***********************************************************************
!**                                                                  ***
!**    Formal Parameters :                                           ***
!**                                                                  ***
!***********************************************************************

      integer, intent(in) ::  l,lmat,lindex,ia1,myproc,nproc,nvt
      integer,intent(inout) ::  nmsg
      integer,intent(in) ::   ilin(nilin),lmatbk(nproc),ptrmbk(nproc+1)
      integer, intent(in) ::  info(ia1,ia2),index(lindex),tids(nproc)
      double precision, intent(in) ::  MAT(lmat)
      double precision, intent(out) ::  rhs(l)
      logical, intent(in) ::  lsym
!***********************************************************************
!**                                                                  ***
!**    List of Formal Parameters :                                   ***
!**                                                                  ***
!**                                                                  ***
!---------I------I-----I------------------------------------------------
! NAME    I TYPE I I/O I MEANING
!---------I------I-----I------------------------------------------------
! l       I int  I in  I dimension of the matrix
!---------I------I-----I------------------------------------------------
! llocal  I int  I in  I local number of matrix rows
!---------I------I-----I------------------------------------------------
! lmat    I int  I in  I number of matrix elements
!---------I------I-----I------------------------------------------------
! lindex  I int  I in  I number of indices
!---------I------I-----I------------------------------------------------
! info    I int  I in  I array with matrix-structure-information
!---------I------I-----I------------------------------------------------
! index   I int  I in  I array with indices
!---------I------I-----I------------------------------------------------
! MAT     I  dp  I in  I array with matrix-elements
!---------I------I-----I------------------------------------------------
! lsym    I l    I in  I indicates if matrix is symmetric
!---------I------I-----I------------------------------------------------
! ierr    I int  I out I error-indication
!---------I------I-----I------------------------------------------------
!**                                                                  ***
!**                                                                  ***
!**    Local Variables :                                             ***
!**                                                                  ***
      integer          i,idoku,istride,lout,to,mids,lmyblk
      integer          iw(nproc),iermax,ihelp
      double precision dw(5*l),x(l),xmax,gscp,gmax,rhlp
      external         LL6AXO
      double precision, parameter ::  zero = 0.0

!***********************************************************************
!**                                                                  ***
!**    List of Local Variables :                                     ***
!**                                                                  ***
!**                                                                  ***
!---------I------I-----I------------------------------------------------
! NAME    I TYPE I I/O I MEANING
!---------I------I-----I------------------------------------------------
!   dw    I  dp  I  -  I work array, array: dw(5*l)
!---------I------I-----I------------------------------------------------
!***********************************************************************


!***********************************************************************
!**    Start of Calculation                                          ***
!***********************************************************************

!     ierr = 0000
      lout  = ilin(12)
      idoku = ilin(13)
      lmyblk = lmatbk(myproc)

!*********************************************************
!   Calculation of the right hand side rhs so that
!   x=(1,2**(1/3),3**(1/3),...,l**(1/3))
!   is the solution of the linear system.
!*********************************************************
      do i=1,lmyblk
        x(i) = dble(ptrmbk(myproc)+i)**(1./3.)
        if (mod(ilin(19),10000) >= 1000) xexact(i) = x(i)
      enddo
      call LL4SCP(lmyblk,1,l,1,x,1,x,1,gscp,rhlp,.true.,
     &            myproc,nproc,tids,nmsg)
      gscp = sqrt(gscp)

      call LL4RNM(lmyblk,1,l,x,gmax,rhlp,myproc,nproc,tids,nmsg)

      if ((idoku > 0) .and. (myproc == 1)) then
        write(lout,1015)
        write(lout,1525)
        write(lout,1021)
      endif
      if (idoku < 0) then
        write(lout,1015)
        write(lout,1525)
        write(lout,1021)
      endif
      if (l > 8) then
        istride = (l+7)/8
      else
        istride = 1
      endif
      if ((idoku > 0) .and. (myproc == 1)) then
        write(lout,1528) istride
        write(lout,1530) (x(i),i=1,l,istride)
        write(lout,1526) gscp
        write(lout,1527) gmax
      endif
      if (idoku < 0) then
        write(lout,1528) istride
        write(lout,1530) (x(i),i=1,l,istride)
        write(lout,1526) gscp
        write(lout,1527) gmax
      endif

!*********************************************************
!   now send the solution from processor 1 to all processors
!*********************************************************
!
!     do i=2,nproc
!       to = tids(i)
!       call MPSNDA(to,nmsg+i-1,lmatbk(i)*ireal,
!    &              x(ptrmbk(i)+1),mids,ierr)
!       call MPSNDW(to,nmsg+i-1,lmatbk(i)*ireal,
!    &              x(ptrmbk(i)+1),mids,ierr)
!     enddo
!     nmsg = nmsg+nproc
!
!     call LL4INM(1,1,1,ierr,iermax,ihelp,myproc,nproc,tids,nmsg)
!     if (iermax /= 0) then
!       ierr = iermax
!       goto 999
!     endif

      do i=1,nproc
	iw(i) = 1
      enddo

      if (lsym) then
        call LL3ASXO(mat,x,rhs,dw,ptrmbk,lmatbk,
     #               l,nproc,tids,myproc,iw,.false.,nvt,
     #               lmat,lindex,ia1,info,index,nmsg)
      else
        call LL3AXO(LL6AXO,mat,x,rhs,dw,ptrmbk,lmatbk,
     #              l,nproc,tids,myproc,iw,.false.,nvt,
     #              lmat,lindex,ia1,ia2,info,index,nmsg)
      endif

!***********************************************************************
!**    End of Calculation                                            ***
!***********************************************************************

 1015 format('   -------------------------------------------------')
 1525 format('  ||  The linear system Ax=b will be solved with   ||'/
     &       '  ||  a right_hand_side b so that holds:           ||'/
     &       '  ||    x = (1,2**(1/3),...,l**(1/3))              ||')
 1021 format('   ------------------------------------------------- '//)
 1526 format(' ',/
     &       '  The Euclidian norm of the solution is:',/
     &       '   ||x||2   = ',d10.4)
 1527 format('  The Maximum norm of the solution is:',/
     &       '   ||x||max = ',d10.4//)
 1528 format('  Each',i5,'th element of the solution is: ')
 1530 format(1x,4(d17.10,2x),/,1x,4(d17.10,2x))

       r e t u r n
!-----END OF SET_RHS_IGUESS -------------------------------------------
       e    n    d
