!**********************************************************************
!        1         2         3         4         5         6         7*
!**********************************************************************
!**                                                                 ***
!**                                                                 ***
      subroutine SHIFTL(pmat,lpmat,pix,lpix,infoL,infoU,ia1LU,nvtLU,
     *                  nvtL,infnew,l,ilin,myproc,nproc,ierr)
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**   S H I F T L  resorts the packed rows of L into packed columns ***
!**                columns; the values of L must be sorted in       ***
!**                ascending order; CAUTION: the packed columns of  ***
!**                U are mixed together with the packed rows of L.  ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**    Copyright (C) 1999 University Karlsruhe                      ***
!**    PROGRAMMER: H. Haefner                                       ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**     THE IMPLICIT NONE-INSTRUCTION HAS THE AIM,                  ***
!**     THAT ALL VARIABLES MUST BE DECLARED EXPLICITLY.             ***
!**                                                                 ***
      USE constants

      implicit none
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**      FORMAL PARAMETERS :                                        ***
!**      -----------------                                          ***
!**                                                                 ***
      integer,intent(in) ::  lpmat,lpix,ia1LU,l,myproc
      integer,intent(in) ::  nvtLU,nproc
      integer,intent(in) ::  ilin(nilin)
      integer,intent(inout) ::  infnew(l,2),ierr
      integer,intent(out) ::  nvtL
      integer,intent(inout) ::  pix(lpix),infoL(ia1LU,ia2LU)
      integer,intent(inout) ::  infoU(ia1LU,ia2LU)
      double precision,intent(inout):: pmat(lpmat)
!**                                                                 ***
!**                                                                 ***
!**      LOCAL PARAMETERS :                                         ***
!**      -----------------                                          ***
!**                                                                 ***
      integer    lbuf,libuf
      integer, dimension(:), allocatable ::  ibuf
      double precision, dimension(:), allocatable ::  buf
      integer           ioff(nvtLU)
      integer           i,j,k,nocols,bsize,lout
      integer           addaL,iarL,indcL,lvtL
      integer           ptrluold,ptrlunew,ptrlu11,matdiff
      integer           firstix,lvtbuf,nvtbuf,ptrbuf
      integer           ktmp,ptrlu,runacc
      integer, parameter ::  ixmark = -999
      double precision, parameter ::  matmark = 0.0
      logical           lcopy
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**   LIST OF LOCAL PARAMETERS :                                    ***
!**   -------------------------                                     ***
!**                                                                 ***
!--------I------I-----I------------------------------------------------
! NAME   I TYPE I I/O I     MEANING
!--------I------I-----I------------------------------------------------
!--------I------I-----I------------------------------------------------
! ptrlu  I  I   I i/o I  local size of matrix array
!--------I------I-----I------------------------------------------------
! matmarkI  I   I in  I  marker for matrix elements to be removed
!--------I------I-----I------------------------------------------------
! ixmark I  I   I in  I  marker for index elements to be removed
!--------I------I-----I------------------------------------------------

!
!**** START OF CALCULATION :
!     ---------------------
!

      bsize    = ilin(7)
      lout     = ilin(12)
      runacc   = ilin(21)
      ptrlu    = ilin(37)
      ptrlunew = ptrlu
      nocols   = 0

!
!***  Create buffer <buf> for L and buffer <ibuf> for indices of L
!

      ierr  = 0

      if ((bsize < 1) .or. (bsize > 1000000)) then
        bsize = 5
      endif

      lbuf = MIN(ptrlu+l,int(bsize*1.e6/ireal))
      allocate (buf(lbuf),stat=ierr)
      libuf = lbuf
      allocate (ibuf(libuf),stat=ierr)

      if (ierr /= 0) then
        write(lout,1000)
        write(lout,1010)
        write(lout,1100)
	goto 999
      endif

!
!**** Computation of minimal and maximal row index per column
!
      do j=1,l
        infnew(j,1) = l + 1
        infnew(j,2) = 0
      enddo
      do i=1,nvtLU       ! for all rows (distributed blockwise onto proc.)
        lvtL  = infoL(i,2)
        indcL = infoL(i,1)
        do k=1,lvtL
          if (infnew(pix(indcL+k),1) > i) infnew(pix(indcL+k),1) = i
          if (infnew(pix(indcL+k),2) < i) infnew(pix(indcL+k),2) = i
        enddo
      enddo

!
!***  Store the packed rows of L columnwise into the buffer <buf> and
!***  the column indices of L in the buffer <ibuf>
!

      firstix = 1

  111 ioff = 0
      ptrbuf = 0
      nvtbuf = 0
      do j=firstix,l
        nvtbuf = nvtbuf + 1
        lvtbuf = 0
        do i=infnew(j,1),infnew(j,2)
          addaL = infoL(i,1)
          lvtL  = infoL(i,2)
          iarL  = infoL(i,3)
          indcL = addaL
          k=ioff(i)+1         ! pix(indcL+k) >= j
          if ((lvtL-ioff(i) > 0) .and. (pix(indcL+k) == j)) then
            lvtbuf = lvtbuf+1
            buf(ptrbuf+lvtbuf) = pmat(addaL+k)
            pmat(addaL+k) = matmark
            ibuf(ptrbuf+lvtbuf) = iarL + 1
            pix(indcL+k) = ixmark
            ioff(i) = ioff(i) + 1
          endif
        enddo 
        infnew(j,1) = ptrbuf
        infnew(j,2) = lvtbuf
        ptrbuf = ptrbuf + lvtbuf
        if ((ptrbuf+l > lbuf) .or. (ptrbuf+l > libuf)) then
          firstix = j + 1
          lcopy = .false.
          exit
        endif
      enddo 

      if (j > l .or. firstix > l) lcopy = .true.

      do i=1,nvtLU
        infoL(i,2) = infoL(i,2) - ioff(i)
      enddo

!
!***  Garbage collection
!
      ptrluold = ptrlunew
      
      if (runacc == 0) then
        call GCLU(pmat,pix,infoL,infoU,lpmat,lpix,ia1LU,l,l,
     &            matmark,ixmark,ptrlunew,myproc,nproc)
      else
        call GCLUOPT(pmat,pix,infoL,infoU,lpmat,lpix,ia1LU,l,l,
     &               matmark,ixmark,ptrlunew,myproc,nproc)
      endif

      if (nocols == 0) then
	ptrlu11 = ptrlunew
      else
	matdiff = ptrluold - ptrlunew
	do j=1,nocols
          infnew(j,1) = infnew(j,1) - matdiff
        enddo
        do j=ptrluold+1,ptrlu11
          pmat(j-matdiff) = pmat(j)
        enddo
        do j=ptrluold+1,ptrlu11
          pix(j-matdiff) = pix(j)
        enddo
        ptrlu11 = ptrlu11 - matdiff
      endif

!
!***  Copy the buffer <buf> to the end of <pmat> and the buffer <ibuf>
!***  to the end of <pindex>
!
      do j=1,nvtbuf
        nocols = nocols + 1
	do k=1,infnew(nocols,2)
	  pmat(ptrlu11+k) = buf(infnew(nocols,1)+k)
	enddo
	do k=1,infnew(nocols,2)
	  pix(ptrlu11+k) = ibuf(infnew(nocols,1)+k)
	enddo
        infnew(nocols,1) = ptrlu11
	ptrlu11 = ptrlu11 + infnew(nocols,2)
      enddo
        
      if (.not. lcopy) goto 111 
      if (nocols /= l .or. nvtLU > ia1LU) then
	print*,' ERROR in SHIFTL: Missing column of L or nvtLU > ia1LU'
	ierr = 9999
      endif

      nvtL = l

      do i=1,nvtL
        infoL(i,1) = infnew(i,1)
        infoL(i,2) = infnew(i,2)
        infoL(i,3) = i - 1
      enddo

      deallocate(buf,ibuf)

  999 continue

 1000 format(' **************************************************'/
     &       ' ***  ERROR  occured  in  routine SHIFTL:       ***')
 1010 format(' ***     Buffer allocation failed!!!            ***')
 1100 format(' **************************************************')

      R E T U R N
!------END OF SHIFTL----------------------------------------------------        
      E N D                                   
