!**********************************************************************
!        1         2         3         4         5         6         7*
!**********************************************************************
!**                                                                 ***
!**                                                                 ***
      subroutine SHIFTLU(pmat,lpmat,pix,lpix,infoLU,ia1LU,nvtLU,
     *                   endLU,infnew,lglobal,ilin,myproc,
     *                   nproc,tids,nmsg,ierr)
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**   S H I F T L U   resorts the packed rows of L into packed      ***
!**                   columns; the values of L or U must be sorted  ***
!**                   in ascending order!                           ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**    Copyright (C) 2000 University Karlsruhe                      ***
!**    PROGRAMMER: H. Haefner                                       ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**     THE IMPLICIT NONE-INSTRUCTION HAS THE AIM,                  ***
!**     THAT ALL VARIABLES MUST BE DECLARED EXPLICITLY.             ***
!**                                                                 ***
      USE constants

      implicit none
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**      FORMAL PARAMETERS :                                        ***
!**      -----------------                                          ***
!**                                                                 ***
!--------I------I-----I------------------------------------------------
! NAME   I TYPE I I/O I     MEANING
!--------I------I-----I------------------------------------------------
!--------I------I-----I------------------------------------------------
! endLU  I  I   I in  I  local size of matrix array, i.e. pointer to
!        I      I     I  the last element of the matrix
!--------I------I-----I------------------------------------------------
!
      integer,intent(in) ::  lpmat,lpix,ia1LU,lglobal,myproc,nproc
      integer,intent(in) ::  nvtLU
      integer,intent(in) ::  ilin(nilin),tids(nproc)
      integer,intent(inout) ::  ierr,endLU,nmsg
      integer,intent(inout) ::  pix(lpix),infoLU(ia1LU,ia2LU)
      integer,intent(inout) ::  infnew(lglobal,2)
      double precision,intent(inout):: pmat(lpmat)
!**                                                                 ***
!**                                                                 ***
!**      LOCAL PARAMETERS :                                         ***
!**      -----------------                                          ***
!**                                                                 ***
      integer    lbuf,libuf
      integer, dimension(:), pointer ::  ibuf
      double precision, dimension(:), pointer ::  buf
      integer           ioff(nvtLU)
      integer           i,j,k,nocols
      integer           addaL,iarL,indcL,lvtL
      integer           endLUold,endLUnew,endLU11,matdiff
      integer           firstix,ktmp,lvtbuf,nvtbuf,ptrbuf,runacc
      integer, parameter ::  ixmark = -999
      double precision, parameter ::  matmark = 0.0
      logical           lcopy
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**   LIST OF LOCAL PARAMETERS :                                    ***
!**   -------------------------                                     ***
!**                                                                 ***
!--------I------I-----I------------------------------------------------
! NAME   I TYPE I I/O I     MEANING
!--------I------I-----I------------------------------------------------
!--------I------I-----I------------------------------------------------
! ptrlu  I  I   I i/o I  local size of matrix array
!--------I------I-----I------------------------------------------------
! matmarkI  I   I in  I  marker for matrix elements to be removed
!--------I------I-----I------------------------------------------------
! ixmark I  I   I in  I  marker for index elements to be removed
!--------I------I-----I------------------------------------------------
!
!***********************************************************************
!**    Interface of external subroutines                             ***
!***********************************************************************
      interface

       subroutine CREATE_BUFR(buf,bufsize,maxbuf,ilin,myproc,nproc,
     *                        tids,nmsg,ierr)
	USE constants
        implicit none
        integer,intent(in) ::   nproc,myproc,maxbuf
        integer,intent(inout) ::  nmsg
        integer,intent(out) ::  ierr
        integer,intent(in) ::   ilin(nilin),tids(nproc)
        double precision,dimension(:),pointer ::  buf
        integer,intent(out)::   bufsize
       end subroutine

       subroutine CREATE_BUFI2(buf,bufsize,maxbuf,ilin,myproc,nproc,
     *                         tids,nmsg,ierr)
	USE constants
        implicit none
        integer,intent(in) ::   nproc,myproc,maxbuf
        integer,intent(inout) ::  nmsg
        integer,intent(out) ::  ierr
        integer,intent(in) ::   ilin(nilin),tids(nproc)
        integer,dimension(:),pointer ::  buf
        integer,intent(out)::   bufsize
       end subroutine

      end interface

!
!**** START OF CALCULATION :
!     ---------------------
!
      runacc   = ilin(21)
      endLUnew = endLU
      nocols   = 0

!
!***  Create buffer <buf> for L (U) and buffer <ibuf> for indices of L (U)
!
      call CREATE_BUFR(buf,lbuf,endLU+lglobal,ilin,myproc,nproc,
     *                 tids,nmsg,ierr)

      call CREATE_BUFI2(ibuf,libuf,endLU+lglobal,ilin,myproc,nproc,
     *                  tids,nmsg,ierr)
      if (ierr /= 0) goto 999

!
!**** Computation of minimal and maximal row index per column
!
      do j=1,lglobal
        infnew(j,1) = lglobal + 1
        infnew(j,2) = 0
      enddo
      do i=1,nvtLU       ! for all rows (distributed blockwise onto proc.)
        lvtL  = infoLU(i,2)
        indcL = infoLU(i,1)
        do k=1,lvtL
          if (infnew(pix(indcL+k),1) > i) infnew(pix(indcL+k),1) = i
          if (infnew(pix(indcL+k),2) < i) infnew(pix(indcL+k),2) = i
        enddo
      enddo

!
!***  Store the packed rows of L (U) columnwise into the buffer <buf> and
!***  the column indices of L (U) in the buffer <ibuf>
!
      firstix = 1

  111 ioff = 0
      ptrbuf = 0
      nvtbuf = 0
      do j=firstix,lglobal       ! for all columns
        nvtbuf = nvtbuf + 1
        lvtbuf = 0
        do i=infnew(j,1),infnew(j,2)    ! for all rows (originally: i=1,nvtLU)
          addaL = infoLU(i,1)
          lvtL  = infoLU(i,2)
          iarL  = infoLU(i,3)
          indcL = addaL
          k=ioff(i)+1                ! pix(indcL+k) >= j
          if ((lvtL-ioff(i) > 0) .and. (pix(indcL+k) == j)) then
            lvtbuf = lvtbuf+1
            buf(ptrbuf+lvtbuf) = pmat(addaL+k)
            pmat(addaL+k) = matmark
            ibuf(ptrbuf+lvtbuf) = iarL + 1
            pix(indcL+k) = ixmark
            ioff(i) = ioff(i) + 1
          endif
        enddo 
        infnew(j,1) = ptrbuf
        infnew(j,2) = lvtbuf
        ptrbuf = ptrbuf + lvtbuf
        if ((ptrbuf+nvtLU > lbuf) .or. (ptrbuf+nvtLU > libuf)) then
          firstix = j + 1
          lcopy = .false.
          exit
        endif
      enddo 

      if (j > lglobal .or. firstix > lglobal) lcopy = .true.

      if (.not. lcopy) then
        do i=1,nvtLU
          infoLU(i,2) = infoLU(i,2) - ioff(i)
        enddo
      endif

!
!***  Garbage collection
!
      endLUold = endLUnew
      
      if (.not. lcopy) then
        if (runacc == 0) then
          call GCOFFR(pmat,pix,infoLU,lpmat,lpix,ia1LU,nvtLU,
     *                matmark,ixmark,endLUnew)
        else
          call GCOFFRO(pmat,pix,infoLU,lpmat,lpix,ia1LU,nvtLU,
     *                 matmark,ixmark,endLUnew)
        endif
      else
	endlunew = 0
      endif

      if (nocols == 0) then
	endLU11 = endLUnew
      else
	matdiff = endLUold - endLUnew
	do j=1,nocols
          infnew(j,1) = infnew(j,1) - matdiff
        enddo
        do j=endLUold+1,endLU11
          pmat(j-matdiff) = pmat(j)
        enddo
        do j=endLUold+1,endLU11
          pix(j-matdiff) = pix(j)
        enddo
        endLU11 = endLU11 - matdiff
      endif

!
!***  Copy the buffer <buf> to the end of <pmat> and the buffer <ibuf>
!***  to the end of <pindex>
!
      do j=1,nvtbuf
        nocols = nocols + 1
	do k=1,infnew(nocols,2)
	  pmat(endLU11+k) = buf(infnew(nocols,1)+k)
	enddo
	do k=1,infnew(nocols,2)
	  pix(endLU11+k) = ibuf(infnew(nocols,1)+k)
	enddo
        infnew(nocols,1) = endLU11
	endLU11 = endLU11 + infnew(nocols,2)
      enddo
        
      if (.not. lcopy) goto 111 
      if (nocols /= lglobal) then
	print*,' ERROR in SHIFTL: Missing column of L'
	ierr = 9999
      endif

      do i=1,lglobal
        infoLU(i,1) = infnew(i,1)
        infoLU(i,2) = infnew(i,2)
        infoLU(i,3) = i - 1
      enddo

      deallocate(buf,ibuf)

  999 continue

      R E T U R N
!------END OF SHIFTLU---------------------------------------------------        
      E N D                                   
