!**********************************************************************
!        1         2         3         4         5         6         7*
!**********************************************************************
!**                                                                 ***
!**                                                                 ***
      subroutine SHIFTU_END(pmat,lpmat,pix,lpix,infoU,infoL,ia1LU,
     *                      nvtU,nvtL,ilin,lenU,lenL,
     *                      lmat,rlin,myproc,nproc,tids,nmsg,ierr)
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**   S H I F T U _ E N D  shifts the packed rows of U and its      ***
!**                        indices to the end of the matrix and the ***
!**                        index array respectively.                ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**    Copyright (C) 2000 University Karlsruhe                      ***
!**    PROGRAMMER: H. Haefner                                       ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**     THE IMPLICIT NONE-INSTRUCTION HAS THE AIM,                  ***
!**     THAT ALL VARIABLES MUST BE DECLARED EXPLICITLY.             ***
!**                                                                 ***
      USE constants

      implicit none
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**      FORMAL PARAMETERS :                                        ***
!**      -----------------                                          ***
!**                                                                 ***
      integer,intent(in) ::  lmat,lpmat,lpix,ia1LU,nvtU,nvtL,nproc
      integer,intent(in) ::  tids(nproc),myproc
      integer,intent(in) ::  infoL(ia1LU,ia2LU)
      integer,intent(inout) ::  infoU(ia1LU,ia2LU)
      integer,intent(out) ::  lenU,lenL,ierr
      integer,intent(inout) ::  ilin(nilin),pix(lpix),nmsg
      double precision,intent(in):: rlin(nrlin)
      double precision,intent(inout):: pmat(lpmat)
!**                                                                 ***
!**                                                                 ***
!**      LOCAL PARAMETERS :                                         ***
!**      -----------------                                          ***
!**                                                                 ***
      integer           i,idoku,iermax,lout,ptrlu,shiftU
      integer           addaU,addaL,lenUL,glenUL,ihelp(6)
      double precision  lgfac,gfac,rhelp
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**   LIST OF LOCAL PARAMETERS :                                    ***
!**   -------------------------                                     ***
!**                                                                 ***
!--------I------I-----I------------------------------------------------
! NAME   I TYPE I I/O I     MEANING
!--------I------I-----I------------------------------------------------
!--------I------I-----I------------------------------------------------
! ptrlu  I  I   I i/o I  Pointer to the last element of matrix U
!--------I------I-----I------------------------------------------------
!
!
!**** START OF CALCULATION :
!     ---------------------
!

      ierr  = 0

      lout  = ilin(12)
      idoku = ilin(13)
      ptrlu = ilin(37)

      shiftU = lpmat - ptrlu
      if (shiftU <= 0) then
	ierr = 10111
      endif

      addaU = infoU(1,1)
      lenU  = infoU(nvtU,1) + infoU(nvtU,2) - addaU
      addaL = infoL(1,1)
      lenL  = infoL(nvtL,1) + infoL(nvtL,2) - addaL

      lenUL = lenU + LenL
      
      ihelp(1) = lenUL
      ihelp(2) = ierr
      call LL4INM(1,2,1,ihelp(1),ihelp(3),ihelp(5),
     &            myproc,nproc,tids,nmsg)
      glenUL = ihelp(3)
      iermax = ihelp(4)

      if ((idoku > 0) .and. (myproc == 1)) then
        write(lout,1000)
        write(lout,1001) glenUL
        write(lout,1002) rlin(4)
      endif
      if (idoku < 0) then
        write(lout,1000)
        write(lout,1001) glenUL
        write(lout,1002) rlin(4)
      endif

      lgfac = DBLE(lenUL)/DBLE(lmat) + 0.5D0
      call LL4RNM(1,1,1,lgfac,gfac,rhelp,myproc,nproc,tids,nmsg)
      if ((idoku > 0) .and. (myproc == 1)) then
        write(lout,1003) gfac
      endif
      if (idoku < 0) then
        write(lout,1003) gfac
      endif

      do i=lenU,1,-1
	pmat(addaU+shiftU+i) = pmat(addaU+i)
      enddo

      do i=lenU,1,-1
	pix(addaU+shiftU+i) = pix(addaU+i)
      enddo

      do i=1,nvtU
	infoU(i,1) = infoU(i,1) + shiftU
      enddo

      if (iermax > 0) then
	ierr = 10111
        if (myproc == 1) then
          write(lout,1020)
          write(lout,1021)
          write(lout,1030)
        endif
      endif

 1000 format(' ')
 1001 format('   Size of max. local preconditioning matrices U+L :',i11)
 1002 format('   Gauss factor (gaussfac) is set to  .............:',
     &           f11.1)
 1003 format('   Gauss factor (gaussfac) must be set (min.) to ..:',
     &           f11.1,/)
 1020 format(' **************************************************'/
     &       ' ***  ERROR occured in SHIFTU_END (LSOLP):      ***')
 1021 format(' ***   <LINSOL_GAUSS_FACTOR> or <rlin(4)> is    ***'/
     &       ' ***   too small!                               ***')
 1030 format(' **************************************************')

      R E T U R N
!------END OF SHIFTU_END------------------------------------------------
      E N D                                   
