!**********************************************************************
!        1         2         3         4         5         6         7*
!**********************************************************************
!**                                                                 ***
!**                                                                 ***
      subroutine SHIFTU_ROW(pmat,lpmat,pix,lpix,infoL,infoU,ia1LU,
     *                      lmywrp,iw,lglobal,ilin,myproc,nproc,tids,
     *                      nmsg,ierr)
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**   S H I F T U _ R O W  resorts the packed columns of U into     ***
!**                        packed rows; the values of U must be     ***
!**                        sorted in ascending order;               ***
!**                        CAUTION: the packed columns of U are     ***
!**                        mixed together with the packed rows of L.***
!**                        REMARK: there is a vector of U for each  ***
!**                        column of AP (no. of columns is lglobal).***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**    Copyright (C) 1999 University Karlsruhe                      ***
!**    PROGRAMMER: H. Haefner                                       ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**     THE IMPLICIT NONE-INSTRUCTION HAS THE AIM,                  ***
!**     THAT ALL VARIABLES MUST BE DECLARED EXPLICITLY.             ***
!**                                                                 ***
      USE constants

      implicit none
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**      FORMAL PARAMETERS :                                        ***
!**      -----------------                                          ***
!**                                                                 ***
      integer,intent(in) ::  lpmat,lpix,ia1LU,lmywrp,lglobal,nproc
      integer,intent(in) ::  ilin(nilin),tids(nproc),myproc
      integer,intent(inout) ::  ierr,nmsg
      integer,intent(inout) ::  pix(lpix),infoL(ia1LU,ia2LU)
      integer,intent(inout) ::  infoU(ia1LU,ia2LU),iw(lglobal)
      double precision,intent(inout):: pmat(lpmat)
!**                                                                 ***
!**                                                                 ***
!**      LOCAL PARAMETERS :                                         ***
!**      -----------------                                          ***
!**                                                                 ***
      integer    lbuf,libuf
      integer, dimension(:), pointer ::  ibuf
      double precision, dimension(:), pointer ::  buf
      integer           infnew(lmywrp,2)
      integer           i,j,k,norows
      integer           addaL,iarL,indcL,lvtL
      integer           addaU,iacU,indrU,lvtU
      integer           ptrluold,ptrlunew,ptrlu9,matdiff
      integer           firstix,ktmp,lvtbuf,nvtbuf,ptrbuf
      integer           ptrlu,runacc
      integer, parameter ::  ixmark = -999
      double precision, parameter ::  matmark = 0.0
      logical           lcopy
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**   LIST OF LOCAL PARAMETERS :                                    ***
!**   -------------------------                                     ***
!**                                                                 ***
!--------I------I-----I------------------------------------------------
! NAME   I TYPE I I/O I     MEANING
!--------I------I-----I------------------------------------------------
!--------I------I-----I------------------------------------------------
! ptrlu  I  I   I i/o I  local size of matrix array
!--------I------I-----I------------------------------------------------
! matmarkI  I   I in  I  marker for matrix elements to be removed
!--------I------I-----I------------------------------------------------
! ixmark I  I   I in  I  marker for index elements to be removed
!--------I------I-----I------------------------------------------------
!
!***********************************************************************
!**    Interface of external subroutines                             ***
!***********************************************************************
      interface

       subroutine CREATE_BUFR(buf,bufsize,maxbuf,ilin,myproc,nproc,
     *                        tids,nmsg,ierr)
	USE constants
        implicit none
        integer,intent(in) ::   nproc,myproc,maxbuf
        integer,intent(inout) ::  nmsg
        integer,intent(out) ::  ierr
        integer,intent(in) ::   ilin(nilin),tids(nproc)
        double precision,dimension(:),pointer ::  buf
        integer,intent(out)::   bufsize
       end subroutine

       subroutine CREATE_BUFI2(buf,bufsize,maxbuf,ilin,myproc,nproc,
     *                         tids,nmsg,ierr)
	USE constants
        implicit none
        integer,intent(in) ::   nproc,myproc,maxbuf
        integer,intent(inout) ::  nmsg
        integer,intent(out) ::  ierr
        integer,intent(in) ::   ilin(nilin),tids(nproc)
        integer,dimension(:),pointer ::  buf
        integer,intent(out)::   bufsize
       end subroutine

      end interface

!
!**** START OF CALCULATION :
!     ---------------------
!

      runacc   = ilin(21)
      ptrlu    = ilin(37)
      ptrlunew = ptrlu
      norows   = 0

!
!***  Create buffer <buf> for L and buffer <ibuf> for indices of L
!
      call CREATE_BUFR(buf,lbuf,ptrlu+lglobal,ilin,myproc,nproc,
     *                 tids,nmsg,ierr)

      call CREATE_BUFI2(ibuf,libuf,ptrlu+lglobal,ilin,myproc,nproc,
     *                  tids,nmsg,ierr)
      if (ierr /= 0) goto 999

!
!**** Computation of minimal and maximal column index per row for U
!
      do i=1,lmywrp
        infnew(i,1) = lglobal + 1
        infnew(i,2) = 0
      enddo
      do i=1,lglobal     ! for all columns (wrapped around onto proc.)
        lvtU  = infoU(i,2)
        indrU = infoU(i,1)
        do k=1,lvtU
          j = INT((pix(indrU+k)-1)/nproc) + 1
          if (infnew(j,1) > i) infnew(j,1) = i
          if (infnew(j,2) < i) infnew(j,2) = i
        enddo
      enddo

!
!***  Store the packed colums of U rowwise into the buffer <buf> and
!***  the row indices of U in the buffer <ibuf>
!
      firstix = 1

  111 iw = 0
      ptrbuf = 0
      nvtbuf = 0
      do j=firstix,lmywrp    ! for all rows
        nvtbuf = nvtbuf + 1
        lvtbuf = 0
        do i=infnew(j,1),infnew(j,2) ! for all columns (originally: i=1,lglobal)
          addaU = infoU(i,1)
          lvtU  = infoU(i,2)
          iacU  = infoU(i,3)
          indrU = addaU
          k=iw(i)+1
          if ((lvtU-iw(i) > 0) .and.
     &        (pix(indrU+k) == (j-1)*nproc+myproc)) then
            lvtbuf = lvtbuf+1
            buf(ptrbuf+lvtbuf) = pmat(addaU+k)
            pmat(addaU+k) = matmark
            ibuf(ptrbuf+lvtbuf) = iacU + 1
            pix(indrU+k) = ixmark
            iw(i) = iw(i) + 1
          endif
        enddo 
        infnew(j,1) = ptrbuf
        infnew(j,2) = lvtbuf
        ptrbuf = ptrbuf + lvtbuf
        if ((ptrbuf+lglobal > lbuf) .or. (ptrbuf+lglobal > libuf)) then
          firstix = j + 1
          lcopy = .false.
          exit
        endif
      enddo 

      if (j > lmywrp .or. firstix > lmywrp) lcopy = .true.

      do i=1,lglobal
	infoU(i,2) = infoU(i,2) - iw(i)
      enddo

!
!***  Garbage collection
!
      ptrluold = ptrlunew
      
      if (runacc == 0) then
        call GCLU(pmat,pix,infoL,infoU,lpmat,lpix,ia1LU,lmywrp,
     *            lglobal,matmark,ixmark,ptrlunew,myproc,nproc)
      else
        call GCLUOPT(pmat,pix,infoL,infoU,lpmat,lpix,ia1LU,lmywrp,
     *               lglobal,matmark,ixmark,ptrlunew,myproc,nproc)
      endif

      if (norows == 0) then
	ptrlu9 = ptrlunew
      else
	matdiff = ptrluold - ptrlunew
	do j=1,norows
          infnew(j,1) = infnew(j,1) - matdiff
        enddo
        do j=ptrluold+1,ptrlu9
          pmat(j-matdiff) = pmat(j)
        enddo
        do j=ptrluold+1,ptrlu9
          pix(j-matdiff) = pix(j)
        enddo
        ptrlu9 = ptrlu9 - matdiff
      endif

!
!***  Copy the buffer <buf> to the end of <pmat> and the buffer <ibuf>
!***  to the end of <pindex>
!
      do j=1,nvtbuf
        norows = norows + 1
	do k=1,infnew(norows,2)
	  pmat(ptrlu9+k) = buf(infnew(norows,1)+k)
	enddo
	do k=1,infnew(norows,2)
	  pix(ptrlu9+k) = ibuf(infnew(norows,1)+k)
	enddo
        infnew(norows,1) = ptrlu9
	ptrlu9 = ptrlu9 + infnew(norows,2)
      enddo
        
      if (.not. lcopy) goto 111 
      if (norows /= lmywrp) then
	print*,' ERROR in SHIFTU_ROW: Missing row of U'
	ierr = 9999
      endif

      do i=1,lmywrp
        infoU(i,1) = infnew(i,1)
        infoU(i,2) = infnew(i,2)
        infoU(i,3) = (i-1)*nproc + myproc - 1
      enddo

      deallocate(buf,ibuf)

  999 continue

      R E T U R N
!------END OF SHIFTU_ROW------------------------------------------------
      E N D                                   
