!:::::      ,,,,,SSPBO.....  -*- f90 -*-
!
!
!
!**********************************************************************
!        1         2         3         4         5         6         7*
!**********************************************************************
!**                                                                 ***
!**                                                                 ***
module sspbo
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**      program description of SSPBO                               ***
!**                                                                 ***
!**      Band optimizes a matrix using the                          ***
!**      Schoenauer single pass                                     ***
!**      algorithm.                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**    Copyright (C) 1998, 1999 University Karlsruhe                ***
!**    AUTHOR : Detlev Zundel                                       ***
!**                                                                 ***
!**    $Id: sspbo.f90,v 1.14 2002/08/19 15:19:30 ne12 Exp $         ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                 ***
!**********************************************************************

  use constants
  use comm_ifc

  implicit none


  ! Declarations of the used subroutines


  contains

    ! Subroutine : sspboptim
    ! Purpose    : Main subroutine for the band optimizer
    !
    ! Assertions : This routine works only with packed rows
    !              i.e. vector terms of the LINSOL type 9
    subroutine SSPBOPTIM(mat,lmat,b,perm,index,lindex,info,ia1,lsym,l,&
         ilin,lmatbk,myproc,nproc,tids,nmsg,iconv,ierr)

      ! Formal parameters
      integer :: ierr, iconv, lindex, ia1, l, lmat
      integer :: info(:,:), myproc, nproc, nmsg
      integer :: ilin(:), lmatbk(nproc), perm(l), tids(nproc)
      logical :: lsym
      integer, dimension(:) :: index
      double precision, dimension(:) :: mat
      double precision, dimension(:) :: b
      intent (in) :: ia1, l, lmatbk, myproc
      intent (in) :: nproc, lsym, lindex, lmat
      intent (inout) :: iconv, ierr, ilin, nmsg
      intent (inout) :: mat, b, perm

      !
      ! Constants
      !
      integer, parameter ::  msgsize    = 100
      integer, parameter ::  headersize =   3
      integer, parameter ::  datasize   = msgsize - headersize

      double precision, parameter ::  levelwfactor = 4.2



      ! Local variables
      integer :: lfrtid, frtid, ltotid, totid
      integer :: mids, midr
      integer :: mids2, midr2, ierr1, ierr2
      integer :: ptrmbk(nproc+1), ifirstrow(nproc), ilastrow(nproc)
      integer :: endmat, endind
      integer :: mmsin(2), mmsout(2), mmsbuf(2), ihelp(6)
      integer :: i, j, k, p
      integer :: itact, ibasketprocess, ibasketrec
      integer :: maxstoredeg
      integer :: highdeg(l), mncon(l), mnumb(l)

      double precision ::  symbfactor, newsymbfactor

      ! Communication variables
      integer ::  msgbuf1(msgsize), msgbuf2(msgsize)

      ! Local variables for symmetrization step (Step II)
      integer :: maxdeg
      integer, allocatable, dimension(:,:) :: mindlist
      integer, allocatable, dimension(:,:) :: sendbuf, recbuf
      logical :: exist, diagonal
      integer :: colindex, irow, intermed

      integer :: receiver, free
      integer, allocatable, dimension(:) :: cands, ibuf11, ibuf12
      integer, allocatable, dimension(:) :: msgbuf3

      ! Local variables for step III
      integer :: imin, numbmin, noofproc
      integer :: allocsize

      ! Local variables for step IV
      logical :: active
      integer :: iglobnewlast, numbexcess
      integer :: iloc, nodcount, isum, isumcount
      integer :: nlano, m
      integer :: nunod(nproc)
      integer, allocatable, dimension(:) :: par
      integer, allocatable, dimension(:,:) :: indlist
      integer :: ilano(INT(levelwfactor*l)), numb(INT(levelwfactor*l))
      integer :: ncon(INT(levelwfactor*l))
      integer :: iold(INT((levelwfactor+1)*l))
      integer :: isend, isec, irest, iseccount, ipar, icount
      integer :: parsize, numloc
      integer :: sendproc, key, number
      logical :: used(INT(levelwfactor*l))

      ! Local variables used in the ranking process
      integer :: iil, is, i2, i3, ic
      integer :: istack1, istack2, istack3
      integer, allocatable, dimension(:) :: nstack1, nstack2, nstack3
      integer, allocatable, dimension(:) :: ncond
      logical, allocatable, dimension(:) :: usedstack1, usedstack2, &
           usedstack3
      integer :: icomp
      integer, allocatable, dimension(:) :: allnew

      ! Local variables for the resorting
      integer, allocatable, dimension(:) :: permbuf, ioldbuf, inewsort
      integer, allocatable, dimension(:,:) :: idatasend, idatarec
      double precision, allocatable, dimension(:,:) :: rdatasend, rdatarec
      integer, allocatable, dimension(:) :: mnumbnew, mnconnew
      integer, allocatable, dimension(:,:) :: mindlistnew
      integer, allocatable, dimension(:,:) :: mindlistcorrected
      double precision, allocatable, dimension(:,:) :: realelnew
      integer :: idx
      integer, allocatable, dimension(:) :: inewsortbuf
      integer :: elemcount, rperm(l)

      ! Local variables for final resorting
      integer, allocatable, dimension(:) :: lix
      double precision, allocatable, dimension(:) :: matbuf

      ! Timing varibles and verbose level
      double precision :: timecpu, timeelapsed, thlp1, thlp2;
      integer iverbose, lout

      ierr  = 0
      ierr1 = 0

      symbfactor = 2.d0

      ! Verbose level is passed in ilin
      lout    =ilin(12)
      iverbose=ilin(13)

      ! Initialize parameters for the ringshift
      lfrtid=LL9MAP(myproc-1,nproc)
      frtid=tids(lfrtid)
      ltotid=LL9MAP(myproc+1, nproc)
      totid=tids(ltotid)

      ! Initialize ptrmbk, ifirstrow, ilastrow
      ptrmbk(1)=0
      do p=1, nproc
        ptrmbk(p+1)=ptrmbk(p)+lmatbk(p)
        ifirstrow(p)=ptrmbk(p)+1
        ilastrow(p)=ptrmbk(p+1)
      enddo

      if (myproc==1) then
        call SECONDS(timecpu, timeelapsed);
      endif

      ! First we build the mindlist and mncon fields
      ! So we have to determine allocation limits for those

      ! Determine an upper bound for maxdegree
      maxdeg=0
      do i=1, lmatbk(myproc)
        if (info(i,3)>maxdeg) maxdeg=info(i,3)
      enddo
      mmsin(1)=maxdeg
      call LL4INM(1, 1, 1, mmsin, mmsout, mmsbuf, &
           myproc, nproc, tids, nmsg)
      maxstoredeg=mmsout(1)
!???  if (lsym) then
!???    maxdeg=maxstoredeg
!???  else
        ! Approximate upper bound (probably needs to be changed)
 111    maxdeg = INT(symbfactor*maxstoredeg)
!???  endif

      ! allocate the fields
      allocate(mindlist(l, maxdeg), stat=ierr)

      if (ierr /= 0) ierr = maxdeg*l*iint
      ihelp(1) = ierr
      call LL4INM(1,1,1,ihelp(1),ihelp(2),ihelp(3),myproc,nproc,&
                  tids,nmsg)
      if (ihelp(2) /= 0) then
        ierr = ihelp(2)
        iconv = 99
        if (myproc == 1) then
          write(lout,1020)
          write(lout,1021) symbfactor, ierr
          write(lout,1030)
        endif
        goto 9999
      endif

 1010 format(/'  **************************************************'/&
             '  ***  WARNING occured in SSPBO:                 ***')
 1011 format('  ***   matrix storage exceeded during           ***'/&
             '  ***   symmetrization! <symbfactor> = ',f6.1,'    ***'/&
             '  ***   is changed to ',f6.1,' !!!                 ***')
 1020 format(/'  **************************************************'/&
             '  ***  ERROR occured in SSPBO:                   ***')
 1021 format('  ***   <symbfactor> =',f6.1,' has been used to    ***'/&
             '  ***   allocate array <mindlist>; the resultant ***'/&
             '  ***   allocation size of (maximum) ',i10,'  ***'/&
             '  ***   Bytes is too high!                       ***')
 1023 format('  ***   <symbfactor> =',f6.1,' has been used to    ***'/&
             '  ***   allocate arrays <sendbuf> and <recbuf>;  ***'/&
             '  ***   the resultant allocation size of         ***'/&
             '  ***   (maximum) ',i10,' Bytes is too high!  ***')
 1024 format('  ***   the allocation size for array <indlist>  ***'/&
	     '  ***   of (max.) ',i10,' Bytes is too high!  ***')
 1025 format('  ***   the matrix is reducible! This property   ***'/&
	     '  ***   can not be handled by this algorithm!!!  ***')
 1030 format('  **************************************************'/)


      ! Generate the index list
      do i=1, lmatbk(myproc)
        mncon(i)=info(i,3)
        do j=1, mncon(i)
          mindlist(i,j)=index(info(i,6)+j)
        enddo
      enddo

      ! Step I:
      ! Delete the main diagonal
      do i=1, lmatbk(myproc)
        mnumb(i)=i+ifirstrow(myproc)-1
        diagonal=.false.
        do j=1, mncon(i)
          if (mindlist(i,j)==mnumb(i)) then
            diagonal=.true.
            mncon(i)=mncon(i)-1
          endif
          if (diagonal .and. (j<=mncon(i))) then
            mindlist(i,j)=mindlist(i,j+1)
          endif
        enddo
      enddo

      ! Step II:
      ! Symmetrize the connection lists
      ! for LINSOL we have to do this for lsym=.true. AND lsym=.false.
      ! because lsym=.true. implies the storage of only half the matrix!
      if (.true.) then
        ! allocate the comm buffers
        allocate(sendbuf(l,(maxdeg+1)),recbuf(l,(maxdeg+1)), stat=ierr)

        if (ierr /= 0) ierr = 2*l*(maxdeg+1)*ireal
        ihelp(1) = ierr
        call LL4INM(1,1,1,ihelp(1),ihelp(2),ihelp(3),myproc,nproc,&
                    tids,nmsg)
        if (ihelp(2) /= 0) then
          ierr  = ihelp(2)
          iconv = 99
          if (myproc == 1) then
            write(lout,1020)
            write(lout,1023) symbfactor, ierr
            write(lout,1030)
          endif
          goto 9999
        endif

	highdeg = mncon

        ! Prepare the send buffer
        do i=1, lmatbk(myproc)
          sendbuf(i,1)=mncon(i)
          do j=1, mncon(i)
            sendbuf(i,j+1)=mindlist(i,j)
          enddo
        enddo

        ! Symmetrize the index list by ringshift
        do itact=1, nproc
          ibasketprocess=modulo(myproc-itact+nproc,nproc)+1
          ibasketrec=modulo(myproc-itact+nproc-1,p)+1
          if (nproc>1) then
            call MPSNDA(totid, nmsg+itact, (maxdeg+1)*l*iint, &
                 sendbuf, mids, ierr)
            call MPRCVA(frtid, nmsg+itact, (maxdeg+1)*l*iint, &
                 recbuf, midr, ierr)
          endif

          ! Process passing basket
          do i=1, lmatbk(ibasketprocess)
            do j=1, sendbuf(i,1)
              exist=.false.
              colindex=sendbuf(i,j+1)
              if ((colindex>=ifirstrow(myproc)) .and. &
                   (colindex<=ilastrow(myproc))) then
                irow=colindex-ifirstrow(myproc)+1
                do k=1, mncon(irow)
                  if (mindlist(irow,k)==i+ifirstrow(ibasketprocess)-1) then
                    exist=.true.
                    exit
                  endif
                enddo           ! k
                if (.not. exist) then
                  mncon(irow)=mncon(irow)+1
                  highdeg(irow)=highdeg(irow)+1
                  if (mncon(irow) > maxdeg) then
                    mncon(irow)=mncon(irow)-1
		    ierr1 = MAX(ierr1,highdeg(irow))
		  endif
                  mindlist(irow,mncon(irow))=i+ifirstrow(ibasketprocess)-1
                endif
              endif
            enddo               ! j
          enddo                 ! i

          if (nproc>1) then
            call MPSNDW(totid, nmsg+itact, (maxdeg+1)*l*iint, &
                 sendbuf, mids, ierr)
            call MPRCVW(frtid, nmsg+itact, (maxdeg+1)*l*iint, &
                 recbuf, midr, ierr)
            sendbuf=recbuf
          endif

        enddo                   ! itact
        if (nproc>1) nmsg=nmsg+nproc

        deallocate(sendbuf,recbuf)

        ihelp(1) = ierr1
        call LL4INM(1,1,1,ihelp(1),ihelp(2),ihelp(3),myproc,&
		    nproc,tids,nmsg)
        if (ihelp(2) /= 0) then
          newsymbfactor = ihelp(2)/(maxdeg/symbfactor)
          if (myproc == 1) then
            write(lout,1010)
            write(lout,1011) symbfactor, newsymbfactor
            write(lout,1030)
          endif
          symbfactor = newsymbfactor
          deallocate(mindlist)
	  ierr1 = 0
          goto 111
        endif

        ! Sort the index lists
        do i=1, lmatbk(myproc)
          do j=1, mncon(i)-1
            do k=1, mncon(i)-j
              if (mindlist(i,k)>mindlist(i,k+1)) then
                intermed=mindlist(i,k)
                mindlist(i,k)=mindlist(i,k+1)
                mindlist(i,k+1)=intermed
              endif
            enddo
          enddo
        enddo

      endif                     ! Symmetrization of connection lists

      ! Compute true maxdeg
      maxdeg=0
      do i=1, lmatbk(myproc)
        if (mncon(i)>maxdeg) maxdeg=mncon(i)
      enddo
      mmsin(1)=maxdeg
      call LL4INM(1, 1, 1, mmsin, mmsout, mmsbuf, &
           myproc, nproc, tids, nmsg)
      maxdeg=mmsout(1)

      if ((myproc==1).and.(iverbose>0)) then
        call SECONDS(thlp1, thlp2);
        write (lout,*) 'SSPBO'
        write (lout, '(a,f8.2)') &
             '   CPU-Time     for symm.   process  (in seconds):', &
             thlp1-timecpu
        write (lout, '(a,f8.2)') &
             '   Elapsed time for symm.   process  (in seconds):', &
             thlp2-timeelapsed
        timecpu=thlp1;
        timeelapsed=thlp2;
      endif

      ! Allocate fields
      allocsize = INT(levelwfactor*l)
      allocate(indlist(allocsize,maxdeg), stat=ierr)

      if (ierr /= 0) ierr = allocsize*maxdeg*iint
      ihelp(1) = ierr
      call LL4INM(1,1,1,ihelp(1),ihelp(2),ihelp(3),myproc,nproc,&
                  tids,nmsg)
      if (ihelp(2) /= 0) then
        ierr = ihelp(2)
        iconv = 99
        if (myproc == 1) then
          write(lout,1020)
          write(lout,1024) ierr
          write(lout,1030)
        endif
        goto 9999
      endif

      ! Permute main diagonal entries to the front before
      ! everything else
      ! As this was proven to be trivial, no comments are needed
      iglobnewlast=1
      iloc=1

      allocate(cands(nproc), ibuf11(nproc), ibuf12(nproc), stat=ierr)
      if (ierr>0) then
        write(lout,*) 'sspbo: Sorry, cands field ', &
             'can not be allocated on proc', myproc
        stop
      endif
      cands(1:nproc)=0
      do i=1,lmatbk(myproc)
        if (mncon(i)==0) cands(myproc)=cands(myproc)+1
      enddo
      call LL4ISM(1, nproc, 1, cands, ibuf11, ibuf12, &
           myproc, nproc, tids, nmsg)
      cands(1:nproc)=ibuf11(1:nproc)

      allocate(msgbuf3(maxval(cands)), stat=ierr)
      if (ierr>0) then
        write(lout,*) 'sspbo: Sorry, msgbuf3 field ', &
             'can not be allocated on proc', myproc
        stop
      endif

      receiver=1
      free=lmatbk(receiver)
      do p=1,myproc-1
        free=free-cands(p)
        iglobnewlast=iglobnewlast+cands(p)
        if (free<=0) then
          receiver=receiver+1
          free=free+lmatbk(receiver)
        endif
      enddo
      do while(cands(myproc)>0)
        i=1
        isend=0
        do while (.true.)
          if (mncon(i)==0) then
            if (receiver/=myproc) then
              isend=isend+1
              msgbuf3(isend)=i+ifirstrow(myproc)-1
              iglobnewlast=iglobnewlast+1
            else
              iold(iloc)=i+ifirstrow(myproc)-1
              iloc=iloc+1
              iglobnewlast=iglobnewlast+1
            endif
            free=free-1
            cands(myproc)=cands(myproc)-1
          endif
          i=i+1
          if ((i>lmatbk(myproc)).or.(free<=0).or.(cands(myproc)==0)) exit
        enddo                   ! i
        if (receiver/=myproc) then
!          write(lout,*) 'proc: ',myproc,' sending ',isend,' to ',receiver
          call MPSNDA(tids(receiver), myproc, isend*iint, msgbuf3, &
               mids, ierr)
          call MPSNDW(tids(receiver), myproc, isend*iint, msgbuf3, &
               mids, ierr)
        endif                   ! receiver /= myproc
        if (free<=0) then
          receiver=receiver+1
          free=lmatbk(receiver)
        endif
      enddo                     ! cands(myproc)>0
      do p=myproc+1, nproc
        if (receiver==myproc) then
          if (cands(p)>0) then
            isend=min(free,cands(p))
!            write(lout,*) 'proc: ',myproc,' receiving ',isend,' from ',p
            call MPRCVA(tids(p), p, isend*iint, msgbuf3, &
                 mids, ierr)
            call MPRCVW(tids(p), p, isend*iint, msgbuf3, &
                 mids, ierr)
            do i=1,isend
              iold(iloc)=msgbuf3(i)
              iloc=iloc+1
              iglobnewlast=iglobnewlast+1
            enddo
            free=free-isend
          endif
        else
          free=free-cands(p)
          iglobnewlast=iglobnewlast+cands(p)
        endif
        if (free<=0) then
          receiver=receiver+1
          free=free+lmatbk(receiver)
        endif
      enddo

      deallocate(msgbuf3)
      deallocate(cands, ibuf11, ibuf12)

      ! The processor with the first free slots starts the rest of the
      ! processing
      if (myproc==receiver) then
        active=.true.
      else
        active=.false.
      endif

      if (iglobnewlast==ilastrow(nproc)+1) then
        write (lout,*) 'sspbo:  Warning - only main diagonal found - exiting!'
        goto 6000
      endif

      ! Assertion:
      ! We now have the correct information in mncon(:) and
      ! mindlist(:) for the symmetric and unsymmetric case

      ! Step III:
      ! Find start node (this fails for A=diag(...))
      imin=maxdeg+1
      numbmin=ifirstrow(myproc)
      do i=1, lmatbk(myproc)
        if ((mncon(i)>1) .and. (mncon(i)<imin)) then
          imin=mncon(i)
          numbmin=i+ifirstrow(myproc)-1
        endif
      enddo

      ! Globalize the minimum
      if (.not. active) then
        mmsin(1)=imin
        mmsin(2)=numbmin
        call MPSNDA(tids(receiver), myproc, 2*iint, mmsin, mids, ierr)
        call MPSNDW(tids(receiver), myproc, 2*iint, mmsin, mids, ierr)
      else                      ! active
        noofproc=1
        do p=1, nproc
          if (p==myproc) cycle
          call MPRCVA(tids(p), p, 2*iint, mmsin, midr, ierr)
          call MPRCVW(tids(p), p, 2*iint, mmsin, midr, ierr)
          if (mmsin(1)<imin) then
            imin=mmsin(1)
            numbmin=mmsin(2)
            noofproc=p
          endif
        enddo
      endif                     ! active

      ! Assumption:
      ! No we have the first node with minimal degree. It has the number
      ! `numbmin' on processor `noofproc' and the degree `imin'.

      ! Step IV:
      ! Number the nodes
      d1: do                        ! d1 - loops forever
        if (active .and. (myproc==receiver)) then
          ! This is a one-time setup to get things rolling
          iold(iloc)=numbmin
          nlano=1
          ilano(1)=iold(iloc)
          numbexcess=0
          m=imin
          if (noofproc==myproc) then
            ! Start node is on local processor
            do i=1,m
              numb(i)=mindlist(numbmin-ifirstrow(myproc)+1, i)
            enddo
          else                  ! noofproc==myproc
            ! Get neighbour list from `noofproc'
            msgbuf1(1)=myproc   ! procid
            msgbuf1(2)=1        ! msgtype
            msgbuf1(3)=1        ! len
            msgbuf1(4)=numbmin  ! data(1)
            call MPSNDA(tids(noofproc), 0, msgsize*iint, msgbuf1, &
                 mids, ierr)
            allocate(par(m), stat=ierr)
            if (ierr>0) then
              write(lout,*) 'sspbo: (1) Sorry, par field ', &
                   'can not be allocated on proc', myproc
              stop
            endif
            call MPSNDW(tids(noofproc), 0, msgsize*iint, msgbuf1, &
                 mids, ierr)
            call MPRCVA(tids(noofproc), 1, m*iint, par, &
                 midr, ierr)
            call MPRCVW(tids(noofproc), 1, m*iint, par, &
                 midr, ierr)
            do i=1,m
              numb(i)=par(i)
            enddo
            deallocate(par)
          endif                 ! noofproc==myproc

          ! Now everything should be setup for the standard cycle

        endif                   ! active .and. (myproc==receiver)


        if (active) then
          cycles: do            ! endless loop over all levels for one proc
            ! Here comes the code for the active processor
            ! He does the numbering and requests msgs from
            ! other procs

            if (m==0) then
              ierr = 1
	      msgbuf1(1) = -1
	      do i=1,nproc 
	        if (i /= myproc) then
                  call MPSNDA(tids(p), 0, msgsize*iint, msgbuf1, &
                              mids, ierr)
                  call MPSNDW(tids(p), 0, msgsize*iint, msgbuf1, &
                              mids, ierr)
	        endif
	      enddo
              iconv = 99
              if (myproc == 1) then
                write(lout,1020)
                write(lout,1025)
                write(lout,1030)
              endif
              goto 9999
            endif

            do i=1, m
              used(i)=.false.
            enddo
            ! Group new nodes by proc
            do p=1, nproc
              nunod(p)=0
            enddo
            i=1
            d2: do p=1, nproc
              d3: do
                if (numb(i)<=ilastrow(p)) then
                  nunod(p)=nunod(p)+1
                  i=i+1
                else
                  exit d3
                endif
                if (i==(m+1)) then
                  exit d2
                endif
              enddo d3
            enddo d2
            ! Processor `p' now holds `nunod(p)' new nodes
            ! Now request the relevant info from each in turn
            i=0
            do p=1, nproc
              if (nunod(p)==0) cycle
              if (myproc==p) then
                ! The active processor can fill in the info right away
                do nodcount=1, nunod(p)
                  i=i+1
                  ncon(i)=mncon(numb(i)-ifirstrow(myproc)+1)
                  do j=1, ncon(i)
                    indlist(i,j)=mindlist(numb(i)-ifirstrow(myproc)+1,j)
                  enddo
                enddo
              else                ! myproc==p

                ! Request info from processor `p'
                !
                ! At first we request the `ncon' data
                !
                isend=i
                isec=int(nunod(p)/datasize)
                irest=mod(nunod(p), datasize)
                ! Process the "full" messages
                do iseccount=1, isec
                  msgbuf1(1)=myproc     ! procid
                  msgbuf1(2)=2          ! msgtype
                  msgbuf1(3)=datasize   ! len

                  ! Fill in the `numb' data
                  do k=1, datasize
                    msgbuf1(headersize+k)=numb(isend+k)
                  enddo

                  call MPSNDA(tids(p), 0, msgsize*iint, msgbuf1, &
                       mids, ierr)
                  call MPSNDW(tids(p), 0, msgsize*iint, msgbuf1, &
                       mids, ierr)
                  call MPRCVA(tids(p), 1, datasize*iint, msgbuf1, &
                       midr, ierr)
                  call MPRCVW(tids(p), 1, datasize*iint, msgbuf1, &
                       midr, ierr)

                  ! Store away the received data
                  do k=1, datasize
                    ncon(isend+k)=msgbuf1(k)
                  enddo
                  isend=isend+datasize
                enddo
                ! Process the "rest"
                if (irest/=0) then
                  msgbuf1(1)=myproc     ! procid
                  msgbuf1(2)=2          ! msgtype
                  msgbuf1(3)=irest      ! len

                  ! Fill in the `numb' data
                  do k=1, irest
                    msgbuf1(headersize+k)=numb(isend+k)
                  enddo

                  call MPSNDA(tids(p), 0, msgsize*iint, msgbuf1, &
                       mids, ierr)
                  call MPSNDW(tids(p), 0, msgsize*iint, msgbuf1, &
                       mids, ierr)
                  call MPRCVA(tids(p), 1, m*iint, msgbuf1, &
                       midr, ierr)
                  call MPRCVW(tids(p), 1, m*iint, msgbuf1, &
                       midr, ierr)

                  ! Store away the received data
                  do k=1, irest
                    ncon(isend+k)=msgbuf1(k)
                  enddo
                  isend=isend+irest
                endif             ! irest/=0

                !
                ! Now we can request the `indlist' data
                !
                isend=i
                ! Process the "full" messages
                do iseccount=1, isec
                  msgbuf1(1)=myproc     ! procid
                  msgbuf1(2)=1          ! msgtype
                  msgbuf1(3)=datasize   ! len

                  ! Fill in the `numb' data
                  do k=1, datasize
                    msgbuf1(headersize+k)=numb(isend+k)
                  enddo

                  call MPSNDA(tids(p), 0, msgsize*iint, msgbuf1, &
                       mids, ierr)

                  ! Calculate the size of the receive message
                  isum=0
                  do isumcount=isend+1, isend+datasize
                    isum=isum+ncon(isumcount)
                  enddo
                  isend=isend+datasize

                  allocate(par(isum),stat=ierr)
                  if (ierr>0) then
                    write(lout,*) 'sspbo: (2) Sorry, par field ', &
                         'can not be allocated on proc', myproc
                    stop
                  endif

                  call MPSNDW(tids(p), 0, msgsize*iint, msgbuf1, &
                       mids, ierr)
                  call MPRCVA(tids(p), 1, isum*iint, par, &
                       midr, ierr)
                  call MPRCVW(tids(p), 1, isum*iint, par, &
                       midr, ierr)

                  ! Store away the received data
                  ipar=0
                  do icount=1, datasize
                    i=i+1
                    do j=1, ncon(i)
                      ipar=ipar+1
                      indlist(i,j)=par(ipar)
                    enddo
                  enddo
                  deallocate(par)
                enddo             ! all "full" sections

                ! Process the "rest"
                if (irest/=0) then
                  msgbuf1(1)=myproc     ! procid
                  msgbuf1(2)=1          ! msgtype
                  msgbuf1(3)=irest      ! len

                  ! Fill in the `numb' data
                  do k=1, irest
                    msgbuf1(headersize+k)=numb(isend+k)
                  enddo

                  call MPSNDA(tids(p), 0, msgsize*iint, msgbuf1, &
                       mids, ierr)

                  ! Calculate the size of the receive message
                  isum=0
                  do isumcount=isend+1, isend+irest
                    isum=isum+ncon(isumcount)
                  enddo
                  allocate(par(isum), stat=ierr)
                  if (ierr>0) then
                    write(lout,*) 'sspbo: (3) Sorry, par field ', &
                         'can not be allocated on proc', myproc
                    stop
                  endif
                  call MPSNDW(tids(p), 0, msgsize*iint, msgbuf1, &
                       mids, ierr)
                  call MPRCVA(tids(p), 1, isum*iint, par, &
                       midr, ierr)
                  call MPRCVW(tids(p), 1, isum*iint, par, &
                       midr, ierr)

                  ! Store away the received data
                  ipar=0
                  do icount=1, irest
                    i=i+1
                    do j=1, ncon(i)
                      ipar=ipar+1
                      indlist(i,j)=par(ipar)
                    enddo
                  enddo
                  deallocate(par)
                  isend=isend+irest
                endif             ! irest/=0

              endif               ! myproc==p
            enddo                 ! p

            !
            ! Now follows the numbering of the nodes
            !
            ierr2=0
            allocate(ncond(m),stat=ierr)
            ierr2=ierr2+ierr
            allocate(nstack1(m),nstack2(m),nstack3(m),stat=ierr)
            ierr2=ierr2+ierr
            allocate(usedstack1(m),usedstack2(m),usedstack3(m),stat=ierr)
            ierr2=ierr2+ierr
            if (ierr2>0) then
              write(lout,*) 'sspbo: Sorry, ncond ... field ', &
                   'can not be allocated on proc', myproc
              stop
            endif

            do iil=1, nlano       ! d5
              istack1=0
              istack2=0
              istack3=0
              do i=1,m
                if (.not. used(i)) then
                  do j=1, ncon(i)
                    if (ilano(iil)==indlist(i,j)) then
                      istack1=istack1+1
                      used(i)=.true.
                      nstack1(istack1)=i
                      usedstack1(istack1)=.false.
                      exit
                    endif
                  enddo           ! j
                endif             ! not used(i)
              enddo               ! i
              if (istack1==1) then
                ! If we have only one candidate then we're done
                iloc=iloc+1
                iglobnewlast=iglobnewlast+1
                iold(iloc)=numb(nstack1(1))
                usedstack1(1)=.true.
              endif
              if ((istack1>1).and.(iil<nlano)) then
                ! More than one candidate in stack1
                do is=1, istack1
                  do i2=1, ncon(nstack1(is))
                    if (ilano(iil+1)==indlist(nstack1(is),i2)) then
                      istack2=istack2+1
                      nstack2(istack2)=nstack1(is)
                      usedstack1(is)=.true.
                      usedstack2(istack2)=.false.
                      exit
                    endif
                  enddo           ! i2
                enddo             ! is
              endif               ! (istack1>1).and.(iil<nlano)
              if (istack2==1) then
                ! Only one candidate in stack2
                iloc=iloc+1
                iglobnewlast=iglobnewlast+1
                iold(iloc)=numb(nstack2(1))
                usedstack2(1)=.true.
              endif
              if ((istack2>1).and.(iil<nlano-1)) then
                do is=1, istack2
                  do i3=1,ncon(nstack2(is))
                    if (ilano(iil+2)==indlist(nstack2(is),i3)) then
                      istack3=istack3+1
                      nstack3(istack3)=nstack2(is)
                      usedstack2(is)=.true.
                      exit
                    endif
                  enddo           ! i3
                enddo             ! is
              endif               ! (istack2>1).and.(iil<nlano-1)
              if (istack3==1) then
                ! Only one candidate in stack3
                iloc=iloc+1
                iglobnewlast=iglobnewlast+1
                iold(iloc)=numb(nstack3(1))
              endif
              if (istack3>1) then
                ! Sort the candidates in stack3 according to their
                ! degree
                ! Build ncond table
                do i=1, istack3
                  ncond(i)=ncon(nstack3(i))
                enddo
                do i=1, istack3-1
                  do j=1,istack3-i
                    if (ncon(j)>ncon(j+1)) then
                      intermed=nstack3(j+1)
                      nstack3(j+1)=nstack3(j)
                      nstack3(j)=intermed
                      intermed=ncond(j+1)
                      ncond(j+1)=ncond(j)
                      ncond(j)=intermed
                    endif
                  enddo           ! j
                enddo             ! i
                do i=1,istack3
                  iloc=iloc+1
                  iglobnewlast=iglobnewlast+1
                  iold(iloc)=numb(nstack3(i))
                enddo
              endif               ! (istack3>1)
              ! Now handle all unused elements in stack2
              do i=1, istack2
                if (.not. usedstack2(i)) then
                  iloc=iloc+1
                  iglobnewlast=iglobnewlast+1
                  iold(iloc)=numb(nstack2(i))
                endif
              enddo               ! i
              ! Now the unused elements of stack1 must be handled
              do i=1, istack1
                if (.not. usedstack1(i)) then
                  iloc=iloc+1
                  iglobnewlast=iglobnewlast+1
                  iold(iloc)=numb(nstack1(i))
                endif
              enddo               ! i
            enddo                 ! d5

            ! This check verifies the assumption that every node
            ! has been "used" in the numbering
            do i=1, m
              if (.not. used(i)) then
                write(*,*) 'sspbo: Error in processing of nodes. Node ',&
                     i,' still unused.'
                stop
              endif
            enddo

            deallocate(usedstack1,usedstack2,usedstack3)
            deallocate(nstack1,nstack2,nstack3)
            deallocate(ncond)

            ! The active proc has numbered all of his nodes (or more)
            ! We can stop here if we are the last proc
            if ((iglobnewlast>=ilastrow(myproc)).and.(myproc==nproc)) then
              ! Send exit signal to all waiting procs
              do p=1, nproc-1
                msgbuf1(1)=myproc ! procid
                msgbuf1(2)=4      ! msgtype
                call MPSNDA(tids(p), 0, msgsize*iint, msgbuf1, &
                     mids, ierr)
                call MPSNDW(tids(p), 0, msgsize*iint, msgbuf1, &
                     mids, ierr)
              enddo
              exit d1
            endif

            ! Now prepare the list for the next step
            ! Calculate size for the allnew field
            icount=0
            do i=1,m
              icount=icount+ncon(i)
            enddo
            allocate(allnew(icount),stat=ierr)
            if (ierr>0) then
              write(lout,*) 'sspbo: Sorry, allnew field ', &
                   'can not be allocated on proc', myproc
              stop
            endif
            icount=0
            do i=1,m
              do ic=1, ncon(i)
                icount=icount+1
                allnew(icount)=indlist(i,ic)
              enddo
            enddo                 ! i

            ! Eliminate backward connections
            do ic=1, nlano
              do i=1, icount
                if (allnew(i)==ilano(ic)) then
                  allnew(i)=0
                endif
              enddo
            enddo                 ! ic

            ! Eliminate cross connections
            do ic=1, m
              do i=1, icount
                if (allnew(i)==numb(ic)) then
                  allnew(i)=0
                endif
              enddo
            enddo                 ! ic

            ! Eliminate duplicate neighbours
            do i=1, icount-1
              if (allnew(i)/=0) then
                do j=i+1, icount
                  if (allnew(j)==allnew(i)) then
                    allnew(j)=0
                  endif
                enddo
              endif
            enddo                 ! i

            ! Compress allnew inplace (dzu)
            icomp=0
            do i=1, icount
              if (allnew(i)==0) then
                icomp=icomp+1
              elseif (icomp/=0) then
                allnew(i-icomp)=allnew(i)
              endif
            enddo
            icount=icount-icomp

            ! Sort the allnew field
            do i=1, icount-1
              do j=1, icount-i
                if (allnew(j)>allnew(j+1)) then
                  intermed=allnew(j+1)
                  allnew(j+1)=allnew(j)
                  allnew(j)=intermed
                endif
              enddo
            enddo                 ! i

            ! Setup the new tables for the next cycle
            nlano=m
            do i=1, m
              ilano(i)=iold(iloc-m+i)
            enddo
            m=icount
            ! Check if we overflow our fields
            if (m>allocsize) then
              write(*,*) 'sspbo: Overflow of the allocated fields. ',&
                   'Change levelwfactor to >', &
                   m/(allocsize/levelwfactor),' and recompile!'
              stop
            endif
            do i=1, m
              numb(i)=allnew(i)
            enddo
            deallocate(allnew)

            ! If we have numbered all (or more) of our nodes then
            ! we pass on control
            if (iglobnewlast>=ilastrow(myproc)) then
              numbexcess=iglobnewlast-ilastrow(myproc)
              msgbuf1(1)=myproc   ! procid
              msgbuf1(2)=3        ! msgtype
              msgbuf1(3)=4        ! msglen
              msgbuf1(4)=nlano
              msgbuf1(5)=iglobnewlast
              msgbuf1(6)=numbexcess
              msgbuf1(7)=m
              call MPSNDA(tids(myproc+1), 0, msgsize*iint, msgbuf1, &
                   mids, ierr)
              call MPSNDW(tids(myproc+1), 0, msgsize*iint, msgbuf1, &
                   mids, ierr)

              parsize=nlano+numbexcess+m
              allocate(par(parsize),stat=ierr)
              if (ierr>0) then
                write(lout,*) 'sspbo: (4) Sorry, par field ', &
                     'can not be allocated on proc', myproc
                stop
              endif
              ipar=0
              do i=1, nlano
                ipar=ipar+1
                par(ipar)=ilano(i)
              enddo
              do i=1, numbexcess
                ipar=ipar+1
                par(ipar)=iold(lmatbk(myproc)+i)
              enddo
              do i=1,m
                ipar=ipar+1
                par(ipar)=numb(i)
              enddo
              call MPSNDA(tids(myproc+1), 2, parsize*iint, par, &
                   mids, ierr)
              call MPSNDW(tids(myproc+1), 2, parsize*iint, par, &
                   mids, ierr)
              deallocate(par)

              active=.false.
              exit cycles
            endif               ! myproc<nproc
          enddo cycles
        else                    ! active

          ! This is the code for the passive procs.
          ! We wait in an endless loop until we either get control
          ! or the calculation is finished and answer to requests
          ! in the meantime
          do                    ! d4 (endless loop)

            ! Do an open receive
            call MPRCVA(-1, 0, msgsize*iint, msgbuf1, &
                 midr, ierr)
            call MPRCVW(-1, 0, msgsize*iint, msgbuf1, &
                 midr, ierr)

            if (msgbuf1(1) == -1) then
              ierr = 1
              iconv = 99
              if (myproc == 1) then
                write(lout,1020)
                write(lout,1025)
                write(lout,1030)
              endif
              goto 9999
            endif

            sendproc=msgbuf1(1)
            key=msgbuf1(2)
            number=msgbuf1(3)

!            write (*,*) 'sspbo: ', myproc,' received msg with key ',key,&
!                 ' from ', sendproc
            ! According to`key' we have to fulfill different requests
            msgtype: select case (key)

            case(1)
              ! Send list of conections for the requested nodes
              isum=0
              do i=1, number
                isum=isum+mncon(msgbuf1(headersize+i)-ifirstrow(myproc)+1)
              enddo
              allocate(par(isum),stat=ierr)
              if (ierr>0) then
                write(lout,*) 'sspbo: (5) Sorry, par field ', &
                     'can not be allocated on proc', myproc
                stop
              endif
              ipar=0
              do i=1, number
                numloc=msgbuf1(headersize+i)-ifirstrow(myproc)+1
                do j=1, mncon(numloc)
                  ipar=ipar+1
                  par(ipar)=mindlist(numloc,j)
                enddo
              enddo
              call MPSNDA(tids(sendproc), 1, isum*iint, par, &
                   mids, ierr)
              call MPSNDW(tids(sendproc), 1, isum*iint, par, &
                   mids, ierr)
              deallocate(par)

            case (2)
              ! Send number of connections for the requested nodes
              allocate(par(number),stat=ierr)
              if (ierr>0) then
                write(lout,*) 'sspbo: (6) Sorry, par field ', &
                     'can not be allocated on proc', myproc
                stop
              endif
              ipar=0
              do i=1, number
                ipar=ipar+1
                par(ipar)=mncon(msgbuf1(headersize+i)-ifirstrow(myproc)+1)
              enddo
              call MPSNDA(tids(sendproc), 1, number*iint, par, &
                   mids, ierr)
              call MPSNDW(tids(sendproc), 1, number*iint, par, &
                   mids, ierr)
              deallocate(par)

            case (3)
              ! We receive control
              nlano=msgbuf1(headersize+1)
              iglobnewlast=msgbuf1(headersize+2)
              numbexcess=msgbuf1(headersize+3)
              m=msgbuf1(headersize+4)
              parsize=nlano+numbexcess+m
              allocate(par(parsize),stat=ierr)
              if (ierr>0) then
                write(lout,*) 'sspbo: (7) Sorry, par field ', &
                     'can not be allocated on proc', myproc
                stop
              endif
              call MPRCVA(tids(sendproc), 2, parsize*iint, par, &
                   midr, ierr)
              call MPRCVW(tids(sendproc), 2, parsize*iint, par, &
                   midr, ierr)
              ipar=0
              do i=1, nlano
                ipar=ipar+1
                ilano(i)=par(ipar)
              enddo
              iloc=0
              do i=1, numbexcess
                ipar=ipar+1
                iloc=iloc+1
                iold(iloc)=par(ipar)
              enddo
              do i=1,m
                ipar=ipar+1
                numb(i)=par(ipar)
              enddo
              deallocate(par)

              if (numbexcess>=lmatbk(myproc)) then
                numbexcess=numbexcess-lmatbk(myproc)
                msgbuf1(1)=myproc ! procid
                msgbuf1(2)=3      ! msgtype
                msgbuf1(3)=4      ! msglen
                msgbuf1(4)=nlano
                msgbuf1(5)=iglobnewlast
                msgbuf1(6)=numbexcess
                msgbuf1(7)=m
                call MPSNDA(tids(myproc+1), 0, msgsize*iint, msgbuf1, &
                 mids, ierr)
                call MPSNDW(tids(myproc+1), 0, msgsize*iint, msgbuf1, &
                     mids, ierr)

                parsize=nlano+numbexcess+m
                allocate(par(parsize),stat=ierr)
                if (ierr>0) then
                  write(lout,*) 'sspbo: (8) Sorry, par field ', &
                       'can not be allocated on proc', myproc
                  stop
                endif
                ipar=0
                do i=1, nlano
                  ipar=ipar+1
                  par(ipar)=ilano(i)
                enddo
                do i=1, numbexcess
                  ipar=ipar+1
                  par(ipar)=iold(lmatbk(myproc)+i)
                enddo
                do i=1,m
                  ipar=ipar+1
                  par(ipar)=numb(i)
                enddo
                call MPSNDA(tids(myproc+1), 2, parsize*iint, par, &
                     mids, ierr)
                call MPSNDW(tids(myproc+1), 2, parsize*iint, par, &
                     mids, ierr)
                deallocate(par)

              else              ! numbexcess>=lmatbk(myproc)
                ! We gain control so end receive loop
                active=.true.
                cycle d1
              endif             ! numbexcess>=lmatbk(myproc)

            case (4)
              ! We are finished
              exit d1

            endselect msgtype
          enddo                 ! d4
        endif                   ! active
      enddo d1

6000  continue

      deallocate(indlist)

      ! We use the reversed best permutation
      ! A simple flip is not possible because the "opposing" processor
      ! might have less or more nodes than we have so we resort to
      ! another ring shift
      perm(1:l)=iold(1:l)

! H. Haefner 15.10.2001
! Perform resorting of triple (mat,index,info) outside
!     goto 9999             ! inserted



      allocate(permbuf(l),stat=ierr)
      if (ierr>0) then
        write(lout,*) 'sspbo: Sorry, permbuf field ', &
             'can not be allocated on proc', myproc
        stop
      endif
      do itact=1, nproc
        ibasketprocess=modulo(myproc-itact+nproc,nproc)+1
        ibasketrec=modulo(myproc-itact+nproc-1,p)+1
        if (nproc>1) then
          call MPSNDA(totid, nmsg+itact, l*iint, &
               perm, mids, ierr)
          call MPRCVA(frtid, nmsg+itact, l*iint, &
               permbuf, midr, ierr)
        endif

        ! Process passing basket
        do i=1, lmatbk(ibasketprocess)
          j=ilastrow(nproc)+1-(ifirstrow(ibasketprocess)+i-1)
          if ((j>=ifirstrow(myproc)).and.&
               (j<=ilastrow(myproc))) then
            iold(j-ifirstrow(myproc)+1)=perm(i)
          endif
        enddo                 ! i

        if (nproc>1) then
          call MPSNDW(totid, nmsg+itact, l*iint, &
               perm, mids, ierr)
          call MPRCVW(frtid, nmsg+itact, l*iint, &
               permbuf, midr, ierr)

          ! Copy the received data
          perm(1:l)=permbuf(1:l)
        endif
      enddo                   ! itact
      if (nproc>1) nmsg=nmsg+nproc
      deallocate(permbuf)
      perm(1:l)=iold(1:l)

      if ((myproc==1).and.(iverbose>0)) then
        call SECONDS(thlp1, thlp2);
        write (lout, '(a,f8.2)') &
             '   CPU-Time     for numb.   process  (in seconds):', &
             thlp1-timecpu
        write (lout, '(a,f8.2)') &
             '   Elapsed time for numb.   process  (in seconds):', &
             thlp2-timeelapsed
        timecpu=thlp1;
        timeelapsed=thlp2;
      endif

      ! Sorting of nodes for old numbers by sending
      ! iold around the processors once
      ierr2=0
      allocate(inewsort(l),stat=ierr)
      ierr2=ierr2+ierr
      allocate(ioldbuf(l),stat=ierr)
      ierr2=ierr2+ierr
      if (ierr2>0) then
        write(lout,*) 'sspbo: Sorry, inewsort and ioldbuf fields ', &
             'can not be allocated on proc', myproc
        stop
      endif

      do itact=1, nproc
        ibasketprocess=modulo(myproc-itact+nproc,nproc)+1
        ibasketrec=modulo(myproc-itact+nproc-1,p)+1
        if (nproc>1) then
          call MPSNDA(totid, nmsg+itact, l*iint, &
               iold, mids, ierr)
          call MPRCVA(frtid, nmsg+itact, l*iint, &
               ioldbuf, midr, ierr)
        endif

        ! Process passing basket
        do i=1, lmatbk(ibasketprocess)
          if ((iold(i)>=ifirstrow(myproc)).and.&
               (iold(i)<=ilastrow(myproc))) then
            inewsort(iold(i)-ifirstrow(myproc)+1)= &
                 i+ifirstrow(ibasketprocess)-1
          endif
        enddo                 ! i

        if (nproc>1) then
          call MPSNDW(totid, nmsg+itact, l*iint, &
               iold, mids, ierr)
          call MPRCVW(frtid, nmsg+itact, l*iint, &
               ioldbuf, midr, ierr)

          ! Copy the received data
          iold(1:l)=ioldbuf(1:l)
        endif
      enddo                   ! itact
      if (nproc>1) nmsg=nmsg+nproc

      deallocate(ioldbuf)

! H. Haefner 31.10.2001
! Perform resorting of triple (mat,index,info) outside
      perm(1:l)=inewsort(1:l)   ! inserted


      deallocate(mindlist)             ! was not deallocated
      deallocate(inewsort)        ! replaced
      goto 9999                        ! inserted

      ! Permute the matrix and the right hand side
      ierr2=0
      allocate(mnumbnew(l),mnconnew(l),mindlistnew(l, maxstoredeg), &
           stat=ierr)
      ierr2=ierr2+ierr
      allocate(realelnew(l,maxstoredeg), stat=ierr)
      ierr2=ierr2+ierr
      allocate(idatasend(l,(maxstoredeg+2)), rdatasend(l,(maxstoredeg+1)), &
           stat=ierr)
      ierr2=ierr2+ierr
      allocate(idatarec(l,(maxstoredeg+2)), rdatarec(l,(maxstoredeg+1)), &
           stat=ierr)
      ierr2=ierr2+ierr
      if (ierr2>0) then
        write(lout,*) 'sspbo: Sorry, mnumbnew ... fields ', &
        'can not be allocated on proc', myproc
        stop
      endif

      ! Distribute the matrix according to the new numbers
      do i=1, lmatbk(myproc)
        idatasend(i,1)=ifirstrow(myproc)+i-1
        idatasend(i,2)=info(i,3)
        do j=1, info(i,3)
          idatasend(i,j+2)=index(info(i,6)+j)
          rdatasend(i,j)=mat(info(i,2)+j)
        enddo
        rdatasend(i,maxstoredeg+1)=b(i)
      enddo

      do itact=1, nproc
        ibasketprocess=modulo(myproc-itact+nproc,nproc)+1
        ibasketrec=modulo(myproc-itact+nproc-1,p)+1
        if (nproc>1) then
          call MPSNDA(totid, nmsg+itact, l*(maxstoredeg+2)*iint, &
               idatasend, mids, ierr)
          call MPSNDA(totid, nmsg+itact, l*(maxstoredeg+1)*ireal, &
               rdatasend, mids2, ierr2)
          call MPRCVA(frtid, nmsg+itact, l*(maxstoredeg+2)*iint, &
               idatarec, midr, ierr)
          call MPRCVA(frtid, nmsg+itact, l*(maxstoredeg+1)*ireal, &
               rdatarec, midr2, ierr2)
        endif

        ! Process passing basket
        do i=1, lmatbk(myproc)
          if ((iold(i)>=ifirstrow(ibasketprocess)).and.&
               (iold(i)<=ilastrow(ibasketprocess))) then
            idx=iold(i)-ifirstrow(ibasketprocess)+1
            mnumbnew(i)=i+ifirstrow(myproc)-1
            mnconnew(i)=idatasend(idx,2)
            b(i)=rdatasend(idx, maxstoredeg+1)
            do k=1, mnconnew(i)
              mindlistnew(i,k)=idatasend(idx,k+2)
              realelnew(i,k)=rdatasend(idx,k)
            enddo
          endif
        enddo                 ! i

        if (nproc>1) then
          call MPSNDW(totid, nmsg+itact, l*(maxstoredeg+2)*iint, &
               idatasend, mids, ierr)
          call MPSNDW(totid, nmsg+itact, l*(maxstoredeg+1)*ireal, &
               rdatasend, mids2, ierr2)
          call MPRCVW(frtid, nmsg+itact, l*(maxstoredeg+2)*iint, &
               idatarec, midr, ierr)
          call MPRCVW(frtid, nmsg+itact, l*(maxstoredeg+1)*ireal, &
               rdatarec, midr2, ierr2)

          ! Copy the received data
          idatasend=idatarec
          rdatasend=rdatarec
        endif
      enddo                   ! itact
      if (nproc>1) nmsg=nmsg+nproc

      deallocate(idatarec, rdatarec)
      deallocate(idatasend, rdatasend)


      ! Now we have to renumber the index lists through another ring shift
      ! of the new numbers
      ierr2=0
      allocate(inewsortbuf(l), stat=ierr)
      ierr2=ierr2+ierr
      allocate(mindlistcorrected(l, maxstoredeg), stat=ierr)
      ierr2=ierr2+ierr
      if (ierr2>0) then
        write(lout,*) 'sspbo: Sorry, inewsortbuf ', &
        'can not be allocated on proc', myproc
        stop
      endif

      do itact=1, nproc
        ibasketprocess=modulo(myproc-itact+nproc,nproc)+1
        ibasketrec=modulo(myproc-itact+nproc-1,p)+1
        if (nproc>1) then
          call MPSNDA(totid, nmsg+itact, l*iint, &
               inewsort, mids, ierr)
          call MPRCVA(frtid, nmsg+itact, l*iint, &
               inewsortbuf, midr, ierr)
        endif

        ! Process passing basket
        do i=1, lmatbk(myproc)
          do j=1, mnconnew(i)
            if ((mindlistnew(i,j)>=ifirstrow(ibasketprocess)).and.&
                 (mindlistnew(i,j)<=ilastrow(ibasketprocess))) then
              k=mindlistnew(i,j)-ifirstrow(ibasketprocess)+1
              mindlistcorrected(i,j)=inewsort(k)
            endif
          enddo                 ! j
        enddo                   ! i

        if (nproc>1) then
          call MPSNDW(totid, nmsg+itact, l*iint, &
               inewsort, mids, ierr)
          call MPRCVW(frtid, nmsg+itact, l*iint, &
               inewsortbuf, midr, ierr)

          ! Copy the received data
          inewsort=inewsortbuf
        endif
      enddo                   ! itact
      if (nproc>1) nmsg=nmsg+nproc

      deallocate(inewsortbuf)

      ! Now we have to transform the 2d arrays into LINSOL
      ! storage scheme
      elemcount=0
      do i=1, lmatbk(myproc)
        elemcount=elemcount+mnconnew(i)
      enddo

      ! This should be done but it is impossible right now
      !lindex=elemcount
      !lmat=elemcount
      !deallocate(mat,index)
      !allocate(mat(lmat), index(lindex))

      ! Because we cannot reallocate storage we simply check and die
      ! in case...
      if ((elemcount>lmat).or.(elemcount>lindex)) then
        write(*,*) 'on processor ',myproc,' lmat=',lmat, &
             ' lindex=',lindex,' elemcount=',elemcount
        mmsin=1
      else
        mmsin=0
      endif
      call LL4INM(1, 1, 1, mmsin, mmsout, mmsbuf, &
           myproc, nproc, tids, nmsg)
      if (mmsout(1)==1) then
        write(*,*) 'sspbo: Sorry, you loose. ', &
             'Matrix storage exceeded during permutation!'
        stop
      endif

      idx=0
      do i=1, lmatbk(myproc)
        info(i,2)=idx
        info(i,3)=mnconnew(i)
        info(i,5)=mnumbnew(i)-1 ! These numbers are 0-based
        info(i,6)=idx
        do j=1, mnconnew(i)
          mat(idx+j)=realelnew(i,j)
          index(idx+j)=mindlistcorrected(i,j)
        enddo
        idx=idx+mnconnew(i)
      enddo

      deallocate(mindlistcorrected)
      deallocate(realelnew)
      deallocate(mnumbnew,mnconnew,mindlistnew)

      deallocate(inewsort)

      allocate(lix(ilastrow(nproc)), matbuf(ilastrow(nproc)), stat=ierr)
      if (ierr>0) then
        write(lout,*) 'sspbo: Sorry, lix and matbuf fields ', &
        'can not be allocated on proc', myproc
        stop
      endif
      ! Sort the indices because LINSOL needs this for
      ! the subsequent processing
      do i=1,lmatbk(myproc)
        idx  = info(i,2)
        k    = info(i,3)
        call HEAPSORTIX(index(info(i,6)+1),lix,k,.true.)
        do j=1,k
          matbuf(j) = mat(idx+lix(j))
        enddo
        do j=1,k
          mat(idx+j) = matbuf(j)
        enddo
      enddo
      deallocate(lix, matbuf)

      if ((myproc==1).and.(iverbose>0)) then
        call SECONDS(thlp1, thlp2);
        write (lout, '(a,f8.2)') &
             '   CPU-Time     for permut. process  (in seconds):', &
             thlp1-timecpu
        write (lout, '(a,f8.2)') &
             '   Elapsed time for permut. process  (in seconds):', &
             thlp2-timeelapsed
        timecpu=thlp1;
        timeelapsed=thlp2;
        write (*,*) 'SSPBO finished'
      endif

      ! Copy back the changed global variables
      ilin(33) = elemcount      ! endmat
      ilin(34) = elemcount      ! endind

 9999 return

    end subroutine SSPBOPTIM

end module sspbo
