!:::::      ,,,,,STORAGEOPT.....
!
!
!
!**********************************************************************
!        1         2         3         4         5         6         7*
!**********************************************************************
!**                                                                 ***
!**                                                                 ***
      subroutine STORAGEOPT(mat,lmat,index,lindex,info,nvt,ia1,l,lsym,
     &                      ilin,lmatbk,ptrmbk,myproc,nproc,ierr)
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**     S T O R A G E O P T    :  to optim the storage patterns     ***
!**                      from "packed row" to "packed diagonal"     ***
!**                      or "full diagonal" if demands              ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**    Copyright (C) 2002 University Karlsruhe                      ***
!**    PROGRAMMER: Xiangming Quan                                   ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**     THE IMPLICIT NONE-INSTRUCTION HAS THE AIM,                  ***
!**     THAT ALL VARIABLES MUST BE DECLARED EXPLICITLY.             ***
!**                                                                 ***
      USE constants

      implicit none
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**      FORMAL PARAMETERS :                                        ***
!**      -----------------                                          ***
!**                                                                 ***
      integer, intent(in) ::  ia1,l,lmat,lindex,nproc,myproc
      integer, intent(in) ::  ptrmbk(nproc+1),lmatbk(nproc)
      integer, intent(inout) :: nvt
      integer, intent(inout) :: info(ia1,ia2),ilin(nilin),index(lindex)
      integer, intent(out) ::  ierr
      double precision, intent(inout) ::  mat(lmat)
      logical, intent(in) :: lsym
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**   LIST OF IMPORTANT PARAMETERS :                                ***
!**   ------------------------------                                ***
!**                                                                 ***
!--------I------I-----I------------------------------------------------
! NAME   I TYPE I I/O I     MEANING
!--------I------I-----I------------------------------------------------
!--------I------I-----I------------------------------------------------
! mat    I  DP  I i/o I  mat is a one dimentional array which stores
!        I      I     I  the input matrix in different storage patterns
!--------I------I-----I------------------------------------------------
! index  I   I  I i/o I  stores the index vectors
!--------I------I-----I------------------------------------------------
!**                                                                 ***
!**                                                                 ***
!**      LOCAL PARAMETERS :                                         ***
!**      -----------------                                          ***
!**                                                                 ***

      integer, allocatable :: diagel(:), diagbgn(:)
      integer, allocatable :: diagend(:), diagcrf(:), diagcrp(:)
      double precision :: mflops, mflopsfd, mflopspd
      double precision, parameter :: accfac = 0.9, zero = 0.0
      integer lout,itemp,ibwavg
      integer lvtsum, lvtavg,i,diagbwavg

      lout = ilin(12)

      allocate(diagel(2*l-1),diagbgn(2*l-1),diagend(2*l-1))
      allocate(diagcrf(2*l-1),diagcrp(2*l-1))

      call DIAGCOMP(index,lindex,info,ia1,nvt,l,diagel,diagbgn,
     &              diagend,diagbwavg)
      call BWAVG(index,lindex,info,ia1,nvt,ibwavg)

      lvtsum = 0
      do i=1,nvt
	  lvtsum = lvtsum + info(i,3)
      enddo
      lvtavg = NINT(REAL(lvtsum)/REAL(nvt))

      call BMPROW(lvtavg,ibwavg,ilin,mflops,ierr)

      do i = 1,2*l-1
        diagcrf(i) = 0
        diagcrp(i) = 0
      enddo

      do i=1,2*l-1
        if (diagel(i) > 0) then
	  itemp = diagend(i) - diagbgn(i) + 1
          call BMFDIAG(diagel(i),itemp,ilin,mflopsfd,ierr)
	  if (i /= l) then
            call BMPDIAG(diagel(i),itemp,ilin,mflopspd,ierr)
	  else
	    mflopspd = zero
	  endif

	  if (mflopsfd>mflops*accfac .and.
     &        mflopspd <= mflops*accfac) diagcrf(i) = 1
	  if (mflopsfd<=mflops*accfac .and.
     &        mflopspd > mflops*accfac) diagcrp(i) = 1
	  if (mflopsfd>mflops*accfac .and.
     &        mflopspd>mflops*accfac) then
	    if (mflopsfd>mflopspd) then            
	      diagcrf(i) = 1
	    else
	      diagcrp(i) = 1
	    endif
	  endif
	endif
      enddo

      print *,' Following full diags will be created:',diagcrf
      print *,' Following packed diags will be created:',diagcrp

!***  Packed rows --> full or packed diagonals
      call PROW2DIAG(mat,lmat,index,lindex,info,ia1,nvt,ilin,l,
     &               diagcrf,diagcrp,diagbgn,diagend,
     &               diagel)

      ilin(5) = nvt

      ierr = 0
      call SHOWVT(MAT,lmat,index,lindex,info,ia1,nvt,l,lsym,
     &            myproc,nproc,lmatbk,lout,ierr)

!     print *,' Diagonalenbelegung: '
!     do i=1,2*l-1
!       print *,i,diagel(i)
!     enddo

      return
      end
