!:::::      ,,,,,SYMMAT....
!
!
!
!**********************************************************************
!        1         2         3         4         5         6         7*
!**********************************************************************
!**                                                                 ***
!**                                                                 ***
      subroutine SYMMAT(mat,lmat,index,lindex,info,ia1,l,lglobal,ilin,
     &                  ptrmbk,lmatbk,tids,nproc,iconv,ierr)
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**      program description of SYMMAT                              ***
!**                                                                 ***
!**      Mirrors the elements of the matrix below the main diagonal ***
!**      into the upper half.                                       ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**    Copyright (C) 2001 University Karlsruhe                      ***
!**    PROGRAMMER : Hartmut Haefner                                 ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**   the implicit none-instruction has the aim,                    ***
!**   that all variables must be declared explicitly.               ***
!**                                                                 ***
      USE constants

      implicit none
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**      FORMAL PARAMETERS :                                        ***
!**      -----------------                                          ***
!**                                                                 ***
      integer           ia1,iconv,ierr,lindex,lmat
      integer           nproc,l,lglobal,ptrmbk(nproc+1),ilin(nilin)
      integer           info(ia1,ia2),index(lindex)
      integer           tids(nproc),lmatbk(nproc)
      double precision  mat(lmat)
!**********************************************************************
!**                                                                 ***
!**    List of Formal Parameters :                                  ***
!**                                                                 ***
!**                                                                 ***
!---------I------I-----I-----------------------------------------------
! NAME    I TYPE I I/O I MEANING
!---------I------I-----I-----------------------------------------------
! info    I int  I     I array with matrix-structure-information
!---------I------I-----I-----------------------------------------------
! index   I int  I     I array with indices
!---------I------I-----I-----------------------------------------------
!**                                                                 ***
!**                                                                 ***
!**        LOCAL PARAMETERS :                                       ***
!**        ----------------                                         ***
!**                                                                 ***
      integer           i,j,jj,jjoff,jloc,k,kk,lvt,iermax,ixptr
      integer           ierr1,ierr2,ierr3,ierr4
      integer           hixsiz(nproc)
      integer           hixlen(l,nproc-1),hixmlen(l)
      integer           hixmax,endmat,endind,offsetmat,offsetix,offset
      integer           myproc,frtid,totid,ihlp(12),nmsg,lout
      integer           firstindex,lastindex,mids,midr
      integer           joff(l),ixh(lglobal)
      double precision  math(lglobal)
!     double precision  t1,ted,t2
      integer,allocatable,dimension(:,:,:)::  hix
      integer,allocatable,dimension(:,:)::  hixm
      double precision,allocatable,dimension(:,:,:)::  rhix
      double precision,allocatable,dimension(:,:)::  rhixm

!**********************************************************************
!**                                                                 ***
!**    List of Local Variables :                                    ***
!**                                                                 ***
!**                                                                 ***
!---------I------I-----I-----------------------------------------------
! NAME    I TYPE I I/O I MEANING
!---------I------I-----I-----------------------------------------------
! rhixm   I  dp  I --- I array with matrix elements
!         I      I     I array:  rhixm(hixsiz(myproc),lmatbk(myproc))
!         I      I     I
!         I      I     I rhixm(1:hixsiz(myproc),j) = matrix elements of
!         I      I     I   column j in main diagonal block
! hixm    I int  I --- I array with indices
!         I      I     I array:  hixm(hixsiz(myproc),lmatbk(myproc))
!         I      I     I
!         I      I     I hixm(1:hixsiz(myproc),j) = column indices of
!         I      I     I   column j in main diagonal block
! hixmlen I int  I --- I length of column vectors of array hixm (rhixm)
!         I      I     I   array:  hixmlen(l)
!         I      I     I
! rhix    I  dp  I --- I array with matrix elements
!         I      I     I array:  hix(hixmax,l,nproc-1)
!         I      I     I
!         I      I     I rhix(1:hixsiz(i),j,i) = matrix elements of
!         I      I     I   column j in block i (no main diagonal block)
! hix     I int  I --- I array with indices
!         I      I     I array:  hix(hixmax,l,nproc-1)
!         I      I     I
!         I      I     I  hix(1:hixsiz(i),j,i) = column indices of
!         I      I     I   column j in block i (no main diagonal block)
! hixlen  I int  I --- I length of column vectors of array hix (rhix)
!         I      I     I   array:  hixlen(l,nproc-1)
!---------I------I-----I-----------------------------------------------
!**                                                                 ***
!**        CONSTANTS:                                               ***
!**        ---------                                                ***
!**                                                                 ***

      nmsg=ilin(3)
      lout=ilin(12)
      myproc=ilin(17)

      endmat=ilin(33)
      endind=ilin(34)

      offsetmat = lmat - endmat
      offsetix = lindex - endind

      if (offsetmat > 0) then
	do i=endmat,1,-1
	  mat(i+offsetmat) = mat(i)
	enddo
	do i=1,lmatbk(myproc)
	  info(i,2) = info(i,2) + offsetmat
	enddo
      endif
      if (offsetix > 0) then
	do i=endind,1,-1
	  index(i+offsetix) = index(i)
	enddo
	do i=1,lmatbk(myproc)
	  info(i,6) = info(i,6) + offsetix
	enddo
      endif

      hixsiz = 0
!**                                                                 ***
!**   determine, send and receive hixsiz(1:nproc)                   ***
!**                                                                 ***
!     call SECONDS(t1,ted)
      if (nproc == 1) then
	hixmlen = 0
	do k=1,lmatbk(1)  ! for all rows
	  lvt = info(k,3)
	  do jj=1,lvt
	    j = index(info(k,6)+jj)
	    if (k > j) hixmlen(j) = hixmlen(j) + 1
	  enddo
	enddo
	hixsiz(1) = 0
	do j=1,lmatbk(1)
	  if (hixmlen(j) > hixsiz(1)) hixsiz(1) = hixmlen(j)
	enddo
      else

!**                                                                 ***
!**       determination of hixsiz(1:nproc)                          ***
!**                                                                 ***
!**       count the number of elements rowwise                      ***

        joff = 1
        do i=1,nproc
          if (i < myproc) then
	    hixlen = 0
            do k=1,lmatbk(myproc)
	      lvt = info(k,3)
	      jjoff = joff(k)
              do jj=jjoff,lvt
		j = index(info(k,6)+jj)
		jloc = j - ptrmbk(i)
                if (j > ptrmbk(i+1)) exit
	        hixlen(jloc,i) = hixlen(jloc,i) + 1
	        joff(k) = joff(k) + 1
              enddo
            enddo
	    hixsiz(i) = 0
	    do j=1,lmatbk(i)
	      if (hixlen(j,i) > hixsiz(i)) hixsiz(i) = hixlen(j,i)
	    enddo
          endif

          if (i == myproc) then
	    hixmlen = 0
            do k=1,lmatbk(myproc)
	      lvt = info(k,3)
	      jjoff = joff(k)
              do jj=jjoff,lvt
		j = index(info(k,6)+jj)
		jloc = j - ptrmbk(i)
                if (j > ptrmbk(i+1)) exit
                if (ptrmbk(myproc)+k <= j) then
		  joff(k) = joff(k) + 1
		else
		  hixmlen(jloc) = hixmlen(jloc) + 1
		  joff(k) = joff(k) + 1
		endif
              enddo
            enddo
	    hixsiz(i) = 0
	    do j=1,lmatbk(i)
	      if (hixmlen(j) > hixsiz(i)) hixsiz(i) = hixmlen(j)
	    enddo
          endif
        enddo

!**   send (receive) hixsiz(i) to (from) processor i                ***

        do i=1,nproc
          if (i < myproc) then
            totid=tids(i)
            call MPSNDA(totid,nmsg+myproc,iint,hixsiz(i),mids,ierr)
            call MPSNDW(totid,nmsg+myproc,iint,hixsiz(i),mids,ierr)
          else
            frtid=tids(i)
            call MPRCVA(frtid,nmsg+i,iint,hixsiz(i),midr,ierr)
            call MPRCVW(frtid,nmsg+i,iint,hixsiz(i),midr,ierr)
          endif
        enddo
        nmsg=nmsg+nproc+1

!**   determine the maximum of hixsiz(1:nproc)                      ***

        hixmax = 0
        do i=1,nproc
          if ((i /= myproc) .and. (hixsiz(i) > hixmax)) then
            hixmax = hixsiz(i)
          endif
        enddo

      endif      ! if (nproc == 1)


!     call SECONDS(t2,ted)
!     print*,' Determine array sizes:',t2-t1

      ierr = 0; ierr1 = 0; ierr2 = 0; ierr3 = 0; ierr4 = 0

!     call SECONDS(t1,ted)
      allocate(hixm(hixsiz(myproc),lmatbk(myproc)),stat=ierr1)
      allocate(rhixm(hixsiz(myproc),lmatbk(myproc)),stat=ierr2)
      if (nproc > 1) then
	allocate(hix(hixmax,l,nproc-1),stat=ierr3)
	allocate(rhix(hixmax,l,nproc-1),stat=ierr4)
      endif

      if (ierr1 /= 0) ierr1 = hixsiz(myproc)*lmatbk(myproc)
      ihlp(1) = ierr1
      if (ierr2 /= 0) ierr2 = hixsiz(myproc)*lmatbk(myproc)
      ihlp(2) = ierr2
      if (ierr3 /= 0) ierr3 = hixmax*l*(nproc-1)
      ihlp(3) = ierr3
      if (ierr4 /= 0) ierr4 = hixmax*l*(nproc-1)
      ihlp(4) = ierr4
      call LL4INM(1,4,1,ihlp(1),ihlp(5),ihlp(9),myproc,nproc,tids,nmsg)
      if (ihlp(5) /= 0 .or. ihlp(6) /= 0 .or. ihlp(7) /= 0 .or.
     &    ihlp(8) /= 0) then
        if (ihlp(5) /= 0) then
          ierr = ihlp(5)
          iconv = 99
          if (myproc == 1) then
            write(lout,1020)
            write(lout,1022) ierr
            write(lout,1030)
          endif
	endif
        if (ihlp(6) /= 0) then
          ierr = ihlp(6)
          iconv = 99
          if (myproc == 1) then
            write(lout,1020)
            write(lout,1022) ierr
            write(lout,1030)
          endif
	endif
        if (ihlp(7) /= 0) then
          ierr = ihlp(7)
          iconv = 99
          if (myproc == 1) then
            write(lout,1020)
            write(lout,1022) ierr
            write(lout,1030)
          endif
	endif
        if (ihlp(8) /= 0) then
          ierr = ihlp(8)
          iconv = 99
          if (myproc == 1) then
            write(lout,1020)
            write(lout,1022) ierr
            write(lout,1030)
          endif
	endif
      endif
!     call SECONDS(t2,ted)
!     print*,' Allocation of arrays:',t2-t1

!**                                                                 ***
!**       columnwise set up of hix                                  ***
!**                                                                 ***
!**       IMPORTANT: indices must be sorted .and.                   ***
!**                  rows must be sorted!                           ***
!**                                                                 ***

!     call SECONDS(t1,ted)

      joff = 1
      do i=1,nproc

        if (i < myproc) then
	  hixlen = 0
          do k=1,lmatbk(myproc)
	    lvt = info(k,3)
	    jjoff = joff(k)
            do jj=jjoff,lvt
	      j = index(info(k,6)+jj)
              jloc = j - ptrmbk(i)
              if (j > ptrmbk(i+1)) exit
	      hixlen(jloc,i) = hixlen(jloc,i) + 1
	      joff(k) = joff(k) + 1
	      hix(hixlen(jloc,i),jloc,i) = ptrmbk(myproc) + k
	      rhix(hixlen(jloc,i),jloc,i) = mat(info(k,2)+jj)
            enddo
          enddo
        endif
        if (i == myproc) then
	  hixmlen = 0
          do k=1,lmatbk(myproc)
	    lvt = info(k,3)
	    jjoff = joff(k)
            do jj=jjoff,lvt
	      j = index(info(k,6)+jj)
              jloc = j - ptrmbk(i)
              if (j > ptrmbk(i+1)) exit
              if (ptrmbk(myproc)+k <= j) then
	        joff(k) = joff(k) + 1
	      else
	        hixmlen(jloc) = hixmlen(jloc) + 1
	        joff(k) = joff(k) + 1
		hixm(hixmlen(jloc),jloc) = ptrmbk(myproc) + k
	        rhixm(hixmlen(jloc),jloc) = mat(info(k,2)+jj)
	      endif
            enddo
          enddo
        endif

!**                                                                 ***
!**       Send hix(:,:,i) to processor i and receive hix(:,:,i)     ***
!**       from processor i                                          ***
!**                                                                 ***

        if (i < myproc) then
          if (hixsiz(i) > 0) then
            totid=tids(i)
            call MPSNDA(totid,nmsg+myproc+1,l*iint,hixlen(1,i),
     &                  mids,ierr)
            call MPSNDW(totid,nmsg+myproc+1,l*iint,hixlen(1,i),
     &                  mids,ierr)
            call MPSNDA(totid,nmsg+myproc+2,hixmax*l*iint,
     &                  hix(1,1,i),mids,ierr)
            call MPSNDW(totid,nmsg+myproc+2,hixmax*l*iint,
     &                  hix(1,1,i),mids,ierr)
            call MPSNDA(totid,nmsg+myproc+3,hixmax*l*ireal,
     &                  rhix(1,1,i),mids,ierr)
            call MPSNDW(totid,nmsg+myproc+3,hixmax*l*ireal,
     &                  rhix(1,1,i),mids,ierr)
          endif
        endif
        if (i > myproc) then
          if (hixsiz(i)>0) then
            frtid=tids(i)
            call MPRCVA(frtid,nmsg+i+1,l*iint,hixlen(1,i-1),midr,ierr)
            call MPRCVW(frtid,nmsg+i+1,l*iint,hixlen(1,i-1),midr,ierr)
            call MPRCVA(frtid,nmsg+i+2,hixmax*l*iint,
     &                  hix(1,1,i-1),midr,ierr)
            call MPRCVW(frtid,nmsg+i+2,hixmax*l*iint,
     &                  hix(1,1,i-1),midr,ierr)
            call MPRCVA(frtid,nmsg+i+3,hixmax*l*ireal,
     &                  rhix(1,1,i-1),midr,ierr)
            call MPRCVW(frtid,nmsg+i+3,hixmax*l*ireal,
     &                  rhix(1,1,i-1),midr,ierr)
	  endif
        endif
      enddo

      nmsg=nmsg+nproc+3
!     call SECONDS(t2,ted)
!     print*,' Compute array hix:',t2-t1

      if (nproc > 1) then
        do i=myproc,nproc-1
	  hixsiz(i) = hixsiz(i+1)
        enddo
      endif

!**                                                                 ***
!**   rowwise sorting of index and hix into graph                   ***
!**                                                                 ***
!     call SECONDS(t1,ted)

      ixptr = 0
      offset = min(offsetmat,offsetix)

!***  for all rows!
      do i=1,lmatbk(myproc)
        ixh = 0
        math = 0
	kk = 1
        do k=1,info(i,3)
          if (index(info(i,6)+k) < i) kk = kk + 1
          if (index(info(i,6)+k) >= i) then
	    ixh(index(info(i,6)+k)) = index(info(i,6)+k)
	    math(index(info(i,6)+k)) = mat(info(i,2)+k)
          endif
        enddo

	offset = offset + info(i,3)

        do k=1,hixmlen(i)
          ixh(hixm(k,i)) = hixm(k,i)
          math(hixm(k,i)) = math(hixm(k,i)) + rhixm(k,i)
        enddo

!***  for column blocks > myproc!
        do j=myproc,nproc-1
	 if (hixsiz(j) > 0) then
	  do k=1,hixlen(i,j)
	    ixh(hix(k,i,j)) = hix(k,i,j)
	    math(hix(k,i,j)) = math(hix(k,i,j)) + rhix(k,i,j)
	  enddo
	 endif
        enddo

        firstindex = lglobal + 1
	lastindex  = 0
	if (info(i,3) > 0) then
          firstindex = min(firstindex,index(info(i,6)+kk))
          lastindex  = max(lastindex,index(info(i,6)+info(i,3)))
	endif
	if (hixmlen(i) > 0) then
          firstindex = min(firstindex,hixm(1,i))
          lastindex  = max(lastindex,hixm(hixmlen(i),i))
	endif
        do j=myproc,nproc-1
	 if (hixlen(i,j) > 0) then
          firstindex = min(firstindex,hix(1,i,j))
          lastindex = max(lastindex,hix(hixlen(i,j),i,j))
         endif
        enddo

        lvt = 0
	do k=firstindex,lastindex    ! originally k=1,lglobal
	  if (ixh(k) /= 0) then
	    lvt = lvt + 1
	    if (ixptr+lvt > offset) then
	      ierr = 8888
	      goto 999
	    endif
	    index(ixptr+lvt) = ixh(k)
	    mat(ixptr+lvt) = math(k)
	  endif
	enddo
	info(i,2) = ixptr
	info(i,3) = lvt
	info(i,4) = 0
	info(i,5) = ptrmbk(myproc) + i - 1
	info(i,6) = ixptr
	info(i,7) = 0
	ixptr = ixptr + lvt
      enddo

!     call SECONDS(t2,ted)
!     print*,' Merge sort:',t2-t1

      deallocate(hixm,rhixm)
      if (nproc > 1) deallocate(hix,rhix)

  999 call LL4INM(1,1,1,ierr,iermax,ihlp,myproc,nproc,tids,nmsg)
      if (iermax /= 0) then
        ierr = iermax
        if (ierr == 8888) then
          iconv = 99
          if (myproc == 1) then
            write(lout,1020)
            write(lout,1021)
            write(lout,1030)
          endif
	endif
      endif

      ilin(3)  = nmsg

      ilin(33) = ixptr
      ilin(34) = ixptr
      ilin(35) = lmatbk(myproc)
      call LL4ISM(1,3,1,ilin(33),ilin(30),ihlp(1),myproc,nproc,
     &            tids,nmsg)

      ilin(5) = lmatbk(myproc)

 1020 format(' **************************************************'/
     &       ' ***  ERROR occured in SYMMAT:                  ***')
 1021 format(' ***   enlarge matrix and (or) index array!     ***')
 1022 format(' ***   allocation of one of four auxiliary      ***'/
     &       ' ***   arrays failed!                           ***'/
     &       ' ***   The allocation size of (maximum)         ***'/
     &       ' ***   ',i10,' Bytes is too high!            ***')
 1030 format(' **************************************************')
      r e t u r n
!-----END OF SYMMAT-----------------------------------------------------
      e    n    d
