!**********************************************************************
!        1         2         3         4         5         6         7*
!**********************************************************************
!**                                                                 ***
!**                                                                 ***
      subroutine THRES1(mat,lmat,info,ia1,ilin,rlin,thrshld1,nvt,
     &                  myproc,nproc,tids,nmsg)
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**   T H R E S 1   computes the the first threshold value          ***
!**                 for the incomplete Gauss algorithm.             ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**    Copyright (C) 1999 University Karlsruhe                      ***
!**    PROGRAMMER: H. Haefner                                       ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**     THE IMPLICIT NONE-INSTRUCTION HAS THE AIM,                  ***
!**     THAT ALL VARIABLES MUST BE DECLARED EXPLICITLY.             ***
!**                                                                 ***
      USE constants

      implicit none
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**      FORMAL PARAMETERS :                                        ***
!**      -----------------                                          ***
!**                                                                 ***
      integer,intent(in) ::  ia1,nproc,myproc,lmat,nvt
      integer,intent(inout) ::  nmsg
      integer,intent(in) ::  info(ia1,ia2),ilin(nilin),tids(nproc)
      double precision,intent(in) ::  mat(lmat),rlin(nrlin)
      double precision,intent(out)::  thrshld1(nvt)
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**   LIST OF FORMAL PARAMETERS :                                   ***
!**   -------------------------                                     ***
!**                                                                 ***
!**   (see Manual-Pages <lsolp>)                                    ***
!**                                                                 ***
!**                                                                 ***
!**      LOCAL PARAMETERS :                                         ***
!**      -----------------                                          ***
!**                                                                 ***
      integer    count,i,icnt,ihlp,k,gcount,gendmat,lout,idoku
      double precision  thrshlp,thrscomp
      double precision  maxval(nvt)
      double precision, parameter ::  zero = 0., one = 1., errfac = 0.1
      double precision, parameter ::  eps = 1.d-5

!
!**** START OF CALCULATION :
!     ---------------------
!
      icnt    = 1

      lout    = ilin(12)
      idoku   = ilin(13)
      gendmat = ilin(30)

!
!***  This routine only works if holds: nvt = lmyblk
!
      maxval = zero
      do k=1,nvt
	do i=1,info(k,3)
	  if (abs(mat(info(k,2)+i)) > maxval(k))
     &       maxval(k) = abs(mat(info(k,2)+i))
        enddo
      enddo

      thrshlp = rlin(2)
      if (thrshlp < zero .or. thrshlp > one) then
        if (myproc == 1) write(lout,1020)
	thrshlp = zero
      endif

      do k=1,nvt
        thrshld1(k) = maxval(k)*thrshlp
      enddo

  111 count = 0
      do k=1,nvt
	do i=1,info(k,3)
	  if (abs(mat(info(k,2)+i)) <= thrshld1(k))
     &      count = count + 1
        enddo
      enddo

      call LL4ISM(1,1,1,count,gcount,ihlp,myproc,nproc,tids,nmsg)
      thrscomp = DBLE(gcount)/DBLE(gendmat)
      if (thrscomp < eps) thrscomp = eps

      if ((thrscomp > thrshlp*(1-errfac)) .and. (thrscomp <
     &     thrshlp*(1+errfac)) .or. icnt > 2) then
        if ((idoku > 0) .and. (myproc == 1)) then
          write(lout,1000)
          write(lout,1001) rlin(2)
          write(lout,1002) 100.0-thrscomp*100.0
          write(lout,1010)
	endif
        if (idoku < 0) then
          write(lout,1000)
          write(lout,1001) rlin(2)
          write(lout,1002) 100.0-thrscomp*100.0
          write(lout,1010)
	endif
      else
	do k=1,nvt
	  thrshld1(k) = thrshld1(k)*(thrshlp/thrscomp)
	enddo
	icnt = icnt + 1
	goto 111
      endif

 1000 format(/'   -------------------------------------------------- '/
     &       '  ||  Computation of 1st threshold value in THRES1: ||')
 1001 format('  ||    LINSOL_GAUSS_THRES1 (rlin(2)) . :',f10.5,' ||')
 1002 format('  ||    percentage of entries of A                  ||'/
     &       '  ||    to be copied to A(P)  . . . . . :',f7.2,'%   ||')
 1010 format('   -------------------------------------------------- '/)
 1020 format('  WARNING: environment variable LINSOL_GAUSS_THRES1 is'/
     &       '           set to a value smaller than zero or greater'/
     &       '           than one --> the first threshold value is set'/       
     &       '           set to zero, i.e. whole matrix will be copied'/
     &       '           into the preconditioning matrix! '/)

      R E T U R N
!------END OF THRES1------------------------------------------------
      E N D                                   
