!**********************************************************************
!        1         2         3         4         5         6         7*
!**********************************************************************
!**                                                                 ***
!**                                                                 ***
      subroutine THRES2(mat,lmat,infoLU,ia1LU,ilin,rlin,nvtLU,
     &                  myproc,nproc,tids,nmsg)
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**   T H R E S 2   computes the the second threshold value         ***
!**                 for the incomplete Gaussian algorithm.          ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**    Copyright (C) 1999 University Karlsruhe                      ***
!**    PROGRAMMER: H. Haefner                                       ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**     THE IMPLICIT NONE-INSTRUCTION HAS THE AIM,                  ***
!**     THAT ALL VARIABLES MUST BE DECLARED EXPLICITLY.             ***
!**                                                                 ***
      USE constants

      implicit none
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**      FORMAL PARAMETERS :                                        ***
!**      -----------------                                          ***
!**                                                                 ***
      integer,intent(in) ::  ia1LU,nproc,nvtLU,myproc,lmat
      integer,intent(inout) ::  nmsg
      integer,intent(in) ::  infoLU(ia1LU,ia2LU),ilin(nilin)
      integer,intent(in) ::  tids(nproc)
      double precision,intent(in) ::  mat(lmat)
      double precision,intent(inout)::  rlin(nrlin)
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**   LIST OF FORMAL PARAMETERS :                                   ***
!**   -------------------------                                     ***
!**                                                                 ***
!**   (see Manual-Pages <lsolp>)                                    ***
!**                                                                 ***
!**                                                                 ***
!**      LOCAL PARAMETERS :                                         ***
!**      -----------------                                          ***
!**                                                                 ***
      integer    i,k,lout,idoku
      double precision  thrshlp
      double precision  maxval,gmaxval,rhlp,thrshld2
      double precision, parameter ::  zero = 0.0, one = 1.0

!
!**** START OF CALCULATION :
!     ---------------------
!
      lout   = ilin(12)
      idoku  = ilin(13)

      thrshld2 = rlin(3)

!
!***  This routine only works if holds: nvtLU = l
!
      maxval = zero
      do k=1,nvtLU
        do i=1,infoLU(k,2)
	  if (abs(mat(infoLU(k,1)+i)) > maxval)
     &      maxval = abs(mat(infoLU(k,1)+i))
        enddo
      enddo

      call LL4RNM(1,1,1,maxval,gmaxval,rhlp,myproc,nproc,tids,nmsg)

      thrshlp = rlin(3)
      if (thrshlp < zero .or. thrshlp > one) then
        if (myproc == 1) write(lout,1020)
	thrshlp = zero
      endif

      thrshld2 = gmaxval*thrshlp
      if ((idoku > 0) .and. (myproc == 1)) then
        write(lout,1000)
        write(lout,1001) rlin(3)
        write(lout,1002) thrshld2
        write(lout,1010)
      endif
      if (idoku < 0) then
        write(lout,1000)
        write(lout,1001) rlin(3)
        write(lout,1002) thrshld2
        write(lout,1010)
      endif

      rlin(3) = thrshld2

 1000 format(/'   -------------------------------------------------- '/
     &       '  ||  Computation of 2nd threshold value in THRES2: ||')
 1001 format('  ||    LINSOL_GAUSS_THRES2 (rlin(3)) . :',f10.7,' ||')
 1002 format('  ||    threshold value for dropping                ||'/
     &       '  ||    of computed elements during                 ||'/
     &       '  ||    elimination process . . . . . . :',f10.7,' ||')
 1010 format('   -------------------------------------------------- '/)
 1020 format('  Warning: <rlin(2)> or the environment variable'/
     &       '           LINSOL_GAUSS_THRES2 is set to a value'/
     &       '           smaller than zero or greater than one'/
     &       '           --> the second threshold value is'/
     &       '           set to zero, i.e. full fill in! '/)

      R E T U R N
!------END OF THRES2------------------------------------------------
      E N D                                   
