!**********************************************************************
!        1         2         3         4         5         6         7*
!**********************************************************************
!**                                                                 ***
!**                                                                 ***
      subroutine ULnorm(UL,lUL,ixUL,lixUL,infoUL,ia1UL,iw,
     &                  nvtUL,ULtyp,ilin,myproc,nproc,tids,nmsg)
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**   U L N O R M   computes the euclidian and maximum norm of      ***
!**                 the factorized matrices U and L.                ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**    Copyright (C) 2001 University Karlsruhe                      ***
!**    PROGRAMMER: H. Haefner                                       ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                 ***
!**********************************************************************

      USE constants

      implicit none

      integer, intent(in) :: lUL,lixUL,ia1UL,myproc,nproc,nvtUL
      integer, intent(in) :: ilin(nilin)
      integer, intent(inout) :: iw(3*nvtUL),nmsg
      integer, intent(in) :: ixUL(lixUL),infoUL(ia1UL,ia2LU),tids(nproc)
      double precision, intent(in) :: UL(lUL)
      character*1, intent(in) :: ULtyp
 
      integer                    addUL,lvtUL,iacUL,cntmdU,cntcolUL,lenUL
      integer                    gcntmdU,gcntcolUL
      integer                    i,idoku,k,lglobal,lout
      real                       pcntmdU,pcntcolUL
      double precision           maxUL,euclUL,minU,gmaxUL,geuclUL,gminU
      double precision           rhlp
      double precision, parameter ::  zero = 0.0, inf = 9.9D99

      lout    = ilin(12)
      idoku   = ilin(13)
      lglobal = ilin(36)

      maxUL  = zero
      euclUL = zero
      lenUL  = 0

      do i=1,nvtUL
	addUL  = infoUL(i,1)
	lvtUL  = infoUL(i,2)
	lenUL  = lenUL + lvtUL
	do k=1,lvtUL
	  if (abs(UL(addUL+k)) > maxUL) maxUL = abs(UL(addUL+k))
	  euclUL = euclUL + UL(addUL+k)*UL(addUL+k)
        enddo
      enddo

!*** the indices of the index vectors of U must increase monotonically

      if (ULtyp == 'U') then
        cntmdU = 0
	minU  = inf
        do i=1,nvtUL
	  addUL  = infoUL(i,1)
	  lvtUL  = infoUL(i,2)
	  iacUL  = infoUL(i,3) + 1
	  if ((lvtUL > 0) .and. (ixUL(addUL+1) == iacUL)) then
	    if (abs(UL(addUL+1)) < minU) minU = abs(UL(addUL+1)) 
            cntmdU = cntmdU + 1
          endif
        enddo
      endif

      if (nproc > 1) then
	if (nvtUL == lglobal) then
          do i=1,nvtUL
	    iw(i)  = infoUL(i,2)
          enddo
          call LL4INM(1,nvtUL,1,iw,iw(nvtUL+1),iw(2*nvtUL+1),
     &                myproc,nproc,tids,nmsg)
          gcntcolUL = 0
          do i=nvtUL+1,2*nvtUL
	    if (iw(i) > 0) gcntcolUL = gcntcolUL + 1
          enddo
        else
	  cntcolUL = 0
          do i=1,nvtUL
	    if (infoUL(i,2) > 0) cntcolUL = cntcolUL + 1
          enddo
	  call LL4ISM(1,1,1,cntcolUL,gcntcolUL,iw(1),
     &                myproc,nproc,tids,nmsg)
        endif
      else
	cntcolUL = 0
        do i=1,nvtUL
	  if (infoUL(i,2) > 0) cntcolUL = cntcolUL + 1
        enddo
      endif

      if (nproc > 1) then
        call LL4RNM(1,1,1,maxUL,gmaxUL,rhlp,myproc,nproc,tids,nmsg)
	call LL4SUM(1,1,1,euclUL,1,geuclUL,rhlp,myproc,nproc,tids,nmsg)
	if (ULtyp == 'U') then
	  call LL4RMIN(1,1,1,minU,gminU,rhlp,myproc,nproc,tids,nmsg)
	  call LL4ISM(1,1,1,cntmdU,gcntmdU,iw(1),myproc,nproc,tids,nmsg)
	endif
      else
	gmaxUL  = maxUL
	geuclUL = euclUL
	gminU   = minU
	gcntmdU = cntmdU
	gcntcolUL = cntcolUL
      endif

      geuclUL   = SQRT(geuclUL)
      pcntmdU   = REAL(gcntmdU)*100/REAL(lglobal)
      pcntcolUL = REAL(gcntcolUL)*100/REAL(lglobal)


      if ((idoku > 0) .and. (myproc == 1)) then
	write(lout,1000) ULtyp
	write(lout,1006) ULtyp, lenUL
	write(lout,1001) ULtyp, gmaxUL
	write(lout,1002) ULtyp, geuclUL
	if (ULtyp == 'U') then
	  write(lout,1003) gminU
	  write(lout,1004) pcntmdU
	endif
	write(lout,1005) ULtyp, pcntcolUL
      endif
      if (idoku < 0) then
	write(lout,1000) ULtyp
	write(lout,1006) ULtyp, lenUL
	write(lout,1001) ULtyp, gmaxUL
	write(lout,1002) ULtyp, geuclUL
	if (ULtyp == 'U') then
	  write(lout,1003) gminU
	  write(lout,1004) pcntmdU
	endif
	write(lout,1005) ULtyp, pcntcolUL
      endif

      return

 1000 format(//'  STATISTICS OF MATRIX ',A1,':')
 1001 format('   max(',A1,') . . . . . . . . . . . . . . . . . . . . .'
     &       ,G12.5)
 1002 format('   ||',A1,'||2 .........................................'
     &       ,G12.5)
 1003 format('   min(abs(diag(U))). . . . . . . . . . . . . . . .'
     &       ,G12.5)
 1004 format('   rate of elements (/=0) on main diagonal of U ... '
     &       ,F10.2,'%')
 1005 format('   rate of vectors of ',A1,' with vl > 0 . . . . . . . . '
     &       ,F10.2,'%'/)
 1006 format('   number of elements of ',A1,' ........................'
     &       ,I12)
      end  ! ULnorm
