!***********************************************************************
!        1         2         3         4         5         6         7 *
!***********************************************************************
!**                                                                  ***
!**                                                                  ***
      subroutine WRITE_ERROR(x,l,lmatbk,ptrmbk,lout,
     &                       tids,nproc,myproc,nmsg)
!**                                                                  ***
!**                                                                  ***
!***********************************************************************
!**                                                                  ***
!**    WRITE_ERROR    computes the error of the solution and         ***
!**                   writes the error on processor 1                ***
!**                                                                  ***
!***********************************************************************
!**                                                                  ***
!**    Copyright (C) 1999 University Karlsruhe                       ***
!**    PROGRAMMER : H. Haefner                                       ***
!**                                                                  ***
!**    This file is part of LINSOL.                                  ***
!**    LINSOL is free software; you can redistribute it and/or       ***
!**    modify it under the terms of the GNU General Public License   ***
!**    as published by the Free Software Foundation; either          ***
!**    version 2 of the License, or (at your option) any later       ***
!**    version.                                                      ***
!**                                                                  ***
!**    LINSOL is distributed in the hope that it will be useful,     ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty   ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.       ***
!**    See the GNU General Public License for more details.          ***
!**                                                                  ***
!**    You should have received a copy of the GNU General Public     ***
!**    License along with LINSOL; if not, write to the Free          ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,       ***
!**    Boston, MA  02111-1307, USA.                                  ***
!**                                                                  ***
!***********************************************************************
!**                                                                  ***
      implicit none
!**                                                                  *** 
!***********************************************************************
!**                                                                  ***
!**    Formal Parameters :                                           ***
!**                                                                  ***
!***********************************************************************
      integer          l,myproc,lout,nproc,nmsg
      integer          lmatbk(nproc),ptrmbk(nproc+1),tids(nproc)
      double precision x(l)

!***********************************************************************
!**                                                                  ***
!**    List of Formal Parameters :                                   ***
!**                                                                  ***
!**                                                                  ***
!---------I------I-----I------------------------------------------------
! NAME    I TYPE I I/O I MEANING
!---------I------I-----I------------------------------------------------
! l       I int  I in  I dimension of the matrix
!---------I------I-----I------------------------------------------------
!**                                                                  ***
!**                                                                  ***
!**    Local Variables :                                             ***
!**                                                                  ***
      integer          i,j,iconst,lmyblk
      integer          count,imin,imax,gimin,gimax,ihlp
      double precision zero,esum,emax,emin,xsum,xmax,rhlp
      double precision gescp,gemax,gemin,gsum,gmax

      parameter (iconst=4, zero=0.0)
      integer          itop(iconst),ibutom(iconst)
      double precision top(iconst),buttom(iconst)


!***********************************************************************
!**    Start of Calculation                                          ***
!***********************************************************************

      lmyblk = lmatbk(myproc)

      xsum = zero
      do i=1,lmyblk
        xsum = xsum + (dble(ptrmbk(myproc)+i)**(1./3.))**2
      enddo
      call LL4SUM(1,1,1,xsum,1,gsum,rhlp,myproc,nproc,tids,nmsg)
      gsum = sqrt(gsum)

      xmax = dble(ptrmbk(myproc)+lmyblk)**(1./3.)
      call LL4RNM(1,1,1,xmax,gmax,rhlp,myproc,nproc,tids,nmsg)

      do i=1,iconst
        top(i) = zero
        buttom(i) = zero
        itop(i) = 0
        ibutom(i) = 0
      enddo
      do i=1,l
        x(i) = abs(x(i)-dble(ptrmbk(myproc)+i)**(1./3.))
      enddo
      call LL4SCP(lmyblk,1,l,1,x,1,x,1,gescp,rhlp,.true.,
     &            myproc,nproc,tids,nmsg)

      count = 1

      m1: do while (count <= iconst)
            emax = zero
            do i=1,lmyblk
              if (count == 1) then
                if (abs(x(i)) >= emax) then
                  emax = abs(x(i))
                  imax = i
                endif
              else
                if ((abs(x(i)) >= emax) .and.
     &               abs(x(i)) < top(count-1)) then
                  emax = abs(x(i))
                  imax = i
                endif
              endif
            enddo
            call LL4RNM(1,1,1,emax,gemax,rhlp,myproc,nproc,tids,nmsg)
            top(count) = gemax

            if (emax == gemax) then
              imax = imax + ptrmbk(myproc)
            else
              imax = 0
            endif
            call LL4INM(1,1,1,imax,gimax,ihlp,myproc,nproc,tids,nmsg)
            itop(count) = gimax

            count = count + 1
          enddo m1

      count = 1

      m2: do while (count <= iconst)
            emin  = top(1)
            do i=1,lmyblk
              if (count == 1) then
                if (abs(x(i)) <= emin) then
                  emin = abs(x(i))
                  imin = i
                endif
              else
                if ((abs(x(i)) <= emin) .and.
     &               abs(x(i)) > buttom(count-1)) then
                  emin = abs(x(i))
                  imin = i
                endif
              endif
            enddo
            call LL4RMIN(1,1,1,emin,gemin,rhlp,myproc,nproc,tids,nmsg)
            buttom(count) = gemin

            if (emin == gemin) then
              imin = imin + ptrmbk(myproc)
            else
              imin = 0
            endif
            call LL4MIN(1,1,1,imin,gimin,ihlp,myproc,nproc,tids,nmsg)
            ibutom(count) = gimin
            count = count + 1
          enddo m2

      gescp = sqrt(gescp)/gsum
      gemax = top(1)/gmax

      if (myproc == 1) then
        write(lout,1541)
        write(lout,1539) (itop(i),i=1,iconst)
        write(lout,1540) (top(i),i=1,iconst)
        write(lout,1542)
        write(lout,1539) (ibutom(i),i=1,iconst)
        write(lout,1540) (buttom(i),i=1,iconst)
        write(lout,1527) gescp
        write(lout,1530) gemax
      endif

!***********************************************************************
!**    End of Calculation                                            ***
!***********************************************************************

 1527 format(' ',/
     &       '  The relative Euclidian norm of the error is:',/
     &       '   ||error||2   = ',d10.4)
 1530 format('  The relative Maximum norm of the error is:',/
     &       '   ||error||max = ',d10.4/)
 1539 format(1x,4(i17,2x))
 1540 format(1x,4(d17.10,2x))
 1541 format(' ',//
     &       '  The 4 largest errors are (indices --> errors):')
 1542 format(' ',/
     &       '  The 4 smallest errors are (indices --> errors):')

       r e t u r n
!-----END OF WRITE_ERROR ----------------------------------------------
       e    n    d
