!**********************************************************************
!        1         2         3         4         5         6         7*
!**********************************************************************
!**                                                                 ***
!**                                                                 ***
      subroutine FELBWS(pmat,lpmat,pindex,lpix,infoU,infoL,ia1LU,rhs,x,
     *                  colperm,ptrmbk,l,lglobal,msgblk,lfel,
     *                  myproc,nproc,tids,nmsg,ierr)
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**   F E L B W S   performs the forward elimination if lfel = true ***
!**                              (U*x = rhs)                        ***
!**                 or the backward substitution if lfel = false.   ***
!**                              (L*x = rhs)                        ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**    Copyright (C) 2000 University Karlsruhe                      ***
!**    Programmer: H. Haefner                                       ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**     THE IMPLICIT NONE-INSTRUCTION HAS THE AIM,                  ***
!**     THAT ALL VARIABLES MUST BE DECLARED EXPLICITLY.             ***
!**                                                                 ***
      USE constants

      implicit none
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**      FORMAL PARAMETERS :                                        ***
!**      -----------------                                          ***
!**                                                                 ***
      integer, intent(in) ::  lpmat,lpix,ia1LU,l,lglobal,myproc,nproc
      integer, intent(in) ::  msgblk
      integer, intent(inout) ::  nmsg
      integer, intent(out) ::  ierr
      integer, intent(in) ::  ptrmbk(nproc+1),tids(nproc)
      integer, intent(in) ::  infoU(ia1LU,ia2LU),infoL(ia1LU,ia2LU)
      integer, intent(in) ::  pindex(lpix),colperm(lglobal)
      double precision, intent(in)  ::  pmat(lpmat)
      double precision, intent(in)  ::  rhs(l)
      double precision, intent(out) ::  x(l)
      logical, intent(in) ::  lfel
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**   LIST OF FORMAL PARAMETERS :                                   ***
!**   -------------------------                                     ***
!**                                                                 ***
!--------I------I-----I------------------------------------------------
! NAME   I TYPE I I/O I     MEANING
!--------I------I-----I------------------------------------------------
!--------I------I-----I------------------------------------------------
! colpermI  I   I in  I  permutation vector for the columns of the
!        I      I     I  matrix due to pivoting in the elim. process
!--------I------I-----I------------------------------------------------
!**                                                                 ***
!**                                                                 ***
!**      LOCAL PARAMETERS :                                         ***
!**      -----------------                                          ***
!**                                                                 ***
      integer          adda,i,ii,indr,ipivot,k,kstart,j,jj,lvt,ptrmyb
      double precision xnew(msgblk)
      double precision update(l)
      logical          root
!     real*8,parameter ::  inf_plus  = Z'7FF0000000000000'
!     real*8,parameter ::  inf_minus = Z'FFF0000000000000'

!
!**** START OF CALCULATION :
!     ---------------------
!
      ptrmyb = ptrmbk(myproc)
      ierr = 0

      j = 1

      update = rhs

      if (lfel) then         ! Forward elimination on matrix U
        ipivot = 1
	do i=1,lglobal
	  if (i > ptrmbk(ipivot+1)) ipivot = ipivot + 1
	  if (i > ptrmyb .and. i <= ptrmbk(myproc+1)) then
            adda = infoU(i,1)
            lvt  = infoU(i,2)
	    indr = adda
	    ii = i - ptrmyb
	    if ((lvt > 0) .and. (pindex(indr+1) == i)) then
	      xnew(j) = update(ii)/pmat(adda+1)
	      kstart  = 2 
	    else
	      xnew(j) = update(ii)
	      kstart  = 1 
	    endif
	    if ((ipivot /= nproc) .and.
     &          (j == msgblk) .or. (i == ptrmbk(myproc+1))) then
	      root = .true.
	      call MPSCAT(xnew,ireal*j,root,myproc,nproc,tids,nmsg,ierr)
	    endif
	    include "norec.h"
	    do k=kstart,lvt
	      update(pindex(indr+k)-ptrmyb) =
     &              update(pindex(indr+k)-ptrmyb) - pmat(adda+k)*xnew(j)
	    enddo
            x(ii) = xnew(j)
	  else
	    if ((ipivot /= nproc) .and.
     &          (j == msgblk) .or. (i == ptrmbk(ipivot+1))) then
	      root = .false.
	      call MPSCAT(xnew,ireal*j,root,myproc,nproc,tids,nmsg,ierr)
	    endif
	  endif
!         if (.not. (xnew == xnew) .or. (xnew == inf_plus) .or.
!    &        (xnew == inf_minus)) then
!           ierr = 99999
!           goto 999
!         endif 
	  if ((i <= ptrmyb) .and.
     &        ((j == msgblk) .or. (i == ptrmbk(ipivot+1)))) then
	    do jj=1,j
              adda = infoU(i-j+jj,1)
              lvt  = infoU(i-j+jj,2)
	      indr = adda
	      include "norec.h"
	      do k=1,lvt
	        update(pindex(indr+k)-ptrmyb) =
     &             update(pindex(indr+k)-ptrmyb) - pmat(adda+k)*xnew(jj)
	      enddo
	    enddo
	  endif
	  if ((j == msgblk) .or. (i == ptrmbk(ipivot+1))) then
	    j = 1
	  else
	    j = j + 1
	  endif
	enddo


      else                   ! Backward substitution on matrix L

        ipivot = nproc
	do i=lglobal,1,-1
	  if (i == ptrmbk(ipivot)) ipivot = ipivot - 1
	  if (i > ptrmyb .and. i <= ptrmbk(myproc+1)) then
	    ii = i - ptrmyb
	    xnew(j) = update(ii)
	    if ((ipivot /= 1) .and.
     &          (j == msgblk) .or. (i == ptrmyb+1)) then
	      root = .true.
	      call MPSCAT(xnew,ireal*j,root,myproc,nproc,tids,nmsg,ierr)
	    endif
            adda = infoL(colperm(i),1)
            lvt  = infoL(colperm(i),2)
            indr = adda
	    include "norec.h"
	    do k=1,lvt
	      update(pindex(indr+k)-ptrmyb) =
     &             update(pindex(indr+k)-ptrmyb) - pmat(adda+k)*xnew(j)
	    enddo
            x(ii) = xnew(j)
	  else
	    if ((ipivot /= 1) .and.
     &          (j == msgblk) .or. (i == ptrmbk(ipivot)+1)) then
	      root = .false.
	      call MPSCAT(xnew,ireal*j,root,myproc,nproc,tids,nmsg,ierr)
	    endif
	  endif
!         if (.not. (xnew == xnew) .or. (xnew == inf_plus) .or.
!    &        (xnew == inf_minus)) then
!           ierr = 99999
!           goto 999
!         endif 
	  if ((i > ptrmbk(myproc+1)) .and.
     &        ((j == msgblk) .or. (i == ptrmbk(ipivot)+1))) then
	    do jj=1,j
              adda = infoL(colperm(i-jj+j),1)
              lvt  = infoL(colperm(i-jj+j),2)
	      indr = adda
	      include "norec.h"
	      do k=1,lvt
	        update(pindex(indr+k)-ptrmyb) =
     &            update(pindex(indr+k)-ptrmyb) - pmat(adda+k)*xnew(jj)
	      enddo
	    enddo
	  endif
	  if ((j == msgblk) .or. (i == ptrmbk(ipivot)+1)) then
	    j = 1
	  else
	    j = j + 1
	  endif
	enddo
      endif

  999 R E T U R N
!------END OF FELBWS------------------------------------------------
      E N D                                   
