!**********************************************************************
!        1         2         3         4         5         6         7*
!**********************************************************************
!**                                                                 ***
!**                                                                 ***
      subroutine FELBWS1(pmat,lpmat,pindex,lpix,infoU,infoL,ia1LU,rhs,
     *                   x,colperm,l,lfel,ierr)
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**   F E L B W S 1   performs the forward elimination on           ***
!**                   1 processor if (lfel), i.e. (U*x = rhs),      ***
!**                   or the backward substitution on 1 processor   ***
!**                   if (.not. lfel), i.e. (L*x = rhs).            ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**    Copyright (C) 2000 University Karlsruhe                      ***
!**    Programmer: H. Haefner                                       ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**     THE IMPLICIT NONE-INSTRUCTION HAS THE AIM,                  ***
!**     THAT ALL VARIABLES MUST BE DECLARED EXPLICITLY.             ***
!**                                                                 ***
      USE constants

      implicit none
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**      FORMAL PARAMETERS :                                        ***
!**      -----------------                                          ***
!**                                                                 ***
      integer, intent(in) ::  lpmat,lpix,ia1LU,l
      integer, intent(out) ::  ierr
      integer, intent(in) ::  infoU(ia1LU,ia2LU),infoL(ia1LU,ia2LU)
      integer, intent(in) ::  pindex(lpix),colperm(l)
      double precision, intent(in)  ::  pmat(lpmat)
      double precision, intent(in)  ::  rhs(l)
      double precision, intent(out) ::  x(l)
      logical, intent(in) ::  lfel
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**   LIST OF FORMAL PARAMETERS :                                   ***
!**   -------------------------                                     ***
!**                                                                 ***
!--------I------I-----I------------------------------------------------
! NAME   I TYPE I I/O I     MEANING
!--------I------I-----I------------------------------------------------
!--------I------I-----I------------------------------------------------
! colpermI  I   I in  I  permutation vector for the columns of the
!        I      I     I  matrix due to pivoting in the elim. process
!--------I------I-----I------------------------------------------------
!**                                                                 ***
!**                                                                 ***
!**      LOCAL PARAMETERS :                                         ***
!**      -----------------                                          ***
!**                                                                 ***
      integer          adda,i,indr,k,kstart,lvt
      double precision xnew
      double precision update(l)
!     real*8,parameter ::  inf_plus  = Z'7FF0000000000000'
!     real*8,parameter ::  inf_minus = Z'FFF0000000000000'

!
!**** START OF CALCULATION :
!     ---------------------
!
      ierr = 0

      update = rhs

      if (lfel) then         ! Forward elimination on matrix U
	do i=1,l
          adda = infoU(i,1)
          lvt  = infoU(i,2)
	  indr = adda
          if ((lvt > 0) .and. (pindex(indr+1) == i)) then
            xnew = update(i)/pmat(adda+1)
            kstart  = 2
          else
            xnew = update(i)
            kstart  = 1
          endif
	  include "norec.h"
	  do k=kstart,lvt
	    update(pindex(indr+k)) =
     &             update(pindex(indr+k)) - pmat(adda+k)*xnew
	  enddo
          x(i) = xnew
!         if (.not. (xnew == xnew) .or. (xnew == inf_plus) .or.
!    &        (xnew == inf_minus)) then
!           ierr = 99999
!           goto 999
!         endif 
	enddo
      else                   ! Backward substitution on matrix L
	do i=l,1,-1
	  xnew = update(i)
          adda = infoL(colperm(i),1)
          lvt  = infoL(colperm(i),2)
          indr = adda
	  include "norec.h"
	  do k=1,lvt
	    update(pindex(indr+k)) =
     &             update(pindex(indr+k)) - pmat(adda+k)*xnew
	  enddo
          x(i) = xnew
!         if (.not. (xnew == xnew) .or. (xnew == inf_plus) .or.
!    &        (xnew == inf_minus)) then
!           ierr = 99999
!           goto 999
!         endif 
	enddo
      endif

  999 R E T U R N
!------END OF FELBWS1-----------------------------------------------
      E N D                                   
