!**********************************************************************
!        1         2         3         4         5         6         7*
!**********************************************************************
!**                                                                 ***
!**                                                                 ***
      subroutine FELBWSSYM1(pmat,lpmat,pindex,lpix,infoL,ia1LU,
     *                      rhs,x,l,lfel,ierr)
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**   F E L B W S S Y M 1   performs the forward elimination on 1   ***
!**                         processor; if (lfel) then L(T)*x = rhs; ***
!**                         the backward substitution on 1          ***
!**                         processor is performed, too;            ***
!**                         if (.not. lfel), then L*x = rhs. This   ***
!**                         routine only works on symmatric         ***
!**                         matrices.                               ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**    Copyright (C) 2001 University Karlsruhe                      ***
!**    Programmer: H. Haefner                                       ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**     THE IMPLICIT NONE-INSTRUCTION HAS THE AIM,                  ***
!**     THAT ALL VARIABLES MUST BE DECLARED EXPLICITLY.             ***
!**                                                                 ***
      USE constants

      implicit none
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**      FORMAL PARAMETERS :                                        ***
!**      -----------------                                          ***
!**                                                                 ***
      integer, intent(in) ::  lpmat,lpix,ia1LU,l
      integer, intent(out) ::  ierr
      integer, intent(in) ::  infoL(ia1LU,ia2LU)
      integer, intent(in) ::  pindex(lpix)
      double precision, intent(in)  ::  pmat(lpmat)
      double precision, intent(in)  ::  rhs(l)
      double precision, intent(out) ::  x(l)
      logical, intent(in) ::  lfel
!**                                                                 ***
!**                                                                 ***
!**                                                                 ***
!**      LOCAL PARAMETERS :                                         ***
!**      -----------------                                          ***
!**                                                                 ***
      integer          ioff(l)
      integer          adda,i,k,kstart,lvt
      double precision xnew
      double precision update(l)

!
!**** START OF CALCULATION :
!     ---------------------
!
      ierr = 0
      ioff = 0

      update = rhs

      if (lfel) then    ! Forward elimination on rowwise stored matrix L(T)
	do i=1,l
          adda = infoL(i,1)
          lvt  = infoL(i,2)
          if ((lvt > 0) .and. (pindex(adda+1) == i)) then
            xnew = update(i)/pmat(adda+1)
            kstart  = 2
          else
            xnew = update(i)
            kstart  = 1
          endif
	  include "norec.h"
	  do k=kstart,lvt
	    update(pindex(adda+k)) =
     &             update(pindex(adda+k)) - pmat(adda+k)*xnew
	  enddo
          x(i) = xnew
	enddo
      else              ! Backward substitution on rowwise stored matrix L
	do i=l,1,-1
          adda = infoL(i,1)
          lvt  = infoL(i,2)
          if ((lvt > 0) .and. (pindex(adda+1) == i)) then
            xnew = update(i)/pmat(adda+1)
            kstart  = 2
          else
            xnew = update(i)
            kstart  = 1
          endif
	  do k=i-1,1,-1
            adda = infoL(k,1)
            lvt  = infoL(k,2)
            if ((lvt-ioff(k) >= 1) .and.
     &          (pindex(adda+lvt-ioff(k)) == i)) then
	      update(k) = update(k) - pmat(adda+lvt-ioff(k))*xnew
	      ioff(k) = ioff(k) + 1
	    endif
	  enddo
          x(i) = xnew
	enddo
      endif

  999 R E T U R N
!------END OF FELBWSSYM1--------------------------------------------
      E N D                                   
