!**********************************************************************
!        1         2         3         4         5         6         7*
!**********************************************************************
!**                                                                 ***
!**                                                                 ***
      subroutine FELBWST(pmat,lpmat,pindex,lpix,infoU,infoL,ia1LU,rhs,
     *                   x,colperm,lmatbk,ptrmbk,l,lglobal,lfel,
     *                   myproc,nproc,tids,nmsg,ierr)
!**                                                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**   F E L B W S T   performs the backward substitution if         ***
!**                   lfel = true      (U^T*x = rhs)                ***
!**                   or the forward elimination if lfel = false.   ***
!**                                    (L^T*x = rhs)                ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**    Copyright (C) 2000 University Karlsruhe                      ***
!**    Programmer: H. Haefner                                       ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
!**                                                                 ***
!**********************************************************************
!**                                                                 ***
!**     THE IMPLICIT NONE-INSTRUCTION HAS THE AIM,                  ***
!**     THAT ALL VARIABLES MUST BE DECLARED EXPLICITLY.             ***
!**                                                                 ***
      USE constants

      implicit none
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**      FORMAL PARAMETERS :                                        ***
!**      -----------------                                          ***
!**                                                                 ***
      integer, intent(in) ::  lpmat,lpix,ia1LU,l,lglobal,myproc,nproc
      integer, intent(inout) ::  nmsg
      integer, intent(out) ::  ierr
      integer, intent(in) ::  lmatbk(nproc),ptrmbk(nproc+1),tids(nproc)
      integer, intent(in) ::  infoU(ia1LU,ia2LU),infoL(ia1LU,ia2LU)
      integer, intent(in) ::  pindex(lpix),colperm(lglobal)
      double precision, intent(in)  ::  pmat(lpmat)
      double precision, intent(in)  ::  rhs(l)
      double precision, intent(out) ::  x(l)
      logical, intent(in) ::  lfel
!**                                                                 ***
!**-----------------------------------------------------------------***
!**                                                                 ***
!**   LIST OF FORMAL PARAMETERS :                                   ***
!**   -------------------------                                     ***
!**                                                                 ***
!--------I------I-----I------------------------------------------------
! NAME   I TYPE I I/O I     MEANING
!--------I------I-----I------------------------------------------------
!--------I------I-----I------------------------------------------------
! colpermI  I   I in  I  permutation vector for the columns of the
!        I      I     I  matrix due to pivoting in the elim. process
!--------I------I-----I------------------------------------------------
!**                                                                 ***
!**                                                                 ***
!**      LOCAL PARAMETERS :                                         ***
!**      -----------------                                          ***
!**                                                                 ***
      integer          adda,colix,i,iloc,ii,iiloc,j,kloc,lstart,lvt
      integer          ptrmyb,to,from,mids,midr,swap
      integer          pcnt,pbufs,pbufr
      integer          ioff(lglobal)
      integer          LL9MAP
      double precision xnew
      double precision buf(2*l)
      logical          root

!
!**** START OF CALCULATION :
!     ---------------------
!
      ptrmyb = ptrmbk(myproc)
      ierr = 0

      ioff = 0

      pbufs = 1
      pbufr = l + 1

      do i=1,l
        buf(pbufs+i-1) = rhs(i)
      enddo

      if (lfel) then         ! Backward substitution on matrix U^T

	pcnt   = nproc
	lstart = lglobal
	from   = tids(LL9MAP(myproc-1,nproc))
	to     = tids(LL9MAP(myproc+1,nproc))

  111   if (pcnt > 1) call MPRCVA(from,nmsg+pcnt-1,l*ireal,
     &                            buf(pbufr),midr,ierr) 
	if (myproc == pcnt) then
	  do i=lstart,lstart-lmatbk(myproc)+1,-1
            adda = infoU(i,1)
	    iloc = i - ptrmyb
	    xnew = buf(pbufs+iloc-1)/pmat(adda+1)
	    do iiloc=iloc-1,1,-1
	      ii = iiloc + ptrmyb
	      adda = infoU(ii,1)
              lvt  = infoU(ii,2)
	      if ((lvt-ioff(ii) >= 1) .and.
     &            (pindex(adda+lvt-ioff(ii)) == i)) then
	        buf(pbufs+iiloc-1) = buf(pbufs+iiloc-1) -
     &                          pmat(adda+lvt-ioff(ii))*xnew
		ioff(ii) = ioff(ii) + 1
	      endif
	    enddo
            x(iloc) = xnew
	  enddo
	endif

	lstart = lstart - lmatbk(pcnt)

	do j=pcnt-1,1,-1
	  call MPSNDA(to,nmsg+j,l*ireal,buf(pbufs),mids,ierr)
	  call MPSNDW(to,nmsg+j,l*ireal,buf(pbufs),mids,ierr)
	  call MPRCVW(from,nmsg+j,l*ireal,buf(pbufr),midr,ierr)
	  if (myproc == pcnt) then
	    do kloc=lmatbk(myproc),1,-1
	      do iiloc=lmatbk(j),1,-1
	        ii = iiloc + ptrmbk(j)
	        adda = infoU(ii,1)
                lvt  = infoU(ii,2)
	        if ((lvt-ioff(ii) >= 1) .and.
     &              (pindex(adda+lvt-ioff(ii)) == kloc+ptrmyb)) then
	          buf(pbufr+iiloc-1) = buf(pbufr+iiloc-1) -
     &                                 pmat(adda+lvt-ioff(ii))*x(kloc)
	          ioff(ii) = ioff(ii) + 1
	        endif
	      enddo
	    enddo
	  endif
	  swap  = pbufs
	  pbufs = pbufr
	  pbufr = swap
	  if (j == 1) then
	    from   = tids(LL9MAP(myproc-(nproc-pcnt)-1,nproc))
	    to     = tids(LL9MAP(myproc+nproc-pcnt+1,nproc))
	  endif
	  call MPRCVA(from,nmsg+j-1,l*ireal,buf(pbufr),midr,ierr)
	enddo
	if (pcnt > 1) then
	  call MPSNDA(to,nmsg,l*ireal,buf(pbufs),mids,ierr)
	  call MPSNDW(to,nmsg,l*ireal,buf(pbufs),mids,ierr)
	  call MPRCVW(from,nmsg,l*ireal,buf(pbufr),midr,ierr)
	endif

	pcnt = pcnt - 1
        if (pcnt > 1) goto 111

      else                   ! Forward elimination on matrix L^T

	ioff = 1

	pcnt   = 1
	lstart = 1
	from   = tids(LL9MAP(myproc+1,nproc))
	to     = tids(LL9MAP(myproc-1,nproc))

  222   if (nproc > 1 .and. pcnt < nproc) call MPRCVA(from,nmsg+pcnt,
     &                                    l*ireal,buf(pbufr),midr,ierr) 
	if (myproc == pcnt) then
	  do i=lstart,lstart+lmatbk(myproc)-1
	    iloc = i - ptrmyb
	    xnew = buf(pbufs+iloc-1)
	    do iiloc=iloc+1,lmatbk(myproc)
	      ii = iiloc + ptrmyb
	      adda = infoL(colperm(ii),1)
              lvt  = infoL(colperm(ii),2)
	      if ((ioff(ii) <= lvt) .and.
     &            (pindex(adda+ioff(ii)) == i)) then
	        buf(pbufs+iiloc-1) = buf(pbufs+iiloc-1) -
     &                               pmat(adda+ioff(ii))*xnew
		ioff(ii) = ioff(ii) + 1
	      endif
	    enddo
            x(iloc) = xnew
	  enddo
	endif

	lstart = lstart + lmatbk(pcnt)

	do j=pcnt+1,nproc
	  call MPSNDA(to,nmsg+j-1,l*ireal,buf(pbufs),mids,ierr)
	  call MPSNDW(to,nmsg+j-1,l*ireal,buf(pbufs),mids,ierr)
	  call MPRCVW(from,nmsg+j-1,l*ireal,buf(pbufr),midr,ierr)
	  if (myproc == pcnt) then
	    do kloc=1,lmatbk(myproc)
	      do iiloc=1,lmatbk(j)
	        ii = iiloc + ptrmbk(j)
	        adda = infoL(colperm(ii),1)
                lvt  = infoL(colperm(ii),2)
	        if ((ioff(ii) <= lvt) .and.
     &              (pindex(adda+ioff(ii)) == kloc+ptrmyb)) then
	          buf(pbufr+iiloc-1) = buf(pbufr+iiloc-1) -
     &                                 pmat(adda+ioff(ii))*x(kloc)
	          ioff(ii) = ioff(ii) + 1
	        endif
	      enddo
	    enddo
	  endif
	  swap  = pbufs
	  pbufs = pbufr
	  pbufr = swap
	  if (j == nproc) then
	    from   = tids(LL9MAP(myproc+pcnt,nproc))
	    to     = tids(LL9MAP(myproc-pcnt,nproc))
	  endif
	  call MPRCVA(from,nmsg+j,l*ireal,buf(pbufr),midr,ierr)
	enddo
	if (nproc > 1 .and. pcnt < nproc) then
	  call MPSNDA(to,nmsg+nproc,l*ireal,buf(pbufs),mids,ierr)
	  call MPSNDW(to,nmsg+nproc,l*ireal,buf(pbufs),mids,ierr)
	  call MPRCVW(from,nmsg+nproc,l*ireal,buf(pbufr),midr,ierr)
	endif

	pcnt = pcnt + 1
        if (pcnt < nproc) goto 222

      endif

      nmsg = MOD(nmsg+nproc+1,nmsgmax)

      R E T U R N
!------END OF FELBWST-----------------------------------------------
      E N D                                   
