C:::::      ,,,,,LL3AXB.....
C
C
C
C**********************************************************************
C        1         2         3         4         5         6         7*
C**********************************************************************
C**                                                                 ***
C**                                                                 ***
      subroutine LL3AXB(rs,r,xs,x,b,prec,work,l,lprec,lmyblk,epslin,
     &                  ptrmbk,lmatbk,ptrinf,jump,mat,lmat,index,
     &                  lindex,info,ia1,lsym,ladd,
     &                  method,lstart,myproc,nproc,tid,nmsg,ilin,ierr,
     &                  count,imvm,branch)
C**                                                                 ***
C**                                                                 ***
C**********************************************************************
C**                                                                 ***
C**      L L 3 A X B   computes the norm of the residual and        ***
C**      performs the check, if the stopping criterion is fulfilled.***
C**      If (lcond1 .and. lcond2) then the iteration is stopped.    ***
C**      If (lcond1 .and. .not. lcond2) then a new stopping         ***
C**      criterion is computed.                                     ***
C**      lcond1 = (||rs||2 <= epsrel * ||Nb||2)                     ***
C**      noprec=1:                                                  ***
C**      lcond2 = (||N-1*rs||max <= epslin * ||b||max)              ***
C**      noprec=0:                                                  ***
C**      lcond2 = (||rs||max <= epslin * ||Nb||max)                 ***
C**                                                                 ***
C**********************************************************************
C**                                                                 ***
C**    Copyright (C) 1996 University Karlsruhe                      ***
C**    Programmer: H. Haefner                                       ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
C**                                                                 ***
C**********************************************************************
C**                                                                 ***
      USE constants

      implicit none
C**                                                                 ***
C**-----------------------------------------------------------------***
C**                                                                 ***
C**      FORMAL PARAMETERS :                                        ***
C**      -------------------                                        ***
C**                                                                 ***
      integer            ia1,ierr,l,lmat,lindex,lprec,lmyblk,myproc,
     #                   nproc,nmsg,count,imvm,branch 
      integer            tid(nproc),ilin(nilin)
      double precision   prec(lprec),rs(l),r(l),xs(l),x(l),b(l),
     #                   work(6*l),epslin
      character*10       method
      integer            ptrmbk(nproc+1),lmatbk(nproc),jump(nproc),
     #                   ptrinf(ntyp+1,nproc),info(ia1,ia2),
     #                   index(lindex)
      double precision   mat(lmat)
      logical            ladd,lstart,lsym
C**                                                                 ***
C**-----------------------------------------------------------------***
C**                                                                 ***
C**   LIST OF FORMAL PARAMETERS :                                   ***
C**   ---------------------------                                   ***
C**                                                                 ***
C--------I------I-----I------------------------------------------------
C NAME   I TYPE I I/O I     MEANING
C--------I------I-----I------------------------------------------------
C--------I------I-----I------------------------------------------------
C rs     I  R   I IN  I  Smoothed residual           array: rs(l)
C--------I------I-----I------------------------------------------------
C r      I  R   I IN  I  Residual                    array: r(l)
C--------I------I-----I------------------------------------------------
C xs     I  R   I IN  I  Smoothed solution           array: xs(l)
C--------I------I-----I------------------------------------------------
C x      I  R   I IN  I  Solution                    array: x(l)
C--------I------I-----I------------------------------------------------
C work   I  R   I  -  I  Workarray                   array: work(6*l)
C--------I------I-----I------------------------------------------------
C imvm   I  I   I IN  I  Number of performed matrix-vector-
C        I      I     I  multiplications
C--------I------I-----I------------------------------------------------
C count  I  I   I IN  I  Number of performed iteration steps
C--------I------I-----I------------------------------------------------
C branch I  I   I OUT I  Branch variable for arithmetic goto
C--------I------I-----I------------------------------------------------
C lstart I  L   I IN  I  must be true, if this routine is called
C        I      I     I  before the iteration process.
C--------I------I-----I------------------------------------------------
C method I  S   I IN  I  The name of the routine, which calls this
C        I      I     I  routine.
C--------I------I-----I------------------------------------------------
C**                                                                 ***
C**    All other parameters: see Manual Pages <lsolpp>              ***
C**                                                                 ***
C**-----------------------------------------------------------------***
C**                                                                 ***
C**                                                                 ***
C**        LOCAL PARAMETERS :                                       ***
C**        ----------------                                         ***
C**                                                                 ***
      integer           i,ione,lout,ibborg,ibplbg,ibNb,noprec,idoku1
      integer           unit
      double precision  r0eucl,rseucl,rseucn,rseuco,rsmax,beucl,bmax
      double precision  epsrel,rnorm,esnorm,enorm,rpoly
      double precision  zero,help
      logical           lsmo

      common /norms/ rseucn,rpoly(3),epsrel
C**                                                                 ***
C**                                                                 ***
C**   LIST OF LOCAL PARAMETERS :                                    ***
C**   ---------------------------                                   ***
C**                                                                 ***
C--------I------I-----I------------------------------------------------
C NAME   I TYPE I I/O I     MEANING
C--------I------I-----I------------------------------------------------
C--------I------I-----I------------------------------------------------
C rseucl I  R   I sav I  Euclidian norm of the smoothed residual 
C--------I------I-----I------------------------------------------------
C rsmax  I  R   I sav I  Maximum norm of the smoothed residual     
C--------I------I-----I------------------------------------------------
C beucl  I  R   I sav I  Euclidian norm of the right hand side
C--------I------I-----I------------------------------------------------
C bmax   I  R   I sav I  Maximum norm of the right hand side
C--------I------I-----I------------------------------------------------
C epsrel I  R   I sav I  Stopping factor:
C        I      I     I  lstart=.true. --> epsrel = epslin
C        I      I     I  lstart=.false. --> 
C        I      I     I  epsrel = 0.7*(rseucl/beucl)*(epsrel*bmax/rsmax)
C--------I------I-----I------------------------------------------------
C**                                                                 ***
C**        CONSTANTS:                                               ***
C**        ---------                                                ***
C**                                                                 ***
      parameter  (zero = 0., ione = 1)
C**                                                                 ***
      external   LL6AX
C**                                                                 ***
C**                                                                 ***
      save rseucl,rsmax,beucl,bmax,idoku1
C**-----------------------------------------------------------------***
C**                                                                 ***
C**** START OF CALCULATION :                                        ***
C**   ---------------------                                         ***
C**                                                                 ***
      branch = 0
      noprec = ilin(9)
      lout   = ilin(12)
      lsmo   = ilin(18) .ne. 0
      unit   = ilin(44)

      ibborg = 1 + 2*l
      ibplbg = 1 + 3*l
      ibNb   = 1 + 4*l

C** 
C**** COMPUTATION OF ||r0|| (r0eucl)
C**   
      if (lstart) then
	idoku1 = abs(ilin(13))

C**** ||PL*N*b||2
        if (lsmo) then
          call LL4SCP(lmyblk,ione,l,ione,rs,1,rs,1,r0eucl,help,
     #                .true.,myproc,nproc,tid,nmsg)
        else
          call LL4SCP(lmyblk,ione,l,ione,r,1,r,1,r0eucl,help,
     #                .true.,myproc,nproc,tid,nmsg)
        endif
	r0eucl = sqrt(r0eucl)

C****
C**** ||PL*N*borg||2  i.e. beucl relates to the original system Ax=b
C****
	call LL4SCP(lmyblk,ione,l,ione,prec,ibplbg,prec,ibplbg,
     #              beucl,help,.true.,myproc,nproc,tid,nmsg)
	beucl = sqrt(beucl)

C**** NORMALIZED SYSTEM IS CONSIDERED
	if (noprec .ne. 1) then
C**** ||PL*N*borg||max
          call LL4RNM(lmyblk,ione,l,prec(ibplbg),bmax,help,
     #                myproc,nproc,tid,nmsg)
          if (mod(ilin(19),10000) .ge. 1000) then
            call LL8NME(work,xs,x,r,esnorm,enorm,rnorm,l,lmyblk,
     #                  nproc,myproc,tid,nmsg)
          endif
        else

C**** ORIGINAL SYSTEM IS CONSIDERED
C**** ||borg||max
          call LL4RNM(lmyblk,ione,l,prec(ibborg),bmax,help,
     #                myproc,nproc,tid,nmsg)
          if (mod(ilin(19),10000) .ge. 1000) then
            call LL8NME(work,xs,x,r,esnorm,enorm,rnorm,l,
     #                  lmyblk,nproc,myproc,tid,nmsg)
          endif
        endif

        if (ilin(13) .le. -2) then
          if (mod(ilin(19),10000) .ge. 1000) then
            write(lout,1211) r0eucl,rnorm,esnorm,enorm,
     #                       epsrel*beucl,method
          else
	    write(lout,1111) r0eucl,method,epsrel*beucl
          endif
        elseif ((ilin(13) .ge. 2) .and. (myproc .eq. 1)) then
          if (mod(ilin(19),10000) .ge. 1000) then
            write(lout,1211) r0eucl,rnorm,esnorm,enorm,
     #                       epsrel*beucl,method
	    if (mod(ilin(19),100000) .ge. 10000) then
	      write(unit,*) count,imvm,r0eucl,rnorm,esnorm,enorm
            endif
          else
            write(lout,1111) r0eucl,method,epsrel*beucl
	    if (mod(ilin(19),100000) .ge. 10000) then
	      write(unit,*) count,imvm,r0eucl
            endif
          endif
        endif
        if (r0eucl .eq. zero) then
          if (ilin(13) .lt. 0) write(lout,1000)
          if (ilin(13) .gt. 0) then
            if (myproc .eq. 1) write(lout,1000)
          endif
	  branch = 1
	  goto 999
        endif
	rseucl = r0eucl
C***  SET ONLY FOR BICO
	rseucn = rseucl

C** 
C**** COMPUTATION OF ||rk_smoothed|| (rseucl)
C**   
      else
	rseuco = rseucl
C**** NORMALIZED SYSTEM IS CONSIDERED
	if (noprec .lt. 2) then
	  if (lsmo) then
	    do 130 i = 1,lmyblk
	      work(i) = rs(i)
  130       continue
          else
	    do 131 i = 1,lmyblk
	      work(i) = r(i)
  131       continue
          endif
	else
C**** ORIGINAL SYSTEM IS CONSIDERED
          if (lsym) then
	    if (lsmo) then
	     call LL3ASX(mat,xs,work(1),work(l+1),ptrmbk,
     #                   lmatbk,ptrinf,l,nproc,tid,myproc,jump,
     #                   ladd,lmat,lindex,ia1,info,index,nmsg)
            else
	     call LL3ASX(mat,x,work(1),work(l+1),ptrmbk,
     #                   lmatbk,ptrinf,l,nproc,tid,myproc,jump,
     #                   ladd,lmat,lindex,ia1,info,index,nmsg)
            endif
          else
	    if (lsmo) then
             call LL3AX(LL6AX,mat,xs,work(1),work(l+1),ptrmbk,
     #                  lmatbk,ptrinf,l,nproc,tid,myproc,jump,ladd,
     #                  lmat,lindex,ia1,info,index,nmsg)
            else
             call LL3AX(LL6AX,mat,x,work(1),work(l+1),ptrmbk,
     #                  lmatbk,ptrinf,l,nproc,tid,myproc,jump,ladd,
     #                  lmat,lindex,ia1,info,index,nmsg)
            endif
          endif
          imvm=imvm+1
C****   work(1..l)  = (|N*A*|N*xs - |N*b)
C****   work(1..l)  =  (N*A*xs - N*b)
          include "norec.h"
          do 132 i=1,lmyblk
            work(i) = work(i) - prec(ibNb+i-1)
  132     continue
          if (lsmo) then
            do 133 i=1,lmyblk
              rs(i) = work(i)
  133       continue
          else
            do 134 i=1,lmyblk
              r(i) = work(i)
  134       continue
          endif
        endif
        call LL4SCP(lmyblk,ione,l,ione,work,1,work,1,rseucl,help,
     #              .true.,myproc,nproc,tid,nmsg)
C**** ||PL*N*r(s)||2 or ||Ax(s)-b||2
	rseucl = sqrt(rseucl)

        if (mod(ilin(19),10000) .ge. 1000) then
	  if (noprec .eq. 1) then
            if (lsym) then
	      call LL3ASX(mat,xs,work(1),work(l+1),ptrmbk,
     #                    lmatbk,ptrinf,l,nproc,tid,myproc,jump,
     #                    ladd,lmat,lindex,ia1,info,index,nmsg)
            else
              call LL3AX(LL6AX,mat,xs,work(1),work(l+1),ptrmbk,
     #                   lmatbk,ptrinf,l,nproc,tid,myproc,jump,ladd,
     #                   lmat,lindex,ia1,info,index,nmsg)
            endif
            imvm=imvm+1
C****   work(1..l)  = |N(-1)*(|N*A*|N*x - |N*b)
C****   work(1..l)  =  N(-1)*(N*A*x - N*b)
            include "norec.h"
            do 116 i=1,lmyblk
              work(i) = (work(i)-prec(ibNb+i-1))/prec(i)
  116       continue
          endif
          call LL8NME(work(l+1),xs,x,work(1),esnorm,enorm,rnorm,l,
     #                lmyblk,nproc,myproc,tid,nmsg)
        endif

        if (count .ge. idoku1) then
          if (ilin(13) .le. -2) then
            if (mod(ilin(19),10000) .ge. 1000) then
              write(lout,1201) count,imvm,rseucl,rnorm,esnorm,enorm
            else
	      write(lout,1200) count,imvm,rseucl
            endif
          elseif ((ilin(13) .ge. 2) .and. (myproc .eq. 1)) then
            if (mod(ilin(19),10000) .ge. 1000) then
              write(lout,1201) count,imvm,rseucl,rnorm,esnorm,enorm
	      if (mod(ilin(19),100000) .ge. 10000) then
	        write(unit,*) count,imvm,rseucl,rnorm,esnorm,enorm
              endif
            else
	      write(lout,1200) count,imvm,rseucl
	      if (mod(ilin(19),100000) .ge. 10000) then
	        write(unit,*) count,imvm,rseucl
              endif
            endif
          endif
        endif
        if ((rseucl .gt. rseuco) .and. lsmo .and.
     &      (ilin(1) .ne. 6) .and. (ilin(1) .ne. 11)) then
	  branch = 3
	  goto 999
        endif

        if (rseucl .le. epsrel*beucl) then
          if (count .lt. idoku1) then
            if (ilin(13) .le. -2) then
              if (mod(ilin(19),10000) .ge. 1000) then
                write(lout,1201) count,imvm,rseucl,rnorm,esnorm,enorm
              else
	        write(lout,1200) count,imvm,rseucl
              endif
            elseif ((ilin(13) .ge. 2) .and. (myproc .eq. 1)) then
              if (mod(ilin(19),10000) .ge. 1000) then
                write(lout,1201) count,imvm,rseucl,rnorm,esnorm,enorm
	        if (mod(ilin(19),100000) .ge. 10000) then
	          write(unit,*) count,imvm,rseucl,rnorm,esnorm,enorm
                endif
              else
	        write(lout,1200) count,imvm,rseucl
	        if (mod(ilin(19),100000) .ge. 10000) then
	          write(unit,*) count,imvm,rseucl
                endif
              endif
            endif
          endif

	  if (noprec .eq. 1) then
            if (mod(ilin(19),10000) .lt. 1000) then
              if (lsym) then
	        if (lsmo) then
	          call LL3ASX(mat,xs,work(1),work(l+1),ptrmbk,
     #                        lmatbk,ptrinf,l,nproc,tid,myproc,jump,
     #                        ladd,lmat,lindex,ia1,info,index,nmsg)
                else
	          call LL3ASX(mat,x,work(1),work(l+1),ptrmbk,
     #                        lmatbk,ptrinf,l,nproc,tid,myproc,jump,
     #                        ladd,lmat,lindex,ia1,info,index,nmsg)
                endif
              else
	        if (lsmo) then
                  call LL3AX(LL6AX,mat,xs,work(1),work(l+1),ptrmbk,
     #                       lmatbk,ptrinf,l,nproc,tid,myproc,jump,
     #                       ladd,lmat,lindex,ia1,info,index,nmsg)
                else
                  call LL3AX(LL6AX,mat,x,work(1),work(l+1),ptrmbk,
     #                       lmatbk,ptrinf,l,nproc,tid,myproc,jump,
     #                       ladd,lmat,lindex,ia1,info,index,nmsg)
                endif
              endif
              imvm=imvm+1
              include "norec.h"
              do 113 i=1,lmyblk
                work(i) = (work(i)-prec(ibNb+i-1))/prec(i)
  113         continue
	    endif
	  endif
          call LL4RNM(lmyblk,ione,l,work,rsmax,help,
     #                myproc,nproc,tid,nmsg)
          if (rsmax .le. epslin*bmax) then
	    if (noprec .eq. 0) then
	     if (ilin(13) .lt. 0) write(lout,1400) rsmax,epslin*bmax
	     if (ilin(13) .gt. 0) then
	       if (myproc .eq. 1) write(lout,1400) rsmax,epslin*bmax
             endif
            else
	     if (ilin(13) .lt. 0) write(lout,1410) rsmax,epslin*bmax
	     if (ilin(13) .gt. 0) then
	       if (myproc .eq. 1) write(lout,1410) rsmax,epslin*bmax
             endif
            endif
	    branch = 1
          else
            epsrel = 0.7 * rseucl * bmax * epsrel /(beucl * rsmax)
	    if (noprec .eq. 0) then
	     if (ilin(13).lt.0) write(lout,1300) rsmax, epslin*bmax,
     #                                           epsrel*beucl
	     if (ilin(13) .gt. 0) then
	       if (myproc.eq.1) write(lout,1300) rsmax, epslin*bmax,
     #                                           epsrel*beucl
             endif
            else
	     if (ilin(13).lt.0) write(lout,1310) rsmax, epslin*bmax,
     #                                           epsrel*beucl
	     if (ilin(13) .gt. 0) then
	       if (myproc.eq.1) write(lout,1310) rsmax, epslin*bmax,
     #                                           epsrel*beucl
             endif
            endif
          endif
        endif

	if (imvm .ge. ilin(2)) then
	  branch = 2
          if (count .lt. idoku1) then
            if (ilin(13) .le. -2) then
              if (mod(ilin(19),10000) .ge. 1000) then
                write(lout,1201) count,imvm,rseucl,rnorm,esnorm,enorm
              else
	        write(lout,1200) count,imvm,rseucl
              endif
            elseif ((ilin(13) .ge. 2) .and. (myproc .eq. 1)) then
              if (mod(ilin(19),10000) .ge. 1000) then
                write(lout,1201) count,imvm,rseucl,rnorm,esnorm,enorm
	        if (mod(ilin(19),100000) .ge. 10000) then
	          write(unit,*) count,imvm,rseucl,rnorm,esnorm,enorm
                endif
              else
	        write(lout,1200) count,imvm,rseucl
	        if (mod(ilin(19),100000) .ge. 10000) then
	          write(unit,*) count,imvm,rseucl
                endif
              endif
            endif
          endif
        endif

        if (count .ge. idoku1) then
	  idoku1 = count + abs(ilin(13))
        endif

C***  SET ONLY FOR BICO
	rseucn = rseucl
      endif
C**                                                                 ***
C**                                                                 ***
C**** END OF CALCULATION                                            ***
C**   ------------------                                            ***
C**                                                                 ***
  999 r e t u r n
 1000 format('  Starting defect is zero; initial guess is the solution'
     &/' ')
 1111 format('  Starting defect   (L2-Norm) :',g12.4,a10/
     &       '  Stopping criterion     (L2) :',g12.4,'   Method ')
 1211 format('      ||s||:',e8.2,'  ||r||:',e8.2,'  ||es||:',e8.2,
     &       '||e||:',e8.2/'  Stopping criterion (L2) :',g12.4,
     &        a10,'Method')
 1200 format('  Step:',i8,'   MVMs:',i8,'   Defect:',g12.4)
 1201 format('  Step:',i8,'   MVMs:',i8/
     &       '      ||s||:',e8.2,'  ||r||:',e8.2,'  ||es||:',e8.2,
     &       '||e||:',e8.2)
 1300 format('  Maximum of defect (normalized system) >',
     &       ' epslin*||(PL)*N*b||max'/33x,e8.2,' > ',e8.2,
     &       /'  New stopping criterion (L2) :',e10.4)
 1310 format('  Maximum of defect (original system) >',
     &       ' epslin*||b||max'/31x,e8.2,' > ',e8.2,
     &       /'  New stopping criterion (L2) :',e10.4)
 1400 format('  Maximum of defect (normalized system) <',
     &       ' epslin*||(PL)*N*b||max'/33x,e8.2,' < ',e8.2,
     &       /'  LSOLPP iteration complete.')
 1410 format('  Maximum of defect (original system) <',
     &       ' epslin*||b||max'/31x,e8.2,' < ',e8.2,
     &       /'  LSOLPP iteration complete.')
C**                                                                 ***
C-----END OF LL3AXB----------------------------------------------------
      e    n    d
