C:::::      ,,,,,LL4RNM.....
C
C
C
C**********************************************************************
C        1         2         3         4         5         6         7*
C**********************************************************************
C**                                                                 ***
C**                                                                 ***
      subroutine LL4RNM(nc,n,l,v,maxnrm,rwork,
     #                  myproc,nproc,tids,nmsg)
C**                                                                 ***
C**                                                                 ***
C**********************************************************************
C**                                                                 ***
C**      L L 4 R N M   computes the maximum norm of n vectors       ***
C**                    of length nc, which are stored in the        ***
C**                    vector field v.                              ***
C**                                                                 ***
C**********************************************************************
C**                                                                 ***
!**    Copyright (C) 1994 University Karlsruhe                      ***
!**    Programmer: H. Haefner                                       ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
C**                                                                 ***
C**********************************************************************
C**                                                                 ***
      USE constants

      implicit none
C**                                                                 ***
C**-----------------------------------------------------------------***
C**                                                                 ***
C**      FORMAL PARAMETERS :                                        ***
C**      -----------------                                          ***
C**                                                                 ***
      integer            nc,n,l,myproc,nproc,nmsg 
      integer            tids(nproc)
C**                                                                 ***
      double precision   v(l,n),maxnrm(n),rwork(n)
C**                                                                 ***
C**-----------------------------------------------------------------***
C**                                                                 ***
C**   LIST OF FORMAL PARAMETERS :                                   ***
C**   -------------------------                                     ***
C**                                                                 ***
C--------I------I-----I------------------------------------------------
C NAME   I TYPE I I/O I     MEANING
C--------I------I-----I------------------------------------------------
C--------I------I-----I------------------------------------------------
C nc     I  I   I in  I  number of elements of the vectors in v
C--------I------I-----I------------------------------------------------
C n      I  I   I in  I  number of vectors in vector field v
C--------I------I-----I------------------------------------------------
C l      I  I   I in  I  dimension of one vector in v
C--------I------I-----I------------------------------------------------
C v      I  R   I in  I  vector set                    array : v(l,n)
C--------I------I-----I------------------------------------------------
C rwork  I  R   I  -  I  real work array           array : rwork(n)
C--------I------I-----I------------------------------------------------
C maxnrm I  R   I out I  maximum norms             array : maxnrm(n)
C--------I------I-----I------------------------------------------------
C nproc  I  I   I in  I  number of processors (processes)
C--------I------I-----I------------------------------------------------
C myproc I  I   I in  I  logical process(or) number
C--------I------I-----I------------------------------------------------
C tids   I  I   I in  I  (see Manual pages <lsolpp>)
C--------I------I-----I------------------------------------------------
C nmsg   I  I   I in  I  message counter
C--------I------I-----I------------------------------------------------
C**                                                                 ***
C**                                                                 ***
C**        LOCAL PARAMETERS :                                       ***
C**        ----------------                                         ***
C**                                                                 ***
      integer           i,il,lstr
      external          LL9RNM
C**                                                                 ***
C**-----------------------------------------------------------------***
C**                                                                 ***
C
C
C**** START OF CALCULATION :
C     ---------------------
C
      do 10 i=1,n
        maxnrm(i) = abs(v(1,i))
          do 10 il=2,nc
            maxnrm(i) = max(maxnrm(i),abs(v(il,i)))
   10 continue
C**                                                                 ***
C**   REDUCTION AND BROADCASTING OF MAXIMAL VALUES                  ***
C**                                                                 ***
      if (nproc .ne. 1) then
        lstr = n*ireal
        call REDUCE(LL9RNM,rwork,maxnrm,lstr,myproc,nproc,tids,nmsg)
      endif
C
C**** END OF CALCULATION
C     ------------------
C
      r e t u r n
C-----END OF LL4RNM---------------------------------------------------
      e    n    d
