C:::::      ,,,,,LL8OPS.....
C
C
C
C**********************************************************************
C        1         2         3         4         5         6         7*
C**********************************************************************
C**                                                                 ***
C**                                                                 ***
      subroutine LL8OPS(l,rlin,ilin,ispoly,ms,noop,nproc)
C**                                                                 ***
C**                                                                 ***
C**********************************************************************
C**                                                                 ***
C**      L L 8 O P S   computes the number of operations for the    ***
C**                    iterative solvers.                           ***
C**                                                                 ***
C**********************************************************************
C**                                                                 ***
!**    Copyright (C) 1997 University Karlsruhe                      ***
!**    Programmer: H. Haefner                                       ***
!**                                                                 ***
!**    This file is part of LINSOL.                                 ***
!**    LINSOL is free software; you can redistribute it and/or      ***
!**    modify it under the terms of the GNU General Public License  ***
!**    as published by the Free Software Foundation; either         ***
!**    version 2 of the License, or (at your option) any later      ***
!**    version.                                                     ***
!**                                                                 ***
!**    LINSOL is distributed in the hope that it will be useful,    ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty  ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      ***
!**    See the GNU General Public License for more details.         ***
!**                                                                 ***
!**    You should have received a copy of the GNU General Public    ***
!**    License along with LINSOL; if not, write to the Free         ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,      ***
!**    Boston, MA  02111-1307, USA.                                 ***
C**                                                                 ***
C**********************************************************************
C**                                                                 ***
      USE constants

      implicit none
C**                                                                 ***
C**-----------------------------------------------------------------***
C**                                                                 ***
C**      FORMAL PARAMETERS :                                        ***
C**      -----------------                                          ***
C**                                                                 ***
      integer            ilin(nilin)
      integer            l,ispoly(4),ms,nproc
      double precision   rlin(nrlin),noop
C**                                                                 ***
C**-----------------------------------------------------------------***
C**                                                                 ***
C**   LIST OF FORMAL PARAMETERS :                                   ***
C**   -------------------------                                     ***
C**                                                                 ***
C--------I------I-----I------------------------------------------------
C NAME   I TYPE I I/O I     MEANING
C--------I------I-----I------------------------------------------------
C--------I------I-----I------------------------------------------------
Cilin(1) I  I   I in  I  user-defined method selection parameter
C--------I------I-----I------------------------------------------------
Cilin(10)I  I   I in  I  number of matrix-vector-multiplications
C--------I------I-----I------------------------------------------------
Cilin(30)I  I   I in  I  global size of matrix
C--------I------I-----I------------------------------------------------
C ms     I  I   I in  I  internal method selection parameter
C--------I------I-----I------------------------------------------------
C l      I  I   I in  I  length of the above mentioned arrays
C--------I------I-----I------------------------------------------------
Cispoly  I  I   I in  I  number of iteration steps for polyalg. (PA)
C--------I------I-----I------------------------------------------------
C noop   I  DP  I out I  number of operations for iterative solvers
C--------I------I-----I------------------------------------------------
C**                                                                 ***
C**                                                                 ***
C**        LOCAL PARAMETERS :                                       ***
C**        ----------------                                         ***
C**                                                                 ***
      integer           imvm,istep
      double precision  lglb, glpL, glpU
      logical           lsmo
C**                                                                 ***
C----------------------------------------------------------------------
C
C
C**** START OF CALCULATION :
C     ---------------------
C
      imvm  = ilin(10)
      istep = ilin(11)
      lsmo  = ilin(18) .ne. 0

      lglb = DBLE(ilin(36))
      if (nproc == 1) then
	glpL = DBLE(ilin(37))
	glpU = DBLE(ilin(38))
      else
	glpL = rlin(9)
	glpU = rlin(10)
      endif

C**** Performing smoothing needs 10*l+1 operations 
C**** Performing smoothing for QMR needs 8*l+6 operations 
C**** Performing stopping criterion needs 4*l operations (start) and
C****            2*l operations each time it is called during iteration
C
C
      if ((ms .eq. 1) .and. (ilin(1) .eq. 1)) then
	if (ilin(8) < 63) then
	  if (lsmo) then
            noop = 2.d0*DBLE(imvm)*DBLE(ilin(30)) +
     &             DBLE(istep)*DBLE(31*l+13.5) +
     &             istep/20*DBLE(12*l+1) + 4*l
	  else
            noop = 2.d0*DBLE(imvm)*DBLE(ilin(30)) +
     &             DBLE(istep)*DBLE(31*l+13.5) +
     &             istep/20*DBLE(2*l) + 4*l
	  endif 
	else
	  if (lsmo) then
            noop = 2.d0*DBLE(imvm)*DBLE(ilin(30)) +
     &             DBLE(istep)*DBLE(31*l+13.5) +
     &             istep/20*DBLE(12*l+1) + 4*l +
     &             lglb + 2.d0*(glpU - lglb) + 2.d0*glpL
	  else
            noop = 2.d0*DBLE(imvm)*DBLE(ilin(30)) +
     &             DBLE(istep)*DBLE(31*l+13.5) +
     &             istep/20*DBLE(2*l) + 4*l +
     &             lglb + 2.d0*(glpU - lglb) + 2.d0*glpL
	  endif 
	endif 
        goto 999
      endif

      if ((ms .eq. 2) .and. (ilin(1) .eq. 2)) then
	if (ilin(8) < 63) then
	  if (lsmo) then
            noop = 2.d0*DBLE(imvm)*DBLE(ilin(30)) +
     &             DBLE(istep)*DBLE(16*l+3) +
     &             DBLE(istep)*DBLE(12*l+1) + 4*l
	  else
            noop = 2.d0*DBLE(imvm)*DBLE(ilin(30)) +
     &             DBLE(istep)*DBLE(16*l+3) +
     &             DBLE(istep)*DBLE(2*l) + 4*l
	  endif
	else
	  if (lsmo) then
            noop = 2.d0*DBLE(imvm)*DBLE(ilin(30)) +
     &             DBLE(istep)*DBLE(16*l+3) +
     &             DBLE(istep)*DBLE(12*l+1) + 4*l +
     &             lglb + 2.d0*(glpU - lglb) + 2.d0*glpL
	  else
            noop = 2.d0*DBLE(imvm)*DBLE(ilin(30)) +
     &             DBLE(istep)*DBLE(16*l+3) +
     &             DBLE(istep)*DBLE(2*l) + 4*l +
     &             lglb + 2.d0*(glpU - lglb) + 2.d0*glpL
	  endif
	endif 
        goto 999
      endif

      if (ms .eq. 3) then
	if (lsmo) then
	  noop = 2.d0*DBLE(imvm)*DBLE(ilin(30)) +
     &           DBLE(istep)*DBLE(20*l+5) +
     &           DBLE(istep)*DBLE(12*l+1) + 4*l
	else
	  noop = 2.d0*DBLE(imvm)*DBLE(ilin(30)) +
     &           DBLE(istep)*DBLE(20*l+5) +
     &           DBLE(istep)*DBLE(2*l) + 4*l
	endif
        goto 999
      endif

      if ((ms .eq. 4) .and. (ilin(1) .eq. 4)) then
	if (lsmo) then
          noop = 2.d0*DBLE(imvm)*DBLE(ilin(30)) +
     &           DBLE(istep)*DBLE(16*l+5) +
     &           DBLE(istep)*DBLE(12*l+1) + 4*l
	else
          noop = 2.d0*DBLE(imvm)*DBLE(ilin(30)) +
     &           DBLE(istep)*DBLE(16*l+5) +
     &           DBLE(istep)*DBLE(2*l) + 4*l
	endif
        goto 999
      endif

      if (ms .eq. 5) then
	if (lsmo) then
	  noop = 2.d0*DBLE(imvm)*DBLE(ilin(30)) +
     &           DBLE(istep)*DBLE(17*l+2) +
     &           DBLE(istep)*DBLE(12*l+1) + 4*l
	else
	  noop = 2.d0*DBLE(imvm)*DBLE(ilin(30)) +
     &           DBLE(istep)*DBLE(17*l+2) +
     &           DBLE(istep)*DBLE(2*l) + 4*l
	endif
        goto 999
      endif

      if (ms .eq. 6) then
	if (ilin(8) < 63) then
	  if (lsmo) then
	    noop = 2.d0*DBLE(imvm)*DBLE(ilin(30)) +
     &             DBLE(istep)*DBLE(16*l+3) +
     &             DBLE(istep)*DBLE(10*l+6) + 4*l
	  else
	    noop = 2.d0*DBLE(imvm)*DBLE(ilin(30)) +
     &             DBLE(istep)*DBLE(16*l+3) +
     &             DBLE(istep)*DBLE(2*l) + 4*l
	  endif
	else
	  if (lsmo) then
	    noop = 2.d0*DBLE(imvm)*DBLE(ilin(30)) +
     &             DBLE(istep)*DBLE(16*l+3) +
     &             DBLE(istep)*DBLE(10*l+6) + 4*l +
     &             lglb + 2.d0*(glpU - lglb) + 2.d0*glpL
	  else
	    noop = 2.d0*DBLE(imvm)*DBLE(ilin(30)) +
     &             DBLE(istep)*DBLE(16*l+3) +
     &             DBLE(istep)*DBLE(2*l) + 4*l +
     &             lglb + 2.d0*(glpU - lglb) + 2.d0*glpL
	  endif
	endif
        goto 999
      endif

      if (ms .eq. 7) then
	if (ilin(8) < 63) then
	  if (lsmo) then
	    noop = 2.d0*DBLE(imvm)*DBLE(ilin(30)) +
     &             DBLE(istep)*DBLE(46*l+17) +
     &             DBLE(istep)*DBLE(12*l+1) + 4*l
	  else
	    noop = 2.d0*DBLE(imvm)*DBLE(ilin(30)) +
     &             DBLE(istep)*DBLE(46*l+17) +
     &             DBLE(istep)*DBLE(2*l) + 4*l
	  endif
	else
	  if (lsmo) then
	    noop = 2.d0*DBLE(imvm)*DBLE(ilin(30)) +
     &             DBLE(istep)*DBLE(46*l+17) +
     &             DBLE(istep)*DBLE(12*l+1) + 4*l +
     &             lglb + 2.d0*(glpU - lglb) + 2.d0*glpL
	  else
	    noop = 2.d0*DBLE(imvm)*DBLE(ilin(30)) +
     &             DBLE(istep)*DBLE(46*l+17) +
     &             DBLE(istep)*DBLE(2*l) + 4*l +
     &             lglb + 2.d0*(glpU - lglb) + 2.d0*glpL
	  endif
	endif
        goto 999
      endif

      if (ms .eq. 9) then
	if (lsmo) then
	  noop = 2.d0*DBLE(imvm)*DBLE(ilin(30)) +
     &           DBLE(istep)*DBLE(16*l+5) +
     &           DBLE(istep)*DBLE(12*l+1) + 4*l
	else
	  noop = 2.d0*DBLE(imvm)*DBLE(ilin(30)) +
     &           DBLE(istep)*DBLE(16*l+5) +
     &           DBLE(istep)*DBLE(2*l) + 4*l
	endif
        goto 999
      endif

      if (ms .eq. 15 .and. ilin(1) .eq. 15) then
	if (lsmo) then
	  noop = 2.d0*DBLE(imvm)*DBLE(ilin(30)) +
     &           DBLE(istep)*DBLE(28*l+3) +
     &           istep/5*DBLE(12*l+1+5*l+1) + 4*l + 4*l+1
	else
	  noop = 2.d0*DBLE(imvm)*DBLE(ilin(30)) +
     &           DBLE(istep)*DBLE(28*l+3) +
     &           istep/5*DBLE(2*l+5*l+1) + 4*l + 4*l+1
	endif
        goto 999
      endif

      if (ms .eq. 16 .and. ilin(1) .eq. 16) then
	if (lsmo) then
	  noop = 2.d0*DBLE(imvm)*DBLE(ilin(30)) +
     &           DBLE(istep)*DBLE(94*l+3) +
     &           istep/20*DBLE(12*l+1+5*l+1) + 4*l + 4*l+1
	else
	  noop = 2.d0*DBLE(imvm)*DBLE(ilin(30)) +
     &           DBLE(istep)*DBLE(94*l+3) +
     &           istep/20*DBLE(2*l+5*l+1) + 4*l + 4*l+1
	endif
        goto 999
      endif

      if (ms .eq. 100 .and. ilin(1) .eq. 100) then
	if (ilin(8) < 63) then
	  if (lsmo) then
	    noop = 2.d0*DBLE(imvm)*(2.d0*DBLE(ilin(30)-l)+l)+
     &             DBLE(istep)*DBLE(16*l+5) +
     &             DBLE(istep)*DBLE(12*l+1) + 4*l
	  else
	    noop = 2.d0*DBLE(imvm)*(2.d0*DBLE(ilin(30)-l)+l)+
     &             DBLE(istep)*DBLE(16*l+5) +
     &             DBLE(istep)*DBLE(2*l) + 4*l
	  endif
	else
	  if (lsmo) then
	    noop = 2.d0*DBLE(imvm)*(2.d0*DBLE(ilin(30)-l)+l)+
     &             DBLE(istep)*DBLE(16*l+5) +
     &             DBLE(istep)*DBLE(12*l+1) + 4*l +
     &             2.d0*lglb + 2.d0*(glpL - lglb)
	  else
	    noop = 2.d0*DBLE(imvm)*(2.d0*DBLE(ilin(30)-l)+l)+
     &             DBLE(istep)*DBLE(16*l+5) +
     &             DBLE(istep)*DBLE(2*l) + 4*l +
     &             2.d0*lglb + 2.d0*(glpL - lglb)
	  endif
	endif
        goto 999
      endif

      if (ms .eq. 101 .and. ilin(1) .eq. 101) then
	if (lsmo) then
	  noop = 2.d0*DBLE(imvm)*(2.d0*DBLE(ilin(30)-l)+l)+
     &           DBLE(istep)*DBLE(22*l+3) +
     &           istep/abs(ilin(13))*DBLE(12*l+1+3*l+1) + 4*l + 4*l+1
	else
	  noop = 2.d0*DBLE(imvm)*(2.d0*DBLE(ilin(30)-l)+l)+
     &           DBLE(istep)*DBLE(22*l+3) +
     &           istep/abs(ilin(13))*DBLE(3*l) + 4*l + 4*l+1
	endif
        goto 999
      endif

      if (ilin(1) .eq. 200 .or. ilin(1) .eq. 201 .or. ilin(1) .eq. 202
     &    .or. ilin(1) .eq. 203 .or. ilin(1) .eq. 210 .or.
     &    ilin(1) .eq. 211) then
	if (lsmo) then
          noop = 2.d0*DBLE(imvm)*DBLE(ilin(30)) +
     &           DBLE(ispoly(1))*DBLE(31*l+13.5) +
     &           ispoly(1)/20*DBLE(12*l+1) + 4*l +
     &           DBLE(ispoly(2))*DBLE(16*l+3) +
     &           DBLE(ispoly(2))*DBLE(12*l+1) + 4*l +
     &           DBLE(ispoly(3))*DBLE(28*l+3) +
     &           ispoly(3)/5*DBLE(12*l+1+5*l+1) + 4*l + 4*l+1 +
     &           DBLE(ispoly(4))*DBLE(16*l+5) +
     &           DBLE(ispoly(4))*DBLE(12*l+1) + 4*l
	else
          noop = 2.d0*DBLE(imvm)*DBLE(ilin(30)) +
     &           DBLE(ispoly(1))*DBLE(31*l+13.5) +
     &           ispoly(1)/20*DBLE(2*l) + 4*l +
     &           DBLE(ispoly(2))*DBLE(16*l+3) +
     &           DBLE(ispoly(2))*DBLE(2*l) + 4*l +
     &           DBLE(ispoly(3))*DBLE(28*l+3) +
     &           ispoly(3)/5*DBLE(2*l+5*l+1) + 4*l + 4*l+1 +
     &           DBLE(ispoly(4))*DBLE(16*l+5) +
     &           DBLE(ispoly(4))*DBLE(2*l) + 4*l
	endif
        goto 999
      endif

      if (ilin(1) .eq. 204 .or. ilin(1) .eq. 212) then
	if (lsmo) then
          noop = 2.d0*DBLE(imvm)*DBLE(ilin(30)) +
     &           DBLE(ispoly(1))*DBLE(31*l+13.5) +
     &           ispoly(1)/20*DBLE(12*l+1) + 4*l +
     &           DBLE(ispoly(2))*DBLE(16*l+3) +
     &           DBLE(ispoly(2))*DBLE(12*l+1) + 4*l +
     &           DBLE(ispoly(3))*DBLE(94*l+3) +
     &           ispoly(3)/20*DBLE(12*l+1+5*l+1) + 4*l + 4*l+1 +
     &           DBLE(ispoly(4))*DBLE(16*l+5) +
     &           DBLE(ispoly(4))*DBLE(12*l+1) + 4*l
	else
          noop = 2.d0*DBLE(imvm)*DBLE(ilin(30)) +
     &           DBLE(ispoly(1))*DBLE(31*l+13.5) +
     &           ispoly(1)/20*DBLE(2*l) + 4*l +
     &           DBLE(ispoly(2))*DBLE(16*l+3) +
     &           DBLE(ispoly(2))*DBLE(2*l) + 4*l +
     &           DBLE(ispoly(3))*DBLE(94*l+3) +
     &           ispoly(3)/20*DBLE(2*l+5*l+1) + 4*l + 4*l+1 +
     &           DBLE(ispoly(4))*DBLE(16*l+5) +
     &           DBLE(ispoly(4))*DBLE(2*l) + 4*l
	endif
        goto 999
      endif

      if (ilin(1) .eq. 220 .or. ilin(1) .eq. 221) then
	if (lsmo) then
	  noop = 2.d0*DBLE(imvm)*(2.d0*DBLE(ilin(30)-l)+l)+
     &           DBLE(ispoly(3))*DBLE(22*l+3) +
     &           ispoly(3)/abs(ilin(13))*DBLE(12*l+1+3*l+1)+8*l+1 +
     &           DBLE(ispoly(4))*DBLE(16*l+5) +
     &           DBLE(ispoly(4))*DBLE(12*l+1) + 4*l
	else
	  noop = 2.d0*DBLE(imvm)*(2.d0*DBLE(ilin(30)-l)+l)+
     &           DBLE(ispoly(3))*DBLE(22*l+3) +
     &           ispoly(3)/abs(ilin(13))*DBLE(3*l) + 8*l + 1 +
     &           DBLE(ispoly(4))*DBLE(16*l+5) +
     &           DBLE(ispoly(4))*DBLE(2*l) + 4*l
	endif
        goto 999
      endif

  999 continue 
C
C**** END OF CALCULATION
C     ------------------
C
      r e t u r n
C-----END OF LL8OPS---------------------------------------------------
      e    n    d
