!***********************************************************************
!        1         2         3         4         5         6         7 *
!***********************************************************************
!**                                                                  ***
!**                                                                  ***
      subroutine LL8STATP(l,lglobal,lmat,mat,lindex,lprec,liprec,prec,
     &                    iprec,info,ia1,nvt,ilin,rlin,lsym,maxnvp,
     &                    idoku,lout,myproc,nproc,tids,nmsg)
!**                                                                  ***
!**                                                                  ***
!***********************************************************************
!**                                                                  ***
!**    LL8STATP        enters statistical parameters into array      ***
!**                    <ilin> and prints statistical information     ***
!**                                                                  ***
!***********************************************************************
!**                                                                  ***
!**    Copyright (C) 1997 University Karlsruhe                       ***
!**    Programmer: H. Haefner                                        ***
!**                                                                  ***
!**    This file is part of LINSOL.                                  ***
!**    LINSOL is free software; you can redistribute it and/or       ***
!**    modify it under the terms of the GNU General Public License   ***
!**    as published by the Free Software Foundation; either          ***
!**    version 2 of the License, or (at your option) any later       ***
!**    version.                                                      ***
!**                                                                  ***
!**    LINSOL is distributed in the hope that it will be useful,     ***
!**    but WITHOUT ANY WARRANTY; without even the implied warranty   ***
!**    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.       ***
!**    See the GNU General Public License for more details.          ***
!**                                                                  ***
!**    You should have received a copy of the GNU General Public     ***
!**    License along with LINSOL; if not, write to the Free          ***
!**    Software Foundation, Inc., 59 Temple Place - Suite 330,       ***
!**    Boston, MA  02111-1307, USA.                                  ***
!**                                                                  ***
!***********************************************************************
!**                                                                  ***
      USE constants

      implicit none
!**                                                                  *** 
!***********************************************************************
!**                                                                  ***
!**    Formal Parameters :                                           ***
!**                                                                  ***
!***********************************************************************
      integer            l,lglobal,lmat,lindex,lout,ia1,idoku,nvt
      integer            lprec,liprec
      integer            maxnvp,myproc,nproc,nmsg
      integer            info(ia1,ia2),ilin(nilin),tids(nproc)
      integer            iprec(liprec)
      double precision   mat(lmat),prec(lprec),rlin(nrlin)
      logical            lsym
!***********************************************************************
!**                                                                  ***
!**    List of Formal Parameters :                                   ***
!**                                                                  ***
!**                                                                  ***
!---------I------I-----I------------------------------------------------
! NAME    I TYPE I I/O I MEANING
!---------I------I-----I------------------------------------------------
! l       I int  I in  I dimension of the matrix
!---------I------I-----I------------------------------------------------
! lmat    I int  I in  I number of nonzero elements
!---------I------I-----I------------------------------------------------
! lindex  I int  I in  I number of indices
!---------I------I-----I------------------------------------------------
! nvt     I int  I in  I total number of vector-terms
!---------I------I-----I------------------------------------------------
! ia1     I int  I in  I 1.dimension of info-array
!---------I------I-----I------------------------------------------------
! info    I int  I in  I array with matrix-structure-information
!---------I------I-----I------------------------------------------------
! ilin    I int  I out I array with LINSOL-parameters
!---------I------I-----I------------------------------------------------
!**                                                                  ***
!**                                                                  ***
!**    Local Variables :                                             ***
!**                                                                  ***
      integer         i,j,maxi,maxind,lpL,lpU,ih(102)
      integer*8       glpL,glpU,ih8(6)
      integer         imin(ntyp),imax(ntyp),icnt(ntyp)
      double precision  ravg(ntyp), rh(2*ntyp)

!***********************************************************************
!**                                                                  ***
!**    List of Local Variables :                                     ***
!**                                                                  ***
!**                                                                  ***
!---------I------I-----I------------------------------------------------
! NAME    I TYPE I I/O I MEANING
!---------I------I-----I------------------------------------------------
!  imin   I int  I  -  I Minimal v-length of vector terms of type 2-ntyp
!---------I------I-----I------------------------------------------------
!  ravg   I DP   I  -  I Average v-length of vector terms of type 2-ntyp
!---------I------I-----I------------------------------------------------
!  imax   I int  I  -  I Maximal v-length of vector terms of type 2-ntyp
!---------I------I-----I------------------------------------------------
!***********************************************************************


!***********************************************************************
!**    Start of Calculation                                          ***
!***********************************************************************


!***********************************************************************
!**    Statistics, if not printed in main program, on one processor  ***
!***********************************************************************

      if ((ilin(50) .ne. -1 .or. ilin(8) >= 63) .and.
     &    (nproc .eq. 1)) then

       imin = l+1
       ravg = 0.0
       imax = 0
       icnt = 0

!** Vector length statistics
       do 110 i=1,nvt
	if (info(i,1) .eq. 1 .or. info(i,1) .eq. 10) then
	  do j=1,info(i,3)
	    if (mat(info(i,2)+j) /= 0.d0) imax(1) = imax(1) + 1 
	  enddo
	  ravg(1) = ravg(1)+info(i,3)
	  icnt(1) = icnt(1)+1
        endif
	if (info(i,1) .eq. 2 .or. info(i,1) .eq. 20) then
	  if (info(i,3) .lt. imin(2)) imin(2) = info(i,3)
	  if (info(i,3) .gt. imax(2)) imax(2) = info(i,3)
	  ravg(2) = ravg(2)+info(i,3)
	  icnt(2) = icnt(2)+1
        endif
	if (info(i,1) .eq. 3 .or. info(i,1) .eq. 30) then
	  if (info(i,3) .lt. imin(3)) imin(3) = info(i,3)
	  if (info(i,3) .gt. imax(3)) imax(3) = info(i,3)
	  ravg(3) = ravg(3)+info(i,3)
	  icnt(3) = icnt(3)+1
        endif
	if (info(i,1) .eq. 4 .or. info(i,1) .eq. 40) then
	  if (info(i,3) .lt. imin(4)) imin(4) = info(i,3)
	  if (info(i,3) .gt. imax(4)) imax(4) = info(i,3)
	  ravg(4) = ravg(4)+info(i,3)
	  icnt(4) = icnt(4)+1
        endif
	if (info(i,1) .eq. 5 .or. info(i,1) .eq. 50) then
	  if (info(i,3) .lt. imin(5)) imin(5) = info(i,3)
	  if (info(i,3) .gt. imax(5)) imax(5) = info(i,3)
	  ravg(5) = ravg(5)+info(i,3)
	  icnt(5) = icnt(5)+1
        endif
	if (info(i,1) .eq. 6 .or. info(i,1) .eq. 60) then
	  if (info(i,3) .lt. imin(6)) imin(6) = info(i,3)
	  if (info(i,3) .gt. imax(6)) imax(6) = info(i,3)
	  ravg(6) = ravg(6)+info(i,3)
	  icnt(6) = icnt(6)+1
        endif
	if (info(i,1) .eq. 7 .or. info(i,1) .eq. 70) then
	  if (info(i,3) .lt. imin(7)) imin(7) = info(i,3)
	  if (info(i,3) .gt. imax(7)) imax(7) = info(i,3)
	  ravg(7) = ravg(7)+info(i,3)
	  icnt(7) = icnt(7)+1
        endif
	if (info(i,1) .eq. 8 .or. info(i,1) .eq. 80) then
	  if (info(i,3) .lt. imin(8)) imin(8) = info(i,3)
	  if (info(i,3) .gt. imax(8)) imax(8) = info(i,3)
	  ravg(8) = ravg(8)+info(i,3)
	  icnt(8) = icnt(8)+1
        endif
	if (info(i,1) .eq. 9 .or. info(i,1) .eq. 90) then
	  if (info(i,3) .lt. imin(9)) imin(9) = info(i,3)
	  if (info(i,3) .gt. imax(9)) imax(9) = info(i,3)
	  ravg(9) = ravg(9)+info(i,3)
	  icnt(9) = icnt(9)+1
        endif
	if (info(i,1) .eq. 11 .or. info(i,1) .eq. 110) then
	  if (info(i,3) .lt. imin(11)) imin(11) = info(i,3)
	  if (info(i,3) .gt. imax(11)) imax(11) = info(i,3)
	  ravg(11) = ravg(11)+info(i,3)
	  icnt(11) = icnt(11)+1
        endif
	if (info(i,3) .lt. imin(10)) imin(10) = info(i,3)
	if (info(i,3) .gt. imax(10)) imax(10) = info(i,3)
	ravg(10) = ravg(10)+info(i,3)
  110  continue

       icnt(10) = nvt
       do 120 i=1,ntyp
	if (icnt(i) .eq. 0) imin(i) = 0
  120  continue
       do 130 i=1,ntyp
	if (icnt(i) .ne. 0) then
	  ravg(i) = ravg(i)/icnt(i)
	endif
  130  continue

       if (idoku .gt. 0) then
	write(lout,1000)
	write(lout,1001) lmat
	write(lout,1002) ilin(30)
	write(lout,1003) lindex
	write(lout,1004) ilin(31)
	write(lout,1005) ia1
	write(lout,1006) ia2
	if (ilin(8) >= 63) then
	  if (.not. lsym) then
	    lpU = iprec(iprec(10)+lglobal) +
     &            iprec(iprec(10)+2*lglobal) - iprec(iprec(10)+1)
	    ilin(38) = lpU
	  endif
	  lpL = iprec(iprec(13)+lglobal) +
     &          iprec(iprec(13)+2*lglobal) - iprec(iprec(13)+1)
	  ilin(37) = lpL
	  write(lout,1101) lprec
	  write(lout,1102) liprec
	  write(lout,1103) iprec(1)
	  if (.not. lsym) write(lout,1104) lpU
	  write(lout,1105) lpL
	  write(lout,1106) rlin(4)
	  if (lsym) then
	    write(lout,1110) DBLE(lpL)/DBLE(ilin(33)) + 0.1
	  else
	    write(lout,1110) DBLE(lpL+lpU)/DBLE(ilin(33)) + 0.1
	  endif
	  write(lout,1108) iprec(3)
	  write(lout,1109) ia2LU
	endif
	write(lout,1030) cachsz
	write(lout,1031) cusfac
	write(lout,1032) maxnvp
	write(lout,1008) nvt
	write(lout,1009) imin(10)
	write(lout,1010) ravg(10)
	write(lout,1011) imax(10)
	write(lout,1007)
	write(lout,1212) icnt(1)
	if (icnt(1) .ne. 0) then
	  write(lout,1010) ravg(1)
	  write(lout,1211) imax(1)
	endif
	write(lout,1007)
	write(lout,1012) icnt(2)
	if (icnt(2) .ne. 0) then
	  write(lout,1009) imin(2)
	  write(lout,1010) ravg(2)
	  write(lout,1011) imax(2)
	endif
	write(lout,1007)
	write(lout,1013) icnt(3)
	if (icnt(3) .ne. 0) then
	  write(lout,1009) imin(3)
	  write(lout,1010) ravg(3)
	  write(lout,1011) imax(3)
	endif
	write(lout,1007)
	write(lout,1014) icnt(4)
	if (icnt(4) .ne. 0) then
	  write(lout,1009) imin(4)
	  write(lout,1010) ravg(4)
	  write(lout,1011) imax(4)
	endif
	write(lout,1007)
	write(lout,1015) icnt(5)
	if (icnt(5) .ne. 0) then
	  write(lout,1009) imin(5)
	  write(lout,1010) ravg(5)
	  write(lout,1011) imax(5)
	endif
	write(lout,1007)
	write(lout,1016) icnt(6)
	if (icnt(6) .ne. 0) then
	  write(lout,1009) imin(6)
	  write(lout,1010) ravg(6)
	  write(lout,1011) imax(6)
	endif
	write(lout,1007)
	write(lout,1017) icnt(7)
	if (icnt(7) .ne. 0) then
	  write(lout,1009) imin(7)
	  write(lout,1010) ravg(7)
	  write(lout,1011) imax(7)
	endif
	write(lout,1007)
	write(lout,1018) icnt(8)
	if (icnt(8) .ne. 0) then
	  write(lout,1009) imin(8)
	  write(lout,1010) ravg(8)
	  write(lout,1011) imax(8)
	endif
	write(lout,1007)
	write(lout,1019) icnt(9)
	if (icnt(9) .ne. 0) then
	  write(lout,1009) imin(9)
	  write(lout,1010) ravg(9)
	  write(lout,1011) imax(9)
	endif
	write(lout,1007)
	write(lout,1020) icnt(11)
	if (icnt(11) .ne. 0) then
	  write(lout,1009) imin(11)
	  write(lout,1010) ravg(11)
	  write(lout,1011) imax(11)
	endif
	write(lout,1021)
       endif
      endif


!***********************************************************************
!**    Statistics on many processors                                 ***
!***********************************************************************

      if (nproc .gt. 1) then

       imin = l+1
       ravg = 0.0
       imax = 0
       icnt = 0

!** Vector length statistics
       do 160 i=1,nvt
	if (info(i,1) .eq. 1 .or. info(i,1) .eq. 10) then
	  do j=1,info(i,3)
	    if (mat(info(i,2)+j) /= 0.d0) imax(1) = imax(1) + 1 
	  enddo
	  ravg(1) = ravg(1)+info(i,3)
	  icnt(1) = icnt(1)+1
        endif
	if (info(i,1) .eq. 2 .or. info(i,1) .eq. 20) then
	  if (info(i,3) .lt. imin(2)) imin(2) = info(i,3)
	  if (info(i,3) .gt. imax(2)) imax(2) = info(i,3)
	  ravg(2) = ravg(2)+info(i,3)
	  icnt(2) = icnt(2)+1
        endif
	if (info(i,1) .eq. 3 .or. info(i,1) .eq. 30) then
	  if (info(i,3) .lt. imin(3)) imin(3) = info(i,3)
	  if (info(i,3) .gt. imax(3)) imax(3) = info(i,3)
	  ravg(3) = ravg(3)+info(i,3)
	  icnt(3) = icnt(3)+1
        endif
	if (info(i,1) .eq. 4 .or. info(i,1) .eq. 40) then
	  if (info(i,3) .lt. imin(4)) imin(4) = info(i,3)
	  if (info(i,3) .gt. imax(4)) imax(4) = info(i,3)
	  ravg(4) = ravg(4)+info(i,3)
	  icnt(4) = icnt(4)+1
        endif
	if (info(i,1) .eq. 5 .or. info(i,1) .eq. 50) then
	  if (info(i,3) .lt. imin(5)) imin(5) = info(i,3)
	  if (info(i,3) .gt. imax(5)) imax(5) = info(i,3)
	  ravg(5) = ravg(5)+info(i,3)
	  icnt(5) = icnt(5)+1
        endif
	if (info(i,1) .eq. 6 .or. info(i,1) .eq. 60) then
	  if (info(i,3) .lt. imin(6)) imin(6) = info(i,3)
	  if (info(i,3) .gt. imax(6)) imax(6) = info(i,3)
	  ravg(6) = ravg(6)+info(i,3)
	  icnt(6) = icnt(6)+1
        endif
	if (info(i,1) .eq. 7 .or. info(i,1) .eq. 70) then
	  if (info(i,3) .lt. imin(7)) imin(7) = info(i,3)
	  if (info(i,3) .gt. imax(7)) imax(7) = info(i,3)
	  ravg(7) = ravg(7)+info(i,3)
	  icnt(7) = icnt(7)+1
        endif
	if (info(i,1) .eq. 8 .or. info(i,1) .eq. 80) then
	  if (info(i,3) .lt. imin(8)) imin(8) = info(i,3)
	  if (info(i,3) .gt. imax(8)) imax(8) = info(i,3)
	  ravg(8) = ravg(8)+info(i,3)
	  icnt(8) = icnt(8)+1
        endif
	if (info(i,1) .eq. 9 .or. info(i,1) .eq. 90) then
	  if (info(i,3) .lt. imin(9)) imin(9) = info(i,3)
	  if (info(i,3) .gt. imax(9)) imax(9) = info(i,3)
	  ravg(9) = ravg(9)+info(i,3)
	  icnt(9) = icnt(9)+1
        endif
	if (info(i,1) .eq. 11 .or. info(i,1) .eq. 110) then
	  if (info(i,3) .lt. imin(11)) imin(11) = info(i,3)
	  if (info(i,3) .gt. imax(11)) imax(11) = info(i,3)
	  ravg(11) = ravg(11)+info(i,3)
	  icnt(11) = icnt(11)+1
        endif
	if (info(i,3) .lt. imin(10)) imin(10) = info(i,3)
	if (info(i,3) .gt. imax(10)) imax(10) = info(i,3)
	ravg(10) = ravg(10)+info(i,3)
  160  continue

       icnt(10) = nvt
       do 170 i=1,ntyp
	if (icnt(i) .eq. 0) imin(i) = 0
  170  continue
       do 180 i=1,ntyp
	if (icnt(i) .ne. 0) then
	  ravg(i) = ravg(i)/icnt(i)
	endif
  180  continue

       if (ilin(8) >= 63) then
	 ih(2) = iprec(iprec(13)+lglobal) +
     &           iprec(iprec(13)+2*lglobal) - iprec(iprec(13)+1)
	 if (lsym) then
	   ih(1) = 0
	 else
	   ih(1) = iprec(iprec(10)+lglobal) +
     &             iprec(iprec(10)+2*lglobal) - iprec(iprec(10)+1)
	 endif

         call LL4INM(1,2,1,ih(1),ih(3),ih(5),myproc,nproc,
     &               tids,nmsg)
	 lpU = ih(3)
	 lpL = ih(4)

         ih8(1) = ih(1)
         ih8(2) = ih(2)
         call LL4ISM8(1,2,1,ih8(1),ih8(3),ih8(5),myproc,nproc,
     &                tids,nmsg)
	 glpU = ih8(3)
	 rlin(10) = DBLE(glpU)
	 glpL = ih8(4)
	 rlin(9) = DBLE(glpL)
       endif

       ih(1) = ia1
       ih(2) = nvt
       ih(3) = icnt(1)
       ih(4) = icnt(2)
       ih(5) = icnt(3)
       ih(6) = icnt(4)
       ih(7) = icnt(5)
       ih(8) = icnt(6)
       ih(9) = icnt(7)
       ih(10) = icnt(8)
       ih(11) = icnt(9)
       ih(12) = icnt(11)
       ih(13) = imin(1)
       ih(14) = imin(2)
       ih(15) = imin(3)
       ih(16) = imin(4)
       ih(17) = imin(5)
       ih(18) = imin(6)
       ih(19) = imin(7)
       ih(20) = imin(8)
       ih(21) = imin(9)
       ih(22) = imin(10)
       ih(23) = imin(11)
       ih(24) = imax(1)
       ih(25) = imax(2)
       ih(26) = imax(3)
       ih(27) = imax(4)
       ih(28) = imax(5)
       ih(29) = imax(6)
       ih(30) = imax(7)
       ih(31) = imax(8)
       ih(32) = imax(9)
       ih(33) = imax(10)
       ih(34) = imax(11)
       call LL4INM(1,34,1,ih(1),ih(35),ih(69),myproc,nproc,
     &             tids,nmsg)

       call LL4RNM(1,ntyp,1,ravg(1),rh(1),rh(ntyp+1),myproc,nproc,
     &             tids,nmsg)


       if ((idoku .gt. 0) .and. (myproc .eq. 1)) then
	write(lout,1000)
	write(lout,1002) ilin(30)
	write(lout,1004) ilin(31)
	write(lout,1008) ilin(32)
	if (ilin(8) >= 63) then
	  write(lout,1101) lprec
	  write(lout,1102) liprec
	  write(lout,1103) iprec(1)
	  if (.not. lsym) write(lout,1111) glpU
	  write(lout,1112) glpL
	  if (.not. lsym) write(lout,1113) lpU
	  write(lout,1114) lpL
	  write(lout,1106) rlin(4)
	  if (lsym) then
	    write(lout,1110) DBLE(glpL)/DBLE(ilin(30))
	  else
	    write(lout,1110) DBLE(glpL+glpU)/DBLE(ilin(30))
	  endif
	  write(lout,1108) iprec(3)
	  write(lout,1109) ia2LU
	endif
	write(lout,1041) lmat
	write(lout,1042) ilin(33)
	write(lout,1043) lindex
	write(lout,1044) ilin(34)
	write(lout,1045) ih(1)
	write(lout,1006) ia2
	write(lout,1030) cachsz
	write(lout,1031) cusfac
	write(lout,1032) maxnvp
	write(lout,1048) ih(2)
	write(lout,1069) ih(22)
	write(lout,1070) rh(10)
	write(lout,1071) ih(33)
	write(lout,1007)
	write(lout,1252) ih(3)
	if (ih(3) .ne. 0) then
	  write(lout,1010) rh(1)
	  write(lout,1211) ih(24)
	endif
	write(lout,1007)
	write(lout,1052) ih(4)
	if (ih(4) .ne. 0) then
	  write(lout,1069) ih(14)
	  write(lout,1070) rh(2)
	  write(lout,1071) ih(25)
	endif
	write(lout,1007)
	write(lout,1053) ih(5)
	if (ih(5) .ne. 0) then
	  write(lout,1069) ih(15)
	  write(lout,1070) rh(3)
	  write(lout,1071) ih(26)
	endif
	write(lout,1007)
	write(lout,1054) ih(6)
	if (ih(6) .ne. 0) then
	  write(lout,1069) ih(16)
	  write(lout,1070) rh(4)
	  write(lout,1071) ih(27)
	endif
	write(lout,1007)
	write(lout,1055) ih(7)
	if (ih(7) .ne. 0) then
	  write(lout,1069) ih(17)
	  write(lout,1070) rh(5)
	  write(lout,1071) ih(28)
	endif
	write(lout,1007)
	write(lout,1056) ih(8)
	if (ih(8) .ne. 0) then
	  write(lout,1069) ih(18)
	  write(lout,1070) rh(6)
	  write(lout,1071) ih(29)
	endif
	write(lout,1007)
	write(lout,1057) ih(9)
	if (ih(9) .ne. 0) then
	  write(lout,1069) ih(19)
	  write(lout,1070) rh(7)
	  write(lout,1071) ih(30)
	endif
	write(lout,1007)
	write(lout,1058) ih(10)
	if (ih(10) .ne. 0) then
	  write(lout,1069) ih(20)
	  write(lout,1070) rh(8)
	  write(lout,1071) ih(31)
	endif
	write(lout,1007)
	write(lout,1059) ih(11)
	if (ih(11) .ne. 0) then
	  write(lout,1069) ih(21)
	  write(lout,1070) rh(9)
	  write(lout,1071) ih(32)
	endif
	write(lout,1007)
	write(lout,1060) ih(12)
	if (ih(12) .ne. 0) then
	  write(lout,1069) ih(22)
	  write(lout,1070) rh(11)
	  write(lout,1071) ih(33)
	endif
	write(lout,1021)
       endif

       if (idoku .lt. 0) then
	write(lout,1000)
	write(lout,1002) ilin(30)
	write(lout,1004) ilin(31)
	write(lout,1008) ilin(32)
	write(lout,1041) lmat
	write(lout,1042) ilin(33)
	write(lout,1043) lindex
	write(lout,1044) ilin(34)
	write(lout,1045) ih(1)
	write(lout,1006) ia2
	write(lout,1030) cachsz
	write(lout,1031) cusfac
	write(lout,1032) maxnvp
	write(lout,1048) ih(2)
	write(lout,1069) ih(22)
	write(lout,1070) rh(10)
	write(lout,1071) ih(33)
	write(lout,1007)
	write(lout,1252) ih(3)
	if (ih(3) .ne. 0) then
	  write(lout,1010) rh(1)
	  write(lout,1211) ih(24)
	endif
	write(lout,1052) ih(4)
	if (ih(4) .ne. 0) then
	  write(lout,1069) ih(14)
	  write(lout,1070) rh(2)
	  write(lout,1071) ih(25)
	endif
	write(lout,1007)
	write(lout,1053) ih(5)
	if (ih(5) .ne. 0) then
	  write(lout,1069) ih(15)
	  write(lout,1070) rh(3)
	  write(lout,1071) ih(26)
	endif
	write(lout,1007)
	write(lout,1054) ih(6)
	if (ih(6) .ne. 0) then
	  write(lout,1069) ih(16)
	  write(lout,1070) rh(4)
	  write(lout,1071) ih(27)
	endif
	write(lout,1007)
	write(lout,1055) ih(7)
	if (ih(7) .ne. 0) then
	  write(lout,1069) ih(17)
	  write(lout,1070) rh(5)
	  write(lout,1071) ih(28)
	endif
	write(lout,1007)
	write(lout,1056) ih(8)
	if (ih(8) .ne. 0) then
	  write(lout,1069) ih(18)
	  write(lout,1070) rh(6)
	  write(lout,1071) ih(29)
	endif
	write(lout,1007)
	write(lout,1057) ih(9)
	if (ih(9) .ne. 0) then
	  write(lout,1069) ih(19)
	  write(lout,1070) rh(7)
	  write(lout,1071) ih(30)
	endif
	write(lout,1007)
	write(lout,1058) ih(10)
	if (ih(10) .ne. 0) then
	  write(lout,1069) ih(20)
	  write(lout,1070) rh(8)
	  write(lout,1071) ih(31)
	endif
	write(lout,1007)
	write(lout,1059) ih(11)
	if (ih(11) .ne. 0) then
	  write(lout,1069) ih(21)
	  write(lout,1070) rh(9)
	  write(lout,1071) ih(32)
	endif
	write(lout,1007)
	write(lout,1060) ih(12)
	if (ih(12) .ne. 0) then
	  write(lout,1069) ih(22)
	  write(lout,1070) rh(11)
	  write(lout,1071) ih(33)
	endif
	write(lout,1021)
       endif
      endif

!***********************************************************************
!**    End of Calculation                                            ***
!***********************************************************************

 1000 format(/'  LSOLPP STATISTICS: ')
 1001 format('   Dimension of matrix array (lmat) ...............:',i10)
 1002 format('   Size of (global) matrix  . . . . . . . . . . . .:',i10)
 1003 format('   Dimension of index array (lindex) ..............:',i10)
 1004 format('   Size of (global) index array . . . . . . . . . .:',i10)
 1005 format('   First Dimension of info array (ia1) ............:',i10)
 1006 format('   Second Dimension of info array (ia2) . . . . . .:',i10)
 1101 format('   Dimension of preconditioning array (lprec) .....:',i10)
 1102 format('   Dimension of prec. index and info array (liprec):',i10)
 1103 format('   Dimension of (local) prec.-matrix array (lpmat) :',i10)
 1104 format('   Size of preconditioning matrix U . . . . . . . .:',i10)
 1105 format('   Size of preconditioning matrix L ...............:',i10)
 1106 format('   Gauss factor (gaussfac) is set to  . . . . . . .:',
     &           f10.1)
!1107 format('   Gauss factor (gaussfac) must be set (min.) to ..:',
!    &           f10.1)
 1108 format('   First Dimension of info array (ia1LU)  . . . . .:',i10)
 1109 format('   Second Dimension of info array (ia2LU) .........:',i10)
 1110 format('   Fill-in factor is ..............................:',
     &           f10.1)
 1111 format('   Size of global preconditioning matrix U  . . . .:',i10)
 1112 format('   Size of global preconditioning matrix L ........:',i10)
 1113 format('   Size of maximal local preconditioning matrix U .:',i10)
 1114 format('   Size of maximal local preconditioning matrix L .:',i10)
 1007 format('   -------------------------------------------------')
 1008 format('   Number of (global) vector terms ................:',i10)
 1009 format('   Minimal vector length . . . . . . . . . . . . . :',i10)
 1010 format('   Average vector length ..........................:',
     &       f10.2)
 1011 format('   Maximal vector length . . . . . . . . . . . . . :',i10)
 1211 format('   Number of non-zero elements . . . . . . . . . . :',i10)
 1212 format('   Number of (global) vector terms of type 1 ......:',i10)
 1012 format('   Number of (global) vector terms of type 2 ......:',i10)
 1013 format('   Number of (global) vector terms of type 3 ......:',i10)
 1014 format('   Number of (global) vector terms of type 4 ......:',i10)
 1015 format('   Number of (global) vector terms of type 5 ......:',i10)
 1016 format('   Number of (global) vector terms of type 6 ......:',i10)
 1017 format('   Number of (global) vector terms of type 7 ......:',i10)
 1018 format('   Number of (global) vector terms of type 8 ......:',i10)
 1019 format('   Number of (global) vector terms of type 9 ......:',i10)
 1020 format('   Number of (global) vector terms of type 11 .....:',i10)
 1021 format('   -------------------------------------------------'//)
 1030 format('   Size of the cache in 64-bit words ..............:',i10)
 1031 format('   Cache use factor (greater 0 and less or equal 1):',
     &       f10.2)
 1032 format('   Number of cache stripes per matrix (block) . . .:',i10)
 1041 format('   Dimension of local matrix array (lmat) .........:',i10)
 1042 format('   Size of local matrix . . . . . . . . . . . . . .:',i10)
 1043 format('   Dimension of local index array (lindex) ........:',i10)
 1044 format('   Size of local index array . . . . . . . . . . . :',i10)
 1045 format('   Max. first dimension of local info array (ia1)  :',i10)
 1048 format('   Maximal number of local vector terms ...........:',i10)
 1252 format('   Maximal number of local vector terms of type 1  :',i10)
 1052 format('   Maximal number of local vector terms of type 2  :',i10)
 1053 format('   Maximal number of local vector terms of type 3  :',i10)
 1054 format('   Maximal number of local vector terms of type 4  :',i10)
 1055 format('   Maximal number of local vector terms of type 5  :',i10)
 1056 format('   Maximal number of local vector terms of type 6  :',i10)
 1057 format('   Maximal number of local vector terms of type 7  :',i10)
 1058 format('   Maximal number of local vector terms of type 8  :',i10)
 1059 format('   Maximal number of local vector terms of type 9  :',i10)
 1060 format('   Maximal number of local vector terms of type 11 :',i10)
 1069 format('   Maximal minimal vector length . . . . . . . . . :',i10)
 1070 format('   Maximal average vector length ..................:',
     &       f10.2)
 1071 format('   Maximal vector length (over all processors) . . :',i10)
       r e t u r n
!---- END OF LL8STATP --------------------------------------------------
       e    n    d
